/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/
#ifndef __SAMPLE_H__
#define __SAMPLE_H__

#include "idt82V3911.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*---------------------------------------------------------------------
 * F U N C T I O N   P R O T O T Y P E S
 *---------------------------------------------------------------------*/
/**
 * @brief Sample configuration 1
 *
 *      IN3 19.44MHz -> DPLL #1 -> OUT6 25 MHz
 *      IN3 19.44MHz -> DPLL #1 -> OUT7 125 MHz
 *      IN4 25MHz    -> DPLL #2 -> OUT2 19.44 MHz
 *      APLL1 XTAL1 at 25MHz
 *
 * @param hdlr Device driver handle
 */
void IDTSample_ConfigureSampleConfig1(T_idtDrvHdlr hdlr);

/**
 * @brief Sample configuration 2
 *
 *      IN3 25 MHz    -> DPLL #1 -> OUT1 19.44 MHz
 *      IN3 25 MHz    -> DPLL #1 -> OUT3 77.76 MHz
 *                       DPLL #1 -> OUT6 155.52 MHz
 *      IN4 19.44 MHz -> DPLL #2 -> OUT8 25 MHz
 *      APLL1 XTAL1 at 24.8832MHz
 *      APLL2 XTAL2 at 25MHz
 *
 * @param hdlr Device driver handle
 */
void IDTSample_ConfigureSampleConfig2(T_idtDrvHdlr hdlr);

/**
 * @brief Sample configuration 3
 *
 *      IN3 25 MHz    -> DPLL #1 -> OUT1 19.44 MHz
 *      IN4 25 MHz    -> DPLL #1 -> OUT8 622.08 MHz
 *      IN5 19.44 MHz -> DPLL #2 -> OUT6 25 MHz
 *      APLL1 XTAL1 at 25MHz
 *      APLL2 XTAL2 at 24.8832MHz
 *      Hitless switch is enabled across input ports { IN3, IN4 }
 *
 * @param hdlr Device driver handle
 */
void IDTSample_ConfigureSampleConfig3(T_idtDrvHdlr hdlr);

/**
 * @brief Sample configuration 4
 *
 *      IN1 10 MHz -> DPLL #1 -> OUT 6 25 MHz
 *      IN2 10 MHz -> DPLL #1 -> OUT 7 156.25 MHz
 *      IN3 25 MHz -> DPLL #2 -> OUT 1 2.048 MHz
 *      IN4 25 MHz -> DPLL #2 -> OUT 2 25 MHz
 *      APLL1 XTAL1 at 25MHz
 *      Hitless switch is enabled across input ports { IN1, IN2 }
 *
 * @param hdlr Device driver handle
 */
void IDTSample_ConfigureSampleConfig4(T_idtDrvHdlr hdlr);

/**
 * @brief Sample configuration 5
 *
 *      IN4 10 MHz    -> DPLL #1 -> OUT6 25 MHz
 *                    -> DPLL #1 -> OUT7 625 MHz
 *      EX_SYNC2 1PPS ->         -> FRSYNC 1PPS
 *      APLL1 XTAL1 at 25MHz
 *
 * @param hdlr Device driver handle
 */
void IDTSample_ConfigureSampleConfig5(T_idtDrvHdlr hdlr);

/**
 * @brief Sample configuration 6
 *
 *      IN4 6.48 MHz  -> DPLL #1 -> OUT1 19.44 MHz
 *                    -> DPLL #1 -> OUT6 155.52 MHz
 *      EX_SYNC2 8kHz ->         -> FRSYNC 8 kHz
 *      APLL1 XTAL1 at 24.8832 MHz
 *
 * @param hdlr Device driver handle
 */
void IDTSample_ConfigureSampleConfig6(T_idtDrvHdlr hdlr);

/**
 * @brief Sample configuration 7
 *
 *      IN1 25 MHz -> DPLL #1 -> OUT1 10 MHz
 *      IN3 10 MHz -> DPLL #1 -> OUT2 1PPS
 *                    DPLL #1 -> OUT3 125 MHz
 *                    DPLL #1 -> OUT4 8 KHz
 *                    DPLL #1 -> OUT5 8 KHz
 *                    DPLL #1 -> OUT6 156.25 MHz
 *      APLL1 XTAL1 at 25 MHz
 *
 * @param hdlr Device driver handle
 */
void IDTSample_ConfigureSampleConfig7(T_idtDrvHdlr hdlr);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __SAMPLE_H__ */
