/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/
#include "osNumeric.h"
#include "osAdapt.h"

#include "idt82V3911.h"
#include "access.h"

#include "ReadWrite_sim.h"


/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/

/*********************************************************************
 * F U N C T I O N    P R O T O T Y P E S
 *********************************************************************/
void Init_SimWrapper(void *usrData);
void DeInit_SimWrapper(void *usrData);
T_osUint8 ReadByte_SimWrapper(void       *usrData,
                              T_osUint8  devAddr,
                              T_osUint8  nAddr);
void WriteByte_SimWrapper(void       *usrData,
                          T_osUint8  devAddr,
                          T_osUint8  nAddr,
                          T_osUint8  nData);

/*********************************************************************
 * V A R I A B L E S    &    C O N S T A N T S
 *********************************************************************/
static T_idtApllXtal xtals[IDT_DRV_MAX_APLL_XTAL];


/* Simulated */
const T_idtDrvAccess SimAccessMethods = {
    Init_SimWrapper,
    DeInit_SimWrapper,
    ReadByte_SimWrapper,
    WriteByte_SimWrapper,
    NULL 
};

/*********************************************************************
 * F U N C T I O N    I M P L E M E N T A T I O N
 *********************************************************************/

/**
 * @brief Function to wrap both common and APLL HAL
 * initialization functions.
 * @param usrData User data optionally used for initialization.
 */
void Init_SimWrapper(void *usrData)
{
    Init_Sim(usrData);
}

/**
 * @brief Function to wrap both common and APLL HAL
 * deinitialization functions.
 * @param usrData User data optionally used for de-initialization
 */
void DeInit_SimWrapper(void *usrData)
{
    DeInit_Sim(usrData);
}

/**
 * @brief Function to wrap both common and APLL HAL read
 * register functions.
 * @param usrData User data optionally used for reading registers
 * @param devAddr Device Address (I2C addr)
 * @param nAddr Register offset
 * @return Read register
 */
T_osUint8 ReadByte_SimWrapper(void       *usrData,
                              T_osUint8  devAddr,
                              T_osUint8  nAddr)
{
    return RDByte_Sim(usrData, devAddr, nAddr);
}

/**
 * @brief Function to wrap both common and APLL HAL write
 * register functions.
 * @param usrData User data optionally used for writing registers
 * @param devAddr Device Address (I2C addr)
 * @param nAddr Register offset
 * @param nData Register value to write
 */
void WriteByte_SimWrapper(void       *usrData,
                          T_osUint8  devAddr,
                          T_osUint8  nAddr,
                          T_osUint8  nData)
{
    WRByte_Sim(usrData, devAddr, nAddr, nData);
}


/**
 * @brief Utility function to initialize device driver for simulator
 * access.
 *
 * This function is used for TCL extension library
 *
 * @param xtalList List of XTAL
 * @param coreI2CAddr Device I2C address
 * @return Device driver handle
 */
T_idtDrvHdlr IDTSample_InitDriverSimulator(T_idtApllXtalList *xtalList,
                                           T_osUint8      coreI2CAddr)
{
	T_idtDrvHdlr hdlr;
	T_osUint8 numberOfXtal = 0;
	int i;

	if (((*xtalList).xtal1Apll1) && ((*xtalList).xtal1Apll1Freq != APLL_XTAL_FREQ_MAX))
	{
		xtals[numberOfXtal].apllXtalId = APLL_XTAL_1_APLL1;
		xtals[numberOfXtal].apllXtalFreq = (*xtalList).xtal1Apll1Freq;
		numberOfXtal++;
	}

	if (((*xtalList).xtal3Apll1) && ((*xtalList).xtal3Apll1Freq != APLL_XTAL_FREQ_MAX))
	{
		xtals[numberOfXtal].apllXtalId = APLL_XTAL_3_APLL1;
		xtals[numberOfXtal].apllXtalFreq = (*xtalList).xtal3Apll1Freq;
		numberOfXtal++;
	}

	if (((*xtalList).xtal2Apll2) && ((*xtalList).xtal2Apll2Freq != APLL_XTAL_FREQ_MAX))
	{
		xtals[numberOfXtal].apllXtalId = APLL_XTAL_2_APLL2;
		xtals[numberOfXtal].apllXtalFreq = (*xtalList).xtal2Apll2Freq;
		numberOfXtal++;
	}
	
	if (((*xtalList).xtal4Apll2) && ((*xtalList).xtal4Apll2Freq != APLL_XTAL_FREQ_MAX))
	{
		xtals[numberOfXtal].apllXtalId = APLL_XTAL_4_APLL2;
		xtals[numberOfXtal].apllXtalFreq = (*xtalList).xtal4Apll2Freq;
		numberOfXtal++;
	}

	printf("%s: numberOfXtal: %d\n", __FUNCTION__, numberOfXtal);

	for (i = 0; i < numberOfXtal; i++)
	{
		printf("%s: xtal ID: %d\n", __FUNCTION__, xtals[i].apllXtalId);
		printf("%s: xtal frequency: %d\n", __FUNCTION__, xtals[i].apllXtalFreq);
	}

	hdlr = IDTGeneral_InitDriver("dev82V3911", 
                                 SimAccessMethods, 
                                 IDT_82V3911,
                                 xtals,
                                 numberOfXtal,
                                 coreI2CAddr, /* I2C Address (doesn't
                                                 matter) */
                                 1); /* Using high level API */

	OS_ASSERT_PTR_NULL(hdlr);

	IDTGeneral_InitDevice(hdlr);

	return hdlr;	
}

/**
 * @brief Utility function to reset APLL XTAL list to unconfigured value
 *
 *
 * This function is used for TCL extension library
 *
 * @param xtalList List of XTAL
 */
void IDTSample_ResetXtalList(T_idtApllXtalList *xtalList)
{
	(*xtalList).xtal1Apll1 = 0;
	(*xtalList).xtal1Apll1Freq = APLL_XTAL_FREQ_MAX;
	(*xtalList).xtal3Apll1 = 0;
	(*xtalList).xtal3Apll1Freq = APLL_XTAL_FREQ_MAX;
	(*xtalList).xtal2Apll2 = 0;
	(*xtalList).xtal2Apll2Freq = APLL_XTAL_FREQ_MAX;
	(*xtalList).xtal4Apll2 = 0;
	(*xtalList).xtal4Apll2Freq = APLL_XTAL_FREQ_MAX;
}

/**
 * @brief Utility function to change log verbosity level
 * access.
 *
 * This function is used for TCL extension library
 *
 * @param level Log verbosity level
 */
void IDTSample_SetLogVerbosity(int level)
{
	OS_SETLOGLVL(level);
}

/**
 * @brief Utility function to get current log verbosity level
 *
 * This function is used for TCL extension library
 */
int IDTSample_GetLogVerbosity(void)
{
	return OS_GETLOGLVL();
}
