/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdarg.h>

#include "Hal.h"
#include "save.h"
#include "load.h"
/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/
static FILE *filePtr;  /**< Global file pointer */
/** Table of registers to read when outputting a *rgf file */
/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

/**
 * @brief Wrapper read function to read from a file.
 * @param s String variable save to 
 * @param size Maximum size of string to save to.
 * @return pointer to string, null on error
 */
static char *IDTSample_ReadFromFile(char *s, int size)
{
    char * rc;

    rc = fgets(s,size,filePtr);
    if (feof(filePtr) || (strlen(s)==0))
        return NULL;
    return rc;
}

/**
 * @brief Wrapper write function to output to a file
 * @param format Printf format
 * @param ... Variable arguments 
 * @return number of character written
 */
static int IDTSample_WriteToFile(const char *format, ...)
{
    va_list fmtargs;
    int rc;

    va_start(fmtargs,format);
    rc = vfprintf(filePtr,format,fmtargs);
    va_end(fmtargs);

    return rc;
}

/**
 * @brief Function to output register contents of a device to standard
 * IO or to a file.
 * @param hdlr Device driver handler
 * @param filename Output file name, NULL for standard IO
 */
void IDTSample_SaveRgfFile(T_idtDrvHdlr hdlr,
                           char         *filename)
{
    if (!filename)
    {
        /* No file name specified, output to standard I/O */
        saveVersion2(printf, hdlr);
    }
    else
    {
        /* Output to specified file name */
        filePtr = fopen(filename,"w+t");
        if (!filePtr)
        {
            printf("Unable to save to file %s\n", filename);
            return;
        }

        /* Save registers to file */
        saveVersion2(IDTSample_WriteToFile, hdlr);

        fclose(filePtr);
    }
}

/**
 * @brief Function to read in a file and program device registers. 
 * @param hdlr Device driver handler
 * @param filename Input file name to load register settings from.
 */
void IDTSample_LoadRgfFile(T_idtDrvHdlr hdlr,
                           char         *filename)
{
    if (!filename) return;

    /* Open file for reading */
    filePtr = fopen(filename,"r+t");
    if (!filePtr)
    {
        printf("Unable to load file %s\n", filename);
        return;
    }

    /* Load contents of file and write to registers */
    loadVersion2(IDTSample_ReadFromFile, hdlr);

    fclose(filePtr);
}
