/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/
#include <stdlib.h>

#include "idt82V3911.h"
#include "profiles.h"
#include "inputFreq.h"
#include "outputFreq.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

/**
 * Sample Configuration 1 (see header file for description)
 */
void IDTSample_ConfigureSampleConfig1(T_idtDrvHdlr hdlr)
{
    /* Optional - Disable all outputs */
    IDTHlApi_DisableAllOutputs(hdlr);

    /* Since we are configuring priorities - let's clear default
       input priorities first */
    IDTHlApi_DisableAllInputPriorities(hdlr);

    /* DPLL #1 */
    /* ------- */
    {
        /* Configure input 3, 19.44MHz 
         * Priority 1 @ DPLL #1 
         */
        IDTHlApi_ConfigureInputFrequency(hdlr, 3, 19440);
        IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_1, 1);

        /* Configure DPLL for G8262 Option 1 */
        IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

        /* Configure output 6 (25 MHz) using DPLL #1 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 6,
                                 OutFreq_25000k,
                                 DPLL_INSTANCE_1,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG0,
                                 APLL_OUT_SIG_LVPECL,
                                 APLL_CLK_SEL_INTERNAL);

        /* Configure output 7 (125 MHz) using DPLL #1 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 7,
                                 OutFreq_125000k,
                                 DPLL_INSTANCE_1,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG0,
                                 APLL_OUT_SIG_LVPECL,
                                 APLL_CLK_SEL_INTERNAL);
    }

    /* DPLL #2 */
    /* ------- */
    {
        /* Configure input 4, 25MHz 
         * Priority 1 @ DPLL #2 
         */
        IDTHlApi_ConfigureInputFrequency(hdlr, 4, 25000);
        IDTInput_SetPriority(hdlr, 4, DPLL_INSTANCE_2, 1);

        /* Configure DPLL for G8262 Option 1 */
        IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_2);

        /* Configure output 2 (19.44MHz) using DPLL #2 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 2,
                                 OutFreq_19440k,
                                 DPLL_INSTANCE_2,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG_MAX,
                                 APLL_OUT_SIG_MAX,
                                 APLL_CLK_SEL_MAX);

        /* Optional - since port only supports CMOS */
        IDTOutput_SetOuputpuInterface(hdlr, 2, OUTPUTIF_CMOS); 
    }

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
    IDTOutput_SetOutputEnable(hdlr, 7, 1);
}

/**
 * Sample Configuration 2 (see header file for description)
 */
void IDTSample_ConfigureSampleConfig2(T_idtDrvHdlr hdlr)
{
    /* Optional - Disable all outputs */
    IDTHlApi_DisableAllOutputs(hdlr);

    /* Since we are configuring priorities - let's clear default
       input priorities first */
    IDTHlApi_DisableAllInputPriorities(hdlr);

    /* DPLL #1 */
    /* ------- */
    {
        /* Configure input 3, 25MHz 
         * Priority 1 @ DPLL #1 
         */
        IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);
        IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_1, 1);

        /* Configure DPLL for G8262 Option 1 */
        IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

        /* Configure output 1 (19.44 MHz) using DPLL #1 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 1,
                                 OutFreq_19440k,
                                 DPLL_INSTANCE_1,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG_MAX,
                                 APLL_OUT_SIG_MAX,
                                 APLL_CLK_SEL_MAX);

       /* Optional - since port only supports CMOS */
        IDTOutput_SetOuputpuInterface(hdlr, 1, OUTPUTIF_CMOS); 

        /* Configure output 3 (77.76 MHz) using DPLL #1
         * Output interface CMOS
         */
        IDTHlApi_ConfigureOutput(hdlr,
                                 3,
                                 OutFreq_77760k,
                                 DPLL_INSTANCE_1,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG_MAX,
                                 APLL_OUT_SIG_MAX,
                                 APLL_CLK_SEL_MAX);

       /* Optional - since port only supports CMOS */
        IDTOutput_SetOuputpuInterface(hdlr, 3, OUTPUTIF_CMOS); 

        /* Configure output 6 (155.52 MHz) using DPLL #1 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 6,
                                 OutFreq_155520k,
                                 DPLL_INSTANCE_1,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG0,
                                 APLL_OUT_SIG_LVPECL,
                                 APLL_CLK_SEL_INTERNAL);
    }

    /* DPLL #2 */
    /* ------- */
    {
        /* Configure input 4, 19.44 MHz 
         * Priority 1 @ DPLL #2 
         */
        IDTHlApi_ConfigureInputFrequency(hdlr, 4, 19440);
        IDTInput_SetPriority(hdlr, 4, DPLL_INSTANCE_2, 1);

        /* Configure DPLL for G8262 Option 1 */
        IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_2);

        /* Configure output 8 (25 MHz) using DPLL #2 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 8,
                                 OutFreq_25000k,
                                 DPLL_INSTANCE_2,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG0,
                                 APLL_OUT_SIG_LVPECL,
                                 APLL_CLK_SEL_INTERNAL);
    }

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
    IDTOutput_SetOutputEnable(hdlr, 8, 1);
}

/**
 * Sample Configuration 3 (see header file for description)
 */
void IDTSample_ConfigureSampleConfig3(T_idtDrvHdlr hdlr)
{
    /* Optional - Disable all outputs */
    IDTHlApi_DisableAllOutputs(hdlr);

    /* Since we are configuring priorities - let's clear default
       input priorities first */
    IDTHlApi_DisableAllInputPriorities(hdlr);

    /* DPLL #1 */
    /* ------- */
    {
        /* Configure input 3, 25MHz 
         * Priority 1 @ DPLL #1 
         */
        IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);
        IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_1, 1);

        /* Configure input 4, 25MHz 
         * Priority 2 @ DPLL #1 
         */
        IDTHlApi_ConfigureInputFrequency(hdlr, 4, 25000);
        IDTInput_SetPriority(hdlr, 4, DPLL_INSTANCE_1, 2);

        /* Hitless switching ENABLED */
        IDTDpll_SetHitlessSwitchingFeature(hdlr, 
                                           1, /* Enable */
                                           0); /* Allow revertive
                                                  switching */

        /* Configure DPLL for G8262 Option 1 */
        IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

        /* Configure output 1 (19.44 MHz) using DPLL #1 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 1,
                                 OutFreq_19440k,
                                 DPLL_INSTANCE_1,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG_MAX,
                                 APLL_OUT_SIG_MAX,
                                 APLL_CLK_SEL_MAX);

        /* Optional - since port only supports CMOS */
        IDTOutput_SetOuputpuInterface(hdlr, 1, OUTPUTIF_CMOS); 

        /* Configure output 8 (622.08 MHz) using DPLL #1 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 8,
                                 OutFreq_622080k,
                                 DPLL_INSTANCE_1,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG0,
                                 APLL_OUT_SIG_LVPECL,
                                 APLL_CLK_SEL_INTERNAL);
    }

    /* DPLL #2 */
    /* ------- */
    {
        /* Configure input 5, 19.44 MHz 
         * Priority 1 @ DPLL #2 
         */
        IDTHlApi_ConfigureInputFrequency(hdlr, 5, 19440);
        IDTInput_SetPriority(hdlr, 5, DPLL_INSTANCE_2, 1);

        /* Configure DPLL for G8262 Option 1 */
        IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_2);

        /* Configure output 6 (25 MHz) using DPLL #2 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 6,
                                 OutFreq_25000k,
                                 DPLL_INSTANCE_2,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG0,
                                 APLL_OUT_SIG_LVPECL,
                                 APLL_CLK_SEL_INTERNAL);
    }

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
    IDTOutput_SetOutputEnable(hdlr, 8, 1);
}

/**
 * Sample Configuration 4 (see header file for description)
 */
void IDTSample_ConfigureSampleConfig4(T_idtDrvHdlr hdlr)
{
    /* Optional - Disable all outputs */
    IDTHlApi_DisableAllOutputs(hdlr);

    /* Since we are configuring priorities - let's clear default
       input priorities first */
    IDTHlApi_DisableAllInputPriorities(hdlr);

    /* DPLL #1 */
    /* ------- */
    {
        /* Configure input 3, 10MHz 
         * Priority 1 @ DPLL #1 
         */
        IDTHlApi_ConfigureInputFrequency(hdlr, 1, 10000);
        IDTInput_SetPriority(hdlr, 1, DPLL_INSTANCE_1, 1);

        /* Configure input 4, 10MHz 
         * Priority 2 @ DPLL #1 
         */
        IDTHlApi_ConfigureInputFrequency(hdlr, 2, 10000);
        IDTInput_SetPriority(hdlr, 2, DPLL_INSTANCE_1, 2);

        /* Hitless switching ENABLED */
        IDTDpll_SetHitlessSwitchingFeature(hdlr, 
                                           1, /* Enable */
                                           0); /* Allow revertive
                                                  switching */

        /* Configure DPLL for G8262 Option 1 */
        IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

        /* Configure output 6 (25 MHz) using DPLL #1 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 6,
                                 OutFreq_25000k,
                                 DPLL_INSTANCE_1,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG0,
                                 APLL_OUT_SIG_LVPECL,
                                 APLL_CLK_SEL_INTERNAL);

        /* Configure output 7 (156.25 MHz) using DPLL #1 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 7,
                                 OutFreq_156250k,
                                 DPLL_INSTANCE_1,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG0,
                                 APLL_OUT_SIG_LVPECL,
                                 APLL_CLK_SEL_INTERNAL);
    }

    /* DPLL #2 */
    /* ------- */
    {
        /* Configure input 3, 25 MHz 
         * Priority 1 @ DPLL #2
         */
        IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);
        IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_2, 1);

        /* Configure input 4, 25 MHz 
         * Priority 2 @ DPLL #2
         */
        IDTHlApi_ConfigureInputFrequency(hdlr, 4, 25000);
        IDTInput_SetPriority(hdlr, 4, DPLL_INSTANCE_2, 2);

        /* Configure DPLL for G8262 Option 1 */
        IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_2);

        /* Configure output 1 (2.048 MHz) using DPLL #2 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 1,
                                 OutFreq_2048k,
                                 DPLL_INSTANCE_2,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG_MAX,
                                 APLL_OUT_SIG_MAX,
                                 APLL_CLK_SEL_MAX);

        /* Optional - since port only supports CMOS */
        IDTOutput_SetOuputpuInterface(hdlr, 1, OUTPUTIF_CMOS); 

        /* Configure output 2 (25 MHz) using DPLL #2 */
        IDTHlApi_ConfigureOutput(hdlr,
                                 2,
                                 OutFreq_25000k,
                                 DPLL_INSTANCE_2,
                                 sonetGigeMode_Matching,
                                 APLL_CONFIG_MAX,
                                 APLL_OUT_SIG_MAX,
                                 APLL_CLK_SEL_MAX);

        /* Optional - since port only supports CMOS */
        IDTOutput_SetOuputpuInterface(hdlr, 2, OUTPUTIF_CMOS); 
    }

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
    IDTOutput_SetOutputEnable(hdlr, 7, 1);
}

/**
 * Sample Configuration 5 (see header file for description)
 */
void IDTSample_ConfigureSampleConfig5(T_idtDrvHdlr hdlr)
{
    /* Optional - Disable all outputs */
    IDTHlApi_DisableAllOutputs(hdlr);

    /* Since we are configuring priorities - let's clear default
       input priorities first */
    IDTHlApi_DisableAllInputPriorities(hdlr);

    /* Configure input 4, 10MHz 
     * Priority 1 @ DPLL #1 
     */
    IDTHlApi_ConfigureInputFrequency(hdlr, 4, 10000);
    IDTInput_SetPriority(hdlr, 4, DPLL_INSTANCE_1, 1);

    /* Configure EX_SYNC1
     */
    IDTInput_SetSyncFrequency(hdlr, SYNC_1PPS);
    IDTInput_SetSyncSampling(hdlr,
                             2, /* EX_SYNC2 */
                             externalSyncSampling_onTarget); /* 0 UI */
    IDTInput_SetSyncEdge(hdlr,
                         externalSyncEdge_FallingEdge); /* Align to
                                                           falling
                                                           edge */
    IDTInput_SetSyncBypass(hdlr,
                           externalSyncBypass_AutoSel); /* Auto Sync
                                                           to IN4 */
    IDTInput_SetSyncEnable(hdlr,
                           externalSyncEnable_Enable); /* Eanble */

    /* Configure DPLL for G8262 Option 1 */
    IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

    /* Configure output 6 (25 MHz) using DPLL #1 */
    IDTHlApi_ConfigureOutput(hdlr,
                             6,
                             OutFreq_25000k,
                             DPLL_INSTANCE_1,
                             sonetGigeMode_Matching,
                             APLL_CONFIG0,
                             APLL_OUT_SIG_LVPECL,
                             APLL_CLK_SEL_INTERNAL);

    /* Configure output 7 (625 MHz) using DPLL #1 */
    IDTHlApi_ConfigureOutput(hdlr,
                             7,
                             OutFreq_625000k,
                             DPLL_INSTANCE_1,
                             sonetGigeMode_Matching,
                             APLL_CONFIG0,
                             APLL_OUT_SIG_LVPECL,
                             APLL_CLK_SEL_INTERNAL);

    /* Configure FRSYNC
     *    1 PPS, Non-inverted pulse sync to raising edge of EX_SYNC
     */
    IDTOutput_SetFrameSync(hdlr, frameSync_FRSYNC,
                           1, /* Enable */
                           0, /* Non-inverted */
                           1); /* Pulse */
    IDTOutput_SetFrameSyncPulseEdge(hdlr,
                                    1); /* Rising Edge */

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
    IDTOutput_SetOutputEnable(hdlr, 7, 1);
}

/**
 * Sample Configuration 6 (see header file for description)
 */
void IDTSample_ConfigureSampleConfig6(T_idtDrvHdlr hdlr)
{
    /* Optional - Disable all outputs */
    IDTHlApi_DisableAllOutputs(hdlr);

    /* Since we are configuring priorities - let's clear default
       input priorities first */
    IDTHlApi_DisableAllInputPriorities(hdlr);

    /* Configure input 4, 6.48MHz 
     * Priority 1 @ DPLL #1 
     */
    IDTHlApi_ConfigureInputFrequency(hdlr, 4, 6480);
    IDTInput_SetPriority(hdlr, 4, DPLL_INSTANCE_1, 1);

    /* Configure EX_SYNC1
     */
    IDTInput_SetSyncFrequency(hdlr, SYNC_8kHz);
    IDTInput_SetSyncSampling(hdlr,
                             2, /* EX_SYNC2 */
                             externalSyncSampling_onTarget); /* 0 UI */
    IDTInput_SetSyncEdge(hdlr,
                         externalSyncEdge_FallingEdge); /* Align to
                                                           falling
                                                           edge */
    IDTInput_SetSyncBypass(hdlr,
                           externalSyncBypass_AutoSel); /* Auto Sync
                                                           to IN4 */
    IDTInput_SetSyncEnable(hdlr,
                           externalSyncEnable_Enable); /* Enable */

    /* Configure DPLL for G8262 Option 1 */
    IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

    /* Configure output 1 (19.44 MHz) using DPLL #1 */
    IDTHlApi_ConfigureOutput(hdlr,
                             1,
                             OutFreq_19440k,
                             DPLL_INSTANCE_1,
                             sonetGigeMode_Matching,
                             APLL_CONFIG_MAX,
                             APLL_OUT_SIG_MAX,
                             APLL_CLK_SEL_MAX);

    /* Optional - since port only supports CMOS */
    IDTOutput_SetOuputpuInterface(hdlr, 1, OUTPUTIF_CMOS);

    /* Configure output 6 (155.52 MHz) using DPLL #1 */
    IDTHlApi_ConfigureOutput(hdlr,
                             6,
                             OutFreq_155520k,
                             DPLL_INSTANCE_1,
                             sonetGigeMode_Matching,
                             APLL_CONFIG0,
                             APLL_OUT_SIG_LVPECL,
                             APLL_CLK_SEL_INTERNAL);

    /* Configure MFRSYNC
     *    8kHz, Non-inverted pulse sync to raising edge of EX_SYNC
     */
    IDTOutput_SetFrameSync(hdlr, frameSync_MFRSYNC,
                           1, /* Enable */
                           0, /* Non-inverted */
                           1); /* Pulse */
    IDTOutput_SetFrameSyncPulseEdge(hdlr,
                                    1); /* Rising Edge */

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
}

/**
 * Sample Configuration 7 (see header file for description)
 */
void IDTSample_ConfigureSampleConfig7(T_idtDrvHdlr hdlr)
{
    /* Optional - Disable all outputs */
    IDTHlApi_DisableAllOutputs(hdlr);

    /* Since we are configuring priorities - let's clear default
       input priorities first */
    IDTHlApi_DisableAllInputPriorities(hdlr);

    /* Configure input 1, 25 MHz 
     * Priority 1 @ DPLL #1 
     */
    IDTHlApi_ConfigureInputFrequency(hdlr, 1, 25000);
    IDTInput_SetPriority(hdlr, 1, DPLL_INSTANCE_1, 1);

    /* Configure input 3, 10 MHz 
     * Priority 2 @ DPLL #1 
     */
    IDTHlApi_ConfigureInputFrequency(hdlr, 3, 10000);
    IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_1, 2);

    /* Configure DPLL for G8262 Option 1 */
    IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

    /* Configure output 1 (10 MHz) using DPLL #1 */
    IDTHlApi_ConfigureOutput(hdlr,
                             1,
                             OutFreq_10000k,
                             DPLL_INSTANCE_1,
                             sonetGigeMode_Matching,
                             APLL_CONFIG_MAX,
                             APLL_OUT_SIG_MAX,
                             APLL_CLK_SEL_MAX);

    /* Optional - since port only supports CMOS */
    IDTOutput_SetOuputpuInterface(hdlr, 1, OUTPUTIF_CMOS);

    /* Configure output 2 (1 PPS) using DPLL #1 */
    IDTHlApi_ConfigureOutput(hdlr,
                             2,
                             OutFreq_1,
                             DPLL_INSTANCE_1,
                             sonetGigeMode_Matching,
                             APLL_CONFIG_MAX,
                             APLL_OUT_SIG_MAX,
                             APLL_CLK_SEL_MAX);

    /* Optional - since port only supports CMOS */
    IDTOutput_SetOuputpuInterface(hdlr, 2, OUTPUTIF_CMOS); 

    /* Configure output 3 (125 MHz) using DPLL #1 */
    IDTHlApi_ConfigureOutput(hdlr,
                             3,
                             OutFreq_125000k,
                             DPLL_INSTANCE_1,
                             sonetGigeMode_Matching,
                             APLL_CONFIG_MAX,
                             APLL_OUT_SIG_MAX,
                             APLL_CLK_SEL_MAX);

    /* Optional - since port only supports CMOS */
    IDTOutput_SetOuputpuInterface(hdlr, 3, OUTPUTIF_CMOS); 

    /* Configure output 4 (8 KHz) using DPLL #1 */
    IDTHlApi_ConfigureOutput(hdlr,
                             4,
                             OutFreq_8k,
                             DPLL_INSTANCE_1,
                             sonetGigeMode_Matching,
                             APLL_CONFIG_MAX,
                             APLL_OUT_SIG_MAX,
                             APLL_CLK_SEL_MAX);

    /* Optional - since port only supports CMOS */
    IDTOutput_SetOuputpuInterface(hdlr, 4, OUTPUTIF_CMOS);

    /* Configure output 5 (8 KHz) using DPLL #1 */
    IDTHlApi_ConfigureOutput(hdlr,
                             5,
                             OutFreq_8k,
                             DPLL_INSTANCE_1,
                             sonetGigeMode_Matching,
                             APLL_CONFIG_MAX,
                             APLL_OUT_SIG_MAX,
                             APLL_CLK_SEL_MAX);

    /* Optional - since port only supports CMOS */
    IDTOutput_SetOuputpuInterface(hdlr, 5, OUTPUTIF_CMOS);

    /* Configure output 6 (156.25 MHz) using DPLL #1 */
    IDTHlApi_ConfigureOutput(hdlr,
                             6,
                             OutFreq_156250k,
                             DPLL_INSTANCE_1,
                             sonetGigeMode_Matching,
                             APLL_CONFIG0,
                             APLL_OUT_SIG_LVPECL,
                             APLL_CLK_SEL_INTERNAL);

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 1, 1);
    IDTOutput_SetOutputEnable(hdlr, 2, 1);
    IDTOutput_SetOutputEnable(hdlr, 3, 1);
    IDTOutput_SetOutputEnable(hdlr, 4, 1);
    IDTOutput_SetOutputEnable(hdlr, 5, 1);
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
}
