/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/
#ifndef __APLL_FREQ_PROFILE_H__
#define __APLL_FREQ_PROFILE_H__

#include "osNumeric.h"

#include "Define.h"
#include "ApllTypeDef.h"

#ifdef __cplusplus
extern "C"
{
#endif

/** \addtogroup APLL
 *  @{
 */

/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/

/**
 * @brief Enumerated type listing supported input frequencies
 */
typedef enum
{
    APLL_INPUT_FREQ_19440KHz, /**< 19.4 MHz */
    APLL_INPUT_FREQ_25000KHz, /**< 25.0 MHz */
    APLL_INPUT_FREQ_77760KHz, /**< 77.76 MHz */
    APLL_INPUT_FREQ_125000KHz, /**< 125 MHz */
    APLL_INPUT_FREQ_155520KHz, /**< 155.52 MHz */
    APLL_INPUT_FREQ_25781_DOT_25KHz, /**< 25.78125 MHz */
    APLL_INPUT_FREQ_312500KHz, /**< 312.5 MHz */
    APLL_INPUT_FREQ_128906_DOT_25KHz, /**< 128.90625 MHz */
    APLL_INPUT_FREQ_156250KHz, /**< 156.25 MHz */
    APLL_INPUT_FREQ_8KHz, /**< 8 KHz */
    APLL_INPUT_FREQ_MAX  /**< \internal Not Used */
} T_idtApllInputFreqType;

/**
 * @brief Enumerated type listing supported output frequencies
 */
typedef enum
{
    APLL_OUTPUT_FREQ_24883_DOT_2KHz, /**< 24.8832 MHz */
    APLL_OUTPUT_FREQ_25000KHz, /**< 25.0 MHz */
    APLL_OUTPUT_FREQ_25781_DOT_25KHz, /**< 25.78125 MHz */
    APLL_OUTPUT_FREQ_77760KHz, /**< 77.76 MHz */
    APLL_OUTPUT_FREQ_78125KHz, /**< 78.125 MHz */
    APLL_OUTPUT_FREQ_80566_DOT_40625KHz, /**< 80.56640625 MHz */
    APLL_OUTPUT_FREQ_124416KHz, /**< 124.416 MHz */
    APLL_OUTPUT_FREQ_125000KHz, /**< 125.0 MHz */
    APLL_OUTPUT_FREQ_128906_DOT_25KHz, /**< 128.90625 MHz */
    APLL_OUTPUT_FREQ_155520KHz, /**< 155.52 MHz */
    APLL_OUTPUT_FREQ_156250KHz, /**< 156.250 MHz */
    APLL_OUTPUT_FREQ_161132_DOT_8125KHz, /**< 161.1328125 MHz */
    APLL_OUTPUT_FREQ_311040KHz, /**< 311.040 MHz */
    APLL_OUTPUT_FREQ_312500KHz, /**< 312.50 MHz */
    APLL_OUTPUT_FREQ_322265_DOT_625KHz, /**< 322.265625 MHz */
    APLL_OUTPUT_FREQ_622080KHz, /**< 622.080 MHz */
    APLL_OUTPUT_FREQ_625000KHz, /**< 625.0 MHz */
    APLL_OUTPUT_FREQ_644531_DOT_25KHz, /**< 644.53125 MHz */
    APLL_OUTPUT_FREQ_MAX /**< \internal Not used */
} T_idtApllOutputFreqType;

/**
 * @brief Type definition for mapping output frequency to APLL configuration
 */
typedef struct
{
    T_idtApllInputFreqType          apllInputFreq; /**< Input frequency */
    T_idtApllDividerValue			apllPreDivider; /**< Pre- (input) divider */
    T_idtApllDividerValue			apllFbDivider; /**< Feedback (oscillator) divider */
    T_idtApllXtalType				apllXtalFreq; /**< Oscillator type */
    T_idtApllDividerValue			apllOutputDivider;  /**< Output divider */
    T_idtApllOutputFreqType		    apllOutputFeq; /**< Output frequency type */
    T_idtApllOutputSupportedType	apllOutputs; /**< Output supported (A or B or both) */
} T_idtApllFreqMapping;

/*********************************************************************
 * F U N C T I O N    P R O T O T Y P E S
 *********************************************************************/
 
const T_idtApllFreqMapping* IDTApll_GetConfigListByFreq(T_idtApllOutputFreqType apllOutputFreq);

const T_idtApllFreqMapping* IDTApll_GetConfigListByXtalType(T_idtApllXtalType apllXtalFreq);

T_osBool IDTApll_NullFreqTableEntry(const T_idtApllFreqMapping *apllFreqMapping);

const T_idtApllFreqMapping* IDTApll_GetCurrentApllFreqConfig(T_idtApllId apllId, T_idtApllOutput output);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif /* __cplusplus__ */
#endif /* __APLL_FREQ_PROFILE_H__ */
