/*********************************************************************
 * Copyright (c) 2012-2013 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#ifndef __PUMA_APLL_REGISTER_DEF_H__
#define __PUMA_APLL_REGISTER_DEF_H__

#ifdef __cplusplus
extern "C"
{
#endif
/*! \file RegisterDef.h
 *  @brief This header file contains register and bitfield information for
 *  accessing device.
 *  \see Datasheet for detailed register descriptions.
 */

/** \defgroup RegistersDef Register definitions
 *
 * @{ */


/**
 * @brief Enumerated type listing register offsers and bit field
 * constants.
 */
enum {

/*======================Register Definitions==========================*/
    REG_CONTROL = 0x00, /**< General Control Register */
    REG_CLK_SEL = 0x01, /**< Input Clock Select */
    REG_PDSEL0_LSB = 0x02, /**< Configuration 0 Pre-divider LSB */
    REG_PDSEL0_MSB = 0x03, /**< Configuration 0 Pre-divider MSB */
    REG_PDSEL1_LSB = 0x04, /**< Configuration 1 Pre-divider LSB */
    REG_PDSEL1_MSB = 0x05, /**< Configuration 1 Pre-divider MSB */
    REG_M0_LSB = 0x06, /**< Configuration 0 Feedback divider LSB */
    REG_M0_MSB = 0x07, /**< Configuration 0 Feedback divider MSB */
    REG_M1_LSB = 0x08, /**< Configuration 1 Feedback divider LSB */
    REG_M1_MSB = 0x09, /**< Configuration 1 Feedback divider MSB */
    REG_ODBSELA0 = 0x0A, /**< QA Output Divider Config 0 */
    REG_ODBSELA1 = 0x0B, /**< QA Output Divider Config 1 */
    REG_ODBSELB0 = 0x0C, /**< QB Output Divider Config 0 */
    REG_ODBSELB1 = 0x0D, /**< QB Output Divider Config 1 */
    REG_VCXO_SEL = 0x0E, /**< VCXO Crystal Select */
    REG_CONFIG_QA = 0x0F, /**< Output Configuration QA */
    REG_CONFIG_QB = 0x10, /**< Output Configuration QB */

/*==================End of Register Definitions=======================*/

/*======================Bitfield Definitions==========================*/
    /**
     * CONTROL
     */
    BF_CONFIG_SIZE = 1,
    BF_CONFIG_MASK = 0x02,
    BF_CONFIG_LSB = 1,

    /**
     * CLK_SEL
     */
    BF_CLK_SEL_SIZE = 1,
    BF_CLK_SEL_MASK = 0x01,
    BF_CLK_SEL_LSB = 0,

    /**
     * PDSEL0_LSB
     */
    BF_PDSEL0_LSB_SIZE = 8,
    BF_PDSEL0_LSB_MASK = 0xFF,
    BF_PDSEL0_LSB_LSB = 0,

    /**
     * PDSEL0_MSB
     */
    BF_PDSEL0_MSB_SIZE = 7,
    BF_PDSEL0_MSB_MASK = 0x7F,
    BF_PDSEL0_MSB_LSB = 0,

    /**
     * PDSEL1_LSB
     */
    BF_PDSEL1_LSB_SIZE = 8,
    BF_PDSEL1_LSB_MASK = 0xFF,
    BF_PDSEL1_LSB_LSB = 0,

    /**
     * PDSEL1_MSB
     */
    BF_PDSEL1_MSB_SIZE = 7,
    BF_PDSEL1_MSB_MASK = 0x7F,
    BF_PDSEL1_MSB_LSB = 0,

    /**
     * M0_LSB
     */
    BF_M0_LSB_SIZE = 8,
    BF_M0_LSB_MASK = 0xFF,
    BF_M0_LSB_LSB = 0,

    /**
     * M0_MSB
     */
    BF_M0_MSB_SIZE = 7,
    BF_M0_MSB_MASK = 0x7F,
    BF_M0_MSB_LSB = 0,

    /**
     * M1_LSB
     */
    BF_M1_LSB_SIZE = 8,
    BF_M1_LSB_MASK = 0xFF,
    BF_M1_LSB_LSB = 0,

    /**
     * M1_MSB
     */
    BF_M1_MSB_SIZE = 7,
    BF_M1_MSB_MASK = 0x7F,
    BF_M1_MSB_LSB = 0,

    /**
     * ODBSELA0
     */
    BF_ODBSELA0_SIZE = 3,
    BF_ODBSELA0_MASK = 0x07,
    BF_ODBSELA0_LSB = 0,

    /**
     * ODBSELA1
     */
    BF_ODBSELA1_SIZE = 3,
    BF_ODBSELA1_MASK = 0x07,
    BF_ODBSELA1_LSB = 0,

    /**
     * ODBSELB0
     */
    BF_ODBSELB0_SIZE = 3,
    BF_ODBSELB0_MASK = 0x07,
    BF_ODBSELB0_LSB = 0,

    /**
     * ODBSELB1
     */
    BF_ODBSELB1_SIZE = 3,
    BF_ODBSELB1_MASK = 0x07,
    BF_ODBSELB1_LSB = 0,

    /**
     * VCXO_SEL
     */
    BF_SEL_DOUBLE_SIZE = 1,
    BF_SEL_DOUBLE_MASK = 0x80,
    BF_SEL_DOUBLE_LSB = 7,

    /**
     * VCXO_SEL
     */
    BF_MR_SYNC_SIZE = 1,
    BF_MR_SYNC_MASK = 0x40,
    BF_MR_SYNC_LSB = 6,

    /**
     * VCXO_SEL
     */
    BF_BYPASS_SIZE = 1,
    BF_BYPASS_MASK = 0x20,
    BF_BYPASS_LSB = 5,

    /**
     * VCXO_SEL
     */
    BF_SHUTOFF_SIZE = 1,
    BF_SHUTOFF_MASK = 0x10,
    BF_SHUTOFF_LSB = 4,

    /**
     * VCXO_SEL
     */
    BF_VCXO_SEL_SIZE = 1,
    BF_VCXO_SEL_MASK = 0x01,
    BF_VCXO_SEL_LSB = 0,

    /**
     * CONFIG_QA
     */
    BF_LVDS_QA_SIZE = 1,
    BF_LVDS_QA_MASK = 0x02,
    BF_LVDS_QA_LSB = 1,

    /**
     * CONFIG_QA
     */
    BF_OE_A_SIZE = 1,
    BF_OE_A_MASK = 0x01,
    BF_OE_A_LSB = 0,

    /**
     * CONFIG_QB
     */
    BF_LVDS_QB_SIZE = 1,
    BF_LVDS_QB_MASK = 0x02,
    BF_LVDS_QB_LSB = 1,

    /**
     * CONFIG_QB
     */
    BF_OE_B_SIZE = 1,
    BF_OE_B_MASK = 0x01,
    BF_OE_B_LSB = 0,


/*==================End of Bitfield Definitions=======================*/

    REGMAP_APLL_MAX
};


/** @}*/

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __PUMA_APLL_REGISTER_DEF_H__ */
