/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/
#ifndef __APLL_TYPE_DEF_H__
#define __APLL_TYPE_DEF_H__

#include "osNumeric.h"

#ifdef __cplusplus
extern "C"
{
#endif

/** \addtogroup APLL
 *  @{
 */

/**
 * @brief Define constant indicating invalid divider value
 */
#define INVALID_DIVIDER_VALUE	(0xFFFF)

/**
 * @brief Enumerated type listing APLL instance within device
 */
typedef enum
{
    APLL_INST_1,  /**< APLL #1 */
    APLL_INST_2,  /**< APLL #2 */
    APLL_INST_MAX /**< \internal Not used */
} T_idtApllId;

/**
 * @brief Enumerated type listing supported access methods
 */
typedef enum
{
    APLL_ACCESS_I2C, /**< Device access through I2C */
    APLL_ACCESS_SIM, /**< Simulated device */
    APLL_ACCESS_MAX /**< \internal Not used */
} T_idtApllAccessType;

/**
 * @brief Enumerated type listing possible configuration for each APLL
 */
typedef enum
{
    APLL_CONFIG0 = 0, /**< Configuration #1 */
    APLL_CONFIG1 = 1, /**< Configuration #2*/
    APLL_CONFIG_MAX /**< \internal Not used */
} T_idtApllConfigSel;

/**
 * @brief Enumerated type listing input ports
 */
typedef enum
{
    APLL_CLK_SEL_EXTERNAL, /**< External clock source */
    APLL_CLK_SEL_INTERNAL, /**< Internal clock source */
    APLL_CLK_SEL_MAX /**< \internal Not used */
} T_idtApllInputClkSrc;

/**
 * @brief Enumerated type listing output ports
 */
typedef enum
{
    APLL_OUTPUT_QA, /**< Output port A */
    APLL_OUTPUT_QB, /**< Output port B */
    APLL_OUTPUT_MAX /**< \internal Not used */
} T_idtApllOutput;

/**
 * @brief Enumerated type listing output port status (enable/disable)
 */
typedef enum
{
   APLL_OUTPUT_DISABLE, /**< Disable output port */
   APLL_OUTPUT_ENABLE, /**< Enable output port */
   APLL_OUTPUT_EN_MAX /**< \internal Not used */
} T_idtApllOutputEn;

/**
 * @brief Enumerated type listing output signal type
 */
typedef enum
{
    APLL_OUT_SIG_LVPECL, /**< LVPECL output signal type */
    APLL_OUT_SIG_LVDS, /**< LVDS output signal type */
    APLL_OUT_SIG_MAX /**< \internal Not used */
} T_idtApllOutputSigType;

/**
 * @brief Enumerated type listing frequency supported by the output
 */
typedef enum
{
    APLL_OUT_QA_ONLY, /**< Frequency only supported by output port A  */
    APLL_OUT_QB_ONLY, /**< Frequency only supported by output port B  */
    APLL_OUT_QA_OR_QB, /**< Frequency supported by both output port A and B */
    APLL_OUT_MAX /**< \internal Not used */
} T_idtApllOutputSupportedType;

/**
 * @brief Enumerated type listing output port A divider values
 */
typedef enum
{
    APLL_QA_ODSEL_DIV_25 = 0, /**< Output A divider 25 */
    APLL_QA_ODSEL_DIV_5 = 1, /**< Output A divider 5 */
    APLL_QA_ODSEL_DIV_4 = 2, /**< Output A divider 4 */
    APLL_QA_ODSEL_DIV_2 = 3, /**< Output A divider 2 */
    APLL_QA_ODSEL_DIV_1 = 4, /**< Output A divider 1 */
    APLL_QA_ODSEL_DIV_MAX /**< \internal Not used */
} T_idtApllQaDiv;

/**
 * @brief Enumerated type listing output port B divider values
 */
typedef enum
{
    APLL_QB_ODSEL_DIV_8 = 0, /**< Output B divider 8 */
    APLL_QB_ODSEL_DIV_5 = 1, /**< Output B divider 5 */
    APLL_QB_ODSEL_DIV_4 = 2, /**< Output B divider 4 */
    APLL_QB_ODSEL_DIV_2 = 3, /**< Output B divider 2 */
    APLL_QB_ODSEL_DIV_1 = 4, /**< Output B divider 1 */
    APLL_QB_ODSEL_DIV_MAX /**< \internal Not used */
} T_idtApllQbDiv;

/**
 * @brief Enumerated type listing feedback frequency doubling values
 */
typedef enum
{
    APLL_FREQ_SEL_SINGLE = 0, /**<  Feedback frequency not doubled */
    APLL_FREQ_SEL_DOUBLE = 1, /**<  Feedback frequency doubled */
    APLL_FREQ_SEL_MAX /**< \internal Not used */
} T_idtApllFreqDoublerSel;

/**
 * @brief Enumerated type listing crystals
 */
typedef enum
{
    APLL_XTAL_1_APLL1,	/**< XTAL1 connects to APPL1 */
    APLL_XTAL_2_APLL2,	/**< XTAL2 connects to APPL2 */
    APLL_XTAL_3_APLL1,	/**< XTAL3 connects to APPL1 */
    APLL_XTAL_4_APLL2,	/**< XTAL4 connects to APPL2 */
    APLL_XTAL_MAX /**< \internal Not used */
} T_idtApllXtalId;

/**
 * @brief Enumerated type listing support crystal frequencies
 */
typedef enum
{
    APLL_XTAL_FREQ_25000KHz, /**< Ethernet (25 MHz) */
    APLL_XTAL_FREQ_25781_DOT_25KHz, /**< Ethernet FEC (25.78125 MHz) */
    APLL_XTAL_FREQ_24883_DOT_2KHz, /**< SONET/SDH (24.8832 MHz) */
    APLL_XTAL_FREQ_MAX /**< \internal Not used */
} T_idtApllXtalType;

/**
 * @brief Enumerated type listing Device master reset sync status
 */
typedef enum
{
    APLL_OUT_MR_SYNC = 0, /**< Device out of master reset sync (normal operation) */
    APLL_IN_MR_SYNC = 1, /**< Device in master reset sync */
    APLL_MR_SYNC_MAX /**< \internal Not used */
} T_idtApllMasterResetSync;

/**
 * @brief Enumerated type listing Apll bypass status/setting
 */
typedef enum
{
    APLL_BYPASS_NORMAL_OP = 0, /**< Apll in normal operation */
    APLL_BYPASS_BYPASS = 1, /**< Apll bypass VCXO and connect selected clock input to output A and B */
    APLL_BYPASS_MAX /**<\internal Not used */
} T_idtApllBypass;

/**
 * @brief Enumerated type listing Apll shutoff status/setting
 */
typedef enum
{
    APLL_SHUTOFF_NORMAL_OP = 0, /**< Apll in normal operation */
    APLL_SHUTOFF_SHUTOFF = 1, /**< VCXO current is shut off and Outputs are squelched */
    APLL_SHUTOFF_MAX /**<\internal Not used */
} T_idtApllShutoff;

/**
 * @brief Type define for APLL divider value
 */
typedef T_osUint16 T_idtApllDividerValue;

/**
 * @brief Type define for APLL register address
 */
typedef T_osUint8  T_idtApllRegAddr;

/**
 * @brief Type define for APLL register value mask
 */
typedef T_osUint8  T_idtApllRegMask;

/**
 * @brief Type define for APLL register value
 */
typedef T_osUint8  T_idtApllRegVal;

/**
 * @brief Structure containing APLL crystal id and frequency information
 */
typedef struct
{
	T_idtApllXtalId	apllXtalId; /**< APLL crystal ID */
	T_idtApllXtalType apllXtalFreq; /**< APLL crystal frequency */
} T_idtApllXtal;

/**
 * @}
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __APLL_TYPE_DEF_H__ */
