/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "ApllTypeDef.h"
#include "ApllFreqProfile.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/
/* Pre-Divider */
#define APLL_PRE_DIV_ETH		(19525)
#define APLL_PRE_DIV_SONET		(9775)
#define APLL_PRE_DIV_ETH_FEC	(15625)

/* Feedback Divider */
#define APLL_FB_DIV_ETH			(3124)
#define APLL_FB_DIV_SONET       (3128)
#define APLL_FB_DIV_ETH_FEC		(3125)

/* Output Divider */
#define APLL_OUTPUT_DIV_1		(1)
#define APLL_OUTPUT_DIV_2		(2)
#define APLL_OUTPUT_DIV_4		(4)
#define APLL_OUTPUT_DIV_5		(5)
#define APLL_OUTPUT_DIV_8		(8)
#define APLL_OUTPUT_DIV_25      (25)

#define MAX_CONFIG_COMBO_PER_FREQ       (1)
#define MAX_CONFIG_COMBO_PER_VCXO_TYPE  (6)

/**
 * @brief Utility marco for initializing NULL APLL configuration entry
 */
#define NULL_APLL_FREQ_TABLE_ENTRY	\
{									\
	APLL_INPUT_FREQ_MAX,			\
	0,								\
	0,								\
	APLL_XTAL_FREQ_MAX,				\
	0,								\
	APLL_OUTPUT_FREQ_MAX,			\
	APLL_OUT_MAX					\
}

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/
/**
 * @brief XTAL type based APLL configuration lookup table
 */
static T_idtApllFreqMapping apllFreqMappingVcxoLUT[APLL_XTAL_FREQ_MAX][MAX_CONFIG_COMBO_PER_VCXO_TYPE+1] =
{
	/* 25MHz VCXO - Ethernet frequency family */
	{
        /* 25MHz */
        {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_25,
            APLL_OUTPUT_FREQ_25000KHz,
            APLL_OUT_QA_ONLY
	    },
        /* 78.125MHz */
	    {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_8,
            APLL_OUTPUT_FREQ_78125KHz,
            APLL_OUT_QB_ONLY
	    },
        /* 125MHz */
        {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_5,
            APLL_OUTPUT_FREQ_125000KHz,
            APLL_OUT_QA_OR_QB
	    },
        /* 156.25MHz */
	    {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_4,
            APLL_OUTPUT_FREQ_156250KHz,
            APLL_OUT_QA_OR_QB
        },
        /* 312.5MHz */
	    {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_2,
            APLL_OUTPUT_FREQ_312500KHz,
            APLL_OUT_QA_OR_QB
        },
        /* 625MHz */
	    {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_1,
            APLL_OUTPUT_FREQ_625000KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 25.78125MHz VCXO - Ethernet FEC frequency family */
	{
        /* 25.78125MHz */
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_25,
            APLL_OUTPUT_FREQ_25781_DOT_25KHz,
            APLL_OUT_QA_ONLY
        },
        /* 80.56640625MHz */
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_8,
            APLL_OUTPUT_FREQ_80566_DOT_40625KHz,
            APLL_OUT_QB_ONLY
        },
        /* 128.90625MHz */
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_5,
            APLL_OUTPUT_FREQ_128906_DOT_25KHz,
            APLL_OUT_QA_OR_QB
        },
        /* 161.11328125MHz */
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_4,
            APLL_OUTPUT_FREQ_161132_DOT_8125KHz,
            APLL_OUT_QA_OR_QB
        },
        /* 322.265625MHz */
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_2,
            APLL_OUTPUT_FREQ_322265_DOT_625KHz,
            APLL_OUT_QA_OR_QB
        },
        /* 644.53125MHz */
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_1,
            APLL_OUTPUT_FREQ_644531_DOT_25KHz,
            APLL_OUT_QA_OR_QB
        },
        NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 24.8832MHz VCXO - SONET frequency family */
	{
        /* 24.8832MHz */
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_25,
            APLL_OUTPUT_FREQ_24883_DOT_2KHz,
            APLL_OUT_QA_ONLY
        },
        /* 77.76MHz */
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_8,
            APLL_OUTPUT_FREQ_77760KHz,
            APLL_OUT_QB_ONLY
        },
        /* 124.416MHz */
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_5,
            APLL_OUTPUT_FREQ_124416KHz,
            APLL_OUT_QA_OR_QB
        },
        /* 155.52MHz */
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_4,
            APLL_OUTPUT_FREQ_155520KHz,
            APLL_OUT_QA_OR_QB
        },
        /* 311.04MHz */
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_2,
            APLL_OUTPUT_FREQ_311040KHz,
            APLL_OUT_QA_OR_QB
        },
        /* 622.08MHz */
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_1,
            APLL_OUTPUT_FREQ_622080KHz,
            APLL_OUT_QA_OR_QB
        },
        NULL_APLL_FREQ_TABLE_ENTRY
	}
};

/**
 * @brief Output frequency based APLL configuration lookup table
 */
static T_idtApllFreqMapping apllFreqMappingOutputFreqLUT[APLL_OUTPUT_FREQ_MAX][MAX_CONFIG_COMBO_PER_FREQ+1] =
{
	/* 24.8832MHz */
	{
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_25,
            APLL_OUTPUT_FREQ_24883_DOT_2KHz,
            APLL_OUT_QA_ONLY
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 25MHz */
	{
	    {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_25,
            APLL_OUTPUT_FREQ_25000KHz,
            APLL_OUT_QA_ONLY
        },
		NULL_APLL_FREQ_TABLE_ENTRY,
	},
	/* 25.78125MHz */
	{
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_25,
            APLL_OUTPUT_FREQ_25781_DOT_25KHz,
            APLL_OUT_QA_ONLY
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 77.76MHz */
	{
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_8,
            APLL_OUTPUT_FREQ_77760KHz,
            APLL_OUT_QB_ONLY
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 78.125MHz */
	{
	    {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_8,
            APLL_OUTPUT_FREQ_78125KHz,
            APLL_OUT_QB_ONLY
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 80.56640625MHz */
	{
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_8,
            APLL_OUTPUT_FREQ_80566_DOT_40625KHz,
            APLL_OUT_QB_ONLY
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 124.416MHz */
	{
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_5,
            APLL_OUTPUT_FREQ_124416KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 125MHz */
	{
	    {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_5,
            APLL_OUTPUT_FREQ_125000KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 128.90625MHz */
	{
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_5,
            APLL_OUTPUT_FREQ_128906_DOT_25KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 155.52MHz */
	{
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_4,
            APLL_OUTPUT_FREQ_155520KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 156.25MHz */
	{
	    {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_4,
            APLL_OUTPUT_FREQ_156250KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 161.11328125MHz */
	{
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_4,
            APLL_OUTPUT_FREQ_161132_DOT_8125KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 311.04MHz */
	{
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_2,
            APLL_OUTPUT_FREQ_311040KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 312.5MHz */
	{
	    {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_2,
            APLL_OUTPUT_FREQ_312500KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 322.265625MHz */
	{
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_2,
            APLL_OUTPUT_FREQ_322265_DOT_625KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 622.08MHz */
	{
		{
            APLL_INPUT_FREQ_77760KHz,
            APLL_PRE_DIV_SONET,
            APLL_FB_DIV_SONET,
            APLL_XTAL_FREQ_24883_DOT_2KHz,
            APLL_OUTPUT_DIV_1,
            APLL_OUTPUT_FREQ_622080KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 625MHz */
	{
	    {
            APLL_INPUT_FREQ_156250KHz,
            APLL_PRE_DIV_ETH,
            APLL_FB_DIV_ETH,
            APLL_XTAL_FREQ_25000KHz,
            APLL_OUTPUT_DIV_1,
            APLL_OUTPUT_FREQ_625000KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	},
	/* 644.53125MHz */
	{
		{
            APLL_INPUT_FREQ_128906_DOT_25KHz,
            APLL_PRE_DIV_ETH_FEC,
            APLL_FB_DIV_ETH_FEC,
            APLL_XTAL_FREQ_25781_DOT_25KHz,
            APLL_OUTPUT_DIV_1,
            APLL_OUTPUT_FREQ_644531_DOT_25KHz,
            APLL_OUT_QA_OR_QB
        },
		NULL_APLL_FREQ_TABLE_ENTRY
	}
};

/**
 * @brief Cached APLL configuration
 */
static T_idtApllFreqMapping apllFreqMappingCurrentConfig[APLL_INST_MAX][APLL_OUTPUT_MAX + 1] =
{
    /* APLL 0 */
    {
        NULL_APLL_FREQ_TABLE_ENTRY,
        NULL_APLL_FREQ_TABLE_ENTRY,
        NULL_APLL_FREQ_TABLE_ENTRY
    },
    /* APLL 1 */
    {
        NULL_APLL_FREQ_TABLE_ENTRY,
        NULL_APLL_FREQ_TABLE_ENTRY,
        NULL_APLL_FREQ_TABLE_ENTRY
    }
};

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

/**
 * @brief Function to retrieve output configuration given an output frequency
 * @param apllOutputFreq Output frequency
 * @return Output configuration
 */
const T_idtApllFreqMapping* IDTApll_GetConfigListByFreq(T_idtApllOutputFreqType apllOutputFreq)
{
	OS_TRACE(("Apll output frequency: %d\n", apllOutputFreq));

	if (apllOutputFreq >= APLL_OUTPUT_FREQ_MAX)
	{
		OS_ERROR(("Invalid apll output frequency: %d\n", apllOutputFreq));
		return NULL;
	}
	
	return &apllFreqMappingOutputFreqLUT[apllOutputFreq][0];
}

/**
 * @brief Function to retrieve output configuration gvien the
 * oscilator type (frequency)
 * @param apllVcxoFreq Oscilator type (frequency)
 * @return Output configuration
 */
const T_idtApllFreqMapping* IDTApll_GetConfigListByXtalType(T_idtApllXtalType apllVcxoFreq)
{
	OS_TRACE(("Apll xtal frequency: %d\n", apllVcxoFreq));

    if (apllVcxoFreq >= APLL_XTAL_FREQ_MAX)
	{
		OS_ERROR(("Invalid apll VCXO frequency: %d\n", apllVcxoFreq));
		return NULL;
	}
	
	return &apllFreqMappingVcxoLUT[apllVcxoFreq][0];
}

/**
 * @brief Function to check whether the input APLL frequency mapping
 * is NULL (not configured)
 * @param apllFreqConfig APLL frequency mapping configuration
 * @return  E_osTrue - not configured, E_osFalse - configured
 */
T_osBool IDTApll_NullFreqTableEntry(const T_idtApllFreqMapping *apllFreqConfig)
{
	OS_TRACE(("Enter. apll frequency configuration: %p\n", apllFreqConfig));
	
	OS_ASSERT_PTR_NULL(apllFreqConfig);

	return (APLL_INPUT_FREQ_MAX == apllFreqConfig->apllInputFreq &&
		    !apllFreqConfig->apllFbDivider &&
		    !apllFreqConfig->apllPreDivider &&
		    APLL_XTAL_FREQ_MAX == apllFreqConfig->apllXtalFreq &&
		    !apllFreqConfig->apllOutputDivider &&
		    APLL_OUTPUT_FREQ_MAX == apllFreqConfig->apllOutputFeq &&
		    APLL_OUT_MAX == apllFreqConfig->apllOutputs);
}

/**
 * @brief Function to retrieve the current APLL frequency mapping configuration for the output
 * @param apllId APLL instance
 * @param output APLL output
 * @return APLL frequency mapping configuration
 */
const T_idtApllFreqMapping* IDTApll_GetCurrentApllFreqConfig(T_idtApllId apllId, T_idtApllOutput output)
{
    OS_ASSERT(apllId < APLL_INST_MAX);
    OS_ASSERT(output < APLL_OUTPUT_MAX);
     
    return &apllFreqMappingCurrentConfig[apllId][output];
}
