/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <check.h>

#include "ApllApi.h"
#include "ApllTypeDef.h"

#include "check_main.h"
/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/
typedef enum 
{
    minDiv,
    midDiv,
    maxDiv,
    numDiv
} divIndex_t;

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/
 
/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
START_TEST (test_ApllApiConfigSel)
{
    T_idtApllId apllInstId;
    T_idtApllConfigSel configIn, configOut;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++) 
    {
        for (configIn = APLL_CONFIG0; configIn < APLL_CONFIG_MAX; configIn++)
        {
            IDTApll_SetConfigSel(hdlr, apllInstId, configIn); 
            configOut = IDTApll_GetConfigSel(hdlr, apllInstId);
            fail_unless(configOut == configIn,
                        "Apll set/get config selection not working, output not the same as input, "
                        "apll id: %d, input: %d, output: %d",
                         apllInstId, configIn, configOut);
        }
    }
}
END_TEST

START_TEST (test_ApllApiConfigSelInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllConfigSel configIn,
                      configOutOld,
                      configOutNew;

    /* out of range chip id */
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
        for (configIn = APLL_CONFIG0; configIn < APLL_CONFIG_MAX; configIn++)
        {
            IDTApll_SetConfigSel(hdlr, apllInstId, configIn);
            configOutOld = IDTApll_GetConfigSel(hdlr, apllInstId);
            fail_unless(configOutOld == APLL_CONFIG_MAX,
                    "Apll set/get config selection not working, unexpected return value: %d, "
                    "apll id: %d", configOutOld, apllInstId);
        }
    }

    /* out of range configuration */
    for (configIn = APLL_CONFIG_MAX; configIn <= APLL_CONFIG_MAX+1; configIn++)
    {
        for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
        {
            configOutOld = IDTApll_GetConfigSel(hdlr, apllInstId);
            IDTApll_SetConfigSel(hdlr, apllInstId, configIn);
            configOutNew = IDTApll_GetConfigSel(hdlr, apllInstId);    
            fail_unless(configOutNew == configOutOld,
                    "Apll set/get config selection not working, configuration modified with invalid input, "
                    "apll id: %d, apll config: %d", apllInstId, configIn);
        }
    }
}
END_TEST

START_TEST (test_ApllApiInputClkSrc)
{
    T_idtApllId apllInstId;
    T_idtApllInputClkSrc clkIn, clkOut;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++) 
    {
        for (clkIn = APLL_CLK_SEL_EXTERNAL; clkIn < APLL_CLK_SEL_MAX; clkIn++)
        {
            IDTApll_SetInputClkSrc(hdlr, apllInstId, clkIn);
            clkOut = IDTApll_GetInputClkSrc(hdlr, apllInstId);
            fail_unless(clkOut == clkIn,
                        "Apll set/get input clock source not working, output not the same as input "
                        "apll id: %d, input: %d, output: %d",
                         apllInstId, clkIn, clkOut);
        }
    }
}
END_TEST

START_TEST (test_ApllApiInputClkSrcInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllInputClkSrc clkIn,
                         clkOutOld,
                         clkOutNew;

    /* out of range chip id */
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
        for (clkIn = APLL_CLK_SEL_EXTERNAL; clkIn < APLL_CLK_SEL_MAX; clkIn++)
        {
            IDTApll_SetInputClkSrc(hdlr, apllInstId, clkIn);
            clkOutOld = IDTApll_GetInputClkSrc(hdlr, apllInstId);
            fail_unless(clkOutOld == clkIn,
                    "Apll set/get input clock source not working, unexpected return value: %d, "
                    "apll id: %d", clkOutOld, apllInstId);
        }
    }

    /* out of range input clock selection */
    for (clkIn = APLL_CLK_SEL_MAX; clkIn <= APLL_CLK_SEL_MAX+1; clkIn++)
    {
        for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
        {
            clkOutOld = IDTApll_GetInputClkSrc(hdlr, apllInstId);
            IDTApll_SetInputClkSrc(hdlr, apllInstId, clkIn);
            clkOutNew = IDTApll_GetInputClkSrc(hdlr, apllInstId);    
            fail_unless(clkOutNew == clkOutOld,
                    "Apll set/get input clock source not working, clock selection modified with invalid input, "
                    "apll id: %d, inputClk: %d", apllInstId, clkIn);
        }
    }
}
END_TEST

START_TEST (test_ApllApiPreDiv)
{
    T_idtApllId apllInstId;
    T_idtApllConfigSel apllConfig;
    T_idtApllDividerValue divIn[numDiv], divOut;
    divIndex_t i;

    IDTApll_GetPreDivRange(hdlr, &divIn[minDiv], &divIn[maxDiv]);
    divIn[midDiv] = divIn[minDiv] + (divIn[maxDiv] - divIn[minDiv])/2;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++) 
    {
        for (apllConfig = APLL_CONFIG0; apllConfig < APLL_CONFIG_MAX; apllConfig++)
        {
            for (i = minDiv; i < numDiv; i++)
            {
                IDTApll_SetPreDiv(hdlr, apllInstId, apllConfig, divIn[i]);                 
                divOut = IDTApll_GetPreDiv(hdlr, apllInstId, apllConfig);
                fail_unless(divOut == divIn[i],
                        "Apll set/get pre divider not working, output not the same as input, "
                        "Apll id: %d, apll config: %d, input: %d, output: %d",
                         apllInstId, apllConfig, divIn[i], divOut);
            }
        }
    }
}
END_TEST

START_TEST (test_ApllApiPreDivInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllConfigSel apllConfig;
    T_idtApllDividerValue minDiv,
                        maxDiv,
                        divIn,
                        divOutOld,
                        divOutNew;

    IDTApll_GetPreDivRange(hdlr, &minDiv, &maxDiv);
   
    /* out of range chip id */
    apllConfig = APLL_CONFIG0;
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
         IDTApll_SetPreDiv(hdlr, apllInstId, apllConfig, minDiv);
         divOutOld = IDTApll_GetPreDiv(hdlr, apllInstId, apllConfig);
         fail_unless(divOutOld == INVALID_DIVIDER_VALUE,
                    "Apll set/get pre divider not working, unexpected return value: %d, "
                    "Apll id: %d", divOutOld, apllInstId);
    }

    /* out of range configration selection */
    apllInstId = APLL_INST_1;
    for (apllConfig = APLL_CONFIG_MAX; apllConfig <= APLL_CONFIG_MAX+1; apllConfig++)
    {
        IDTApll_SetPreDiv(hdlr, apllInstId, apllConfig, minDiv);
        divOutOld = IDTApll_GetPreDiv(hdlr, apllInstId, apllConfig);
        fail_unless(divOutOld == INVALID_DIVIDER_VALUE,
                    "Apll set/get pre divider not working, unexpected return value: %d, "
                    "Apll id: %d", divOutOld, apllInstId);
    }

    /* out of range pre-divider */
    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
    {
        for (apllConfig = APLL_CONFIG0; apllConfig < APLL_CONFIG_MAX; apllConfig++)
        {
            for (divIn = 0; divIn < minDiv; divIn++)
            {
                divOutOld = IDTApll_GetPreDiv(hdlr, apllInstId, apllConfig);

                IDTApll_SetPreDiv(hdlr, apllInstId, apllConfig, divIn);
                divOutNew = IDTApll_GetPreDiv(hdlr, apllInstId, apllConfig);    
                fail_unless(divOutNew == divOutOld,
                        "Apll set/get pre divider not working, pre-divider modified with invalid input, "
                        "Apll id: %d, config: %d, preDiv: %d", apllInstId, apllConfig, divIn);
            }

            divIn = maxDiv + 1;
            if (divIn)
            {
                divOutOld = IDTApll_GetPreDiv(hdlr, apllInstId, apllConfig);

                IDTApll_SetPreDiv(hdlr, apllInstId, apllConfig, divIn);
                divOutNew = IDTApll_GetPreDiv(hdlr, apllInstId, apllConfig);    
                fail_unless(divOutNew == divOutOld,
                        "Apll set/get pre divider not working, pre-divider modified with invalid input, "
                        "Apll id: %d, config: %d, preDiv: %d", apllInstId, apllConfig, divIn);
            }
        }
    }
}
END_TEST

START_TEST (test_ApllApiFeedbackDiv)
{
    T_idtApllId apllInstId;
    T_idtApllConfigSel apllConfig;
    T_idtApllDividerValue divIn[numDiv], divOut;
    divIndex_t i;

    IDTApll_GetFeedbackDivRange(hdlr, &divIn[minDiv], &divIn[maxDiv]);
    divIn[midDiv] = divIn[minDiv] + (divIn[maxDiv] - divIn[minDiv])/2;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++) 
    {
        for (apllConfig = APLL_CONFIG0; apllConfig < APLL_CONFIG_MAX; apllConfig++)
        {
            for (i = minDiv; i < numDiv; i++)
            {
                IDTApll_SetFeedbackDiv(hdlr, apllInstId, apllConfig, divIn[i]);                           
                divOut = IDTApll_GetFeedbackDiv(hdlr, apllInstId, apllConfig);
                fail_unless(divOut == divIn[i],
                        "Apll set/get feedback divider not working, output not the same as input, "
                        "Apll id: %d, config: %d, input: %d, output: %d",
                         apllInstId, apllConfig, divIn[i], divOut);
            }
        }
    }
}
END_TEST

START_TEST (test_ApllApiFeedbackDivInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllConfigSel apllConfig;
    T_idtApllDividerValue minDiv,
                        maxDiv,
                        divIn,
                        divOutOld,
                        divOutNew;

    IDTApll_GetFeedbackDivRange(hdlr, &minDiv, &maxDiv);
   
    /* out of range chip id */
    apllConfig = APLL_CONFIG0;
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
         IDTApll_SetFeedbackDiv(hdlr, apllInstId, apllConfig, minDiv);
         divOutOld = IDTApll_GetFeedbackDiv(hdlr, apllInstId, apllConfig);
         fail_unless(divOutOld == INVALID_DIVIDER_VALUE,
                    "Apll set/get feedback divider not working, unexpected return value: %d, "
                    "Apll id: %d", divOutOld, apllInstId);
    }

    /* out of range configration selection */
    apllInstId = APLL_INST_1;
    for (apllConfig = APLL_CONFIG_MAX; apllConfig <= APLL_CONFIG_MAX+1; apllConfig++)
    {
        IDTApll_SetFeedbackDiv(hdlr, apllInstId, apllConfig, minDiv);
        divOutOld = IDTApll_GetFeedbackDiv(hdlr, apllInstId, apllConfig);
        fail_unless(divOutOld == INVALID_DIVIDER_VALUE,
                    "Apll set/get feedback divider not working, unexpected return value: %d, "
                    "Apll id: %d", divOutOld, apllInstId);
    }

    /* out of range pre-divider */
    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
    {
        for (apllConfig = APLL_CONFIG0; apllConfig < APLL_CONFIG_MAX; apllConfig++)
        {
            for (divIn = 0; divIn < minDiv; divIn++)
            {
                divOutOld = IDTApll_GetFeedbackDiv(hdlr, apllInstId, apllConfig);
                IDTApll_SetFeedbackDiv(hdlr, apllInstId, apllConfig, divIn);
                divOutNew = IDTApll_GetFeedbackDiv(hdlr, apllInstId, apllConfig);    
                fail_unless(divOutNew == divOutOld,
                        "Apll set/get feedback divider not working, pre-divider modified with invalid input, "
                        "Apll id: %d, config: %d, preDiv: %d", apllInstId, apllConfig, divIn);
            }

            divIn = maxDiv + 1;
            if (divIn)
            {
                divOutOld = IDTApll_GetFeedbackDiv(hdlr, apllInstId, apllConfig);
                IDTApll_SetFeedbackDiv(hdlr, apllInstId, apllConfig, divIn);
                divOutNew = IDTApll_GetFeedbackDiv(hdlr, apllInstId, apllConfig);    
                fail_unless(divOutNew == divOutOld,
                        "Apll set/get feedback divider not working, pre-divider modified with invalid input, "
                        "Apll id: %d, config: %d, preDiv: %d", apllInstId, apllConfig, divIn);
            }
        }
    }
}
END_TEST

START_TEST (test_ApllApiQaDiv)
{
    T_idtApllId apllInstId;
    T_idtApllConfigSel apllConfig;
    T_idtApllQaDiv divIn, divOut;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
    {
        for (apllConfig = APLL_CONFIG0; apllConfig < APLL_CONFIG_MAX; apllConfig++)
        {
            for (divIn = APLL_QA_ODSEL_DIV_25; divIn < APLL_QA_ODSEL_DIV_MAX; divIn++)
            {
                IDTApll_SetQaDiv(hdlr, apllInstId, apllConfig, divIn);
                divOut = IDTApll_GetQaDiv(hdlr, apllInstId, apllConfig);
                fail_unless(divOut == divIn,
                        "Apll set/get QA divider not working, output does not match input, "
                        "Apll id: %d, config: %d, input: %d, output: %d",
                         apllInstId, apllConfig, divIn, divOut);
            }
        }
    }
}
END_TEST

START_TEST (test_ApllApiQaDivInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllConfigSel apllConfig;
    T_idtApllQaDiv divIn, divOutOld, divOutNew;

    /* out of range chip id */
    apllConfig = APLL_CONFIG0;
    divIn = APLL_QA_ODSEL_DIV_25;
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
        IDTApll_SetQaDiv(hdlr, apllInstId, apllConfig, divIn);
        divOutOld = IDTApll_GetQaDiv(hdlr, apllInstId, apllConfig);
        fail_unless(divOutOld == APLL_QA_ODSEL_DIV_MAX,
                   "Apll set/get QA divider not working, unexpected return value: %d, "
                   "Apll id: %d", divOutOld, apllInstId);
    }

    /* out of range configuration selection */
    apllInstId = APLL_INST_1;
    for (apllConfig = APLL_CONFIG_MAX; apllConfig <= APLL_CONFIG_MAX+1; apllConfig++)
    {
        IDTApll_SetQaDiv(hdlr, apllInstId, apllConfig, divIn);
        divOutOld = IDTApll_GetQaDiv(hdlr, apllInstId, apllConfig);
        fail_unless(divOutOld == APLL_QA_ODSEL_DIV_MAX, 
                    "Apll set/get QA divider not working, unexpected return value: %d, "
                    "Apll id: %d", divOutOld, apllInstId);
    }

    /* out of range output a divider */
    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
    {
        for (apllConfig = APLL_CONFIG0; apllConfig < APLL_CONFIG_MAX; apllConfig++)
        {
            for (divIn = APLL_QA_ODSEL_DIV_MAX; divIn <= APLL_QA_ODSEL_DIV_MAX+1; divIn++)
            {
                divOutOld = IDTApll_GetQaDiv(hdlr, apllInstId, apllConfig);
                IDTApll_SetQaDiv(hdlr, apllInstId, apllConfig, divIn);
                divOutNew = IDTApll_GetQaDiv(hdlr, apllInstId, apllConfig);
                fail_unless(divOutNew == divOutOld,
                        "Apll set/get QA divider not working, output A divider modified with invalid input, "
                        "Apll id: %d, config: %d, qaDiv: %d", apllInstId, apllConfig, divIn);
            }
        }
    }
}
END_TEST

START_TEST (test_ApllApiQbDiv)
{
    T_idtApllId apllInstId;
    T_idtApllConfigSel apllConfig;
    T_idtApllQbDiv divIn, divOut;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
    {
        for (apllConfig = APLL_CONFIG0; apllConfig < APLL_CONFIG_MAX; apllConfig++)
        {
            for (divIn = APLL_QB_ODSEL_DIV_8; divIn < APLL_QB_ODSEL_DIV_MAX; divIn++)
            {
                IDTApll_SetQbDiv(hdlr, apllInstId, apllConfig, divIn);
                divOut = IDTApll_GetQbDiv(hdlr, apllInstId, apllConfig);
                fail_unless(divOut == divIn,
                        "Apll set/get QB divider not working, output does not match input, "
                        "Apll id: %d, config: %d, input: %d, output: %d",
                         apllInstId, apllConfig, divIn, divOut);
            }
        }
    }
}
END_TEST

START_TEST (test_ApllApiQbDivInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllConfigSel apllConfig;
    T_idtApllQbDiv divIn, divOutOld, divOutNew;

    /* out of range chip id */
    apllConfig = APLL_CONFIG0;
    divIn = APLL_QB_ODSEL_DIV_8;
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
        IDTApll_SetQbDiv(hdlr, apllInstId, apllConfig, divIn);
        divOutOld = IDTApll_GetQbDiv(hdlr, apllInstId, apllConfig);
        fail_unless(divOutOld == APLL_QB_ODSEL_DIV_MAX,
                   "Apll set/get QB divider not working, unexpected return value: %d, "
                   "Apll id: %d", divOutOld, apllInstId);
    }

    /* out of range configuration selection */
    apllInstId = APLL_INST_1;
    for (apllConfig = APLL_CONFIG_MAX; apllConfig <= APLL_CONFIG_MAX+1; apllConfig++)
    {
        IDTApll_SetQbDiv(hdlr, apllInstId, apllConfig, divIn);
        divOutOld = IDTApll_GetQbDiv(hdlr, apllInstId, apllConfig);
        fail_unless(divOutOld == APLL_QB_ODSEL_DIV_MAX, 
                    "Apll set/get QB divider not working, unexpected return value: %d, "
                    "Apll id: %d", divOutOld, apllInstId);
    }

    /* out of range output a divider */
    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
    {
        for (apllConfig = APLL_CONFIG0; apllConfig < APLL_CONFIG_MAX; apllConfig++)
        {
            for (divIn = APLL_QA_ODSEL_DIV_MAX; divIn <= APLL_QA_ODSEL_DIV_MAX+1; divIn++)
            {
                divOutOld = IDTApll_GetQbDiv(hdlr, apllInstId, apllConfig);
                IDTApll_SetQbDiv(hdlr, apllInstId, apllConfig, divIn);
                divOutNew = IDTApll_GetQbDiv(hdlr, apllInstId, apllConfig);
                fail_unless(divOutNew == divOutOld,
                        "Apll set/get QB divider not working, output B divider modified with invalid input, "
                        "Apll id: %d, config: %d, qbDiv: %d", apllInstId, apllConfig, divIn);
            }
        }
    }
}
END_TEST

START_TEST (test_ApllApiFreqDoublerSel)
{
    T_idtApllId apllInstId;
    T_idtApllFreqDoublerSel doublerIn, doublerOut;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++) 
    {
        for (doublerIn = APLL_FREQ_SEL_SINGLE; doublerIn < APLL_FREQ_SEL_MAX; doublerIn++)
        {
            IDTApll_SetFreqDoublerSel(hdlr, apllInstId, doublerIn);     
            doublerOut = IDTApll_GetFreqDoublerSel(hdlr, apllInstId);
            fail_unless(doublerOut == doublerIn,
                        "Apll set/get frequency doubler not working, output not the same as input "
                        "Apll id: %d, input: %d, output: %d",
                         apllInstId, doublerIn, doublerOut);
        }
    }
}
END_TEST

START_TEST (test_ApllApiFreqDoublerSelInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllFreqDoublerSel doublerIn,
                          doublerOutOld,
                          doublerOutNew;

    /* out of range chip id */
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
        for (doublerIn = APLL_FREQ_SEL_SINGLE; doublerIn < APLL_FREQ_SEL_MAX; doublerIn++)
        {
            IDTApll_SetFreqDoublerSel(hdlr, apllInstId, doublerIn);
            doublerOutOld = IDTApll_GetFreqDoublerSel(hdlr, apllInstId);
            fail_unless(doublerOutOld == APLL_FREQ_SEL_MAX,
                    "Apll set/get frequency doubler not working, unexpected return value: %d, "
                    "Apll id: %d", doublerOutOld, apllInstId);
        }
    }

    /* out of range double frequency selection */
    for (doublerIn = APLL_CONFIG_MAX; doublerIn <= APLL_CONFIG_MAX+1; doublerIn++)
    {
        for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
        {
            doublerOutOld = IDTApll_GetFreqDoublerSel(hdlr, apllInstId);
            IDTApll_SetFreqDoublerSel(hdlr, apllInstId, doublerIn);
            doublerOutNew = IDTApll_GetFreqDoublerSel(hdlr, apllInstId);
            fail_unless(doublerOutNew == doublerOutOld,
                    "Apll set/get frequency doubler not working, double freq selection modified with invalid input, "
                    "Apll id: %d, double frequency: %d", apllInstId, doublerIn);
        }
    }
}
END_TEST

START_TEST (test_ApllApiXtalId)
{
    T_idtApllId apllInstId;
    T_idtApllXtalId xtalIn, xtalOut;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++) 
    {
        for (xtalIn = APLL_XTAL_1_APLL1; xtalIn < APLL_XTAL_MAX; xtalIn++)
        {
            IDTApll_SetXtalId(hdlr, apllInstId, xtalIn);                            
            xtalOut = IDTApll_GetXtalId(hdlr, apllInstId);
            if (((APLL_INST_1 == apllInstId) && ((APLL_XTAL_1_APLL1 == xtalIn) || (APLL_XTAL_3_APLL1 == xtalIn)))||
                 ((APLL_INST_2 == apllInstId) && ((APLL_XTAL_2_APLL2 == xtalIn) || (APLL_XTAL_4_APLL2 == xtalIn))))
            {
                fail_unless(xtalOut == xtalIn,
                            "Apll set/set xtal id not working, output not the same as input "
                            "Apll id: %d, input: %d, output: %d",
                            apllInstId, xtalIn, xtalOut);
            }
            else
            {
                fail_unless(xtalOut != xtalIn,
                            "Apll set/set xtal id not working, output not the same as input "
                            "Apll id: %d, input: %d, output: %d",
                            apllInstId, xtalIn, xtalOut);
            }
        }
    }
}
END_TEST

START_TEST (test_ApllApiXtalIdInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllXtalId xtalIn,
                xtalOutOld,
                xtalOutNew;

    /* out of range chip id */
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
        for (xtalIn = APLL_XTAL_1_APLL1; xtalIn < APLL_XTAL_MAX; xtalIn++)
        {
            IDTApll_SetXtalId(hdlr, apllInstId, xtalIn);
            xtalOutOld = IDTApll_GetXtalId(hdlr, apllInstId);
            fail_unless(xtalOutOld == APLL_XTAL_MAX,
                    "Apll set/set xtal id not working, unexpected return value: %d, "
                    "Apll id: %d", xtalOutOld, apllInstId);
        }
    }

    /* out of range Xtal ID */
    for (xtalIn = APLL_XTAL_MAX; xtalIn <= APLL_XTAL_MAX+1; xtalIn++)
    {
        for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
        {
            xtalOutOld = IDTApll_GetXtalId(hdlr, apllInstId);
            IDTApll_SetXtalId(hdlr, apllInstId, xtalIn);
            xtalOutNew = IDTApll_GetXtalId(hdlr, apllInstId);
            fail_unless(xtalOutNew == xtalOutOld,
                    "Apll set/set xtal id not working, xtal selection modified with invalid input, "
                    "Apll id: %d, xtal: %d", apllInstId, xtalIn);
        }
    }
}
END_TEST

START_TEST (test_ApllApiOutputSigType)
{
    T_idtApllId apllInstId;
    T_idtApllOutput output;
    T_idtApllOutputSigType sigIn, sigOut;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++) 
    {
        for (output = APLL_OUTPUT_QA; output < APLL_OUTPUT_MAX; output++)
        {
            for (sigIn = APLL_OUT_SIG_LVPECL; sigIn < APLL_OUT_SIG_MAX; sigIn++)
            {
                IDTApll_SetOutputSigType(hdlr, apllInstId, output, sigIn); 
                sigOut = IDTApll_GetOutputSigType(hdlr, apllInstId, output);
                fail_unless(sigOut == sigIn,
                        "Apll set/get output signal type not working, output not the same as input "
                        "Apll id: %d, input: %d, output: %d",
                         apllInstId, sigIn, sigOut);
            }
        }
    }
}
END_TEST

START_TEST (test_ApllApiOutputSigTypeInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllOutput output;
    T_idtApllOutputSigType sigIn,
                         sigOutOld,
                         sigOutNew;

    /* out of range chip id */
    output = APLL_OUTPUT_QA;
    sigIn = APLL_OUT_SIG_LVPECL;
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
        IDTApll_SetOutputSigType(hdlr, apllInstId, output, sigIn);
        sigOutOld = IDTApll_GetOutputSigType(hdlr, apllInstId, output);
        fail_unless(sigOutOld == APLL_OUT_SIG_MAX,
                    "Apll set/get output signal type not working, unexpected return value: %d, "
                    "Apll id: %d", sigOutOld, apllInstId);
    }

    /* out of range output selection */
    apllInstId = APLL_INST_1;
    sigIn = APLL_OUT_SIG_LVPECL;
    for (output = APLL_OUTPUT_MAX; output <= APLL_OUTPUT_MAX+1; output++)
    {
        IDTApll_SetOutputSigType(hdlr, apllInstId, output, sigIn);
        sigOutOld = IDTApll_GetOutputSigType(hdlr, apllInstId, output);
        fail_unless(sigOutOld == APLL_OUT_SIG_MAX,
                    "Apll set/get output signal type not working, unexpected return value: %d, "
                    "Apll id: %d", sigOutOld, apllInstId);
    }

    /* out of range output signal type selection */
    apllInstId = APLL_INST_1;
    output = APLL_OUTPUT_QA;
    for (sigIn = APLL_OUT_SIG_MAX; sigIn <= APLL_OUT_SIG_MAX+1; sigIn++)
    {
        sigOutOld = IDTApll_GetOutputSigType(hdlr, apllInstId, output);

        IDTApll_SetOutputSigType(hdlr, apllInstId, output, sigIn);
        sigOutNew = IDTApll_GetOutputSigType(hdlr, apllInstId, output);
        fail_unless(sigOutOld == sigOutNew,
                    "Apll set/get output signal type not working, output signal selection modified with invalid input, "
                    "Apll id: %d, output signal: %d", apllInstId, sigIn);
    }
}
END_TEST

START_TEST (test_ApllApiOutputEnState)
{
    T_idtApllId apllInstId;
    T_idtApllOutput output;
    T_idtApllOutputEn enIn, enOut;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++) 
    {
        for (output = APLL_OUTPUT_QA; output < APLL_OUTPUT_MAX; output++)
        {
            for (enIn = APLL_OUTPUT_DISABLE; enIn < APLL_OUTPUT_EN_MAX; enIn++)
            {
                IDTApll_SetOutputEnState(hdlr, apllInstId, output, enIn);                           
                enOut = IDTApll_GetOutputEnState(hdlr, apllInstId, output);
                fail_unless(enOut == enIn,
                        "Apll set/get output enable state not working, output not the same as input "
                        "Apll id: %d, input: %d, output: %d",
                         apllInstId, enIn, enOut);
            }
        }
    }
}
END_TEST

START_TEST (test_ApllApiOutputEnStateInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllOutput output;
    T_idtApllOutputEn enIn,
                    enOutOld,
                    enOutNew;

    /* out of range chip id */
    output = APLL_OUTPUT_QA;
    enIn = APLL_OUTPUT_DISABLE;
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
        IDTApll_SetOutputEnState(hdlr, apllInstId, output, enIn);
        enOutOld = IDTApll_GetOutputEnState(hdlr, apllInstId, output);
        fail_unless(enOutOld == APLL_OUTPUT_EN_MAX,
                    "Apll set/get output enable state not working not working, unexpected return value: %d, "
                    "Apll id: %d", enOutOld, apllInstId);
    }

    /* out of range output */
    apllInstId = APLL_INST_1;
    enIn = APLL_OUTPUT_DISABLE;
    for (output = APLL_OUTPUT_MAX; output < APLL_OUTPUT_MAX+1; output++)
    {
        IDTApll_SetOutputEnState(hdlr, apllInstId, output, enIn);
        enOutOld = IDTApll_GetOutputEnState(hdlr, apllInstId, output);
        fail_unless(enOutOld == APLL_OUTPUT_EN_MAX,
                    "Apll set/get output enable state not working, unexpected return value: %d, "
                    "Apll id: %d", enOutOld, apllInstId);
    }

    /* out of range output enable state selection */
    apllInstId = APLL_INST_1;
    enIn = APLL_OUTPUT_DISABLE; 
    for (enIn = APLL_OUTPUT_EN_MAX; enIn <= APLL_OUTPUT_EN_MAX+1; enIn++)
    {
        enOutOld = IDTApll_GetOutputEnState(hdlr, apllInstId, output);

        IDTApll_SetOutputEnState(hdlr, apllInstId, output, enIn);
        enOutNew = IDTApll_GetOutputEnState(hdlr, apllInstId, output);
        fail_unless(enOutOld == enOutNew,
                    "Apll set/get output enable state not working, output state modified with invalid input, "
                    "Apll id: %d, enable: %d", apllInstId, enIn);
    }
}
END_TEST

START_TEST (test_ApllApiMasterResetSync)
{
    T_idtApllId apllInstId;
    T_idtApllMasterResetSync resetIn, resetOut;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++) 
    {
        for (resetIn = APLL_OUT_MR_SYNC; resetIn < APLL_MR_SYNC_MAX; resetIn++)
        {
            IDTApll_SetMasterResetSync(hdlr, apllInstId, resetIn);
            resetOut = IDTApll_GetMasterResetSync(hdlr, apllInstId);
            fail_unless(resetOut == resetIn,
                        "Apll set/get master reset sync not working, output not the same as input "
                        "Apll id: %d, input: %d, output: %d",
                         apllInstId, resetIn, resetOut);
        }
    }
}
END_TEST

START_TEST (test_ApllApiMasterResetSyncInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllMasterResetSync resetIn,
                             resetOutOld,
                             resetOutNew;

    /* out of range chip id */
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
        for (resetIn = APLL_OUT_MR_SYNC; resetIn < APLL_MR_SYNC_MAX; resetIn++)
        {
            IDTApll_SetMasterResetSync(hdlr, apllInstId, resetIn);
            resetOutOld = IDTApll_GetMasterResetSync(hdlr, apllInstId);
            fail_unless(resetOutOld == APLL_MR_SYNC_MAX,
                    "Apll set/get master reset sync not working, unexpected return value: %d, "
                    "Apll id: %d", resetOutOld, apllInstId);
        }
    }

    /* out of range master reset sync value */
    for (resetIn = APLL_MR_SYNC_MAX; resetIn <= APLL_MR_SYNC_MAX+1; resetIn++)
    {
        for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
        {
            resetOutOld = IDTApll_GetMasterResetSync(hdlr, apllInstId);
            IDTApll_SetMasterResetSync(hdlr, apllInstId, resetIn);
            resetOutNew = IDTApll_GetMasterResetSync(hdlr, apllInstId);
            fail_unless(resetOutOld == resetOutNew,
                    "Apll set/get master reset sync not working, output signal selection modified with invalid input, "
                    "Apll id: %d, reset: %d", apllInstId, resetIn);
        }
    }
}
END_TEST

START_TEST (test_ApllApiBypass)
{
    T_idtApllId apllInstId;
    T_idtApllBypass bypassIn, bypassOut;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++) 
    {
        for (bypassIn = APLL_BYPASS_NORMAL_OP; bypassIn < APLL_BYPASS_MAX; bypassIn++)
        {
            IDTApll_SetBypass(hdlr, apllInstId, bypassIn);
            bypassOut = IDTApll_GetBypass(hdlr, apllInstId);
            fail_unless(bypassOut == bypassIn,
                        "Apll set/get bypass not working, output not the same as input "
                        "Apll id: %d, input: %d, output: %d",
                         apllInstId, bypassIn, bypassOut);
        }
    }
}
END_TEST

START_TEST (test_ApllApiBypassInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllBypass bypassIn,
                    bypassOutOld,
                    bypassOutNew;

    /* out of range chip id */
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
        for (bypassIn = APLL_BYPASS_NORMAL_OP; bypassIn < APLL_BYPASS_MAX; bypassIn++)
        {
            IDTApll_SetBypass(hdlr, apllInstId, bypassIn);
            bypassOutOld = IDTApll_GetBypass(hdlr, apllInstId);
            fail_unless(bypassOutOld == APLL_BYPASS_MAX,
                    "Apll set/get bypass not working, unexpected return value: %d, "
                    "Apll id: %d", bypassOutOld, apllInstId);
        }
    }

    /* out of range bypass value */
    for (bypassIn = APLL_BYPASS_MAX; bypassIn <= APLL_BYPASS_MAX+1; bypassIn++)
    {
        for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
        {
            bypassOutOld = IDTApll_GetBypass(hdlr, apllInstId);
            IDTApll_SetBypass(hdlr, apllInstId, bypassIn);
            bypassOutNew = IDTApll_GetBypass(hdlr, apllInstId);
            fail_unless(bypassOutOld == bypassOutNew,
                    "Apll set/get bypass not working, output signal selection modified with invalid input, "
                    "Apll id: %d, reset: %d", apllInstId, bypassIn);
        }
    }
}
END_TEST

START_TEST (test_ApllApiShutoff)
{
    T_idtApllId apllInstId;
    T_idtApllShutoff shutoffIn, shutoffOut;

    for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++) 
    {
        for (shutoffIn = APLL_SHUTOFF_NORMAL_OP; shutoffIn < APLL_SHUTOFF_MAX; shutoffIn++)
        {
            IDTApll_SetShutoff(hdlr, apllInstId, shutoffIn);
            shutoffOut = IDTApll_GetShutoff(hdlr, apllInstId);
            fail_unless(shutoffOut == shutoffIn,
                        "Apll set/get shutoff not working, output not the same as input "
                        "Apll id: %d, input: %d, output: %d",
                         apllInstId, shutoffIn, shutoffOut);
        }
    }
}
END_TEST

START_TEST (test_ApllApiShutoffInvalidInput)
{
    T_idtApllId apllInstId;
    T_idtApllShutoff shutoffIn,
                     shutoffOutOld,
                     shutoffOutNew;

    /* out of range chip id */
    for (apllInstId = APLL_INST_MAX; apllInstId <= APLL_INST_MAX+1; apllInstId++)
    {
        for (shutoffIn = APLL_SHUTOFF_NORMAL_OP; shutoffIn < APLL_SHUTOFF_MAX; shutoffIn++)
        {
            IDTApll_SetShutoff(hdlr, apllInstId, shutoffIn);
            shutoffOutOld = IDTApll_GetShutoff(hdlr, apllInstId);
            fail_unless(shutoffOutOld == APLL_SHUTOFF_MAX,
                    "Apll set/get reset state not working, unexpected return value: %d, "
                    "Apll id: %d", shutoffOutOld, apllInstId);
        }
    }

    /* out of range reset value */
    for (shutoffIn = APLL_SHUTOFF_MAX; shutoffIn <= APLL_SHUTOFF_MAX+1; shutoffIn++)
    {
        for (apllInstId = APLL_INST_1; apllInstId < APLL_INST_MAX; apllInstId++)
        {
            shutoffOutOld = IDTApll_GetShutoff(hdlr, apllInstId);
            IDTApll_SetShutoff(hdlr, apllInstId, shutoffIn);
            shutoffOutNew = IDTApll_GetShutoff(hdlr, apllInstId);
            fail_unless(shutoffOutOld == shutoffOutNew,
                    "Apll set/get shutoff not working, output signal selection modified with invalid input, "
                    "Apll id: %d, reset: %d", apllInstId, shutoffIn);
        }
    }
}
END_TEST

Suite *ApllApi_suite (void)
{
    Suite *s = suite_create("ApllApi");

    TCase *tc_ApllApi = tcase_create("ApllApi");

    tcase_add_checked_fixture (tc_ApllApi,
                               initalizeDriver, 
                               deinitilizeDriver);

    tcase_add_test(tc_ApllApi, test_ApllApiConfigSel);
    tcase_add_test(tc_ApllApi, test_ApllApiConfigSelInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiInputClkSrc);
    tcase_add_test(tc_ApllApi, test_ApllApiInputClkSrcInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiPreDiv);
    tcase_add_test(tc_ApllApi, test_ApllApiPreDivInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiFeedbackDiv);
    tcase_add_test(tc_ApllApi, test_ApllApiFeedbackDivInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiQaDiv);
    tcase_add_test(tc_ApllApi, test_ApllApiQaDivInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiQbDiv);
    tcase_add_test(tc_ApllApi, test_ApllApiQbDivInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiFreqDoublerSel);
    tcase_add_test(tc_ApllApi, test_ApllApiFreqDoublerSelInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiXtalId);
    tcase_add_test(tc_ApllApi, test_ApllApiXtalIdInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiOutputSigType);
    tcase_add_test(tc_ApllApi, test_ApllApiOutputSigTypeInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiOutputEnState);
    tcase_add_test(tc_ApllApi, test_ApllApiOutputEnStateInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiMasterResetSync);
    tcase_add_test(tc_ApllApi, test_ApllApiMasterResetSyncInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiBypass);
    tcase_add_test(tc_ApllApi, test_ApllApiBypassInvalidInput);
    tcase_add_test(tc_ApllApi, test_ApllApiShutoff);
    tcase_add_test(tc_ApllApi, test_ApllApiShutoffInvalidInput);

    suite_add_tcase(s, tc_ApllApi);

    return s;
}
