/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "RegisterDef.h"
#include "osNumeric.h"
#include "Hal.h"
#include "ApllReadWrite_sim.h"

/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/
#define DRV_MEMMAP_SIZE 0x80
#define NONE_APLL_REG_SPACE_FLAG 0x01

int simDebugFlag = 0;

/** Reset values for device */
T_osUint8 simulatedMemMap[2][DRV_MEMMAP_SIZE] = {
  {
    0x91, 0x33, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x00 */
    0x32, 0xA2, 0x89, 0x85, 0x02, 0xFF, 0x3F, 0xD5, /* 0x08 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x10 */
    0x00, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x01, /* 0x18 */
    0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x32, 0x54, /* 0x20 */
    0x76, 0x98, 0xBA, 0xDC, 0xFE, 0x00, 0x0B, 0x23, /* 0x28 */
    0x23, 0x06, 0x04, 0x08, 0x01, 0x06, 0x04, 0x08, /* 0x30 */
    0x01, 0x06, 0x04, 0x08, 0x01, 0x06, 0x04, 0x08, /* 0x38 */
    0x01, 0x00, 0x00, 0x66, 0x66, 0x66, 0x66, 0x66, /* 0x40 */
    0x66, 0x66, 0x00, 0x00, 0xFF, 0x3F, 0x00, 0x00, /* 0x48 */
    0x00, 0x00, 0x81, 0x00, 0x00, 0x01, 0x6F, 0x6F, /* 0x50 */
    0x6B, 0x88, 0x85, 0xA2, 0x44, 0x00, 0x00, 0x00, /* 0x58 */
    0x4C, 0x60, 0x00, 0x00, 0x00, 0x8C, 0xAB, 0x19, /* 0x60 */
    0x00, 0x00, 0x00, 0x0B, 0x06, 0x08, 0x06, 0x04, /* 0x68 */
    0x06, 0x08, 0x40, 0x40, 0x60, 0x00, 0x00, 0x00, /* 0x70 */
    0x06, 0x00, 0x00, 0x00, 0x2B, 0x00, 0x85, 0x05  /* 0x78 */
  },
  { /* Page 1 and T4 overlay registers */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x00 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x08 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x10 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x18 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x32, 0x54, /* 0x20 */
    0x76, 0x98, 0xBA, 0xDC, 0xFE, 0x00, 0x00, 0x00, /* 0x28 */
    0x23, 0x06, 0x04, 0x08, 0x01, 0x00, 0x00, 0x00, /* 0x30 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, /* 0x38 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x40 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x48 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x50 */
    0x00, 0x00, 0x85, 0xA2, 0x00, 0x00, 0x00, 0x00, /* 0x58 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x60 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x68 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 0x70 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  /* 0x78 */
  }
};

T_osUint8 dpllOverlayRegisters[] = {
  REG_IN1_IN2_SEL_PRIORITY_CNFG,    /**< Input 1 and Input 2 Priority
				       Configuration Register */
  REG_IN3_IN4_SEL_PRIORITY_CNFG,    /**< Input 3 and Input 4 Priority
				       Configuration Register */
  REG_IN5_IN6_SEL_PRIORITY_CNFG,    /**< Input 5 and Input 6 Priority
				       Configuration Register */
  REG_IN7_IN8_SEL_PRIORITY_CNFG,    /**< Input 7 and Input 8 Priority
				       Configuration Register */
  REG_IN9_IN10_SEL_PRIORITY_CNFG,   /**< Input 9 and Input 10 Priority
				       Configuration Register */
  REG_IN11_IN12_SEL_PRIORITY_CNFG,  /**< Input 11 and Input 12
				       Priority Configuration
				       Register */
  REG_IN13_IN14_SEL_PRIORITY_CNFG,  /**< Input 13 and Input 14
				       Priority Configuration
				       Register */
  REG_PRIORITY_TABLE1_STS,          /**< The Highest Priority Valid
				       Input Clock and the Selected
				       Input Clock Register */
  REG_PRIORITY_TABLE2_STS,          /**< The Second Highest Priority
				       Valid Input Clock and the
				       Selected Input Clock
				       Register */
  REG_PHASE_LOSS_COARSE_LIMIT_CNFG, /**< Coarse Phase Lock Detector
				       Configuration Register */
  REG_PHASE_LOSS_FINE_LIMIT_CNFG,   /**< Fine Phase Lock Detector
				       Configuration Register */
  REG_HOLDOVER_FREQ_LOW_CNFG,       /**< DPLL Holdover Frequency
				       Register 1 */
  REG_HOLDOVER_FREQ_MID_CNFG,       /**< DPLL Holdover Frequency
				       Register 2 */
  REG_HOLDOVER_FREQ_HIGH_CNFG,      /**< DPLL Holdover Frequency
				       Register 3 */
  REG_CURRENT_FREQ_LOW_STS,         /**< DPLL Current Frequency
				       Register 1 */
  REG_CURRENT_FREQ_MID_STS,         /**< DPLL Current Frequency
				       Register 2 */
  REG_CURRENT_FREQ_HIGH_STS,        /**< DPLL Current Frequency
				       Register 3 */
  REG_CURRENT_DPLL_PHASE_LOW_STS,   /**< Current DPLL Phase Register
				       1 */
  REG_CURRENT_DPLL_PHASE_HIGH_STS   /**< Current DPLL Phase Register
				       2 */

};
static const int dpllOverlayRegistersSize = (sizeof(dpllOverlayRegisters)/
                                             sizeof(T_osUint8));

T_osUint8 page1Registers[] = {
  REG_DFS_OFF_CNFG,     /**< Digital Frequency Synthesizer
			   Configuration Register */
  REG_T0_PPS_CNFG,      /**< T0 1 Pulse Per Second Configuration
			   Register */
  REG_PH_SLOPE_CNFG,    /**< Phase Slope limiting Register */
  REG_T0_ICP_CTRL_CNFG, /**< T0 APLL Charge Pump Current Configuration
			   Register */
  REG_T4_ICP_CTRL_CNFG, /**< T4 APLL Charge Pump Current Configuration
			   Register */
  REG_T4_PPS_CNFG       /**< T4 1 Pulse Per Second Configuration Register */
};
static const int page1RegistersSize = (sizeof(page1Registers)/
                                       sizeof(T_osUint8));

/*********************************************************************
 * F U N C T I O N    I M P L E M E N T A T I O N
 *********************************************************************/
void Init_Sim(void *usrData) 
{
  ApllInit_Sim(usrData);
}

void DeInit_Sim(void *usrData) 
{
  ApllDeInit_Sim(usrData);
}

T_osUint8 RDByte_Sim(void *usrData, T_osUint8 devAddr, T_osUint8 nAddr)
{
  T_osUint8 regVal;
  int   i, page = 0;

  if (devAddr & NONE_APLL_REG_SPACE_FLAG)
  {
      assert(nAddr < DRV_MEMMAP_SIZE);

      /* If DPLL overlayed register, select appropriate "page" */
      for (i=0; i<dpllOverlayRegistersSize; i++) {
        if (nAddr == dpllOverlayRegisters[i]) {
          page = IDTHAL_GetBitfield(simulatedMemMap[0][REG_T4_T0_SEL_CNFG],
    			  BF_T4_T0_SEL_MASK,
    			  BF_T4_T0_SEL_LSB);
          break;
        }
      }

      /* If page1 register, select page 1 */
      for (i=0; i<page1RegistersSize; i++) {
        if (nAddr == page1Registers[i]) {
          page = IDTHAL_GetBitfield(simulatedMemMap[0][REG_PAGE_POINTER_CNFG],
    			 BF_PAGE_POINTER_MASK,
    			 BF_PAGE_POINTER_LSB);
          break;
        }
      }

      /* If input divider register, select page 1 (we're cheating my
       * re-using unsued registers in page 1 - to save memory space.
       *
       * We "steal" registers 00 - 1C for input divider values */
      if (nAddr == REG_PRE_DIVN_LOW_CNFG)
      {
              page = 1;
              i = simulatedMemMap[0][REG_PRE_DIV_CH_CNFG] - 1;
              nAddr = ((i)*2);
      }
      if (nAddr == REG_PRE_DIVN_HIGH_CNFG)
      {
              page = 1;
              i = simulatedMemMap[0][REG_PRE_DIV_CH_CNFG] - 1;
              nAddr = ((i)*2)+1;
      }

      regVal = simulatedMemMap[page][nAddr];
      return regVal;
  }
  else
  {
      return ApllRead_Sim(usrData, devAddr, nAddr);
  }
}

void WRByte_Sim(void *usrData, T_osUint8 devAddr, T_osUint8 nAddr, T_osUint8 nData)
{
  int i, page = 0;

  if (devAddr & NONE_APLL_REG_SPACE_FLAG)
  {
      assert(nAddr < DRV_MEMMAP_SIZE);

      /* If DPLL overlayed register, select appropriate "page" */
      for (i=0; i<dpllOverlayRegistersSize; i++) {
        if (nAddr == dpllOverlayRegisters[i]) {
          page = IDTHAL_GetBitfield(simulatedMemMap[0][REG_T4_T0_SEL_CNFG],
    			  BF_T4_T0_SEL_MASK,
    			  BF_T4_T0_SEL_LSB);
          break;
        }
      }

      /* If page1 register, select page 1 */
      for (i=0; i<page1RegistersSize; i++) {
        if (nAddr == page1Registers[i]) {
          page = IDTHAL_GetBitfield(simulatedMemMap[0][REG_PAGE_POINTER_CNFG],
    			 BF_PAGE_POINTER_MASK,
    			 BF_PAGE_POINTER_LSB);
          break;
        }
      }

      /* If input divider register, select page 1 (we're cheating my
       * re-using unsued registers in page 1 - to save memory space.
       *
       * We "steal" registers 00 - 1C for input divider values */
      if (nAddr == REG_PRE_DIVN_LOW_CNFG)
      {
              page = 1;
              i = simulatedMemMap[0][REG_PRE_DIV_CH_CNFG] - 1;
              nAddr = ((i)*2);
      }
      if (nAddr == REG_PRE_DIVN_HIGH_CNFG)
      {
              page = 1;
              i = simulatedMemMap[0][REG_PRE_DIV_CH_CNFG] - 1;
              nAddr = ((i)*2)+1;
      }

      if (simDebugFlag)
        printf("Reg%02X %d = 0x%02X\n",nAddr,page,nData);

      simulatedMemMap[page][nAddr] = nData;
  }
  else
  {
      ApllWrite_Sim(usrData, devAddr, nAddr, nData);
  }
}
