/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#ifndef __OUTPUTFREQ_H__
#define __OUTPUTFREQ_H__

#include "Define.h"

#ifdef __cplusplus
extern "C"
{
#endif
/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/
/** @brief Enumerated type listing output frequencies */
typedef enum {
        OutFreq_Invalid, /**< Output frequency invalid */
        OutFreq_64k, /**< Output frequency is 64 kHz */
        OutFreq_8k, /**< Output frequency is 8 kHz */
        OutFreq_2k, /**< Output frequency is 2 kHz */
        OutFreq_400, /**< Output frequency is 400 Hz */
        OutFreq_1, /**< Output frequency is 1 Hz (1 PPS) */
        OutFreq_24576k, /**< Output frequency is 24.576 MHz */
        OutFreq_12288k, /**< Output frequency is 12.288 MHz  */
        OutFreq_6144k, /**< Output frequency is 6.144 MHz  */
        OutFreq_4096k, /**< Output frequency is 4.096 MHz  */
        OutFreq_2048k, /**< Output frequency is 2.048 MHz  */
        OutFreq_32768k, /**< Output frequency is 32.768 MHz  */
        OutFreq_16384k, /**< Output frequency is 16.384MHz  */
        OutFreq_8192k, /**< Output frequency is 8.192 MHz  */
        OutFreq_24704k, /**< Output frequency is 24.704 MHz  */
        OutFreq_12352k, /**< Output frequency is 12.352 MHz  */
        OutFreq_6176k, /**< Output frequency is 6.176 MHz  */
        OutFreq_3088k, /**< Output frequency is 3.088 MHz  */
        OutFreq_1544k, /**< Output frequency is 1.544 MHz  */
        OutFreq_34368k, /**< Output frequency is 34.368 MHz  */
        OutFreq_44736k, /**< Output frequency is 44.736 MHz  */
        OutFreq_26000k, /**< Output frequency is 26 MHz  */
        OutFreq_13000k, /**< Output frequency is 13 MHz  */
        OutFreq_30720k, /**< Output frequency is 30.72 MHz  */
        OutFreq_15360k, /**< Output frequency is 15.36 MHz  */
        OutFreq_7680k, /**< Output frequency is 7.68 MHz  */
        OutFreq_3840k, /**< Output frequency is 3.84 MHz  */
        OutFreq_37056k, /**< Output frequency is 37.056 MHz  */
        OutFreq_18528k, /**< Output frequency is 18.528 MHz  */
        OutFreq_9264k, /**< Output frequency is 9.264 MHz  */
        OutFreq_4632k, /**< Output frequency is 4.632 MHz  */
        OutFreq_40000k, /**< Output frequency is 40 MHz  */
        OutFreq_20000k, /**< Output frequency is 20 MHz  */
        OutFreq_10000k, /**< Output frequency is 10 MHz  */
        OutFreq_5000k, /**< Output frequency is 5 MHz  */
        OutFreq_622080k, /**< Output frequency is 622.08 MHz  */
        OutFreq_625000k, /**< Output frequency is 625 MHz  */
        OutFreq_625000k_66_64, /**< Output frequency is 625*(66/64) MHz  */
        OutFreq_311040k, /**< Output frequency is 311.04 MHz  */
        OutFreq_312500k, /**< Output frequency is 312.5 MHz */
        OutFreq_312500k_66_64, /**< Output frequency is 312.5*(66/64) MHz  */
        OutFreq_155520k, /**< Output frequency is 155.52 MHz  */
        OutFreq_156250k, /**< Output frequency is 156.25 MHz  */
        OutFreq_156250k_66_64, /**< Output frequency is 156.25*(66/64) MHz  */
        OutFreq_77760k, /**< Output frequency is 77.76 MHz  */
        OutFreq_51840k, /**< Output frequency is 51.84 MHz  */
        OutFreq_38880k, /**< Output frequency is 38.88 MHz  */
        OutFreq_25920k, /**< Output frequency is 25.92 MHz  */
        OutFreq_125000k, /**< Output frequency is 125 MHz  */
        OutFreq_125000k_66_64, /**< Output frequency is 125*(66/64) MHz  */
        OutFreq_19440k, /**< Output frequency is 19.44 MHz  */
        OutFreq_25000k, /**< Output frequency is 25 MHz  */
        OutFreq_25000k_66_64, /**< Output frequency is 25*(66/64) MHz  */
        OutFreq_6480k, /**< Output frequency is 6.48 MHz  */
        /* Additional frequencies output by APLL */
		OutFreq_24883_DOT_2k, /**< Output frequency is 24.8832 MHz */
		OutFreq_78125k, /**< Output frequency is 78.125 MHz */
		OutFreq_78125k_66_64, /**< Output frequency is 80.56640625 MHz */
		OutFreq_124416k, /**< Output frequency is 124.416 MHz */
        OutFreq_MAX    /**< \internal Not used. */
} outputFrequency_t;


/**
 * @brief Enumerated type listing sonetGige modes
 */
typedef enum E_sonetGigeMode 
{
    sonetGigeMode_Matching, /**< DPLL #1 to APLL #1 and DPLL #2 to
                             * APLL #2 */
    sonetGigeMode_AllFromDpll1, /**< DPLL #1 to APLL #1 and DPLL #1 to
                                 * APLL #2 */
    sonetGigeMode_AllFromDpll2, /**< DPLL #2 to APLL #1 and DPLL #2 to
                                 * APLL #2 */
    sonetGigeMode_MAX /**< \internal Not used */
} T_sonetGigeMode;


/*---------------------------------------------------------------------
 * F U N C T I O N   P R O T O T Y P E S
 *---------------------------------------------------------------------*/

/** Provision output port specifying output freqency and DPLL
 *   to use */
int  IDTHlApi_ConfigureOutput(T_idtDrvHdlr hdlr,
                              T_osUint8 nOutN,
                              outputFrequency_t	nOutFreq,
                              T_idtDpllInstance dpllInstance,
                              T_sonetGigeMode sonetGigeMode,
                              T_idtApllConfigSel apllConfSel,
                              T_idtApllOutputSigType apllOutputSigType,
                              T_idtApllInputClkSrc apllClkSource);

/* Function to disable all outputs. */
void IDTHlApi_DisableAllOutputs(T_idtDrvHdlr hdlr);

/* Function to reset high level API provisioing information */
void IDTHlApi_ClearProvisioning(T_idtDrvHdlr hdlr) ;

void IDTHlApi_PrintOutput(T_idtDrvHdlr hdlr);
void IDTHlApi_PrintInput(T_idtDrvHdlr hdlr);


#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __OUTPUTFREQ_H__ */
