/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#ifndef __OUTPUTFREQ_PRIV_H__
#define __OUTPUTFREQ_PRIV_H__

#include "Define.h"
#include "General.h"
#include "DpllCnfg.h"
#include "Output.h"
#include "Dpll.h"

#include "outputFreq.h"

#ifdef __cplusplus
extern "C"
{
#endif
/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/** @brief Enumerated type listing output path selectors */
typedef enum {
    Selector_Network, /**< Network selector (SDH/SONET) */
    Selector_T0DpllSelA, /**< T0 Dpll selector A (16E1/16T1/GSM/OBSAI) */
    Selector_T0DpllSelB, /**< T0 Dpll selector B (12E1/GPS/E3/T3) */
    Selector_T4DpllSelA, /**< T4 Dpll selector A (16E1/16T1/GSM/GPS) */
    Selector_T4DpllSelB, /**< T4 Dpll selector B (12E1/24T1/E3/T3) */
    Selector_SonetGigEth, /**< APLL selector (T0 622/T4 622/T0 625/
                           *                  T4 625/T0 625(66/64)/
                           *                  T4 625(66/64)) */
    Selector_Any, /**< For 77.76 / Eth always available frequencies */
    Selector_MAX  /**< \internal Not used. */
} pathSelectors_t;

/** @brief Structure containing select configuration.
 * Used to translate between frequency and output path configuration
 */
typedef struct {
    pathSelectors_t selector_m; /**< Path selector */
    int             value_m;    /**< Path selector value */
    T_osUint8       outputDivider_m; /**< Output divider value */
    T_osUint8       outputPathSelector_m; /**< Output path selector */
    T_osUint8       instance_m; /**< instance of this configuration */
} pathSelectorConfig_t;

/** @brief Structure to index to path selector configs */
typedef struct {
        int start_m; /**< Start index */
        int size_m; /**< Number of entries */
} pathSelectorIndex_t;

#if 1
/** @brief set a bit within a bit array */
#define SET_BIT(array,bit,set) \
        array[bit] = set

/** @brief check a bit within a bit array */
#define CHECK_BIT(array,bit) \
        (array[bit] != OutFreq_Invalid)
#else
/** @brief set a bit within a bit array */
#define SET_BIT(array,bit,set)                                 \
        array[(bit)%8] = ((array[(bit)/8] & ~(1<<((bit)%8))) | \
                          (set)?((1<<((bit)%8)):0))

/** @brief check a bit within a bit array */
#define CHECK_BIT(array,bit) \
        (array[(bit)%8] & (1<<((bit)%8)))
#endif


/** @brief Array type for storing frequency usage */
typedef int validFreq_t[OutFreq_MAX];

/*---------------------------------------------------------------------
 * F U N C T I O N   P R O T O T Y P E S
 *---------------------------------------------------------------------*/
void IDTHlApi_ConvertFromFreqListToFreqArray(outputFrequency_t frequencies[],
                       validFreq_t       validFrequencies);
void IDTHlApi_ApplyFrequencies(const outputFrequency_t *frequencies,
                      validFreq_t              validFrequencies,
                      int                      value);
int IDTHlApi_IsFreqArrayInFreqList(const outputFrequency_t *frequencies,
                     validFreq_t              validFrequencies);
int IDTHlApi_IsFrequencyInFreqList(outputFrequency_t        frequency,
                   const outputFrequency_t  validFrequencies[]);

extern const char *IDTHlApi_Frequency2String_c[OutFreq_MAX];
void IDTHlApi_PrintAvailFrequencies(validFreq_t frequencyArray);
void IDTHlApi_PrintFrequencyList(const outputFrequency_t frequencies[]);
void IDTHlApi_PrintConfiguration(T_IDTPathConfiguration pathConfig);
void IDTHlApi_PrintOption(pathSelectorConfig_t option, T_osUint8 printPort);
void IDTHlApi_PrintSelectedOutputPath(T_outputPathType  nPath);
void IDTHlApi_PrintOutputHwConfig(
    networkType_t            network,
    T_osUint8                nOutN,
    T_osUint8                internalPort,
    T_osUint8                isEnabled,
    T_osUint8                hasApll,
    T_idtApllDividerValue    apllDivVal,
    T_idtApllXtalType        xtalType,
    T_osUint8                outDiv,
    T_outputPathType         outPath,
    T_idtDpllInstance        dpllInstOrSonetGigEthInst,
    T_idtDpllInstance        sonetGigEthInput,
    T_idtSonetGigEthOutput   sonetGigEthMode,
    T_idtDpllOutputSelectorA dpllSelA,
    T_idtDpllOutputSelectorB dpllSelB);


#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __OUTPUTFREQ_PRIV_H__ */
