/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <stdlib.h>

#include "Define.h"
#include "Dpll.h"
#include "Hal.h"
#include "Output.h"
#include "DpllCnfg.h"

#include "outputFreq.h"
#include "outputFreq_priv.h"

#include "ApllFreqProfile.h"
/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/


/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/
#define MAX_SUPPORTED_FREQUENCIES 9

/**
 * @brief Macro to reduce size of output path configuration
 * entries. Used to reduce size of table so that it fits inside 80
 * columns.
 * 
 */
#define OUTMUX_ENTRY(sel,selval,div,path,inst)                  \
    { Selector_##sel, selval, div, OutputPath_##path, inst }

/** 
 * @brief Table translating frequency to path configuration setting
 */
static const pathSelectorConfig_t IDTHlApi_freqencyToPathConfig[] = {
    /*           Selector     Selector Value        Div  Output Path     Inst */
    /*                                              Val                       */
    OUTMUX_ENTRY(MAX,         0,                    0x0, MAX,            0), /**< Invalid */
    OUTMUX_ENTRY(Any,         0,                    0xA, 7776kHz,        0), /**< 64 kHz */
    OUTMUX_ENTRY(Any,         0,                    0xA, 7776kHz,        1),
    OUTMUX_ENTRY(Any,         0,                    0xB, 7776kHz,        0), /**< 8 kHz */
    OUTMUX_ENTRY(Any,         0,                    0xB, 7776kHz,        1),
    OUTMUX_ENTRY(Any,         0,                    0xC, 7776kHz,        0), /**< 2 kHz */
    OUTMUX_ENTRY(Any,         0,                    0xC, 7776kHz,        1),
    OUTMUX_ENTRY(Any,         0,                    0xD, 7776kHz,        0), /**< 400  Hz */
    OUTMUX_ENTRY(Any,         0,                    0xD, 7776kHz,        1),
    OUTMUX_ENTRY(Any,         0,                    0xE, 7776kHz,        0), /**< 1 Hz (1 PPS) */
    OUTMUX_ENTRY(Any,         0,                    0xE, 7776kHz,        1),
    OUTMUX_ENTRY(T0DpllSelB,  DPLLOutputSelB_12E1,  0x2, 12E1_E3_T3,     0), /**< 24.576 MHz */
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_12E1,  0x2, 12E1_E3_T3,     1),
    OUTMUX_ENTRY(T0DpllSelB,  DPLLOutputSelB_12E1,  0x3, 12E1_E3_T3,     0), /**< 12.288 MHz */
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_12E1,  0x3, 12E1_E3_T3,     1),
    OUTMUX_ENTRY(T0DpllSelB,  DPLLOutputSelB_12E1,  0x4, 12E1_E3_T3,     0), /**< 6.144 MHz */
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_12E1,  0x4, 12E1_E3_T3,     1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_16E1,  0x6, GSM_16E1_16T1,  0), /**< 4.096 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_16E1,  0x6, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(T0DpllSelB,  DPLLOutputSelB_12E1,  0x5, 12E1_E3_T3,     0),
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_12E1,  0x5, 12E1_E3_T3,     1),
    OUTMUX_ENTRY(Network,     Network_SDH,          0x6, 16E1_16T1,      0),
    OUTMUX_ENTRY(Network,     Network_SDH,          0x6, 16E1_16T1,      1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_16E1,  0x8, GSM_16E1_16T1,  0), /**< 2.048 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_16E1,  0x8, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(T0DpllSelB,  DPLLOutputSelB_12E1,  0x7, 12E1_E3_T3,     0),
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_12E1,  0x7, 12E1_E3_T3,     1),
    OUTMUX_ENTRY(Network,     Network_SDH,          0x8, 16E1_16T1,      0),
    OUTMUX_ENTRY(Network,     Network_SDH,          0x8, 16E1_16T1,      1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_16E1,  0x2, GSM_16E1_16T1,  0), /**< 32.768 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_16E1,  0x2, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(Network,     Network_SDH,          0x2, 16E1_16T1,      0),
    OUTMUX_ENTRY(Network,     Network_SDH,          0x2, 16E1_16T1,      1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_16E1,  0x3, GSM_16E1_16T1,  0), /**< 16.384 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_16E1,  0x3, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(Network,     Network_SDH,          0x3, 16E1_16T1,      0),
    OUTMUX_ENTRY(Network,     Network_SDH,          0x3, 16E1_16T1,      1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_16E1,  0x4, GSM_16E1_16T1,  0), /**< 8.192 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_16E1,  0x4, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(Network,     Network_SDH,          0x4, 16E1_16T1,      0),
    OUTMUX_ENTRY(Network,     Network_SDH,          0x4, 16E1_16T1,      1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_16T1,  0x2, GSM_16E1_16T1,  0), /**< 24.704 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_16T1,  0x2, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(Network,     Network_SONET,        0x2, 16E1_16T1,      0),
    OUTMUX_ENTRY(Network,     Network_SONET,        0x2, 16E1_16T1,      1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_16T1,  0x3, GSM_16E1_16T1,  0), /**< 12.352 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_16T1,  0x3, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(Network,     Network_SONET,        0x3, 16E1_16T1,      0),
    OUTMUX_ENTRY(Network,     Network_SONET,        0x3, 16E1_16T1,      1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_16T1,  0x4, GSM_16E1_16T1,  0), /**< 6.176 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_16T1,  0x4, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_24T1,  0x5, 24T1,           1),
    OUTMUX_ENTRY(Network,     Network_SONET,        0x4, 16E1_16T1,      0),
    OUTMUX_ENTRY(Network,     Network_SONET,        0x4, 16E1_16T1,      1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_16T1,  0x6, GSM_16E1_16T1,  0), /**< 3.088 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_16T1,  0x6, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_24T1,  0x7, 24T1,           1),
    OUTMUX_ENTRY(Network,     Network_SONET,        0x6, 16E1_16T1,      0),
    OUTMUX_ENTRY(Network,     Network_SONET,        0x6, 16E1_16T1,      1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_16T1,  0x8, GSM_16E1_16T1,  0), /**< 1.544 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_16T1,  0x8, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_24T1,  0x9, 24T1,           1),
    OUTMUX_ENTRY(Network,     Network_SONET,        0x8, 16E1_16T1,      0),
    OUTMUX_ENTRY(Network,     Network_SONET,        0x8, 16E1_16T1,      1),
    OUTMUX_ENTRY(T0DpllSelB,  DPLLOutputSelB_E3,    0x2, 12E1_E3_T3,     0), /**< 34.368 MHz */
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_E3,    0x2, 12E1_E3_T3,     1),
    OUTMUX_ENTRY(T0DpllSelB,  DPLLOutputSelB_T3,    0x2, 12E1_E3_T3,     0), /**< 44.736 MHz */
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_T3,    0x2, 12E1_E3_T3,     1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_GSM,   0x2, GSM_16E1_16T1,  0), /**< 26 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_GSM,   0x2, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_GSM,   0x3, GSM_16E1_16T1,  0), /**< 13 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_GSM,   0x3, GSM_16E1_16T1,  1),
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_OBSAI, 0x2, OBSAI,          0), /**< 30.72 MHz */
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_OBSAI, 0x3, OBSAI,          0), /**< 15.36 MHz */
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_OBSAI, 0x4, OBSAI,          0), /**< 7.68 MHz */
    OUTMUX_ENTRY(T0DpllSelA,  DPLLOutputSelA_OBSAI, 0x6, OBSAI,          0), /**< 3.84 MHz */
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_24T1,  0x2, 24T1,           1), /**< 37.056 MHz */
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_24T1,  0x3, 24T1,           1), /**< 18.528 MHz */
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_24T1,  0x4, 24T1,           1), /**< 9.264 MHz */
    OUTMUX_ENTRY(T4DpllSelB,  DPLLOutputSelB_24T1,  0x6, 24T1,           1), /**< 4.632 MHz */
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_GPS,   0x2, GPS,            1), /**< 40 MHz */
    OUTMUX_ENTRY(T0DpllSelB,  DPLLOutputSelB_GPS,   0x2, GPS,            0),
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_GPS,   0x3, GPS,            1), /**< 20 MHz */
    OUTMUX_ENTRY(T0DpllSelB,  DPLLOutputSelB_GPS,   0x3, GPS,            0),
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_GPS,   0x4, GPS,            1), /**< 10 MHz */
    OUTMUX_ENTRY(T0DpllSelB,  DPLLOutputSelB_GPS,   0x4, GPS,            0),
    OUTMUX_ENTRY(T4DpllSelA,  DPLLOutputSelA_GPS,   0x6, GPS,            1), /**< 5 MHz */
    OUTMUX_ENTRY(T0DpllSelB,  DPLLOutputSelB_GPS,   0x6, GPS,            0),

    /*           Selector     Selector Value                 Div  Output Path   Inst */
    /*                                                       Val                     */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x2, SonetGigEth,  0), /**< 622.08 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x2, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE,   0x2, SonetGigEth,  0), /**< 625 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE,   0x2, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE_6664, 0x2, SonetGigEth,  0), /**< 625*(66/64) MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE_6664, 0x2, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x3, SonetGigEth,  0), /**< 311.04 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x3, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE,   0x3, SonetGigEth,  0), /**< 312.5 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE,   0x3, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE_6664, 0x3, SonetGigEth,  0), /**< 312.5*(66/64) MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE_6664, 0x3, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x4, SonetGigEth,  0), /**< 155.52 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x4, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE,   0x4, SonetGigEth,  0), /**< 156.25 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE,   0x4, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE_6664, 0x4, SonetGigEth,  0), /**< 156.25*(66/64) MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE_6664, 0x4, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x6, SonetGigEth,  0), /**< 77.76 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x6, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x7, SonetGigEth,  0), /**< 51.84 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x7, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x8, SonetGigEth,  0), /**< 38.88 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x8, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x9, SonetGigEth,  0), /**< 25.92 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0x9, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE,   0x9, SonetGigEth,  0), /**< 125 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE,   0x9, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE_6664, 0x9, SonetGigEth,  0), /**< 125*(66/64) MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE_6664, 0x9, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0xA, SonetGigEth,  0), /**< 19.44 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0xA, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE,   0xA, SonetGigEth,  0), /**< 25 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE,   0xA, SonetGigEth,  1),
    OUTMUX_ENTRY(Any,         0,                             0x2, Eth,          0),
    OUTMUX_ENTRY(Any,         0,                             0x2, Eth,          1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE_6664, 0xA, SonetGigEth,  0), /**< 25*(66/64) MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_10GbE_6664, 0xA, SonetGigEth,  1),
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0xD, SonetGigEth,  0), /**< 6.48 MHz */
    OUTMUX_ENTRY(SonetGigEth, SonetGigEthOutput_Sonet,   0xD, SonetGigEth,  1)
};

/** @brief Table indexing frequency translation table.
 *
 * We need this because a frequency can be configure multiple ways
 * (multiple entries)) We use entries in this table to index back to
 * the path selection option table (see above)
 */
static const pathSelectorIndex_t IDTHlApi_freqencyToPathConfigIndex[OutFreq_MAX] = {
        { 0, 0 }, /* Invalid */
        { 1, 2 }, /* 64 kHz */
        { 3, 2 }, /* 8 kHz */
        { 5, 2 }, /* 2 kHz */
        { 7, 2 }, /* 400  Hz */
        { 9, 2 }, /* 1 Hz (1 PPS) */
        { 11, 2 }, /* 24.576 MHz */
        { 13, 2 }, /* 12.288 MHz */
        { 15, 2 }, /* 6.144 MHz */
        { 17, 6 }, /* 4.096 MHz */
        { 23, 6 }, /* 2.048 MHz */
        { 29, 4 }, /* 32.768 MHz */
        { 33, 4 }, /* 16.384 MHz */
        { 37, 4 }, /* 8.192 MHz */
        { 41, 4 }, /* 24.704 MHz */
        { 45, 4 }, /* 12.352 MHz */
        { 49, 5 }, /* 6.176 MHz */
        { 54, 5 }, /* 3.088 MHz */
        { 59, 5 }, /* 1.544 MHz */
        { 64, 2 }, /* 34.368 MHz */
        { 66, 2 }, /* 44.736 MHz */
        { 68, 2 }, /* 26 MHz */
        { 70, 2 }, /* 13 MHz */
        { 72, 1 }, /* 30.72 MHz */
        { 73, 1 }, /* 15.36 MHz */
        { 74, 1 }, /* 7.68 MHz */
        { 75, 1 }, /* 3.84 MHz */
        { 76, 1 }, /* 37.056 MHz */
        { 77, 1 }, /* 18.528 MHz */
        { 78, 1 }, /* 9.264 MHz */
        { 79, 1 }, /* 4.632 MHz */
        { 80, 2 }, /* 40 MHz */
        { 82, 2 }, /* 20 MHz */
        { 84, 2 }, /* 10 MHz */
        { 86, 2 }, /* 5 MHz */
        { 88, 2 }, /* 622.08 MHz */
        { 90, 2 }, /* 625 MHz */
        { 92, 2 }, /* 625*(66/64) MHz */
        { 94, 2 }, /* 311.04 MHz */
        { 96, 2 }, /* 312.5 MHz */
        { 98, 2 }, /* 312.5*(66/64) MHz */
        { 100, 2 }, /* 155.52 MHz */
        { 102, 2 }, /* 156.25 MHz */
        { 104, 2 }, /* 156.25*(66/64) MHz */
        { 106, 2 }, /* 77.76 MHz */
        { 108, 2 }, /* 51.84 MHz */
        { 110, 2 }, /* 38.88 MHz */
        { 112, 2 }, /* 25.92 MHz */
        { 114, 2 }, /* 125 MHz */
        { 116, 2 }, /* 125*(66/64) MHz */
        { 118, 2 }, /* 19.44 MHz */
        { 120, 4 }, /* 25 MHz */
        { 124, 2 }, /* 25*(66/64) MHz */
        { 126, 2 },  /* 6.48 MHz */
	    /* Additional frequencies -ONLY supported by APLL */
	    { 0,   0 }, /* 24.8832 MHz */
        { 0,   0 }, /* 78.125 MHz */
        { 0,   0 }, /* 80.56640625 MHz */
        { 0,   0 }  /* 124.416 MHz */
};

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

/**
 * @brief Function to retrieve output path configuration from hardware.
 *
 * @param hdlr Device driver handler
 *
 * @param pathConfig_p Output path configuration
 */
void IDTHlApi_RetrieveOutputPathMuxFromHardware(
    T_idtDrvHdlr            hdlr,
    T_IDTPathConfiguration *pathConfig_p)
{
    T_idtDpllInstance inputDpll;
    T_idtSonetGigEthOutput outputFreq;
    T_idtDpllType dpllType;
    T_osUint8 i;

    pathConfig_p->networkType_m = IDTGeneral_GetNetworkType(hdlr);

    /* Retrieve DPLL selector information */
    pathConfig_p->T0SelA_m = DPLLOutputSelA_MAX;
    pathConfig_p->T0SelB_m = DPLLOutputSelB_MAX;
    pathConfig_p->T4SelA_m = DPLLOutputSelA_MAX;
    pathConfig_p->T4SelB_m = DPLLOutputSelB_MAX;
    for (i=0; i<DPLL_INSTANCE_MAX; i++)
    {
        /* Exit if we've reached the last DPLL */
        dpllType = IDTDpll_GetTypeOfDpll(hdlr, i);
        if (dpllType == Dpll_MAX) break;

        if (dpllType == Dpll_T0) 
        {
            pathConfig_p->T0SelA_m = 
                IDTDpll_GetDpllOutputPathSelectorA(
                    hdlr,
                    (T_idtDpllInstance)i);
            pathConfig_p->T0SelB_m = 
                IDTDpll_GetDpllOutputPathSelectorB(
                    hdlr,
                    (T_idtDpllInstance)i);
        }
        if (dpllType == Dpll_T4) 
        {
            pathConfig_p->T4SelA_m = 
                IDTDpll_GetDpllOutputPathSelectorA(
                    hdlr,
                    (T_idtDpllInstance)i);
            pathConfig_p->T4SelB_m = 
                IDTDpll_GetDpllOutputPathSelectorB(
                    hdlr,
                    (T_idtDpllInstance)i);
        }
    }

    /* Retrieve sonet/gigEth selector information */
    for (i=0; i<SonetGigEthSel_NUMMAX; i++)
    {
        /* Retrieve and translate */
        if (i<hdlr->deviceConfig_m->numberOfSonetGigEthPath_m)
        {
            IDTDpll_GetSonetGigEthOutputPath(hdlr, 
                                             i,  /* Apll instance */
                                             &inputDpll, 
                                             &outputFreq);
            pathConfig_p->sonetGigEthSel_ma[i] = outputFreq;
        }
        else 
        {
            pathConfig_p->sonetGigEthSel_ma[i] = SonetGigEthSel_MAX;
        }
    }
}

static T_idtDpllInstance IDTHlApi_TranslateSonetGigEthModeToDpllInstance(
    T_sonetGigeMode sonetGigeMode,
    T_osUint8       instance)
{
    switch (sonetGigeMode) {
    case sonetGigeMode_Matching:
        return (DPLL_INSTANCE_1 + instance);
        break;
    case sonetGigeMode_AllFromDpll1:
        return  DPLL_INSTANCE_1;
        break;
    case sonetGigeMode_AllFromDpll2:
        return  DPLL_INSTANCE_2;
        break;
    default:
        /* Should never, ever, ever, ever, ever get here. */
        OS_ERROR(("Invalid sonetGigeMode/%d\n",
                  sonetGigeMode));
        return DPLL_INSTANCE_MAX;
        break;
    }
}

/**
 * @brief Function to provision SONET/GigEth mux based on path
 * configuration information
 * @param hdlr Device Drvier handle
 * @param pathConfig_p path configuration to provision against
 * @param sonetGigeMode DPLL<->Sonet/GigEth mux mode
 */
static void IDTHlApi_ConfigureSonetGigEthMux(
    T_idtDrvHdlr            hdlr,
    T_IDTPathConfiguration *pathConfig_p,
    T_sonetGigeMode         sonetGigeMode)
{
    T_osUint8 i;
    T_idtSonetGigEthOutput sonetGigEthSel;
    T_osUint8 count = 0;
    T_idtDpllInstance dpllInstance;
    T_osUint8 firstMuxProvisioned = 0;

    /* Special case if this is the only SonetGigEth mux provisioned */
    for (i=0; i<SonetGigEthSel_NUMMAX; i++)
    {
        if (pathConfig_p->sonetGigEthSel_ma[i] != SonetGigEthOutput_MAX)
        {
            firstMuxProvisioned = i;
            count++;
        }
    }    

    /* No sonet/gigEth muxes provisioned - just return, our job is
       done here */
    if (count==0)
        return;

    /* Only one SONET/GigEth mux provisioned. All other muxes are
       configured identically. Improves jitter */
    if (count == 1)
    {

        /* Since there's only one mux provisioned, firstMuxProvisioned
           variable is used */
        sonetGigEthSel = 
            pathConfig_p->sonetGigEthSel_ma[firstMuxProvisioned];

        dpllInstance = IDTHlApi_TranslateSonetGigEthModeToDpllInstance(
            sonetGigeMode,
            firstMuxProvisioned);

        /* All muxes should be provisioned identically */
        for (i=0; i<SonetGigEthSel_NUMMAX; i++)
        {
            IDTDpll_SetSonetGigEthOutputPath(
                hdlr, 
                i, /* Sonet/GigEth selector */ 
                dpllInstance, 
                sonetGigEthSel);
        }

        /* Mark only one SONET/GigEth mux as used */
        hdlr->currOutPathCfg_mp->sonetGigEthSel_ma[firstMuxProvisioned] = 
            sonetGigEthSel;
        
        /* That's it folks! Show's over. Go home */
        return;
    }

    /* Provision sonet/GigEth selectors individually */
    for (i=0; i<SonetGigEthSel_NUMMAX; i++)
    {
        sonetGigEthSel = pathConfig_p->sonetGigEthSel_ma[i];
        if (sonetGigEthSel != SonetGigEthOutput_MAX) {

            /* Get sonet/GigEth source based on mode */
            dpllInstance = IDTHlApi_TranslateSonetGigEthModeToDpllInstance(
                sonetGigeMode,
                i);

            /* Provision SONET/GigEth Mux and note it's
               configuration */
            IDTDpll_SetSonetGigEthOutputPath(
                hdlr, 
                i, /* Sonet/GigEth selector */ 
                dpllInstance, 
                sonetGigEthSel);
            hdlr->currOutPathCfg_mp->sonetGigEthSel_ma[i] = 
                sonetGigEthSel;
        }
    }
}

/**
 * @brief Function to apply a path output configuration to hardware.
 *
 * @param hdlr Device driver handler 
 *
 * @param pathConfig_p Output path configuration to apply to hardware.
 *
 * @param sonetGigeMode sonetGige mux mode. Defines source for mux.
 */
static void IDTHlApi_ConfigureOutputPathMuxToHardware(
    T_idtDrvHdlr            hdlr,
    T_IDTPathConfiguration *pathConfig_p,
    T_sonetGigeMode         sonetGigeMode)
{
    T_idtDpllType dpllType;
    T_osUint8 i;

    /* We program muxes when the all following 
     *
     *   - Not default (i.e. unprogrammed)
     *   - Not the same as the current value
     */

    /* Provision network */
    if ((pathConfig_p->networkType_m != Network_MAX) &&
        (pathConfig_p->networkType_m != 
         IDTGeneral_GetNetworkType(hdlr))) {
        IDTGeneral_SetNetworkType(hdlr,
                                  pathConfig_p->networkType_m);
        hdlr->currOutPathCfg_mp->networkType_m =  pathConfig_p->networkType_m;
    }

    /* Provision DPLL selector information */
    for (i=0; i<DPLL_INSTANCE_MAX; i++)
    {
        /* Exit if we've reached the last DPLL */
        dpllType = IDTDpll_GetTypeOfDpll(hdlr, i);
        if (dpllType == Dpll_MAX) break;

        if (dpllType == Dpll_T0) 
        {
            if (pathConfig_p->T0SelA_m != DPLLOutputSelA_MAX) {                
                IDTDpll_SetDpllOutputPathSelectorA(hdlr,
                                                   (T_idtDpllInstance)i,
                                                   pathConfig_p->T0SelA_m);
                hdlr->currOutPathCfg_mp->T0SelA_m = pathConfig_p->T0SelA_m;
            }
            if (pathConfig_p->T0SelB_m != DPLLOutputSelB_MAX) {
                IDTDpll_SetDpllOutputPathSelectorB(hdlr,
                                                   (T_idtDpllInstance)i,
                                                   pathConfig_p->T0SelB_m);
                hdlr->currOutPathCfg_mp->T0SelB_m = pathConfig_p->T0SelB_m;
            }
        }

        if (dpllType == Dpll_T4) 
        {
            if (pathConfig_p->T4SelA_m != DPLLOutputSelA_MAX) {
                IDTDpll_SetDpllOutputPathSelectorA(hdlr,
                                                   (T_idtDpllInstance)i,
                                                   pathConfig_p->T4SelA_m);
                hdlr->currOutPathCfg_mp->T4SelA_m = pathConfig_p->T4SelA_m;
            }
            if (pathConfig_p->T4SelB_m != DPLLOutputSelB_MAX) {
                IDTDpll_SetDpllOutputPathSelectorB(hdlr,
                                                   (T_idtDpllInstance)i,
                                                   pathConfig_p->T4SelB_m);
                hdlr->currOutPathCfg_mp->T4SelB_m = pathConfig_p->T4SelB_m;
            }
        }
    }

    /* Provision SONET/GigEth muxes */ 
    IDTHlApi_ConfigureSonetGigEthMux(hdlr, pathConfig_p, sonetGigeMode);
}

/**
 * @brief Function to check whether or not DPLL instance is valid
 *
 * @param hdlr driver handler
 *
 * @param dpllInstance DPLL instance to limit verfication. Specify
 * DPLL_INSTANCE_MAX if any instance is valid
 *
 * @param sonetGigeMode sonetGige mux configuration. \see
 * T_sonetGigeMode
 *
 * @param dpllType DPLL type to verify against
 *
 * @return 0-not valid, 1-valid
 */

static T_osUint8 IDTHlAPI_IsValidDpll(T_idtDrvHdlr      hdlr,
                                      T_idtDpllInstance dpllInstance,
                                      T_sonetGigeMode   sonetGigeMode,
                                      T_idtDpllType     dpllType)
{
    T_osUint8 okCheck = 0;

    if (dpllType == Dpll_T0)
        okCheck = (((dpllInstance == DPLL_INSTANCE_MAX) ||
                    (dpllInstance == DPLL_INSTANCE_1)) &&
                   ((sonetGigeMode == sonetGigeMode_Matching) ||
                    (sonetGigeMode == sonetGigeMode_AllFromDpll1)));
    if (dpllType == Dpll_T4)
        okCheck = (((dpllInstance == DPLL_INSTANCE_MAX) ||
                    (dpllInstance == DPLL_INSTANCE_2)) &&
                   ((sonetGigeMode == sonetGigeMode_Matching) ||
                    (sonetGigeMode == sonetGigeMode_AllFromDpll2)));
    return okCheck;
}

/**
 * @brief Function to compare output path configuration with a output
 * path selection option.
 *
 * @param hdlr driver handler
 *
 * @param pathSetting Output path selection option
 *
 * @param pathConfiguration Output path configuration
 *
 * @param dpllInstance DPLL instance to limit verfication. Specify
 * DPLL_INSTANCE_MAX if any instance is valid
 *
 * @param sonetGigeMode sonetGige mux configuration. \see
 * T_sonetGigeMode
 *
 * @param sonetGigeMode sonetGige mux configuration. \see
 * T_sonetGigeMode
 *
 * @return 1-Equal 0-Not equal
 */
static int IDTHlApi_comparePathConfigration(
    T_idtDrvHdlr                hdlr,
    const pathSelectorConfig_t *pathSetting,
    T_IDTPathConfiguration      pathConfiguration,
    T_idtDpllInstance           dpllInstance,
    T_sonetGigeMode             sonetGigeMode)
{
    T_osUint8 checkOk;

    switch(pathSetting->selector_m)
    {
    case Selector_Network:
        return ((pathConfiguration.networkType_m == 
                 pathSetting->value_m) &&
                (pathConfiguration.networkType_m != 
                 Network_MAX));
        break;
    case Selector_T0DpllSelA:
        /* Limit to a particular DPLL? */
        checkOk = IDTHlAPI_IsValidDpll(hdlr,dpllInstance,sonetGigeMode,Dpll_T0);
        if (checkOk)
            return ((pathConfiguration.T0SelA_m == 
                     pathSetting->value_m) &&
                    (pathConfiguration.T0SelA_m != 
                     DPLLOutputSelA_MAX));
        break;
    case Selector_T0DpllSelB:
        /* Limit to a particular DPLL? */
        checkOk = IDTHlAPI_IsValidDpll(hdlr,dpllInstance,sonetGigeMode,Dpll_T0);
        if (checkOk)
            return ((pathConfiguration.T0SelB_m == 
                     pathSetting->value_m) &&
                    (pathConfiguration.T0SelB_m != 
                     DPLLOutputSelB_MAX));
        break;
    case Selector_T4DpllSelA:
        /* Limit to a particular DPLL? */
        checkOk = IDTHlAPI_IsValidDpll(hdlr,dpllInstance,sonetGigeMode,Dpll_T4);
        if (checkOk)
            return ((pathConfiguration.T4SelA_m == 
                     pathSetting->value_m) &&
                    (pathConfiguration.T4SelA_m != 
                     DPLLOutputSelA_MAX));
        break;
    case Selector_T4DpllSelB:
        /* Limit to a particular DPLL? */
        checkOk = IDTHlAPI_IsValidDpll(hdlr,dpllInstance,sonetGigeMode,Dpll_T4);
        if (checkOk)
            return ((pathConfiguration.T4SelB_m == 
                     pathSetting->value_m) &&
                    (pathConfiguration.T4SelB_m != 
                     DPLLOutputSelB_MAX));
        break;
    case Selector_SonetGigEth:
    {
        T_osUint8 instance;
        T_idtSonetGigEthOutput sonetGigEthSel;
        T_osUint8 okCheck;

        instance = pathSetting->instance_m;

        /* Check for valid instance */
        okCheck = ((instance == dpllInstance) ||
                   (dpllInstance == DPLL_INSTANCE_MAX));

        if (okCheck)
        {
            sonetGigEthSel = pathConfiguration.sonetGigEthSel_ma[instance];
            return ((sonetGigEthSel == pathSetting->value_m) &&
                    (sonetGigEthSel != SonetGigEthOutput_MAX));
        }
        else
        {
            /* We don't allow provisioning to this instance of APLL */
            return 0;
        }
        break;
    }
    case Selector_Any:
        return 1;
        break;
    default:
        /* Not matching !!! */
        return 0;
        break;
    }

    return 0;
}

/** 
 * @brief Function to check if current configurtation already supports
 * frequency.
 *
 * @param hdlr driver handler
 *
 * @param nOutFreq Desired output frequency
 *
 * @param dpllInstance DPLL instance to limit verfication. Specify
 * DPLL_INSTANCE_MAX if any instance is valid
 *
 * @param sonetGigeMode sonetGige mux configuration. \see
 * T_sonetGigeMode
 *
 * @param pathConfig_p Output path mux configuration
 *
 * @param outputPortConfig_p Output port configuration (used to
 * provision output path and divider)
 *
 * @return 1-Not supported, 0-already supported
 */
static int IDTHlApi_IsFrequencyAlreadySupported(
    T_idtDrvHdlr                 hdlr,
    outputFrequency_t            nOutFreq,
    T_idtDpllInstance            dpllInstance,
    T_sonetGigeMode              sonetGigeMode,
    T_IDTPathConfiguration      *pathConfig_p,
    const pathSelectorConfig_t **outputPortConfig_p)
{
    T_osUint16 noCombos;
    T_osUint16 startIdx;
    const pathSelectorIndex_t  *combinations_p;
    T_osUint16 i;

    OS_LOGGER(5,
              ("%s: nOutFreq/%s dpllInstance/%d\n",
               __FUNCTION__,
               IDTHlApi_Frequency2String_c[nOutFreq],
               dpllInstance));

    combinations_p = &IDTHlApi_freqencyToPathConfigIndex[nOutFreq];
    noCombos = combinations_p->size_m;
    startIdx = combinations_p->start_m;

    /* hmmmm ... number of combinations is invalid */
    if (noCombos == 0) return 1;

    /* Look for a match with existing configuration */
    for (i=0; i<noCombos; i++)
    {
        const pathSelectorConfig_t *combination_p;

        combination_p = &IDTHlApi_freqencyToPathConfig[startIdx+i];

        /* Special case for 25 MHz. We only use DPLL 25MHz only as
         * last resort. */
        if (nOutFreq == OutFreq_25000k) {
            T_osUint8 instance;
            T_osUint8 okCheck;

            instance = combination_p->instance_m;
            OS_ASSERT(instance<SonetGigEthSel_NUMMAX);

            /* Check sonet/gige instance number */
            okCheck = ((instance == dpllInstance) ||
                       (dpllInstance == DPLL_INSTANCE_MAX));

            /* Determine if there is a sonet/GigEth mux available for
             * use; take into account instance */
            if (((pathConfig_p->sonetGigEthSel_ma[instance] == 
                  SonetGigEthOutput_MAX)) && 
                (okCheck))
                break;        
        }

        /* Compare combination with configuration - if we have a match
         * then the existing configuration supports frequency, no need
         * to change configuration. */
        if (IDTHlApi_comparePathConfigration(hdlr,
                                             combination_p,
                                             *pathConfig_p,
                                             dpllInstance,
                                             sonetGigeMode))
        {
            /* Output mux configuration already supports this
               frequency */
            *outputPortConfig_p = combination_p;
            return 0;
        }
    }

    /* Not supported */
    return 1;
}

/**
 * @brief Function to try to match output path mux combination to
 * output path mux configuration.
 *
 * @param hdlr driver handler
 *
 * @param nOutFreq Desired output frequency
 *
 * @param dpllInstance DPLL instance to limit verfication. Specify
 * DPLL_INSTANCE_MAX if any instance is valid
 *
 * @param pathConfig_p Output path mux configuration
 *
 * @param outputPortConfig_p Output port configuration (used to
 * provision output path and divider)
 *
 * @return 0-Frequency can be supported, output path configuration
 * modified to support this. 1-Can not support frequency
 */
static int IDTHlApi_TryCombinations(
    T_idtDrvHdlr                 hdlr,
    outputFrequency_t            nOutFreq,
    T_idtDpllInstance            dpllInstance,
    T_sonetGigeMode              sonetGigeMode,
    T_IDTPathConfiguration      *pathConfig_p,
    const pathSelectorConfig_t **outputPortConfig_p)
{
    T_osUint16 noCombos;
    T_osUint16 startIdx;
    const pathSelectorConfig_t *combination_p;
    const pathSelectorIndex_t  *combinations_p;
    T_osUint16 i;
    T_osUint16 j;
    T_osUint8 okCheck = 0;

    OS_LOGGER(5,
              ("%s: nOutFreq/%s dpllInstance/%d\n",
               __FUNCTION__,
               IDTHlApi_Frequency2String_c[nOutFreq],
               dpllInstance));

    combinations_p = &IDTHlApi_freqencyToPathConfigIndex[nOutFreq];
    noCombos = combinations_p->size_m;
    startIdx = combinations_p->start_m;

    /* hmmmm ... number of combinations is invalid */
    if (noCombos == 0) return 1;

    /* Try to see if we can further provision device so that we can
       support new frequency */
    for (i=0; i<noCombos; i++)
    {
        combination_p = &IDTHlApi_freqencyToPathConfig[startIdx+i];

        /* If we have not configured selector - then we can use it. */
        switch(combination_p->selector_m)
        {
        case Selector_Network:
            if (pathConfig_p->networkType_m == Network_MAX)
            {
                pathConfig_p->networkType_m = 
                    (networkType_t)combination_p->value_m;
                *outputPortConfig_p = combination_p;
                return 0;
            }
            break;
        case Selector_T0DpllSelA:

            /* Can we use this selector? */
            okCheck = IDTHlAPI_IsValidDpll(hdlr,dpllInstance,sonetGigeMode,Dpll_T0);

            if ((pathConfig_p->T0SelA_m == DPLLOutputSelA_MAX) &&
                (okCheck))
            {
                pathConfig_p->T0SelA_m = 
                    (T_idtDpllOutputSelectorA)
                    combination_p->value_m;
                *outputPortConfig_p = combination_p;
                return 0;
            }
            break;
        case Selector_T0DpllSelB:

            /* Can we use this selector? */
            okCheck = IDTHlAPI_IsValidDpll(hdlr,dpllInstance,sonetGigeMode,Dpll_T0);

            if ((pathConfig_p->T0SelB_m == DPLLOutputSelB_MAX) &&
                (okCheck))
            {
                pathConfig_p->T0SelB_m = 
                    (T_idtDpllOutputSelectorB)
                    combination_p->value_m;
                *outputPortConfig_p = combination_p;
                return 0;
            }
            break;
        case Selector_T4DpllSelA:

            /* Can we use this selector? */
            okCheck = IDTHlAPI_IsValidDpll(hdlr,dpllInstance,sonetGigeMode,Dpll_T4);

            if ((pathConfig_p->T4SelA_m == DPLLOutputSelA_MAX) &&
                (okCheck))
            {
                pathConfig_p->T4SelA_m = 
                    (T_idtDpllOutputSelectorA)
                    combination_p->value_m;
                *outputPortConfig_p = combination_p;
                return 0;
            }
            break;
        case Selector_T4DpllSelB:

            /* Can we use this selector? */
            okCheck = IDTHlAPI_IsValidDpll(hdlr,dpllInstance,sonetGigeMode,Dpll_T4);

            if ((pathConfig_p->T4SelB_m == DPLLOutputSelB_MAX) &&
                (okCheck))
            {
                pathConfig_p->T4SelB_m = 
                    (T_idtDpllOutputSelectorB)
                    combination_p->value_m;
                *outputPortConfig_p = combination_p;
                return 0;
            }
            break;
        case Selector_SonetGigEth:
            {
                /* Try all sonet/GigEth selectors */
                for (j=0; j<SonetGigEthSel_NUMMAX; j++)
                {
                    /* Check sonet/gige instance number */
                    okCheck = ((j == dpllInstance) ||
                               (dpllInstance == DPLL_INSTANCE_MAX));

                    if ((okCheck) && 
                        (pathConfig_p->sonetGigEthSel_ma[j] == 
                         SonetGigEthOutput_MAX) &&
                        (combination_p->instance_m == j))
                    {
                        pathConfig_p->sonetGigEthSel_ma[j] = 
                            (T_idtSonetGigEthOutput)
                            combination_p->value_m;
                        *outputPortConfig_p = combination_p;
                        return 0;
                    }
                }
            }
            break;
        case Selector_Any:
            *outputPortConfig_p = combination_p;
            return 0;
            break;

        default:
            /* Not matching !!! */
            OS_ERROR(("Unknown output mux %d\n",
                      combination_p->selector_m));
            return 1;
            break;
        }
    }

    /* Tried but failed. */
    *outputPortConfig_p = NULL;
    return 1;
}

/**
 * @brief Function to retrieve current output path multiplexer
 * configuration.
 *
 * @param hdlr driver handler
 *
 * @param pathConfiguration_p Output path mux configuration
 *
 */
static void IDTHlApi_GetCurrentOutputPathMuxConfiguration(
    T_idtDrvHdlr            hdlr,
    T_IDTPathConfiguration *pathConfiguration_p)
{
    T_osUint8 i;

    /* Check if we have initialize HL API data */
    if (hdlr->currOutPathCfg_mp)
    {
        /* Get current output path config */
        pathConfiguration_p->networkType_m = 
            hdlr->currOutPathCfg_mp->networkType_m;
        pathConfiguration_p->T0SelA_m = 
            hdlr->currOutPathCfg_mp->T0SelA_m;
        pathConfiguration_p->T0SelB_m = 
            hdlr->currOutPathCfg_mp->T0SelB_m;
        pathConfiguration_p->T4SelA_m = 
            hdlr->currOutPathCfg_mp->T4SelA_m;
        pathConfiguration_p->T4SelB_m = 
            hdlr->currOutPathCfg_mp->T4SelB_m;
        for (i=0; i<SonetGigEthSel_NUMMAX; i++)
        {
            pathConfiguration_p->sonetGigEthSel_ma[i] = 
                hdlr->currOutPathCfg_mp->sonetGigEthSel_ma[i];
        }
    }
    else
    {
        /* Alright, *someone* forgot to indicate that we are
         * going to use high level API.
         *
         * Because we're nice people, we're going to initialize
         * the data and populate it properly. Just because
         * you're special to us. :-) */

        /* Allocate memory */
        hdlr->currOutPathCfg_mp = (T_IDTPathConfiguration *)
            OS_MALLOC(sizeof(struct S_IDTPathConfiguration));
        OS_ASSERT_PTR_NULL(hdlr->currOutPathCfg_mp);

        /* Populate data from hardware */
        IDTHlApi_RetrieveOutputPathMuxFromHardware(hdlr,
                                                   hdlr->currOutPathCfg_mp);

        /* Ok. A little recursion here. We're going to call
         * the function again, but this time the high API data
         * will be populated properly (since it's been
         * allocated and filled. */
        IDTHlApi_GetCurrentOutputPathMuxConfiguration(hdlr,pathConfiguration_p);
    }
}

/**
 * @brief Function to handle internal output ports 8 and 9
 *
 * @param hdlr driver handler
 *
 * @param nOutN nOutN Output port (1-11)
 *
 * @param nOutFreq Output frequency /see outputFrequency_t for list of
 * supported frequencies
 *
 * @param dpllInstance Dpll Instance (DPLL_INSTANCE_1,
 * DPLL_INSTANCE_2, DPLL_INSTANCE_MAX (equivalent to DPLL_INSTANCE_1))
 *
 * @param internalPort Internal output port number
 *
 * @param apllConfig_p APLL configuration
 *
 * @return 0-ok, !0-Error
 */
static int IDTHlApi_HandlePort8or9(T_idtDrvHdlr              hdlr,
                                   T_osUint8        	 	 nOutN,
                                   outputFrequency_t 	 	 nOutFreq,
                                   T_idtDpllInstance         dpllInstance,
                                   T_osUint8                 internalPort,
                                   T_idtApllConfigPerOutput  *apllConfig_p)
{
    T_outputPathType pathType;

    /* User doesn't care, so we'll pick DPLL #1 */
    if (dpllInstance == DPLL_INSTANCE_MAX)
        dpllInstance = DPLL_INSTANCE_1;

    /* Ports 8/9 don't support all combinations */
    if (internalPort == 9) {
        networkType_t provisionedNetworkType;

        provisionedNetworkType = hdlr->currOutPathCfg_mp->networkType_m;

        switch (nOutFreq) {
        case OutFreq_1544k:
            if (provisionedNetworkType == Network_MAX) {
                IDTGeneral_SetNetworkType(hdlr, Network_SONET);
                hdlr->currOutPathCfg_mp->networkType_m = Network_SONET;
                pathType = OutputPath_16E1_16T1;
            }
            else
                if (provisionedNetworkType == Network_SDH) {
                    OS_ERROR(("%s Port %d only supports 1.544MHz\n",
                              __FUNCTION__, nOutN));
                    return 1;
                }                        
            break;
        case OutFreq_2048k:
            if (provisionedNetworkType == Network_MAX) {
                IDTGeneral_SetNetworkType(hdlr, Network_SDH);
                hdlr->currOutPathCfg_mp->networkType_m = Network_SDH;
                pathType = OutputPath_16E1_16T1;
            }
            else
                if (provisionedNetworkType == Network_SONET) {
                    OS_ERROR(("%s Port %d only supports 2.048MHz\n",
                              __FUNCTION__, nOutN));
                    return 1;
                }                        
            break;
        default:
            OS_ERROR(("%s Port %d only supports 1.544MHz / 2.048MHz\n",
                      __FUNCTION__, nOutN));
            return 1;
            break;
        }
    }
    else {

        /* Port 8 only supports 77.76 MHz */
        pathType = OutputPath_7776kHz;
    }

    /* No mux configuration is required. Ports 8/9 have fixed input
       sources */

    /* Apply output port divider */
    IDTOutput_SetOutputConfig(hdlr,
                              nOutN,
                              dpllInstance,
                              pathType,
                              2, /* Divider doesn't matter for these
                                    output ports */
                              apllConfig_p);


    /* everything ok */
    return 0;
}

/**
 * @brief Function to retrieve APLL configuration (if valid).
 *
 * @param hdlr driver handler
 *
 * @param nOutN nOutN Output port (1-11)
 *
 * @param nOutFreq_p Output frequency /see outputFrequency_t for list of
 * supported frequencies (may be modified by APLL code)
 *
 * @param apllConfSel   Selects the APLL configuration.
 *                      If output path does not have APLL, then this
 *                      parameter is unchecked, you can set to any
 *                      value.
 *                      If output path has APLL, then this parameter
 *                      is used to select the configuration to
 *                      program; if this parameter is set to
 *                      APLL_CONFIG_MAX, then non-active configuration
 *                      will be programmed and becomes active.
 *
 * @param apllOutputSigType   APLL output signal type, set to
 *                            APLL_OUT_SIG_MAX if unused.
 *
 * @param apllClkSource APLL clock source (internal or external), set
 *                      to APLL_CLK_SEL_MAX if unused.
 *
 * @param apllConfig_p Retrieved APLL configuration
 *
 * @return 0-Ok, !0-Error
 */
static int IDTHlApi_GetApllConfig (
    T_idtDrvHdlr      		  hdlr,
    T_osUint8        		  nOutN,
    outputFrequency_t 		 *nOutFreq_p,
    T_idtApllConfigSel		  apllConfSel,
    T_idtApllOutputSigType	  apllOutputSigType,
    T_idtApllInputClkSrc      apllClkSource,
    T_idtApllConfigPerOutput *apllConfig_p)
{
    T_idtOutputApllConfig outputApll;
    T_idtApllId apllInst;
    const T_idtApllFreqMapping *apllFreqEntry_p = NULL;
    T_idtApllXtalId xtalId;

    OS_ASSERT_PTR_NULL(apllConfig_p);

    IDT_GET_OUTPUT_APLL_CONFIG(hdlr, nOutN, outputApll);

    /* Ignore if we're output port is not connect to apll */
    if (!APLL_CONFIGURED(outputApll))
        return 0;
    
    apllInst = outputApll.apllInst;
    apllFreqEntry_p = IDTApll_GetApllFreqTableEntry(hdlr, 
                                                    outputApll, 
                                                    *nOutFreq_p);
    if (!apllFreqEntry_p)
    {
        OS_ERROR(("%s Unable to find valid APLL "
                  "frequency configuration for output: %d"
                  ", frequency: %d\n",
                  __FUNCTION__, nOutN, *nOutFreq_p));
        return 1;
    }

    /* Figure out DPLL output frequency based on APLL input */
    *nOutFreq_p = IDTApll_ApllInput2DpllOutput(apllFreqEntry_p->apllInputFreq);
    if (OutFreq_Invalid == *nOutFreq_p)
    {
        OS_ERROR(("%s Unable to convert APLL input "
                  "frequency %d to a valid DPLL output\n",
                  __FUNCTION__, apllFreqEntry_p->apllInputFreq));
        return 1;
    }

    /* Check if the select frequency has matching xtal configured */
    xtalId = IDTApll_GetXtalIdFromXtalFreq(hdlr, 
                                           apllInst, 
                                           apllFreqEntry_p->apllXtalFreq);
    if (APLL_XTAL_MAX == xtalId)
    {
        OS_ERROR(("%s xtal is not configured with required frequency: %d\n",
                  __FUNCTION__, apllFreqEntry_p->apllXtalFreq));
        return 1;
    }

    if (APLL_CONFIG_MAX == apllConfSel)
    {
        /* caller didn't specify which configuration, program the
           non-active one */
        apllConfig_p->apllCfg = IDTApll_GetNonActiveApllConfig(hdlr, 
                                                               apllInst);
    }
    else
    {
        apllConfig_p->apllCfg = apllConfSel;
    }

    apllConfig_p->inputClk  = apllClkSource;
    apllConfig_p->preDiv    = apllFreqEntry_p->apllPreDivider;
    apllConfig_p->fbDiv     = apllFreqEntry_p->apllFbDivider;
    apllConfig_p->outputDiv = apllFreqEntry_p->apllOutputDivider;
    /* According to the test, frequency doubler should NEVER be
       set as it cause large spur at 25MHz */
    apllConfig_p->dbleSel   = APLL_FREQ_SEL_SINGLE;
    apllConfig_p->xtal      = xtalId;
    apllConfig_p->sigType   = apllOutputSigType;
    apllConfig_p->enOutput  = APLL_OUTPUT_DISABLE; /* Disabled at
                                                      configuration
                                                      time */

    /* Everything ok */
    return 0;
}

/**
 * @brief Function to provision output port.
 *
 * @param hdlr driver handler
 *
 * @param nOutN Output port (1-11)
 *
 * @param nOutFreq Output frequency /see outputFrequency_t for list of
 * supported frequencies
 *
 * @param dpllInstance Dpll Instance (DPLL_INSTANCE_1,
 * DPLL_INSTANCE_2, specify DPLL_INSTANCE_MAX for no specific DPLL)
 *
 * @param sonetGigeMode sonetGige mux configuration. \see
 * T_sonetGigeMode This parameter and dpllInstance determine what
 * ouptut path(s) software may use to generate specified output
 * frequency. The following table illustrates the available paths, 
 * <table>
 *   <tr><td>dpllInstance</td>     <td>sonetGigeMode</td><td>Avail. Paths</td></tr>
 *   <tr><td>DPLL_INSTANCE_1</td>  <td>Matching</td>     <td>DPLL#1->SonetGigEth#1 only</td></tr>
 *   <tr><td>DPLL_INSTANCE_2</td>  <td>Matching</td>     <td>DPLL#2->SonetGigEth#2 only</td></tr>
 *   <tr><td>DPLL_INSTANCE_MAX</td><td>Matching</td>     <td>DPLL#1->SonetGigEth#1 or DPLL#2->SonetGigEth#2</td></tr>
 *   <tr><td>DPLL_INSTANCE_1</td>  <td>AllFromDpll1</td> <td>DPLL#1->SonetGigEth#1 only</td></tr>
 *   <tr><td>DPLL_INSTANCE_2</td>  <td>AllFromDpll1</td> <td>DPLL#1->SonetGigEth#2 only</td></tr>
 *   <tr><td>DPLL_INSTANCE_MAX</td><td>AllFromDpll1</td> <td>DPLL#1->SonetGigEth#1 or DPLL#1->SonetGigEth#2</td></tr>
 *   <tr><td>DPLL_INSTANCE_1</td>  <td>AllFromDpll2</td> <td>DPLL#2->SonetGigEth#1 only</td></tr>
 *   <tr><td>DPLL_INSTANCE_2</td>  <td>AllFromDpll2</td> <td>DPLL#2->SonetGigEth#2 only</td></tr>
 *   <tr><td>DPLL_INSTANCE_MAX</td><td>AllFromDpll2</td> <td>DPLL#2->SonetGigEth#1 or DPLL#2->SonetGigEth#2</td></tr>
 * </table>
 *
 * @param apllConfSel Selects the APLL configuration, set to
 *                    APLL_CONFIG_MAX if output port does not go
 *                    through APLL, _OR_ if you want to program backup
 *                    APLL configuration
 *
 * @param apllOutputSigType APLL output signal type, set to
 *                          APLL_OUT_SIG_MAX if output port does not
 *                          go through APLL
 *
 * @param apllClkSource APLL clock source (internal or external), set
 *                      to APLL_CLK_SEL_MAX if output port does not go
 *                      through APLL
 *
 * @return 0-OK, 1-Error, unable to configure output path mux
 */
int IDTHlApi_ConfigureOutput(
    T_idtDrvHdlr      		hdlr,
    T_osUint8        		nOutN,
    outputFrequency_t 		nOutFreq,
    T_idtDpllInstance 		dpllInstance,
    T_sonetGigeMode         sonetGigeMode,
    T_idtApllConfigSel		apllConfSel,
    T_idtApllOutputSigType	apllOutputSigType,
    T_idtApllInputClkSrc	apllClkSource)
{
    T_IDTPathConfiguration pathConfiguration;
    const pathSelectorConfig_t *outputPortConfig_p = NULL;
    T_idtApllConfigPerOutput apllConfig;
    T_osUint8 internalPort;

    int rc;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT((nOutN<=IDT_DRV_MAX_OUTPUT)&&(nOutN>0));
    OS_ASSERT((nOutFreq>=OutFreq_Invalid)&&(nOutFreq<OutFreq_MAX));
    OS_ASSERT(dpllInstance<=DPLL_INSTANCE_MAX);
    OS_ASSERT(sonetGigeMode<sonetGigeMode_MAX);
    if (dpllInstance != DPLL_INSTANCE_MAX)
        OS_ASSERTS((IDTDpll_GetTypeOfDpll(hdlr, dpllInstance) != Dpll_MAX),
                   ("Invalid DPLL instance %d\n",dpllInstance));

    OS_LOGGER(5,
              ("%s dev/%s nOutN/%d nOutFreq/%s dpllInstance/%d "
               "sonetGigeMode/%d apllConfSel/%d apllOutputSigType/%d "
               "apllClkSource/%d\n",
               __FUNCTION__, hdlr->name_m, nOutN, 
               IDTHlApi_Frequency2String_c[nOutFreq], 
               dpllInstance, sonetGigeMode, apllConfSel, 
               apllOutputSigType, apllClkSource));

    if (hdlr->deviceConfig_m->outputFreqValidFunc_m)
    {
        OS_ASSERTS((hdlr->deviceConfig_m->outputFreqValidFunc_m(
                        hdlr, nOutN, nOutFreq, NULL)),
                   ("Invalid output frequency/%d for "
                    "output port/%d dev/%s\n",
                    nOutFreq, nOutN, hdlr->name_m));
    }

    /* There's a fair amount of code here to provision output
     * frequencies.
     *
     * DON'T PANIC.
     *
     * The majority of the code is used to determine provisioning for
     * output path. If you know ahead of time your configuration - it
     * simplifies the code by 90% (i.e. you can directly program your
     * configuration) rather than rely on the below code to setup the
     * device.
     *
     * Steps to provision output port
     *    1. Configure APLL/DPLL output path multiplexing
     *          IDTGeneral_SetNetworkType
     *          IDTDpll_SetDpllOutputPathSelectorA
     *          IDTDpll_SetDpllOutputPathSelectorB
     *          IDTDpll_SetSonetGigEthOutputPath
     *    2. Configure output port path selection and dividers
     *          IDTOutput_SetOutputConfig
     *
     * There is sample code that illustrates how to provision without
     * using this function.
     *
     * This function is primarily used when the final configuration is
     * unknown. It "auto" selects mux configurations such that it
     * allows maximum flexibility.
     *
     * OK. So, you don't want to figure out your output path mux
     * configuration. Here's what the below code does:
     *
     *    (1) Retrieve APLL configuration information (if required)
     *    (2) Handle special case of (internal) ports 8/9 
     *    (3) Get provisioning output path mux configuration
     *    (4) Determine if desired frequency is supported by retrieve
     *    path mux configuration. If it is not, then try to find an a
     *    configuration that supports both desired frequency and
     *    existing frequencies.
     *    (5) Apply output path mux configuration to hardware
     *    (6) Provision output port configuration
     */

    /* (1) Retrieve APLL configuration information (if required)
     * --------------------------------------------------------- */
    rc = IDTHlApi_GetApllConfig(hdlr,
                                nOutN,
                                &nOutFreq,
                                apllConfSel,
                                apllOutputSigType,
                                apllClkSource,
                                &apllConfig);
    if (rc != 0)
    {
        /* Assumed that called function reported error and we're just
         * passing the error along */
        return rc;
    }

    /* (2) Handle special case of (internal) ports 8/9 
     * ----------------------------------------------- */
    IDT_TRANSLATE_EXT_TO_INT_OUTPORT(hdlr,nOutN, internalPort);
    if ((internalPort == 8) || (internalPort == 9)) {

        /* After handling port 8/9 no other processing is required */
        return IDTHlApi_HandlePort8or9(hdlr,
                                       nOutN,
                                       nOutFreq,
                                       dpllInstance,
                                       internalPort,
                                       &apllConfig);
    }

    /* (3) Get provisioning output path mux configuration
     * -------------------------------------------------- */
    /* Get current output path mux config (cached) */
    IDTHlApi_GetCurrentOutputPathMuxConfiguration(hdlr,
                                                  &pathConfiguration);


    /* (4) Determine if desired frequency is supported by retrieve
     *  path mux configuration. If it is not, then try to find an a
     *  configuration that supports both desired frequency and
     *  existing frequencies.
     *  ----------------------------------------------------------- */
    rc = IDTHlApi_IsFrequencyAlreadySupported(hdlr,
                                              nOutFreq,
                                              dpllInstance,
                                              sonetGigeMode, 
                                              &pathConfiguration,
                                              &outputPortConfig_p);
    if (rc != 0)
    {
        /* Existing configuration does not support requested
           frequency. We need to see if we can further provision to
           support requested frequency. */
        rc = IDTHlApi_TryCombinations(hdlr,
                                      nOutFreq,
                                      dpllInstance,
                                      sonetGigeMode, 
                                      &pathConfiguration,
                                      &outputPortConfig_p);
        if (rc != 0) {
            /* Unable to configure output path mux. Return error */
            IDTHlApi_PrintConfiguration(pathConfiguration);
            OS_ERROR(("%s Unable to create configuration for %s\n",
                      __FUNCTION__,
                      IDTHlApi_Frequency2String_c[nOutFreq]));
            return 1;
        }
    }

    /* (5) Apply output path mux configuration to hardware
     * --------------------------------------------------- */
    IDTHlApi_ConfigureOutputPathMuxToHardware(hdlr,
                                              &pathConfiguration,
                                              sonetGigeMode);

    /* (6) Provision output port configuration 
     * ---------------------------------------- */
    /* Apply output port divider */
    IDTOutput_SetOutputConfig(hdlr,
                              nOutN,
                              (DPLL_INSTANCE_1 + outputPortConfig_p->instance_m),
                              outputPortConfig_p->outputPathSelector_m,
                              outputPortConfig_p->outputDivider_m,
                              &apllConfig);

    /* Clean up and exit */
    OS_LOGGER(9, ("%s - DONE!\n", __FUNCTION__));

    /* Everything ok */
    return 0;
}
                     
/**
 * @brief Function to disable all outputs.
 *
 * @param hdlr driver handler
 */
void IDTHlApi_DisableAllOutputs(T_idtDrvHdlr hdlr) 
{
    int i;

    OS_ASSERT_PTR_NULL(hdlr);

    /* Cycle through and disable all populated outputs */
    for (i=1; i<=IDT_DRV_MAX_OUTPUT; i++) {
        if (IDT_EXT_OUTPORT_POPULATED(hdlr, i)) {
            IDTOutput_SetOutputEnable(hdlr,
                                      i, /* external output port */
                                      0); /* 0-disable */
        }
    }
}

/**
 * @brief Function to clear saved provisioning used by high level API
 *
 * @param hdlr Device driver handle
 */ 
void IDTHlApi_ClearProvisioning(T_idtDrvHdlr hdlr) 
{
    int i;

    OS_LOGGER(5,
              ("%s dev/%s\n", 
               __FUNCTION__, hdlr->name_m));
    OS_ASSERT_PTR_NULL(hdlr);

    if (!hdlr->currOutPathCfg_mp) 
    {
        hdlr->currOutPathCfg_mp = 
            OS_MALLOC(sizeof(struct S_IDTPathConfiguration));
        OS_ASSERT_PTR_NULL(hdlr->currOutPathCfg_mp);
    }

    /* Initialze structure */
    hdlr->currOutPathCfg_mp->networkType_m = Network_MAX;
    hdlr->currOutPathCfg_mp->T0SelA_m = DPLLOutputSelA_MAX;
    hdlr->currOutPathCfg_mp->T0SelB_m = DPLLOutputSelB_MAX;
    hdlr->currOutPathCfg_mp->T4SelA_m = DPLLOutputSelA_MAX;
    hdlr->currOutPathCfg_mp->T4SelB_m = DPLLOutputSelB_MAX;
    for (i=0; i<DPLL_INSTANCE_MAX; i++)
        hdlr->currOutPathCfg_mp->sonetGigEthSel_ma[i] = SonetGigEthOutput_MAX;
}
