/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "Define.h"
#include "Output.h"
#include "Input.h"
#include "General.h"

#include "outputFreq.h"
#include "outputFreq_priv.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

typedef enum {
    outFreqLookupIdx_Dpll77p76, /**< DPLL 77.76MHz */
    outFreqLookupIdx_Dpll12E1, /**< DPLL 12E1 */
    outFreqLookupIdx_Dpll16E1, /**< DPLL 16E1 */
    outFreqLookupIdx_Dpll16T1, /**< DPLL 16T1 */
    outFreqLookupIdx_DpllE3, /**< DPLL E3 */
    outFreqLookupIdx_DpllT3, /**< DPLL T3 */
    outFreqLookupIdx_DpllGsm, /**< DPLL GSM */
    outFreqLookupIdx_DpllObsai, /**< DPLL OBSAI */
    outFreqLookupIdx_DpllEth, /**< DPLL 24T1 */   
    outFreqLookupIdx_Dpll24T1, /**< DPLL 24T1 */
    outFreqLookupIdx_DpllGps,  /**< DPLL GPS */
    outFreqLookupIdx_SonetGigEthSonet, /**< SONET/GIGE SONET */
    outFreqLookupIdx_SonetGigEthEth, /**< SONET/GIGE ETH */
    outFreqLookupIdx_SonetGigEthFec, /**< SONET/GIGE FEC */
    outFreqLookupIdx_MAX /**< \internal Not used */
} T_outFreqLookupIdx;

/**
 * @brief Structure used to translate APLL configuration 
 * and output port frequency
 */
typedef struct {
    T_idtApllDividerValue apllDivVal_m; /**< Divider value */

  /* Array of supported frequencies */
   outputFrequency_t frequencies_m[APLL_XTAL_FREQ_MAX];
} T_apllOutputFrequencyEntry;

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/
const char *IDTHlApi_Frequency2String_c[OutFreq_MAX] = {
    "Invalid", /**< OutFreq_Invalid */
    "64 kHz", /**< OutFreq_64k */
    "8 kHz", /**< OutFreq_8k */
    "2 kHz", /**< OutFreq_2k */
    "400  Hz", /**< OutFreq_400 */
    "1 PPS", /**< OutFreq_1 */
    "24.576 MHz", /**< OutFreq_24576k */
    "12.288 MHz", /**< OutFreq_12288k */
    "6.144 MHz", /**< OutFreq_6144k */
    "4.096 MHz", /**< OutFreq_4096k */
    "2.048 MHz", /**< OutFreq_2048k */
    "32.768 MHz", /**< OutFreq_32768k */
    "16.384 MHz", /**< OutFreq_16384k */
    "8.192 MHz", /**< OutFreq_8192k */
    "24.704 MHz", /**< OutFreq_24704k */
    "12.352 MHz", /**< OutFreq_12352k */
    "6.176 MHz", /**< OutFreq_6176k */
    "3.088 MHz", /**< OutFreq_3088k */
    "1.544 MHz", /**< OutFreq_1544k */
    "34.368 MHz", /**< OutFreq_34368k */
    "44.736 MHz", /**< OutFreq_44736k */
    "26 MHz", /**< OutFreq_26000k */
    "13 MHz", /**< OutFreq_13000k */
    "30.72 MHz", /**< OutFreq_30720k */
    "15.36 MHz", /**< OutFreq_15360k */
    "7.68 MHz", /**< OutFreq_7680k */
    "3.84 MHz", /**< OutFreq_3840k */
    "37.056 MHz", /**< OutFreq_37056k */
    "18.528 MHz", /**< OutFreq_18528k */
    "9.264 MHz", /**< OutFreq_9264k */
    "4.632 MHz", /**< OutFreq_4632k */
    "40 MHz", /**< OutFreq_40000k */
    "20 MHz", /**< OutFreq_20000k */
    "10 MHz", /**< OutFreq_10000k */
    "5 MHz", /**< OutFreq_5000k */
    "622.08 MHz", /**< OutFreq_622080k */
    "625 MHz", /**< OutFreq_625000k */
    "625*(66/64) MHz", /**< OutFreq_625000k_66_64 */
    "311.04 MHz", /**< OutFreq_311040k */
    "312.5 MHz", /**< OutFreq_312500k */
    "312.5*(66/64) MHz", /**< OutFreq_312500k_66_64 */
    "155.52 MHz", /**< OutFreq_155520k */
    "156.25 MHz", /**< OutFreq_156250k */
    "156.25*(66/64) MHz", /**< OutFreq_156250k_66_64 */
    "77.76 MHz", /**< OutFreq_77760k */
    "51.84 MHz", /**< OutFreq_51840k */
    "38.88 MHz", /**< OutFreq_38880k */
    "25.92 MHz", /**< OutFreq_25920k */
    "125 MHz", /**< OutFreq_125000k */
    "125*(66/64) MHz", /**< OutFreq_125000k_66_64 */
    "19.44 MHz", /**< OutFreq_19440k */
    "25 MHz", /**< OutFreq_25000k */
    "25*(66/64) MHz", /**< OutFreq_25000k_66_64 */
    "6.48 MHz", /**< OutFreq_6480k */
    "24.8832", /**< OutFreq_24883_DOT_2k */
    "78.125 MHz", /**< OutFreq_78125k */
    "78.125*(66/64) MHz", /**< OutFreq_78125k_66_64 */
    "124.416 MHz" /**< OutFreq_124416k */
};

static const char *IDTHlApi_Selector2String[Selector_MAX] = {
    "Network      ", /**< Selector_Network */
    "T0 DPLL Sel A", /**< Selector_T0DpllSelA */
    "T0 DPLL Sel B", /**< Selector_T0DpllSelB */
    "T4 DPLL Sel A", /**< Selector_T4DpllSelA */
    "T4 DPLL Sel B", /**< Selector_T4DpllSelB */
    "SONET/GIGETH ",  /**< Selector_Apll */
    "Any          "  /**< Any */
};

static const char *IDTHlApi_NetworkSelector2String[Network_MAX+1] = {
    "SDH  ", /**< Network_SDH */
    "SONET", /**< Network_SONET */
    "N/A" /**< Network_MAX */
};

static const char *IDTHlApi_DpllSelectorA[DPLLOutputSelA_MAX+1] = {
    "16E1", /**< DPLLOuptutSelA_16E1 */
    "16T1", /**< DPLLOutputSelA_16T1 */
    "GSM", /**< DPLLOutputSelA_GSM */
    "OBSAI", /**< DPLLOutputSelA_OBSAI */
    "GPS", /**< DPLLOutputSelA_GPS */
    "N/A" /**< DPLLOutputSelA_MAX */
};

static const char *IDTHlApi_DpllSelectorB[DPLLOutputSelB_MAX+1] = {
    "12E1", /**< DPLLOutputSelB_12E1 */
    "GPS", /**< DPLLOutputSelB_GPS */
    "E3", /**< DPLLOutputSelB_E3 */
    "T3", /**< DPLLOutputSelB_T3 */
    "24T1", /**< DPLLOutputSelB_24T1 */
    "N/A" /**< DPLLOutputSelB_MAX */
};

static const char *IDTHlApi_SonetGigEthSelector[SonetGigEthOutput_MAX+1] = {
    "SONET", /**< APLLOutput_622080kHz */
    "ETH", /**< APLLOutput_625000kHz */
    "FEC", /**< APLLOutput_625_6664MHz */
    "N/A" /**< APLLOutput_MAX */
};

static const char *OutputPathTnPath2String_ca[OutputPath_MAX] = {
    "SONET/GIGE", /* OutputPath_SonetGigEth */
    "77.76", /* OutputPath_7776kHz */
    "ETH", /* OutputPath_Eth */
    "16E1/16T1", /* OutputPath_16E1_16T1 */
    "SEL A", /* OutputPath_12E1_E3_T3  */
    "SEL B", /* OutputPath_GSM_16E1_16T1 */
    "GPS", /* OutputPath_GPS */
    "OBSAI", /* OutputPath_OBSAI */
    "24T" /* OutputPath_24T1 */
};

static const outputFrequency_t outputFreqLookup[outFreqLookupIdx_MAX][16] = {

    /* DPLL 77.76 */
    { OutFreq_MAX, OutFreq_MAX, OutFreq_MAX, OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX, OutFreq_MAX, OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX, OutFreq_64k, OutFreq_8k,
      OutFreq_2k,  OutFreq_400, OutFreq_1,   OutFreq_MAX },

    /* DPLL 12E1 */
    { OutFreq_MAX,   OutFreq_MAX,   OutFreq_24576k, OutFreq_12288k,
      OutFreq_6144k, OutFreq_4096k, OutFreq_MAX,    OutFreq_2048k,
      OutFreq_MAX,   OutFreq_MAX,   OutFreq_MAX,    OutFreq_MAX,
      OutFreq_MAX,   OutFreq_MAX,   OutFreq_MAX,    OutFreq_MAX }, 

    /* DPLL 16E1 */
    { OutFreq_MAX,   OutFreq_MAX, OutFreq_32768k, OutFreq_16384k,
      OutFreq_8192k, OutFreq_MAX, OutFreq_4096k,  OutFreq_MAX,
      OutFreq_2048k, OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX,
      OutFreq_MAX,   OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX },

    /* DPLL 16T1 */
    { OutFreq_MAX,   OutFreq_MAX, OutFreq_24704k, OutFreq_12352k,
      OutFreq_6176k, OutFreq_MAX, OutFreq_3088k,  OutFreq_MAX,
      OutFreq_1544k, OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX,
      OutFreq_MAX,   OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX }, 

    /* DPLL E3 */
    { OutFreq_MAX, OutFreq_34368k, OutFreq_MAX, OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX, OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX, OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX, OutFreq_MAX },

    /* DPLL T3 */
    { OutFreq_MAX, OutFreq_44736k, OutFreq_MAX, OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX, OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX, OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX, OutFreq_MAX }, 

    /* DPLL GSM */
    { OutFreq_MAX, OutFreq_MAX, OutFreq_26000k, OutFreq_13000k,
      OutFreq_MAX, OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX }, 

    /* DPLL OBSAI */
    { OutFreq_MAX,   OutFreq_MAX, OutFreq_30720k, OutFreq_15360k,
      OutFreq_7680k, OutFreq_MAX, OutFreq_3840k,  OutFreq_MAX,
      OutFreq_MAX,   OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX,
      OutFreq_MAX,   OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX },

    /* DPLL ETH */
    { OutFreq_MAX, OutFreq_MAX, OutFreq_25000k, OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX,
      OutFreq_MAX, OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX }, 

    /* DPLL 24T1 */
    { OutFreq_MAX,   OutFreq_MAX,   OutFreq_37056k, OutFreq_18528k,
      OutFreq_9264k, OutFreq_MAX,   OutFreq_4632k,  OutFreq_3088k,
      OutFreq_MAX,   OutFreq_1544k, OutFreq_MAX,    OutFreq_MAX,
      OutFreq_MAX,   OutFreq_MAX,   OutFreq_MAX,    OutFreq_MAX },

    /* DPLL GPS */
    { OutFreq_MAX,    OutFreq_MAX, OutFreq_40000k, OutFreq_20000k,
      OutFreq_10000k, OutFreq_MAX, OutFreq_5000k,  OutFreq_MAX,
      OutFreq_MAX,    OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX,
      OutFreq_MAX,    OutFreq_MAX, OutFreq_MAX,    OutFreq_MAX }, 

    /* SONET/GIGE SONET */
    { OutFreq_MAX,     OutFreq_622080k, 
      OutFreq_622080k, OutFreq_311040k,
      OutFreq_155520k, OutFreq_MAX,     
      OutFreq_77760k,  OutFreq_51840k,
      OutFreq_38880k,  OutFreq_25920k,  
      OutFreq_19440k,  OutFreq_MAX,
      OutFreq_MAX,     OutFreq_6480k,   
      OutFreq_MAX,     OutFreq_MAX },

    /* SONET/GIGE ETH */
    { OutFreq_MAX,     OutFreq_625000k, 
      OutFreq_625000k, OutFreq_312500k,
      OutFreq_156250k, OutFreq_MAX,     
      OutFreq_MAX,     OutFreq_MAX,
      OutFreq_MAX,     OutFreq_125000k, 
      OutFreq_25000k,  OutFreq_MAX,
      OutFreq_MAX,     OutFreq_MAX,     
      OutFreq_MAX,     OutFreq_MAX }, 

    /* SONET/GIGE FEC */
    { OutFreq_MAX,           OutFreq_625000k_66_64, 
      OutFreq_625000k_66_64, OutFreq_312500k_66_64,
      OutFreq_156250k_66_64, OutFreq_MAX, 
      OutFreq_MAX,           OutFreq_MAX,
      OutFreq_MAX,           OutFreq_125000k_66_64, 
      OutFreq_25000k_66_64,  OutFreq_MAX,
      OutFreq_MAX,           OutFreq_MAX, 
      OutFreq_MAX,           OutFreq_MAX }  
};

/**
 * @brief Table to translate between APLL configuration to output
 * frequency
 */
static const T_apllOutputFrequencyEntry apllOutputFreqLookup[] = {
    /*DIV    ETH             FEC                    SONET        
     *---------------------------------------------------------- */
    { 25, { OutFreq_25000k,  OutFreq_25000k_66_64,  OutFreq_24883_DOT_2k } },
    { 8,  { OutFreq_78125k,  OutFreq_78125k_66_64,  OutFreq_77760k } },
    { 5,  { OutFreq_125000k, OutFreq_125000k_66_64, OutFreq_124416k } },
    { 4,  { OutFreq_156250k, OutFreq_156250k_66_64, OutFreq_155520k } },
    { 2,  { OutFreq_312500k, OutFreq_312500k_66_64, OutFreq_311040k } },
    { 1,  { OutFreq_625000k, OutFreq_625000k_66_64, OutFreq_622080k } }
};

/**
 * @brief Size of APLL translation table (see above)
 */
static const T_osUint8 apllOutputFreqLookupSize = 
    (sizeof( apllOutputFreqLookup)/sizeof(T_apllOutputFrequencyEntry));
/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
/**
 * @brief Function to print frequency array in string format
 * @param frequencyArray Frequency array
 */
void IDTHlApi_PrintAvailFrequencies(validFreq_t frequencyArray)
{
    int i;
    int count = 0;

    for (i=0; i<OutFreq_MAX; i++)
    {
        if (CHECK_BIT(frequencyArray,i))
        {
            printf("%20s ",
                   IDTHlApi_Frequency2String_c[i]);
            if ((count%3==2)&&(i<OutFreq_MAX-1)) printf("\n");
            count++;
        }
    }
    printf("\n");
}

/**
 * @brief Function to print frequency list in string format
 * @param frequencies Frequency list
 */
void IDTHlApi_PrintFrequencyList(const outputFrequency_t frequencies[])
{
    int i;

    /* Process list */
    if (frequencies[0] == OutFreq_Invalid) return;
    for (i=0; (frequencies[i] != OutFreq_Invalid); i++)
    {
        if ((i%3==0)&&(i!=0)) printf("\n");
        printf("%20s ",
               IDTHlApi_Frequency2String_c[frequencies[i]]);
    }
    printf("\n");
}

/**
 * @brief Function to print output path configuration to a string format.
 * @param pathConfig Output path config
 */
void IDTHlApi_PrintConfiguration(T_IDTPathConfiguration pathConfig)
{
    T_osUint8 i;

    printf("Output Path Configuration:\n");
    printf("    Network      : %s\n",
           IDTHlApi_NetworkSelector2String[pathConfig.networkType_m]);
    printf("    T0 DPLL SelA : %s\n",
           IDTHlApi_DpllSelectorA[pathConfig.T0SelA_m]);
    printf("            SelB : %s\n",
           IDTHlApi_DpllSelectorB[pathConfig.T0SelB_m]);
    printf("    T4 DPLL SelA : %s\n",
           IDTHlApi_DpllSelectorA[pathConfig.T4SelA_m]);
    printf("            SelB : %s\n",
           IDTHlApi_DpllSelectorB[pathConfig.T4SelB_m]);

    for (i=0; i<SonetGigEthSel_NUMMAX; i++)
        printf("   Sonet/GigE %2d : %s\n",
               i+1,
               IDTHlApi_SonetGigEthSelector[pathConfig.sonetGigEthSel_ma[i]]);
}

/**
 * @brief Function to print output path configuration option to string
 * format.
 * @param option Output path configuration option.
 * @param printPort Flag to indicate whether port information is also
 * displayed
 */
void IDTHlApi_PrintOption(pathSelectorConfig_t option, T_osUint8 printPort)
{
    switch (option.selector_m)
    {
    case Selector_Network:
        printf("%s : %s",
               IDTHlApi_Selector2String[option.selector_m],
               IDTHlApi_NetworkSelector2String[option.value_m]);
        break;
    case Selector_T0DpllSelA:
        printf("%s : %s",
               IDTHlApi_Selector2String[option.selector_m],
               IDTHlApi_DpllSelectorA[option.value_m]);
        break;
    case Selector_T0DpllSelB:
        printf("%s : %s",
               IDTHlApi_Selector2String[option.selector_m],
               IDTHlApi_DpllSelectorB[option.value_m]);
        break;
    case Selector_T4DpllSelA:
        printf("%s : %s",
               IDTHlApi_Selector2String[option.selector_m],
               IDTHlApi_DpllSelectorA[option.value_m]);
        break;
    case Selector_T4DpllSelB:
        printf("%s : %s",
               IDTHlApi_Selector2String[option.selector_m],
               IDTHlApi_DpllSelectorB[option.value_m]);
        break;
    case Selector_SonetGigEth:
        printf("%s : %s",
               IDTHlApi_Selector2String[option.selector_m],
               IDTHlApi_SonetGigEthSelector[option.value_m]);
        break;
    case Selector_Any:
        printf("%s : -",
               IDTHlApi_Selector2String[option.selector_m]);
        break;
    default:
        break;

    }

    if (printPort)
        printf(" { div/0x%02X path/%d inst/%d }", 
               option.outputDivider_m, 
               option.outputPathSelector_m,
               option.instance_m);
    printf("\n");
}

/**
 * @brief Function to display selected output port path.
 *
 * @param nPath output port path
 */
void IDTHlApi_PrintSelectedOutputPath(T_outputPathType  nPath)
{
    printf("%15s",
           OutputPathTnPath2String_ca[nPath]);
}

static T_outFreqLookupIdx IDTHlApi_TranslateToLookupIdx(
    networkType_t            network,
    T_outputPathType         outPath,
    T_idtSonetGigEthOutput   sonetGigeMode,
    T_idtDpllOutputSelectorA dpllSelA,
    T_idtDpllOutputSelectorB dpllSelB)
{
    switch (outPath)
    {
    case OutputPath_SonetGigEth:
        switch(sonetGigeMode)
        {
        case SonetGigEthOutput_Sonet:
            return outFreqLookupIdx_SonetGigEthSonet;
            break;
        case SonetGigEthOutput_10GbE:
            return outFreqLookupIdx_SonetGigEthEth;
            break;
        case SonetGigEthOutput_10GbE_6664:
            return outFreqLookupIdx_SonetGigEthFec;
            break;
        default:
            /* unhandled */
            return outFreqLookupIdx_MAX;
            break;
        }
        break;

    case OutputPath_7776kHz:
        return outFreqLookupIdx_Dpll77p76;
        break;

    case OutputPath_Eth:
        return outFreqLookupIdx_DpllEth;
        break;

    case OutputPath_16E1_16T1:
        switch (network)
        {
        case Network_SONET:
            return outFreqLookupIdx_Dpll16T1;
            break;
        case Network_SDH:
            return outFreqLookupIdx_Dpll16E1;
            break;
        default:
            /* Something went wrong */
            return outFreqLookupIdx_MAX;
            break;
        }
        break;

    case OutputPath_24T1:
        return outFreqLookupIdx_Dpll24T1;
        break;

    case OutputPath_12E1_E3_T3:
        switch(dpllSelB)
        {
        case DPLLOutputSelB_12E1:
            return outFreqLookupIdx_Dpll12E1;
            break;
        case DPLLOutputSelB_E3:
            return outFreqLookupIdx_DpllE3;
            break;
        case DPLLOutputSelB_T3:
            return outFreqLookupIdx_DpllT3;
            break;
        default:
            /* WTF?! */
            return outFreqLookupIdx_MAX;
            break;
        }
        break;

    case OutputPath_OBSAI:
        return outFreqLookupIdx_DpllObsai;
        break;

    case OutputPath_GSM_16E1_16T1:
        switch (dpllSelA)
        {
        case DPLLOutputSelA_16E1:
            return outFreqLookupIdx_Dpll16E1;
            break;
        case DPLLOutputSelA_16T1:
            return outFreqLookupIdx_Dpll16T1;
            break;
        case DPLLOutputSelA_GSM:
            return outFreqLookupIdx_DpllGsm;
        default:
            /* Code has gone sideways */
            return outFreqLookupIdx_MAX;
            break;
        }
        break;

    case OutputPath_GPS:
        return outFreqLookupIdx_DpllGps;
        break;

    default:
        /* Whoooooa... */
        return outFreqLookupIdx_MAX;
        break;
 
    }

    /* Never get here. */
    return outFreqLookupIdx_MAX;
}

static void IDTHlApi_PrintOutputHwConfigHeader(networkType_t network)
{
    printf("NETWORK: %s\n",
           IDTHlApi_NetworkSelector2String[network]);
    printf("OUT  DIV  XTAL DIV  PATH                  "
           "DPLL  | Frequency\n"
           "=========================================="
           "======|===================\n");
}

static void IDTHlApi_PrintInputHwConfigHeader(networkType_t network)
{
    printf("NETWORK: %s\n",
           IDTHlApi_NetworkSelector2String[network]);
    printf(" IN HF  Divider Value  DPLL Freq  Priority"
           "  | Frequency\n"
           "=========================================="
           "==|============\n");
}

void IDTHlApi_PrintOutputHwConfig(
    networkType_t            network,
    T_osUint8                nOutN,
    T_osUint8                internalPort,
    T_osUint8                isEnabled,
    T_osUint8                hasApll,
    T_idtApllDividerValue    apllDivVal,
    T_idtApllXtalType        xtalType,
    T_osUint8                outDiv,
    T_outputPathType         outPath,
    T_idtDpllInstance        dpllInstOrSonetGigEthInst,
    T_idtDpllInstance        sonetGigEthInput,
    T_idtSonetGigEthOutput   sonetGigeMode,
    T_idtDpllOutputSelectorA dpllSelA,
    T_idtDpllOutputSelectorB dpllSelB)
{
    T_outFreqLookupIdx lookupIdx;
    outputFrequency_t calcFrequency;

    printf(" %2d  ",
           nOutN);

    if (hasApll)
    {
        if (!isEnabled)
            printf ("--- ----- ");
        else
        {
            printf("%3d ",
                   apllDivVal);

            switch (xtalType)
            {
            case APLL_XTAL_FREQ_25000KHz:
                printf("ETH   ");
                break;
            case APLL_XTAL_FREQ_25781_DOT_25KHz:
                printf("FEC   ");
                break;
            case APLL_XTAL_FREQ_24883_DOT_2KHz:
                printf("SONET ");
                break;
            default:
                printf("????? ");
                break;
            }
        }
    }
    else
        printf ("--- ----- ");

    if ((internalPort != 8) && (internalPort != 9))
        printf("0x%01X %-10s ",
               outDiv,
               OutputPathTnPath2String_ca[outPath]);
    else
        printf("--- %-10s ",
               OutputPathTnPath2String_ca[outPath]);

    if (outPath == OutputPath_SonetGigEth)
        printf("(%d)%-5s (%d)",
               dpllInstOrSonetGigEthInst,
               IDTHlApi_SonetGigEthSelector[sonetGigeMode],
               sonetGigEthInput);
    else
        printf("(-)----- (%d)",
               dpllInstOrSonetGigEthInst);

    switch (outPath)
    {

    case OutputPath_SonetGigEth:
        printf("77.76 ");
        break;

    case OutputPath_7776kHz:
    case OutputPath_Eth:
        printf("%-5s ",
               OutputPathTnPath2String_ca[outPath]);
        break;

    case OutputPath_16E1_16T1:
        printf("%-5s ",
               network==Network_SDH?"16E1":"16T1");
        break;

    case OutputPath_24T1:
    case OutputPath_12E1_E3_T3:
        printf("%-5s ",
               IDTHlApi_DpllSelectorB[dpllSelB]);
        break;
    case OutputPath_OBSAI:
    case OutputPath_GSM_16E1_16T1:
        printf("%-5s ",
               IDTHlApi_DpllSelectorA[dpllSelA]);
        break;

    case OutputPath_GPS:
        printf("GPS   ");
        break;

    default:
        printf(" ???  ");
        break;
    }

    /* Output high */
    if (outDiv == 0)
    {
        printf("| SQUELCH LOW        \n");
        return;
    }

    /* Output low */
    if (outDiv == 15)
    {
        printf("| SQUELCH HIGH       \n");
        return;
    }

    /* Special case of internal port 8 */
    if (internalPort == 8)
    {
        if (!isEnabled)
            printf("| DISABLED           \n");
        else
            printf("| AMI SIGNAL         \n");
        return;
    }

    /* Special case of internal port 9 */
    if (internalPort == 9)
    {
        if (!isEnabled)
            printf("| DISABLED           \n");
        else
            if (network == Network_SDH)
                printf("| 2.048 MHz          \n");
            else
                printf("| 1.544 MHz          \n");
        return;
    }

    if (hasApll)
    {
        T_osUint8 i;

        if (!isEnabled)
        {
            printf("| DISABLED           \n");
            return;
        }

        /* Different frequency calculations coming from APLL */
        for (i=0; 
             i<apllOutputFreqLookupSize;
             i++)
        {
            if (apllOutputFreqLookup[i].apllDivVal_m == apllDivVal)
            {
                calcFrequency = apllOutputFreqLookup[i].frequencies_m[xtalType];

                if (calcFrequency != OutFreq_MAX)
                    printf("| %-19s\n",
                           IDTHlApi_Frequency2String_c[calcFrequency]);
                else
                    printf("| UNKNOWN            \n");
                return;
            }
        }
        printf("| UNKNOWN            \n");
        return;
    }

    lookupIdx = IDTHlApi_TranslateToLookupIdx(network,
                                              outPath,
                                              sonetGigeMode,
                                              dpllSelA,
                                              dpllSelB);
    if (lookupIdx != outFreqLookupIdx_MAX)
        calcFrequency = outputFreqLookup[lookupIdx][outDiv];
    else
    {
        printf("UNKNOWN            \n");
        return;
    }

    printf("| %-19s\n",
           IDTHlApi_Frequency2String_c[calcFrequency]);
}


void IDTHlApi_PrintInputHwConfig(
    T_osUint8                        nInputN,
    T_osUint8                        internalPort,
    networkType_t                    networkType,
    T_osUint8                        isEnable,
    T_idtHighFreqencyDivisor         nUsage,
    T_osUint16                       nFactor,
    T_idtInputDividerMux             nDivider,
    T_idtInputFrequencyBitfieldValue nFrequency,
    T_osUint8                        numberOfDpll,
    T_osUint8                        priorities[])
{
    T_osUint32 calcFrequency = 1;
    T_osUint8 i;

    printf(" %2d  ", nInputN);

    if (!isEnable)
    {
        printf("-  ------  -----  ---------- ---- ---- | DISABLED\n");
        return;
    }

    if ((internalPort == 5) || (internalPort == 6))
    {
        switch (nUsage)
        {
        case HF_Bypass:
            printf("1  ");
            break;
        case HF_Divide_4:
            printf("4  ");
            calcFrequency *= 4;
            break;
        case HF_Divide_5:
            printf("5  ");
            calcFrequency *= 5;
            break;
        default:
            OS_ERROR(("Unsupported high frequency divider %d\n",
                      nUsage));
            return;
            break;
        }
    }
    else
        printf("-  ");

    switch (nDivider)
    {
    case Divider_Bypass:
        printf("BYPASS  ");
        calcFrequency *= 1;
        break;

    case Divider_DivN:
        printf("DIVN    ");
        calcFrequency *= (nFactor+1);
        break;

    case Divider_Lock8k:
        printf("LOCK8k  ");
        calcFrequency *= 1;
        break;

    default:
        OS_ERROR(("Unsupported input divider type %d\n",
                  nDivider));
        return;
        break;
    }

    if (nDivider != Divider_DivN)
        printf("-----  ");
    else
        printf("%5d  ",nFactor);

    switch (nFrequency)
    {	
    case Freq_8K:
        calcFrequency *= 8000;
        printf("%-10s","8 kHz");
        break;
	case Freq_1544M:
        {
            if (networkType == Network_SONET)
            {
                calcFrequency *= 1544000;
                printf("%-10s","1.544 MHz");
            }
            if (networkType == Network_SDH)
            {
                calcFrequency *= 2048000;
                printf("%-10s","2.048 MHz");
            }
            break;
        }
	case Freq_648M:
        calcFrequency *= 6480000;
        printf("%-10s","6.48 MHz");
        break;
	case Freq_1944M:
        calcFrequency *= 19440000;
        printf("%-10s","19.44 MHz");
        break;
	case Freq_2592M:
        calcFrequency *= 25920000;
        printf("%-10s","25.92 MHz");
        break;
	case Freq_3888M:
        calcFrequency *= 38880000;
        printf("%-10s","38.88 MHz");
        break;
	case Freq_2K:
        calcFrequency *= 2000;
        printf("%-10s","2 kHz");
        break;
	case Freq_4K:
        calcFrequency *= 4000;
        printf("%-10s","4 kHz");
        break;
	case Freq_1PPS:
        calcFrequency *= 1;
        printf("%-10s","1 PPS");
        break;
	case Freq_625M:
        calcFrequency *= 6250000;
        printf("%-10s","6.25 MHz");
        break;
	case Freq_10M:
        calcFrequency *= 10000000;
        printf("%-10s","10 MHz");
        break;
    default:
        OS_ERROR(("Unsupported input DPLL frequency %d\n",
                  nFrequency));
        return;
        break;
    }


    for (i=0; i<numberOfDpll; i++)
    {
        T_osUint8 remainder;

        printf(" %d:%2d",
               i, priorities[i]);
        remainder = i%2;
        if (remainder==1) 
        {
            printf(" | ");
            if (((numberOfDpll%2)==0) && 
                (i==(numberOfDpll-1)))
                break;
            else
                printf("\n                                 ");
        }
        else 
        {
            if (((numberOfDpll%2)==1) && 
                (i==(numberOfDpll-1)))
            {
                printf("      | ");
                break;
            }
        }
    }

    if (calcFrequency < 1000000)
        printf("%7.3f kHz\n",(float)(calcFrequency/1000.0));
    else
        printf("%7.3f MHz\n",(float)(calcFrequency/1000000.0));
}


void IDTHlApi_PrintInput(T_idtDrvHdlr hdlr)
{
    T_osUint8 nInputN;
    T_osUint8 internalPort;

    T_osUint8 isEnabled;
    networkType_t network;

    T_idtHighFreqencyDivisor nUsage;
    T_osUint16 nFactor;
    T_idtInputDividerMux nDivider;
    T_idtInputFrequencyBitfieldValue nFrequency;

    T_osUint8 i;
    T_osUint8 numberOfDpll;
    T_osUint8 priorities[DPLL_INSTANCE_MAX];

    network = IDTGeneral_GetNetworkType(hdlr);
    IDTHlApi_PrintInputHwConfigHeader(network);
    for (nInputN=1; nInputN<=IDT_DRV_MAX_INPUT; nInputN++)
    {
        if (!IDT_EXT_INPORT_POPULATED(hdlr,nInputN))
            continue;

        IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr,nInputN,internalPort);

        if ((internalPort == 1) || (internalPort == 2))
            continue;

        isEnabled = IDTInput_GetInputEnable(hdlr,nInputN);

        nDivider = IDTInput_GetPreDivider(hdlr,nInputN);
        if (nDivider == Divider_DivN)
            nFactor = IDTInput_GetDivisionFactor(hdlr,nInputN);
        else
            nFactor = 1;
        nFrequency = IDTInput_GetInputFrequency(hdlr, nInputN);
        if ((internalPort == 5) || (internalPort == 6))
            nUsage = IDTInput_GetInputHFdivider(hdlr, nInputN);
        else
            nUsage = 1;

        numberOfDpll = 0;
        for (i=0; i<DPLL_INSTANCE_MAX; i++)
        {
            if (IDTDpll_GetTypeOfDpll(hdlr,i) != Dpll_MAX)
                priorities[numberOfDpll++] = 
                    IDTInput_GetPriority(hdlr,nInputN,i);
        }

        IDTHlApi_PrintInputHwConfig(nInputN,
                                    internalPort,
                                    network,
                                    isEnabled,
                                    nUsage,
                                    nFactor,
                                    nDivider,
                                    nFrequency,
                                    numberOfDpll,
                                    priorities);
    }
}

void IDTHlApi_PrintOutput(T_idtDrvHdlr hdlr)
{
    T_osUint8 nOutN;

    networkType_t network;

    T_osUint8 internalPort;
    T_osUint8 isEnabled;
    T_osUint8 hasApll;

    T_idtDpllInstance nDpll;
    T_outputPathType  nPath;
    T_osUint8 nFactor;

    T_idtApllConfigPerOutput apllConfig;
    T_idtApllDividerValue apllDivVal;
    T_idtApllXtalType xtalType;

    T_idtDpllInstance inputDpll;
    T_idtSonetGigEthOutput outputFreq;

    T_idtDpllOutputSelectorA selAVal;
    T_idtDpllOutputSelectorB selBVal;

    network = IDTGeneral_GetNetworkType(hdlr);
    IDTHlApi_PrintOutputHwConfigHeader(network);

    for (nOutN=1; nOutN<=IDT_DRV_MAX_OUTPUT; nOutN++)
    {
        T_idtOutputApllConfig outputApll;

        if (!IDT_EXT_OUTPORT_POPULATED(hdlr, nOutN))
            continue;

        IDT_TRANSLATE_EXT_TO_INT_OUTPORT(hdlr,nOutN,internalPort);
        IDT_GET_OUTPUT_APLL_CONFIG(hdlr, nOutN, outputApll);

        isEnabled = IDTOutput_GetOutputEnable(hdlr,nOutN);
        hasApll = APLL_CONFIGURED(outputApll);

        /* Retrieve Output Port information */
        IDTOutput_GetOutputConfig(hdlr, nOutN, 
                                  &nDpll,
                                  &nPath,
                                  &nFactor,
                                  &apllConfig);
        if (hasApll)
        {
            T_osUint8 i;

            apllDivVal = apllConfig.outputDiv;

            /* Look for XTAL frequency */
            xtalType = APLL_XTAL_FREQ_MAX;
            for (i=0; i<hdlr->numberOfApllXtal_m; i++)
            {
                if (hdlr->xtalList[i].apllXtalId == apllConfig.xtal)
                    xtalType = hdlr->xtalList[i].apllXtalFreq;
            }
        }
        else
        {
            apllDivVal = 0;
            xtalType = APLL_XTAL_FREQ_MAX;
        }

        if (nPath == OutputPath_SonetGigEth)
        {
            /* Through SONET/GigEth mux */
            IDTDpll_GetSonetGigEthOutputPath(hdlr,
                                             nDpll,
                                             &inputDpll,
                                             &outputFreq);
            selAVal = IDTDpll_GetDpllOutputPathSelectorA(hdlr,
                                                         inputDpll);
            selBVal = IDTDpll_GetDpllOutputPathSelectorB(hdlr,
                                                         inputDpll);
        }
        else
        {
            /* Direct from DPLL */
            selAVal = IDTDpll_GetDpllOutputPathSelectorA(hdlr,
                                                         nDpll);
            selBVal = IDTDpll_GetDpllOutputPathSelectorB(hdlr,
                                                         nDpll);
        }

        /* Print out output port entry */
        IDTHlApi_PrintOutputHwConfig(network,
                                     nOutN,
                                     internalPort,
                                     isEnabled,
                                     hasApll,
                                     apllDivVal,
                                     xtalType,
                                     nFactor,
                                     nPath,
                                     nDpll,
                                     inputDpll,
                                     outputFreq,
                                     selAVal,
                                     selBVal);
    }


}


/*


OUT  DIV  XTAL DIV  PATH        APLL      DPLL  | Frequency
================================================|===============
 XX   XX XXXXX 0xX XXXXXXXXXX (X)XXXXX (X)XXXXX | XXXXXXXXXXXXXX     
 XX  --- ----- 0xX XXXXXXXXXX (-)----- (X)XXXXX | XXXXXXXXXXXXXX     


 IN HF  Divider Value  DPLL Freq  Priority  | Frequency 
============================================|============
 XX  X  XXXXXX  XXXXX  XXXXXXXXXX XXXX XXXX | XXXXXXXXXXX
 XX  X  XXXXXX  XXXXX  XXXXXXXXXX 0:XX 1:XX | XXX.XXX MHz 
 XX  X  XXXXXX  XXXXX  XXXXXXXXXX 0:XX 1:XX |
                                  0:XX 1:XX | XXX.XXX MHz 
 XX  X  XXXXXX  XXXXX  XXXXXXXXXX 0:XX 1:XX | XXX.XXX kHz

 */
