/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "Define.h"
#include "Output.h"

#include "outputFreq.h"
#include "outputFreq_priv.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/


/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/


/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
/**
 * @brief Convert from a frequency list to frequency array 
 * @param frequencies Frequency list to convert
 * @param validFrequencies Output frequency array
 */
void IDTHlApi_ConvertFromFreqListToFreqArray(outputFrequency_t frequencies[],
                       validFreq_t       validFrequencies)
{
        int i = 0;

        /* Zero output frequency array */
        memset(validFrequencies,0,sizeof(validFreq_t));

        /* Process list */
        if (frequencies[0] == OutFreq_Invalid) return;
        for (i=0; (frequencies[i] != OutFreq_Invalid); i++)
        {
                SET_BIT(validFrequencies,frequencies[i],1);
        } 
}

/**
 * @brief Function to set status of frequency in frequency array
 * @param frequencies Constant frequency list
 * @param validFrequencies
 * @param value Status to set for frequency array
 */
void IDTHlApi_ApplyFrequencies(const outputFrequency_t *frequencies,
                      validFreq_t              validFrequencies,
                      int                      value)
{
        int i = 0;

        /* Process list */
        if (frequencies[0] == OutFreq_Invalid) return;
        for (i=0; (frequencies[i] != OutFreq_Invalid); i++)
        {
                SET_BIT(validFrequencies,frequencies[i],value);
        } 
}

/**
 * @brief Function to compare group of frequencies with used frequency
 * list
 * @param frequencies List to frequencies to check for
 * @param validFrequencies List of frequencies to check against
 * @return 0-Not in frequency list, 1-All in frequency list
 */
int IDTHlApi_IsFreqArrayInFreqList(const outputFrequency_t  frequencies[],
                     validFreq_t              validFrequencies)
{
        int i = 0;

        /* Process list */
        if (frequencies[0] == OutFreq_Invalid) return 0;
        for (i=0; (frequencies[i] != OutFreq_Invalid); i++)
        {
                if (!CHECK_BIT(validFrequencies,frequencies[i]))
						return 0;
        } 

        /* All frequencies in array */
		return 1;
}


/**
 * @brief Function to verify that frequency is within frequency list.
 * @param frequency Frequency to check
 * @param validFrequencies Frequency list to check against
 * @return 1-Frequency is in list, 0-Frequency is not in list
 */
int IDTHlApi_IsFrequencyInFreqList(outputFrequency_t        frequency,
                   const outputFrequency_t  validFrequencies[])
{
        int i = 0;

        /* Process list */
        if (frequency == OutFreq_Invalid) return 0;
        for (i=0; (validFrequencies[i] != OutFreq_Invalid); i++)
        {
                if (frequency == validFrequencies[i])
                        return 1;
        }        
        return 0;
}

