/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#ifndef __DPLL_H__
#define __DPLL_H__

#include "osNumeric.h"
#include "Define.h"
#include "DpllCnfg.h"
#include "General.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \defgroup DPLL DPLL Private header file
 *
 * @brief Functions in this group are related to DPLL provisioning.
 *
 * @{
 */

/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/
/** @brief Enumerated type listing T0 DPLL selector A options */
        typedef enum
        {
                T0DpllSelA_16E1, /**< 32.768MHz */
                T0DpllSelA_16T1, /**< 24.704MHz */
                T0DpllSelA_GSM, /**< 26MHz */
                T0DpllSelA_OBSAI, /**< 30.72MHz */
                T0DpllSelA_MAX /**< \internal Not used. */
        } T_idtT0DpllSelectorA;

/** @brief Enumerated type listing T0 DPLL selector B options */
        typedef enum
        {
                T0DpllSelB_12E1, /**< 24.576MHz */
                T0DpllSelB_GPS, /**< GPS */
                T0DpllSelB_E3, /**< 34.368MHz */
                T0DpllSelB_T3, /**< 44.736MHz */
                T0DpllSelB_MAX /**< \internal Not used. */
        } T_idtT0DpllSelectorB;

/** @brief Enumerated type listing T4 DPLL selector A options */
        typedef enum
        {
                T4DpllSelA_16E1, /**< 32.768MHz */
                T4DpllSelA_16T1, /**< 24.704MHz */
                T4DpllSelA_GSM, /**< 26MHz */
                T4DpllSelA_GPS, /**< GPS */
                T4DpllSelA_MAX /**< \internal Not used. */
        } T_idtT4DpllSelectorA;

/** @brief Enumerated type listing T4 DPLL selector B options */
        typedef enum
        {
                T4DpllSelB_12E1, /**< 24.576MHz */
                T4DpllSelB_24T1, /**< 37.056MHz */
                T4DpllSelB_E3, /**< 34.368MHz */
                T4DpllSelB_T3, /**< 44.736MHz*/
                T4DpllSelB_MAX /**< \internal Not used. */
        } T_idtT4DpllSelectorB;

/** @brief Enumerated type listing Sonet/GigE path selection options */
typedef enum
{
    SonetGigEthSel_T0DpllSonet = 0, /**< From T0, 622.08MHz */
    SonetGigEthSel_T4DpllSonet = 4, /**< From T4, 622.08MHz */
    SonetGigEthSel_T0Dpll10GbE = 8, /**< From T0, 625.00MHz */
    SonetGigEthSel_T0Dpll10GbE_6664 = 9, /**< From T0, 625*66/64MHz */
    SonetGigEthSel_T4Dpll10GbE = 12, /**< From T4, 625.00MHz */
    SonetGigEthSel_T4Dpll10GbE_6664 = 13, /**< From T4, 625*66/64MHz */
    SonetGigEthSel_MAX /**< \internal Not used. */
} T_idtSonetGigEthSelector;

/** @brief Constant defining number of instances of SonetGigEth
 * selectors */
#define SonetGigEthSel_NUMMAX 2

/** @brief Structure containing output path configuration */
typedef struct S_IDTPathConfiguration {
        networkType_t networkType_m; /**< Network type */
        T_idtDpllOutputSelectorA T0SelA_m; /**< T0 Output Path Selector A */
        T_idtDpllOutputSelectorB T0SelB_m; /**< T0 Output Path Selector B */
        T_idtDpllOutputSelectorA T4SelA_m; /**< T4 Output Path Selector A */
        T_idtDpllOutputSelectorB T4SelB_m; /**< T4 Output Path Selector B */

        /**  Sonet/GigE Output Path Selector */
        T_idtSonetGigEthOutput sonetGigEthSel_ma[SonetGigEthSel_NUMMAX];
} T_IDTPathConfiguration;

/** @brief Output path configuration no path constant */
extern const T_IDTPathConfiguration IDTDpll_pathConfigurationNULL_c;

/*********************************************************************
 * G L O B A L    V A R I A B L E S
 *********************************************************************/
extern const T_idtSonetGigEthSelector IDTDpll_sonetGigEthPathConfig2Bitfield[Dpll_MAX][SonetGigEthOutput_MAX];

/*********************************************************************
 * F U N C T I O N    P R O T O T Y P E S
 *********************************************************************/

/**
 * @}
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __DPLL_H__ */
