/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#ifndef __DPLLCNFG_H__
#define __DPLLCNFG_H__

#include "osNumeric.h"
#include "Define.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \defgroup DPLLCnfg DPLL Function Module
 *
 * @brief Functions in this group are related to DPPL provisioning.
 *
 * @{
 */

/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/
/**
 * @brief Enumerated list indicating frequency monitoring factor
 */
typedef enum
{
    FreqMonFactor_0p0032 = 0, /**< 0.0032 ppm */
    FreqMonFactor_0p0064 = 1, /**< 0.0064 ppm */
    FreqMonFactor_0p0127 = 2, /**< 0.0127 ppm */
    FreqMonFactor_0p0257 = 3, /**< 0.0257 ppm */
    FreqMonFactor_0p0514 = 4, /**< 0.0514 ppm */
    FreqMonFactor_0p1030 = 5, /**< 0.103  ppm */
    FreqMonFactor_0p2060 = 6, /**< 0.206  ppm */
    FreqMonFactor_0p4120 = 7, /**< 0.412  ppm */
    FreqMonFactor_0p8230 = 8, /**< 0.823  ppm */
    FreqMonFactor_1p6460 = 9, /**< 1.646  ppm */
    FreqMonFactor_3p2920 = 10, /**< 3.292  ppm */
    FreqMonFactor_3p8100 = 11, /**< 3.81   ppm */
    FreqMonFactor_4p6000 = 12, /**< 4.6    ppm */
    FreqMonFactor_MAX /**< \internal Not used */
} T_idtFreqMonFactor;

/**
 * @brief Enumerated list used for selecting bandwidth limiting stages
 */
typedef enum
{
    dpllBandwidthLimitStart, /**< Start stage bandwidth limiting stage */
    dpllBandwidthLimitAcquire, /**< Acquire stage bandwidth limiting stage */
    dpllBandwidthLimitLocked, /**< Locked stage bandwidth limiting stage */
    dpllBandwidthLimit_MAX /**< \internal Not used. */
} T_idtBandwidthLimitStage;

/**
 * @brief Enumerated list showing options for bandwidth damping
 * factor.
 */
typedef enum
{
    DampingFactor_1p2 = 1, /**< 1.2 */
    DampingFactor_2p5 = 2, /**< 2.5 */
    DampingFactor_5 = 3, /**< 5 */
    DampingFactor_10 = 4, /**< 10 */
    DampingFactor_20 = 5, /**< 20 */
    DampingFactor_MAX /**< \internal Not Used */
} T_idtDampingFactor;

/**
 * @brief List of options for DPLL bandwidth
 */
typedef enum
{
    dpllBandwidth_0p5mHz = 0, /**< 0.5 mHz */
    dpllBandwidth_1mHz = 1, /**<  1  mHz */
    dpllBandwidth_2mHz = 2, /**<  2  mHz */
    dpllBandwidth_4mHz = 3, /**<  4  mHz */
    dpllBandwidth_8mHz = 4, /**<  8  mHz */
    dpllBandwidth_15mHz = 5, /**< 15  mHz */
    dpllBandwidth_30mHz = 6, /**< 30  mHz */
    dpllBandwidth_60mHz = 7, /**< 60  mHz */
    dpllBandwidth_0p1Hz = 8, /**< 0.1 Hz */
    dpllBandwidth_0p3Hz = 9, /**< 0.3 Hz */
    dpllBandwidth_0p6Hz = 10, /**< 0.6 Hz */
    dpllBandwidth_1p2Hz = 11, /**< 1.2 Hz */
    dpllBandwidth_2p5Hz = 12, /**< 2.5 Hz */
    dpllBandwidth_4Hz = 13, /**<  4  Hz */
    dpllBandwidth_8Hz = 14, /**<  8  Hz */
    dpllBandwidth_18Hz = 15, /**< 18  Hz */
    dpllBandwidth_35Hz = 16, /**< 35  Hz */
    dpllBandwidth_70Hz = 17, /**< 70  Hz */
    dpllBandwidth_560Hz = 18, /**< 560 Hz */
    dpllBandwidth_MAX /**< \internal Not used */
} T_idtDpllBandwidth;

/**
 * @brief Enumerated type listing phase slope
 */
typedef enum
{
    phaseSlope_gr1244st3 = 0, /**< GR-1244 Stratum3 61us/s */
    phaseSlope_gr1244st3e = 1, /**< GR-1244 Stratum3E, G2-253 Stratum 3 (855ns/s) */
    phaseSlope_gr813 = 2, /**< GR-813,G.8262 (7.5us/s) */
    phaseSlope_noLimit = 3, /**< No limit */
    phaseSlope_MAX /**< \internal Not used. */
} T_idtPhaseSlope;

/**
 * @brief Enumerated type for DPLL operation mode
 */
typedef enum
{
    Automatic_Mode = 0, /**< Automatic mode */
    Force_FreeRun_Mode = 1, /**< Force to Free-Run mode */
    Force_Holdover_Mode = 2, /**< Force to Holdover mode */
    Force_Locked_Mode = 4, /**< Force to Locked mode */
    Force_Pre_Locked2_Mode = 5, /**< Force to Pre-Locked2 mode */
    Force_Pre_Locked_Mode = 6, /**< Force to Pre-Lock mode */
    Force_Lost_Phase_Mode = 7, /**< Force to Lost-Phase mode */
    Dco_Mode = 8, /**< DCO mode */
    DpllOperMode_MAX /**< \internal Not used */
} T_idtDpllOperMode;

/**
 * @brief Enumerated type listing coarse phase limits
 */
typedef enum
{
    coarsePhaseLimit_1 = 0, /**< +/- 1 UI */
    coarsePhaseLimit_3 = 1, /**< +/- 3 UI */
    coarsePhaseLimit_7 = 2, /**< +/- 7 UI */
    coarsePhaseLimit_15 = 3, /**< +/- 15 UI */
    coarsePhaseLimit_31 = 4, /**< +/- 31 UI */
    coarsePhaseLimit_63 = 5, /**< +/- 63 UI */
    coarsePhaseLimit_127 = 6, /**< +/- 127 UI */
    coarsePhaseLimit_255 = 7, /**< +/- 255 UI */
    coarsePhaseLimit_511 = 8, /**< +/- 511 UI */
    coarsePhaseLimit_1023 = 9, /**< +/- 1023 UI (Only valid for T0) */
    coarsePhaseLimit_MAX /**< \internal Not used */
} T_idtCoarsePhaseLimit;


/**
 * @brief Enumerated type listing fine phase limits
 */
typedef enum
{
    finePhaseLimit_45_90degree = 1, /**< +/- 45-90 degrees */
    finePhaseLimit_90_180degree = 2, /**< +/- 90-180 degrees */
    finePhaseLimit_180_360degree = 3, /**< +/- 180-360 degrees */
    finePhaseLimit_20_25nanosec = 4, /**< +/- 20-25 nanoseconds */
    finePhaseLimit_60_65nanosec = 5, /**< +/- 60-65 nanoseconds */
    finePhaseLimit_120_125nanosec = 6, /**< +/- 120-125 nanoseconds */
    finePhaseLimit_950_955nanosec = 7, /**< +/- 950-955 nanoseconds */
    finePhaseLimit_MAX /**< \internal Not used. */
} T_idtFinePhaseLimit;

/** @brief Enumerated type listing holdover modes  */
typedef enum
{
    Holdover_Instantaneous = 0, /**< Automatic instantaneous mode */
    Holdover_Slow_Average = 2, /**< Automatic slow average mode */
    Holdover_Fast_Average = 3, /**< Automatic fast average mode */
    Holdover_Manual = 4, /**< Manual mode */
    Holdover_MAX /**< \internal Not Used */
} T_idtDpllHoldover;

/** @brief Enumerated type listing temp-holdover modes  */
typedef enum
{
    Temp_Same_As_Full_Holdover = 0, /**< Same as the DPLL holdover mode */
    Temp_Holdover_Instantaneous = 1, /**< Automatic instantaneous */
    Temp_Holdover_Fast_Average = 2, /**< Automatic fast average */
    Temp_Holdover_Slow_Average = 3, /**< Automatic slow average */
    Temp_Holdover_MAX /**< \internal Not used. */
} T_idtTemp_holdover;

/** @brief Enumerated type listing DPLL operational modes */
typedef enum
{
    OperDpllMode_FreeRun = 1, /**< Free-Run Mode */
    OperDpllMode_HoldOver = 2, /**< HoldOver Mode */
    OperDpllMode_Locked = 4, /**< Locked Mode */
    OperDpllMode_PreLocked2 = 5, /**< Pre-Locked 2 Mode */
    OperDpllMode_PreLocked = 6, /**< Pre-Locked Mode */
    OperDpllMode_LostPhase = 7 /**< Lost Phase Mode */
} T_idtOperDpllMode;

/**
 * @brief Enumerated type listing possible Sonet/GigE output frequencies
 */
typedef enum
{
    SonetGigEthOutput_Sonet, /**< SONET/SDH 622.08MHz output */
    SonetGigEthOutput_10GbE, /**< Ethernet 625 MHz output */
    SonetGigEthOutput_10GbE_6664, /**< Ethernet OTN 625(66/64) MHz output */
    SonetGigEthOutput_MAX /**< \internal Not used */
} T_idtSonetGigEthOutput;

/**
 * @brief Enumerated type listing DPLL selector A paths
 */
typedef enum {
    DPLLOutputSelA_16E1, /**< 16E1 Output path */
    DPLLOutputSelA_16T1, /**< 16T1 Output path */
    DPLLOutputSelA_GSM, /**< GSM Output path */
    DPLLOutputSelA_OBSAI, /**< OBSAI Output path */
    DPLLOutputSelA_GPS, /**< GPS Output path */
    DPLLOutputSelA_MAX /**< \internal Not used. */
} T_idtDpllOutputSelectorA;

/**
 * @brief Enumerated type listing DPLL selector B paths
 */
typedef enum {
        DPLLOutputSelB_12E1, /**< 12E1 Output path */
        DPLLOutputSelB_GPS, /**< GPS Output path  */
        DPLLOutputSelB_E3, /**< E3 Output path  */
        DPLLOutputSelB_T3, /**< T3 Output path  */
        DPLLOutputSelB_24T1, /**< 24T1 Output path  */
        DPLLOutputSelB_MAX /**< \internal Not used. */
} T_idtDpllOutputSelectorB;

/** @brief Conversion from parts per trillion to register unit value
 * for DCO offset. */
#define DCO_PPT2DCOUNITS 11

/** @brief Max DCO value
 * \see IDTDpll_SetHoldoverFreq/IDTDpll_GetHoldoverFreq */
#define DCO_MAX_PPT      92274677

/** @brief Max DCO value
 * \see IDTDpll_SetHoldoverFreq/IDTDpll_GetHoldoverFreq */
#define DCO_MIN_PPT      -92274688

/** 
 * @brief Enumerated type listing PPS phase options 
 */
typedef enum {
    PpsPhasePostion_0degree, /**< 0 degrees */
    PpsPhasePostion_50ns,    /**< 50 nanoseconds */
    PpsPhasePostion_100ns,   /**< 100 nanaoseconds */
    PpsPhasePostion_MAX      /**< \internal Not used */
} T_idtDpllPpsPhasePostion;

/**
 * @brief Enumerated type listing PPS pulse width
 */
typedef enum {
    PpsPulseWidth_0pt5sec, /**< 0.5 seconds */
    PpsPulseWidth_10ns, /**< 10 nanoseconds */
    PpsPulseWidth_200ns, /**< 200 nanoseconds */
    PpsPulseWidth_400ns, /**< 400 nanoseconds */
    PpsPulseWidth_800ns, /**< 800 nanoseconds */
    PpsPulseWidth_1us, /**< 1 microsecond */
    PpsPulseWidth_20us, /**< 20 microseconds */
    PpsPulseWidth_50us, /**< 20 microseconds */
    PpsPulseWidth_100us, /**< 100 microseconds */
    PpsPulseWidth_200us, /**< 200 microseconds */
    PpsPulseWidth_1pt2ms, /** 1.2 miliseonds */
    PpsPulseWidth_800us, /**< 800 microseconds */
    PpsPulseWidth_MAX /**< \internal Not used */ 
} T_idtDpllPpsPulseWidth;

/**
 * @brief Utility macro to check if DPLL instance is supported and to
 * translated from DPLL instance to DPLL type
 * @param hdlr Device driver handle
 * @param ext External DPLL instance number (0..N)
 * @param internal Internal DPLL instance
 */
#define IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr,ext,internal)        \
        do {                                \
            OS_ASSERT((ext)<DPLL_INSTANCE_MAX);            \
            OS_ASSERTS((hdlr->deviceConfig_m->pllExt2IntXlate_ma[(ext)]<Dpll_MAX), \
                    ("Unsupported DPLL instance %d\n",(ext)));  \
                    internal = hdlr->deviceConfig_m->pllExt2IntXlate_ma[(ext)];    \
        } while(0)

/*********************************************************************
 * F U N C T I O N    P R O T O T Y P E S
 *********************************************************************/

/*********************************************************************
 * Type of DPLL
 */
T_idtDpllType IDTDpll_GetTypeOfDpll(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll);

/*********************************************************************
 * Input Clock Selection
 */
/* Set the DPLL as automatic reference clock selection; */
void IDTDpll_SetAutoSelInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                             T_osUint8 mode);
T_osUint8 IDTDpll_GetAutoSelInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get auto input selection status */

/* Set the DPLL to force to select a specified reference clock; */
void IDTDpll_SetForceSelInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_osUint8 nInputNo);
T_osUint8 IDTDpll_GetForceSelInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get forced input clock selection */

T_osUint8 IDTDpll_Get1stPriorityInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll);/* Get input number of a valid clock with the highest priority; */
T_osUint8 IDTDpll_Get2ndPriorityInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get input number of a valid clock with the second highest priority; */
T_osUint8 IDTDpll_Get3rdPriorityInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get input number of a valid clock with the third highest priority; */
T_osUint8 IDTDpll_GetCurrentSelectInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get the current selected input clock; */

/*********************************************************************
 * DPLL Mode of Operation
 */
void IDTDpll_SetDpllOperMod(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_idtDpllOperMode nMode); /* Set the DPLL working at the specified operating mode; */
T_idtDpllOperMode IDTDpll_GetDpllOperMod(T_idtDrvHdlr hdlr,
        T_idtDpllInstance nDpll); /* Get specified operating mode for a DPLL */

/* Inquire if the DPLL is in lock state */
T_osUint8 IDTDpll_IsDpllLocked(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll);

/*********************************************************************
 * Bandwidth Damping
 */
/* DPLL Bandwidth and Damping Factor; */
/* Set the bandwidth and damping factor */
void IDTDpll_SetBandWidthDamping(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_idtBandwidthLimitStage stage, T_idtDpllBandwidth nBandWidth,
        T_idtDampingFactor nDamping);

/* Get provisioning bandwidth and damping factor */
void IDTDpll_GetBandWidthDamping(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_idtBandwidthLimitStage stage, T_idtDpllBandwidth *nBandWidth_p,
        T_idtDampingFactor *nDamping_p);

void IDTDpll_SetAutoSelBandWidth(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_osUint8 nEnable); /* Set device to select a suitable bandwidth and damping factor automatically; */
T_osUint8 IDTDpll_GetAutoSelBandWidth(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get auto bandwidth/damping factor status */

void IDTDpll_SetDpllFrozen(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_osUint8 nEnable); /* Set DPLL integral path value frozen */
T_osUint8 IDTDpll_GetDpllFrozen(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get DPLL integral path frozen value  */

/*********************************************************************
 * Phase Loss Detection
 */
void IDTDpll_SetPhaseCoarseDetector(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_osUint8 nEnable, T_osUint8 nWide, T_osUint8 nMultiPhase,
        T_osUint8 nMultiPh8kEn); /* Set the coarse phase loss detector */
void IDTDpll_GetPhaseCoarseDetector(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_osUint8 *nEnable_p, T_osUint8 *nWide_p, T_osUint8 *nMultiPhase_p,
        T_osUint8 *nMultiPh8kEn_p); /* Get the coarse phase loss detector */

void IDTDpll_SetPhaseCoarseLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_idtCoarsePhaseLimit nLimit); /* Set the limitation of the coarse phase lose detector; */
T_idtCoarsePhaseLimit IDTDpll_GetPhaseCoarseLimit(T_idtDrvHdlr hdlr,
        T_idtDpllInstance nDpll); /* Get the limitation of the coarse phase lose detector; */

/* Fine phase loss detector; */
void IDTDpll_SetPhaseFineDetectorEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_osUint8 nEnable); /* Set the fine phase lose detector enable; */
T_osUint8 IDTDpll_GetPhaseFineDetectorEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get the fine phase lose detector enable; */
void IDTDpll_SetPhaseFineLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_idtFinePhaseLimit nLimit); /* Set the limitation of the fine phase lose detector; */
T_idtFinePhaseLimit IDTDpll_GetPhaseFineLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get the limitation of the fine phase lose detector; */

/* Fast loss switch */
void IDTDpll_SetFastLossSwitch(T_idtDrvHdlr hdlr, T_osUint8 nEnable); /* Set reference clock switch if a fast loss occurs; */
T_osUint8 IDTDpll_GetFastLossSwitch(T_idtDrvHdlr hdlr); /* Get reference clock switch if a fast loss occurs; */

/* Holdover frequency calculation and setting; */
void IDTDpll_SetHoldoverMode(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, T_idtDpllHoldover nMode,
        T_osUint8 nReadMode); /* Set calculation mode of holdover frequency and read back mode; */
void IDTDpll_GetHoldoverMode(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, T_idtDpllHoldover *nMode_p,
        T_osUint8 *nReadMode_p); /* Get calculation mode of holdover frequency and read back mode; */

void IDTDpll_SetTempHoldoverMode(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_idtTemp_holdover nMode); /* Set calculation mode of temporary-holdover frequency; */
T_idtTemp_holdover IDTDpll_GetTempHoldoverMode(T_idtDrvHdlr hdlr,
        T_idtDpllInstance nDpll); /* Get calculation mode of temporary-holdover frequency; */


long IDTDpll_GetHoldoverFreq(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get hold over frequency offset */
void IDTDpll_SetHoldoverFreq(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, long pptOffset); /* Set DCO frequency offset */

/* Current DPLL frequency offset */
long IDTDpll_GetCurrentDpllFreqOffset(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get current frequency offset of DPLL (in parts per trillion) */

/* Limitation of soft and hard alarm for DPLL */
void IDTDpll_SetDpllSoftAlarmLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, T_osUint8 nLimit); /* Set the limitation of Soft alarm of DPLL; */
T_osUint8 IDTDpll_GetDpllSoftAlarmLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get the limitation of Soft alarm of DPLL; */

void IDTDpll_SetDpllHardAlarmLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, T_osUint16 nLimit); /* Set the limitation of Hard alarm of DPLL; */
T_osUint16 IDTDpll_GetDpllHardAlarmLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get the limitation of Hard alarm of DPLL; */

void IDTDpll_SetDpllHardAlarmEnable(T_idtDrvHdlr hdlr, T_osUint8 nEnable); /* Set DPLL in unlocked state when the Hard alarm raised; */
T_osUint8 IDTDpll_GetDpllHardAlarmEnable(T_idtDrvHdlr hdlr); /* Get DPLL in unlocked state when the Hard alarm raised; */

/* Current phase offset */
T_osUint16 IDTDpll_GetCurrentPhaseError(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get the current phase error of DPLL */

/*********************************************************************
 * DPLL Output Path Selection
 */
void IDTDpll_SetSonetGigEthOutputPath(T_idtDrvHdlr hdlr, T_osUint8 instance, T_idtDpllInstance inputDpll, T_idtSonetGigEthOutput outputFreq);
void IDTDpll_GetSonetGigEthOutputPath(T_idtDrvHdlr hdlr, T_osUint8 instance, T_idtDpllInstance *inputDpll_p, T_idtSonetGigEthOutput *outputFreq_p);

void IDTDpll_SetDpllOutputPathSelectorA(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_idtDpllOutputSelectorA path);
T_idtDpllOutputSelectorA IDTDpll_GetDpllOutputPathSelectorA(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll);

void IDTDpll_SetDpllOutputPathSelectorB(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, T_idtDpllOutputSelectorB path);
T_idtDpllOutputSelectorB IDTDpll_GetDpllOutputPathSelectorB(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll);

void IDTDpll_SetDpllEthPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, T_osUint8 nDisable); /* Set DPLL ETH path Enable/Disable */
T_osUint8 IDTDpll_GetDpllEthPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get DPLL ETH path Enable/Disable */

void IDTDpll_SetDpllSelBPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, T_osUint8 nDisable); /* Set DPLL Selector B Enable/Disable */
T_osUint8 IDTDpll_GetDpllSelBPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get DPLL Selector B Enable/Disable */

void IDTDpll_SetDpll16E116T1Enable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, T_osUint8 nDisable); /* Set DPLL 16E1/16T1 path Enable/Disable */
T_osUint8 IDTDpll_GetDpll16E116T1Enable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get DPLL 16E1/16T1 path Enable/Disable */

void IDTDpll_SetDpllSelAPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, T_osUint8 nDisable); /* Set DPLL Selector A Enable/Disable */
T_osUint8 IDTDpll_GetDpllSelAPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get DPLL Selector A Enable/Disable */

/*********************************************************************
 * Frequency Monitor
 */
/* Set frequency monitoring factor */
void IDTDpll_SetFrequencyMonitoringFactor(T_idtDrvHdlr hdlr,
        T_idtFreqMonFactor freqMonFactor);
T_idtFreqMonFactor IDTDpll_GetFrequencyMonitoringFactor(T_idtDrvHdlr hdlr); /* Get frequency monitoring factor */

void IDTDpll_SetHardAlarmThreshold(T_idtDrvHdlr hdlr, T_osUint8 accepting, T_osUint8 rejecting); /* Set hard alarm monitoring thresholds */
void IDTDpll_GetHardAlarmThreshold(T_idtDrvHdlr hdlr, T_osUint8 *accepting_p, T_osUint8 *rejecting_p); /* Get hard alarm monitoring thresholds */

void IDTDpll_SetSoftAlarmThreshold(T_idtDrvHdlr hdlr, T_osUint8 accepting, T_osUint8 rejecting); /* Set soft alarm monitoring thresholds */
void IDTDpll_GetSoftAlarmThreshold(T_idtDrvHdlr hdlr, T_osUint8 *accepting_p, T_osUint8 *rejecting_p); /* Get soft alarm monitoring thresholds */


void IDTDpll_SetIcpCtrl(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, T_osUint8 nIcp); /* Set ICP Control */
T_osUint8 IDTDpll_GetIcpCtrl(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get input change pump (ICP) value */

void IDTDpll_SetPhaseSlope(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
        T_idtPhaseSlope nPhaseSlope); /* Set DPLL phase slope */
T_idtPhaseSlope IDTDpll_GetPhaseSlope(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll); /* Get DPLL phase slope */

/*  Enable/Disable 1 PPS fast lock */
void IDTDpll_SetPpsFastLock(T_idtDrvHdlr hdlr, T_osUint8 nEnableFreq,
        T_osUint8 nEnablePhase);
void IDTDpll_GetPpsFastLock(T_idtDrvHdlr hdlr, T_osUint8 *nEnableFreq_p,
                    T_osUint8 *nEnablePhase_p);

void IDTDpll_SetPhaseTransient(T_idtDrvHdlr hdlr, T_osUint8 nEnable); /* Set the configuration when phase transient occurs */
T_osUint8 IDTDpll_GetPhaseTransient(T_idtDrvHdlr hdlr); /* Get the configuration when phase transient occurs */

void IDTDpll_SetHitlessSwitchingFeature(T_idtDrvHdlr hdlr, T_osUint8 nEnable, T_osUint8 nFrozen); /* Set the configuration of hitless switching feature */
void IDTDpll_GetHitlessSwitchingFeature(T_idtDrvHdlr hdlr, T_osUint8 *nEnable_p,
 T_osUint8 *nFrozen_p); /* Get the configuration of hitless switching feature */

/*Phase offset */
void IDTDpll_SetPhaseOffset(T_idtDrvHdlr hdlr, T_osUint16 nOffset); /*Set the phase offset value; */
T_osUint16 IDTDpll_GetPhaseOffset(T_idtDrvHdlr hdlr); /*Get the phase offset value; */

/*Set PPS pulse & phase for DPLL */
void IDTDpll_SetPpsPhase(T_idtDrvHdlr hdlr,
        T_idtDpllInstance nDpll,
                         T_idtDpllPpsPhasePostion nPhase, 
                         T_idtDpllPpsPulseWidth   nPulse); 
/*Get PPS pulse & phase for DPLL */
void IDTDpll_GetPpsPhase(T_idtDrvHdlr hdlr,
        T_idtDpllInstance nDpll,
                         T_idtDpllPpsPhasePostion *nPhase_p, 
                         T_idtDpllPpsPulseWidth   *nPulse_p); 

/**
 * @}
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __DPLLCNFG_H__ */
