/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#ifndef __INPUT_H__
#define __INPUT_H__

#include "osNumeric.h"
#include "Define.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \defgroup Input Input Function Module
 *
 * @brief Functions in this group are related to input port provisioning.
 *
 * This module also include frequency monitoring controls too.
 *
 * @{
 */

/*********************************************************************
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *********************************************************************/

/** @brief Enumerated type listing input divider modes */
typedef enum
{
	Divider_Bypass = 0, /**< Bypass divider mode */
	Divider_Lock8k = 1, /**< Lock 8kHz  divider mode */
	Divider_DivN = 2, /**< Divide by N (DivN) mode */
	Divider_MAX /**< \internal No used */
} T_idtInputDividerMux;

/** @brief Enumerated type listing high frequency (>155.52MHz) dividers */
typedef enum
{
	HF_Bypass = 0, /**< Bypass high frequency divider */
	HF_Divide_4 = 1, /**< HF Divide by 4 */
	HF_Divide_5 = 2, /**< HF Divide by 5 */
	HF_MAX /**< \internal No used */
} T_idtHighFreqencyDivisor;

/* For Input clock status */
#define Input_Phase_Lock_Alarm	1
#define Input_No_Activity_Alarm	2
#define Input_Freq_Hard_Alarm	4

/** @brief Enumerated type listing input frequencies */
typedef enum
{
	Freq_8K = 0, /* Input frequency is 8kHz*/
	Freq_1544M = 1, /* Input frequency is 1.544MHz */
	Freq_2048M = 1, /* Input frequency is 2.048MHz */
	Freq_648M = 2, /* Input frequency is 6.48MHz */
	Freq_1944M = 3, /* Input frequency is 19.44MHz */
	Freq_2592M = 4, /* Input frequency is 25.92MHz */
	Freq_3888M = 5, /* Input frequency is 38.88MHz */
	Freq_2K = 9, /* Input frequency is 2kHz */
	Freq_4K = 10, /* Input frequency is 4kHz */
	Freq_1PPS = 11, /* Input frequency is 1 pulse/sec */
	Freq_625M = 12, /* Input frequency is 6.25MHz */
	Freq_10M = 13, /* Input frequency is 10MHz */
	Freq_MAX /* \internal Not used */
} T_idtInputFrequencyBitfieldValue;

/** @brief Enumerated type listing input frequency families */
typedef enum
{
    InFreqFamily_8K = 0, /* Input frequency is 8kHz*/
    InFreqFamily_1544M = 1, /* Input frequency is 1.544MHz */
	InFreqFamily_2048M = 2, /* Input frequency is 2.048MHz */
	InFreqFamily_648M = 3, /* Input frequency is 6.48MHz */
	InFreqFamily_1944M = 4, /* Input frequency is 19.44MHz */
	InFreqFamily_2592M = 5, /* Input frequency is 25.92MHz */
	InFreqFamily_3888M = 6, /* Input frequency is 38.88MHz */
	InFreqFamily_2K = 7, /* Input frequency is 2kHz */
	InFreqFamily_4K = 8, /* Input frequency is 4kHz */
	InFreqFamily_1PPS = 9, /* Input frequency is 1 pulse/sec */
	InFreqFamily_625M = 10, /* Input frequency is 6.25MHz */
	InFreqFamily_10M = 11, /* Input frequency is 10MHz */
	InFreqFamily_MAX /* \internal Not used */
} T_idtInputFrequencyFamily;

/** @brief Enumerated type listing AMI Input frequencies */
typedef enum 
{
    AmiFreq_64kHzPlus8kHz, /**< 64 kHz + 8 kHz signal */
    AmiFreq_64kHzPlus400Hz, /**< 64 kHz + 400 Hz signal */
    AmiFreq_MAX /**< \internal Not used */
} T_idtAmiInputFrequencyBitFieldValue;

/**
 * @brief Enumerated type listing possible input sync frequencies
 */
typedef enum {
    SYNC_8kHz, /**< 8 kHz sync input */
    SYNC_1PPS, /**< 1 PPS sync input */
    SYNC_4kHz, /**< 4 kHz sync input */
    SYNC_2kHz, /**< 2 kHz sync input */
    SYNC_MAX   /**< \internal Not used */
} T_idtInputSyncFreq;

/** 
 * @brief Enumerated type listing sampling of input external sync
 */
typedef enum {
    externalSyncSampling_onTarget, /**< On target (0 UI) */
    externalSyncSampling_0dot5Early, /**< 0.5 UI early */
    externalSyncSampling_1dot0Late, /**< 1.0 UI late */
    externalSyncSampling_0dot5Late, /**< 0.5 UI late */
    externalSyncSampling_MAX /**< \internal Not used */
} T_externalSyncSampling;

/**
 * @brief Enumerated type listing possible external sync alignment
 * options
 */
typedef enum {
    externalSyncEdge_FallingEdge, /**< External sync align to falling edge */
    externalSyncEdge_RisingEdge, /**< External sync align to rising edge */
    externalSyncEdge_MAX /**< \internal Not Used */
} T_externalSyncEdge;

/**
 * @brief Enumerate type listing ranges for external sync alarm
 *
 */
typedef enum {
    externalSyncAlarmRange_1, /**< +/- 1 UI range */
    externalSyncAlarmRange_2, /**< +/- 2 UI range */
    externalSyncAlarmRange_3, /**< +/- 3 UI range */
    externalSyncAlarmRange_4, /**< +/- 4 UI range */
    externalSyncAlarmRange_5, /**< +/- 5 UI range */
    externalSyncAlarmRange_6, /**< +/- 6 UI range */
    externalSyncAlarmRange_7, /**< +/- 7 UI range */
    externalSyncAlarmRange_8, /**< +/- 8 UI range */
    externalSyncAlarmRange_MAX /**< \internal Not used */
} T_externalSyncAlarmRange;

/**
 * @brief Enumerated type listing output synchronization with respect to 
 */
typedef enum {
    externalSyncBypass_ExSync1, /**< Synchronized to external sync 1 */
    externalSyncBypass_AutoSel, /**< Auto synchronize using DPLL #1.
                                 * If DPLL #1 select IN1 or IN4 then
                                 * synchronize to external sync1 If
                                 * DPLL #1 select IN2 then synchronize
                                 * to external sync 2. If no input
                                 * selected by DPLL #1, external sync
                                 * are not synchronized
                                 */
    externalSyncBypass_MAX /**< \internal Not used */
}  T_externalSyncBypass;

/**
 * @brief Enumerated type listing enable options for external input
 * sync
 */
typedef enum {
    externalSyncEnable_Disable, /**< Disable external sync */
    externalSyncEnable_Enable, /**< Enable external sync */
    externalSyncEnable_Auto, /**< Enable/Disable external sync based
                                on DPLL #1 loacking to (internal) input 4 */
    externalSyncEnable_MAX /**< \internal Not used */
} T_externalSyncEnable;

/**
 * @brief Utility macro to check if input port is supported and to
 * translated from external port to internal port
 * @param hdlr Device driver handle
 * @param ext External input port number
 * @param internal Internal input number
 */
#define IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr,ext,internal)	 \
do {								                         \
	OS_ASSERT((ext)<=IDT_DRV_MAX_INPUT);		    	     \
	OS_ASSERT((ext)>0);      	                             \
	OS_ASSERTS(hdlr->deviceConfig_m->inputExt2IntXlate_ma[(ext)],	\
                ("Unsupported input port %d\n",(ext)));             \
	internal = hdlr->deviceConfig_m->inputExt2IntXlate_ma[(ext)];	\
} while(0)

/**
 * @brief Utility macro to check if input port is populated
 * @param hdlr Device driver handle
 * @param ext External input port number
 * @return a positive number - popluated, 0 - not populated
 */
#define IDT_EXT_INPORT_POPULATED(hdlr,ext) \
    (hdlr->deviceConfig_m->inputExt2IntXlate_ma[(ext)])

/*********************************************************************
 * F U N C T I O N    P R O T O T Y P E S
 *********************************************************************/

void IDTInput_SetPriority(T_idtDrvHdlr hdlr, T_osUint8 nInputNo, T_idtDpllInstance nDpll, T_osUint8 nPriority); /* Set the priority of the specified port */
T_osUint8 IDTInput_GetPriority(T_idtDrvHdlr hdlr, T_osUint8 nInputNo, T_idtDpllInstance nDpll); /* Get the priority of the specified port */

void IDTInput_SetInputFrequency(T_idtDrvHdlr hdlr, T_osUint8 nInputNo,
        T_idtInputFrequencyBitfieldValue nFrequency); /* Indicate the frequency of the specified input port */
T_idtInputFrequencyBitfieldValue IDTInput_GetInputFrequency(T_idtDrvHdlr hdlr,
        T_osUint8 nInputNo); /* Retrieve provisioned input port frequency */

/* Select AMI Input signal */
void IDTInput_SetAmiInputFrequency(
    T_idtDrvHdlr                        hdlr,
    T_osUint8                           nInputNo,
    T_idtAmiInputFrequencyBitFieldValue amiSignal);
/* Retrieve AMI Input signal */
T_idtAmiInputFrequencyBitFieldValue IDTInput_GetAmiInputFrequency(
    T_idtDrvHdlr hdlr,
    T_osUint8    nInputNo);

void IDTInput_SetActivityMonitor(T_idtDrvHdlr hdlr, T_osUint8 nInputNo,
        T_osUint8 nBucket); /* Select one of the four activity monitor configurations for the specified input port */
T_osUint8 IDTInput_GetActivityMonitor(T_idtDrvHdlr hdlr, T_osUint8 nInputNo); /* Get selected activity monitor configuration */

void IDTInput_SetBucketConfig(T_idtDrvHdlr hdlr, T_osUint8 nBucket, T_osUint8 nUpper,
        T_osUint8 nLower, T_osUint8 nSize, T_osUint8 nDecay); /* Set the configuration to Leaky Buckets */
void IDTInput_GetBucketConfig(T_idtDrvHdlr hdlr, T_osUint8 nBucket,
        T_osUint8 *nUpper_p, T_osUint8 *nLower_p, T_osUint8 *nSize_p,
        T_osUint8 *nDecay_p); /* Get the configuration of Leaky Buckets */

void IDTInput_SetPreDivider(T_idtDrvHdlr hdlr, T_osUint8 nInputNo,
        T_idtInputDividerMux nDivider); /* Select or bypass pre-divider for an input clock; */
T_idtInputDividerMux IDTInput_GetPreDivider(T_idtDrvHdlr hdlr, T_osUint8 nInputNo); /* Get selected pre-divider for input port */

void IDTInput_SetDivisionFactor(T_idtDrvHdlr hdlr, T_osUint8 nInputNo,
        T_osUint16 iFactor); /* Set the dividen factor to pre-divider assigned to a certain input; */
T_osUint16 IDTInput_GetDivisionFactor(T_idtDrvHdlr hdlr, T_osUint8 nInputNo); /* Get the dividen factor for pre-divider assigned input port */

void IDTInput_SetInputHFdivider(T_idtDrvHdlr hdlr, T_osUint8 nInputNo,
        T_idtHighFreqencyDivisor nUsage); /* Set the setting for HF divider */
T_idtHighFreqencyDivisor IDTInput_GetInputHFdivider(T_idtDrvHdlr hdlr,
        T_osUint8 nInputNo); /* Get the setting for HF divider */

T_osUint8 IDTInput_GetInputFreqOffset(T_idtDrvHdlr hdlr, T_osUint8 nInputNo); /* Get the frequency offset of the specified input clock */
T_osUint8 IDTInput_GetInputClockStatus(T_idtDrvHdlr hdlr, T_osUint8 nInputNo); /* Get the status of the specified input clock; */
T_osUint8 IDTInput_GetInputClockValidation(T_idtDrvHdlr hdlr, T_osUint8 nInputNo); /* Get the validation of all input clocks */

void IDTInput_EnableAllInputs(T_idtDrvHdlr hdlr); /* Enable to select anyone of input clocks */
void IDTInput_DisableAllInputs(T_idtDrvHdlr hdlr); /* Disable to select anyone of input clocks */

void IDTInput_SetInputEnable(T_idtDrvHdlr hdlr, T_osUint8 nInputNo,
        T_osUint8 enable); /* Enable/Disable to select a specified input port */
T_osUint8 IDTInput_GetInputEnable(T_idtDrvHdlr hdlr, T_osUint8 nInputNo); /* Get enable/disable status for input port */

void IDTInput_SetSyncFrequency(T_idtDrvHdlr hdlr, T_idtInputSyncFreq syncFreq); /* Set input sync frequency */
T_idtInputSyncFreq IDTInput_GetSyncFrequency(T_idtDrvHdlr hdlr); /* Get input sync frequency */

void IDTInput_SetSyncSampling(T_idtDrvHdlr hdlr,
                              T_osUint8 exSyncN,
                              T_externalSyncSampling sampling); /* External sync sampling */
T_externalSyncSampling IDTInput_GetSyncSampling(T_idtDrvHdlr hdlr,
                                                T_osUint8 exSyncN); /* External sync sampling */

void IDTInput_SetSyncEdge(T_idtDrvHdlr hdlr,
                          T_externalSyncEdge edge); /* External sync alignment */
T_externalSyncEdge IDTInput_GetSyncEdge(T_idtDrvHdlr hdlr); /* External sync alignment */


void IDTInput_SetSyncAlarmRange(T_idtDrvHdlr hdlr,
                                T_externalSyncAlarmRange range); /* Sync alarm range */
T_externalSyncAlarmRange IDTInput_GetSyncAlarmRange(T_idtDrvHdlr hdlr); /* Sync alarm range */

void IDTInput_SetSyncBypass(T_idtDrvHdlr hdlr,
                            T_externalSyncBypass bypass); /* Sync bypass control */
T_externalSyncBypass IDTInput_GetSyncBypass(T_idtDrvHdlr hdlr); /* Sync by pass control */

void IDTInput_SetSyncEnable(T_idtDrvHdlr hdlr,
                            T_externalSyncEnable enable); /* Sync enable */
T_externalSyncEnable IDTInput_GetSyncEnable(T_idtDrvHdlr hdlr); /* Sync enable */

T_idtInputFrequencyFamily IDTInput_InFreqBitValue2Family(T_idtDrvHdlr hdlr, /* Translate frequency bit value to frequency family */
                                    T_idtInputFrequencyBitfieldValue inFreqBitValue);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __INPUT_H__ */
