/*********************************************************************
 * Copyright (c) 2012-2013 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#ifndef __PUMA_CMN_REGISTER_DEF_H__
#define __PUMA_CMN_REGISTER_DEF_H__

#ifdef __cplusplus
extern "C"
{
#endif
/*! \file RegisterDef.h
 *  @brief This header file contains register and bitfield information for
 *  accessing device.
 *  \see Datasheet for detailed register descriptions.
 */

/** \defgroup RegistersDef Register definitions
 *
 * @{ */


/**
 * @brief Enumerated type listing register offsers and bit field
 * constants.
 */
enum {

/*======================Register Definitions==========================*/
    REG_ID_LOW = 0x00, /**< Device ID Register 1, Low byte */
    REG_ID_HIGH = 0x01, /**< Device ID Register 2, High byte */
    REG_MPU_PIN_STS = 0x02, /**< MPU Pin Status Register */
    REG_NOMINAL_FREQ_LOW_CNFG = 0x04, /**< Crystal Oscillator Frequency Offset Calibration Configuration Register 1,value[7:0] */
    REG_NOMINAL_FREQ_MID_CNFG = 0x05, /**< Crystal Oscillator Frequency Offset Calibration Configuration Register 2,value[15:8] */
    REG_NOMINAL_FREQ_HIGH_CNFG = 0x06, /**< Crystal Oscillator Frequency Offset Calibration Configuration Register 3,value[23:16] */
    REG_T4_T0_SEL_CNFG = 0x07, /**< T0/T4 Register Selection Configuration */
    REG_PHASE_ALARM_TIME_OUT_CNFG = 0x08, /**< Phase Lock Alarm Time Out Configuration Register */
    REG_INPUT_MODE_CNFG = 0x09, /**< Input Mode Control Register */
    REG_DIFFERENTIAL_IN_OUT_CNFG = 0x0A, /**< Differential Input/Output Port Configuration Register */
    REG_MON_SW_HS_CNFG = 0x0B, /**< Frequency Monitoring,Master Clock Switch,Input Clock Switch,HS Configuration Register */
    REG_INTERRUPT_CNFG = 0x0C, /**< Interrupt Configuration Register */
    REG_INTERRUPTS1_STS = 0x0D, /**< Interrupt Status Register 1 */
    REG_INTERRUPTS2_STS = 0x0E, /**< Interrupt Status Register 2 */
    REG_INTERRUPTS3_STS = 0x0F, /**< Interrupt Status Register 3 */
    REG_INTERRUPTS1_MASK_CNFG = 0x10, /**< Interrupt Mask Register 1 */
    REG_INTERRUPTS2_MASK_CNFG = 0x11, /**< Interrupt Mask Register 2 */
    REG_INTERRUPTS3_MASK_CNFG = 0x12, /**< Interrupt Mask Register 3 */
    REG_MS_SL_CTRL_CNFG = 0x13, /**< Master Slave Control */
    REG_IN1_CNFG = 0x14, /**< Input 1 Configuration Register */
    REG_IN2_CNFG = 0x15, /**< Input 2 Configuration Register */
    REG_IN3_CNFG = 0x16, /**< Input 3 Configuration Register */
    REG_IN4_CNFG = 0x17, /**< Input 4 Configuration Register */
    REG_IN5_IN6_HF_DIV_CNFG = 0x18, /**< Input 5 and Input 6 High Frequency divider Configuration Register */
    REG_IN5_CNFG = 0x19, /**< Input 5 Configuration Register */
    REG_IN6_CNFG = 0x1A, /**< Input 6 Configuration Register */
    REG_IN7_CNFG = 0x1B, /**< Input 7 Configuration Register */
    REG_IN8_CNFG = 0x1C, /**< Input 8 Configuration Register */
    REG_IN9_CNFG = 0x1D, /**< Input 9 Configuration Register */
    REG_IN10_CNFG = 0x1E, /**< Input 10 Configuration Register */
    REG_IN11_CNFG = 0x1F, /**< Input 11 Configuration Register */
    REG_IN12_CNFG = 0x20, /**< Input 12 Configuration Register */
    REG_IN13_CNFG = 0x21, /**< Input 13 Configuration Register */
    REG_IN14_CNFG = 0x22, /**< Input 14 Configuration Register */
    REG_PRE_DIV_CH_CNFG = 0x23, /**< Input Clock Select for Frequency division Configuration Register */
    REG_PRE_DIVN_LOW_CNFG = 0x24, /**< DivN Divider Configuration Register 1, Low byte */
    REG_PRE_DIVN_HIGH_CNFG = 0x25, /**< DivN Divider Configuration Register 2, High byte */
    REG_IN1_IN2_SEL_PRIORITY_CNFG = 0x26, /**< Input 1 and Input 2 Priority Configuration Register */
    REG_IN3_IN4_SEL_PRIORITY_CNFG = 0x27, /**< Input 3 and Input 4 Priority Configuration Register */
    REG_IN5_IN6_SEL_PRIORITY_CNFG = 0x28, /**< Input 5 and Input 6 Priority Configuration Register */
    REG_IN7_IN8_SEL_PRIORITY_CNFG = 0x29, /**< Input 7 and Input 8 Priority Configuration Register */
    REG_IN9_IN10_SEL_PRIORITY_CNFG = 0x2A, /**< Input 9 and Input 10 Priority Configuration Register */
    REG_IN11_IN12_SEL_PRIORITY_CNFG = 0x2B, /**< Input 11 and Input 12 Priority Configuration Register */
    REG_IN13_IN14_SEL_PRIORITY_CNFG = 0x2C, /**< Input 13 and Input 14 Priority Configuration Register */
    REG_PAGE_POINTER_CNFG = 0x2D, /**< Page Pointer Configuration Register */
    REG_FREQ_MON_FACTOR_CNFG = 0x2E, /**< Frequency Monitor Factor Configuration Register */
    REG_HARD_FREQ_MON_THRESHOLD_CNFG = 0x2F, /**< Hard Frequency Alarm Threshold for the Non-Selected Input Clocks Configuration Register */
    REG_SOFT_FREQ_MON_THRESHOLD_CNFG = 0x30, /**< Soft Frequency Alarm Threshold for the Selected Input Clock Configuration Register */
    REG_UPPER_THRESHOLD_0_CNFG = 0x31, /**< Upper Threshold 0 Configuration Register */
    REG_LOWER_THRESHOLD_0_CNFG = 0x32, /**< Lower Threshold 0 Configuration Register */
    REG_BUCKET_SIZE_0_CNFG = 0x33, /**< Leaky Bucket Size 0 Configuration Register */
    REG_DECAY_RATE_0_CNFG = 0x34, /**< Decay Rate 0 Configuration Register */
    REG_UPPER_THRESHOLD_1_CNFG = 0x35, /**< Upper Threshold 1 Configuration Register */
    REG_LOWER_THRESHOLD_1_CNFG = 0x36, /**< Lower Threshold 1 Configuration Register */
    REG_BUCKET_SIZE_1_CNFG = 0x37, /**< Leaky Bucket Size 1 Configuration Register */
    REG_DECAY_RATE_1_CNFG = 0x38, /**< Decay Rate 1 Configuration Register */
    REG_UPPER_THRESHOLD_2_CNFG = 0x39, /**< Upper Threshold 2 Configuration Register */
    REG_LOWER_THRESHOLD_2_CNFG = 0x3A, /**< Lower Threshold 2 Configuration Register */
    REG_BUCKET_SIZE_2_CNFG = 0x3B, /**< Leaky Bucket Size 2 Configuration Register */
    REG_DECAY_RATE_2_CNFG = 0x3C, /**< Decay Rate 2 Configuration Register */
    REG_UPPER_THRESHOLD_3_CNFG = 0x3D, /**< Upper Threshold 3 Configuration Register */
    REG_LOWER_THRESHOLD_3_CNFG = 0x3E, /**< Lower Threshold 3 Configuration Register */
    REG_BUCKET_SIZE_3_CNFG = 0x3F, /**< Leaky Bucket Size 3 Configuration Register */
    REG_DECAY_RATE_3_CNFG = 0x40, /**< Decay Rate 3 Configuration Register */
    REG_IN_FREQ_READ_CH_CNFG = 0x41, /**< Input Clock Selection for Frequency Monitoring Result Read Register */
    REG_IN_FREQ_READ_STS = 0x42, /**< Input Clock Frequency Offset Register */
    REG_IN1_IN2_STS = 0x43, /**< Input 1 and Input 2 Status Register */
    REG_IN3_IN4_STS = 0x44, /**< Input 3 and Input 4 Status Register */
    REG_IN5_IN6_STS = 0x45, /**< Input 5 and Input 6 Status Register */
    REG_IN7_IN8_STS = 0x46, /**< Input 7 and Input 8 Status Register */
    REG_IN9_IN10_STS = 0x47, /**< Input 9 and Input 10 Status Register */
    REG_IN11_IN12_STS = 0x48, /**< Input 11 and Input 12 Status Register */
    REG_IN13_IN14_STS = 0x49, /**< Input 13 and Input 14 Status Register */
    REG_INPUT_VALID1_STS = 0x4A, /**< Input Clock Validation Register 1 */
    REG_INPUT_VALID2_STS = 0x4B, /**< Input Clock Validation Register 2 */
    REG_REMOTE_INPUT_VALID1_CNFG = 0x4C, /**< Remote Input Clock Validation Register 1 */
    REG_REMOTE_INPUT_VALID2_CNFG = 0x4D, /**< Remote Input Clock Validation Register 2 */
    REG_PRIORITY_TABLE1_STS = 0x4E, /**< The Highest Priority Valid Input Clock and the Selected Input Clock Register */
    REG_PRIORITY_TABLE2_STS = 0x4F, /**< The Second Highest Priority Valid Input Clock and the Selected Input Clock Register */
    REG_T0_INPUT_SEL_CNFG = 0x50, /**< T0 Input Clock Selection Configuration Register */
    REG_T4_INPUT_SEL_CNFG = 0x51, /**< T4 Input Clock Selection Configuration Register */
    REG_OPERATING_STS = 0x52, /**< Operating Status Register */
    REG_T0_OPERATION_MODE_CNFG = 0x53, /**< T0 DPLL Operation Mode Selection Register */
    REG_T4_OPERATION_MODE_CNFG = 0x54, /**< T4 DPLL Operation Mode Selection Register */
    REG_T0_DPLL_APLL_PATH_CNFG = 0x55, /**< T0 DPLL APLL Path Configuration Register */
    REG_T0_DPLL_START_BW_DAMPING_CNFG = 0x56, /**< T0 DPLL Start Bandwidth and Damping Factor Configuration Register */
    REG_T0_DPLL_ACQ_BW_DAMPING_CNFG = 0x57, /**< T0 DPLL Acquisition Bandwidth and Damping Factor Configuration Register */
    REG_T0_DPLL_LOCKED_BW_DAMPING_CNFG = 0x58, /**< T0 DPLL Locked Bandwidth and Damping Factor Configuration Register */
    REG_T0_BW_OVERSHOOT_CNFG = 0x59, /**< T0 DPLL Bandwidth Overshoot Configuration Register */
    REG_PHASE_LOSS_COARSE_LIMIT_CNFG = 0x5A, /**< Coarse Phase Lock Detector Configuration Register */
    REG_PHASE_LOSS_FINE_LIMIT_CNFG = 0x5B, /**< Fine Phase Lock Detector Configuration Register */
    REG_T0_HOLDOVER_MODE_CNFG = 0x5C, /**< T0 DPLL Holdover Mode Configuration Register */
    REG_HOLDOVER_FREQ_LOW_CNFG = 0x5D, /**< DPLL Holdover Frequency Register 1 */
    REG_HOLDOVER_FREQ_MID_CNFG = 0x5E, /**< DPLL Holdover Frequency Register 2 */
    REG_HOLDOVER_FREQ_HIGH_CNFG = 0x5F, /**< DPLL Holdover Frequency Register 3 */
    REG_T4_DPLL_APLL_PATH_CNFG = 0x60, /**< T4 DPLL APLL Path Configuration Register */
    REG_T4_DPLL_LOCKED_BW_DAMPING_CNFG = 0x61, /**< T4 DPLL Locked Bandwidth and Damping Factor Configuration Register */
    REG_CURRENT_FREQ_LOW_STS = 0x62, /**< DPLL Current Frequency Register 1 */
    REG_CURRENT_FREQ_MID_STS = 0x63, /**< DPLL Current Frequency Register 2 */
    REG_CURRENT_FREQ_HIGH_STS = 0x64, /**< DPLL Current Frequency Register 3 */
    REG_DPLL_FREQ_SOFT_LIMIT_CNFG = 0x65, /**< DPLL Soft Frequency Alarm Limit Configuration Register */
    REG_DPLL_FREQ_HARD_LIMIT_LOW_CNFG = 0x66, /**< DPLL Hard Frequency Alarm Limit Configuration Register 1 */
    REG_DPLL_FREQ_HARD_LIMIT_MID_CNFG = 0x67, /**< DPLL Hard Frequency Alarm Limit Configuration Register 2 */
    REG_CURRENT_DPLL_PHASE_LOW_STS = 0x68, /**< Current DPLL Phase Register 1 */
    REG_CURRENT_DPLL_PHASE_HIGH_STS = 0x69, /**< Current DPLL Phase Register 2 */
    REG_OUT1_FREQ_CNFG = 0x6B, /**< Output 1 Configuration Register */
    REG_OUT2_FREQ_CNFG = 0x6C, /**< Output 2 Configuration Register */
    REG_OUT3_FREQ_CNFG = 0x6D, /**< Output 3 Configuration Register */
    REG_OUT4_FREQ_CNFG = 0x6E, /**< Output 4 Configuration Register */
    REG_OUT5_FREQ_CNFG = 0x6F, /**< Output 5 Configuration Register */
    REG_OUT6_FREQ_CNFG = 0x70, /**< APLL 1 Clock Frequency Configuration */
    REG_OUT7_FREQ_CNFG = 0x71, /**< APLL 2 Clock Frequency Configuration */
    REG_OUT8_FREQ_CNFG = 0x72, /**< Output 8 Configuration Register */
    REG_OUT9_FREQ_CNFG = 0x73, /**< Output 9 Configuration Register */
    REG_FR_MFR_SYNC_CNFG = 0x74, /**< Frame Sync and Multi-frame Sync Output Configuration Register */
    REG_PHASE_MON_CNFG = 0x78, /**< Phase Transient Monitor Configuration Register */
    REG_PHASE_OFFSET_LOW_CNFG = 0x7A, /**< Phase Offset Control Register 1 */
    REG_PHASE_OFFSET_HIGH_CNFG = 0x7B, /**< Phase Offset Control Register 2 */
    REG_SYNC_MONITOR_CNFG = 0x7C, /**< Sync Monitor Configuration Register */
    REG_SYNC_PHASE_CNFG = 0x7D, /**< Sync Phase Configuration Register */
    REG_PROTECTION_CNFG = 0x7E, /**< Registers Access Mode Selection Register */
    REG_MPU_SEL_CNFG = 0x7F, /**< MPU Interface Mode Selection Register */

/* Page 1 Registers */
    REG_DFS_OFF_CNFG = 0x30, /**< Digital Frequency Synthesizer Configuration Register */
    REG_T0_PPS_CNFG = 0x31, /**< T0 1 Pulse Per Second Configuration Register */
    REG_PH_SLOPE_CNFG = 0x32, /**< Phase Slope limiting Register */
    REG_T0_ICP_CTRL_CNFG = 0x33, /**< T0 APLL Charge Pump Current Configuration Register */
    REG_T4_ICP_CTRL_CNFG = 0x34, /**< T4 APLL Charge Pump Current Configuration Register */
    REG_T4_PPS_CNFG = 0x3E, /**< T4 1 Pulse Per Second Configuration Register */
/*==================End of Register Definitions=======================*/

/*======================Bitfield Definitions==========================*/
    /**
     * ID_LOW
     */
    BF_ID_A_SIZE = 8,
    BF_ID_A_MASK = 0xFF,
    BF_ID_A_LSB = 0,

    /**
     * ID_HIGH
     */
    BF_ID_B_SIZE = 8,
    BF_ID_B_MASK = 0xFF,
    BF_ID_B_LSB = 0,

    /**
     * MPU_PIN_STS
     */
    BF_MPU_PIN_STS_SIZE = 1,
    BF_MPU_PIN_STS_MASK = 0x01,
    BF_MPU_PIN_STS_LSB = 0,

    /**
     * NOMINAL_FREQ_LOW_CNFG
     */
    BF_NOMINAL_FREQ_VALUE_A_SIZE = 8,
    BF_NOMINAL_FREQ_VALUE_A_MASK = 0xFF,
    BF_NOMINAL_FREQ_VALUE_A_LSB = 0,

    /**
     * NOMINAL_FREQ_MID_CNFG
     */
    BF_NOMINAL_FREQ_VALUE_B_SIZE = 8,
    BF_NOMINAL_FREQ_VALUE_B_MASK = 0xFF,
    BF_NOMINAL_FREQ_VALUE_B_LSB = 0,

    /**
     * NOMINAL_FREQ_HIGH_CNFG
     */
    BF_NOMINAL_FREQ_VALUE_C_SIZE = 8,
    BF_NOMINAL_FREQ_VALUE_C_MASK = 0xFF,
    BF_NOMINAL_FREQ_VALUE_C_LSB = 0,

    /**
     * T4_T0_SEL_CNFG
     */
    BF_T4_T0_SEL_SIZE = 1,
    BF_T4_T0_SEL_MASK = 0x10,
    BF_T4_T0_SEL_LSB = 4,

    /**
     * PHASE_ALARM_TIME_OUT_CNFG
     */
    BF_MULTI_FACTOR_SIZE = 2,
    BF_MULTI_FACTOR_MASK = 0xC0,
    BF_MULTI_FACTOR_LSB = 6,

    /**
     * PHASE_ALARM_TIME_OUT_CNFG
     */
    BF_TIMEOUT_VALUE_SIZE = 6,
    BF_TIMEOUT_VALUE_MASK = 0x3F,
    BF_TIMEOUT_VALUE_LSB = 0,

    /**
     * INPUT_MODE_CNFG
     */
    BF_AUTO_EXT_SYNC_EN_SIZE = 1,
    BF_AUTO_EXT_SYNC_EN_MASK = 0x80,
    BF_AUTO_EXT_SYNC_EN_LSB = 7,

    /**
     * INPUT_MODE_CNFG
     */
    BF_EXT_SYNC_EN_SIZE = 1,
    BF_EXT_SYNC_EN_MASK = 0x40,
    BF_EXT_SYNC_EN_LSB = 6,

    /**
     * INPUT_MODE_CNFG
     */
    BF_PH_ALARM_TIMEOUT_SIZE = 1,
    BF_PH_ALARM_TIMEOUT_MASK = 0x20,
    BF_PH_ALARM_TIMEOUT_LSB = 5,

    /**
     * INPUT_MODE_CNFG
     */
    BF_SYNC_FREQ_SIZE = 2,
    BF_SYNC_FREQ_MASK = 0x18,
    BF_SYNC_FREQ_LSB = 3,

    /**
     * INPUT_MODE_CNFG
     */
    BF_IN_SONET_SDH_SIZE = 1,
    BF_IN_SONET_SDH_MASK = 0x04,
    BF_IN_SONET_SDH_LSB = 2,

    /**
     * INPUT_MODE_CNFG
     */
    BF_MASTER_SLAVE_SIZE = 1,
    BF_MASTER_SLAVE_MASK = 0x02,
    BF_MASTER_SLAVE_LSB = 1,

    /**
     * INPUT_MODE_CNFG
     */
    BF_REVERTIVE_MODE_SIZE = 1,
    BF_REVERTIVE_MODE_MASK = 0x01,
    BF_REVERTIVE_MODE_LSB = 0,

    /**
     * DIFFERENTIAL_IN_OUT_CNFG
     */
    BF_OSC_EDGE_SIZE = 1,
    BF_OSC_EDGE_MASK = 0x04,
    BF_OSC_EDGE_LSB = 2,

    /**
     * DIFFERENTIAL_IN_OUT_CNFG
     */
    BF_OUT7_PECL_LVDS_SIZE = 1,
    BF_OUT7_PECL_LVDS_MASK = 0x02,
    BF_OUT7_PECL_LVDS_LSB = 1,

    /**
     * DIFFERENTIAL_IN_OUT_CNFG
     */
    BF_OUT6_PECL_LVDS_SIZE = 1,
    BF_OUT6_PECL_LVDS_MASK = 0x01,
    BF_OUT6_PECL_LVDS_LSB = 0,

    /**
     * MON_SW_HS_CNFG
     */
    BF_FREQ_MON_CLK_SIZE = 1,
    BF_FREQ_MON_CLK_MASK = 0x80,
    BF_FREQ_MON_CLK_LSB = 7,

    /**
     * MON_SW_HS_CNFG
     */
    BF_LOS_FLAG_TO_TDO_SIZE = 1,
    BF_LOS_FLAG_TO_TDO_MASK = 0x40,
    BF_LOS_FLAG_TO_TDO_LSB = 6,

    /**
     * MON_SW_HS_CNFG
     */
    BF_ULTR_FAST_SW_SIZE = 1,
    BF_ULTR_FAST_SW_MASK = 0x20,
    BF_ULTR_FAST_SW_LSB = 5,

    /**
     * MON_SW_HS_CNFG
     */
    BF_EXT_SW_SIZE = 1,
    BF_EXT_SW_MASK = 0x10,
    BF_EXT_SW_LSB = 4,

    /**
     * MON_SW_HS_CNFG
     */
    BF_HS_FREZ_SIZE = 1,
    BF_HS_FREZ_MASK = 0x08,
    BF_HS_FREZ_LSB = 3,

    /**
     * MON_SW_HS_CNFG
     */
    BF_HS_EN_SIZE = 1,
    BF_HS_EN_MASK = 0x04,
    BF_HS_EN_LSB = 2,

    /**
     * MON_SW_HS_CNFG
     */
    BF_FREQ_MON_HARD_EN_SIZE = 1,
    BF_FREQ_MON_HARD_EN_MASK = 0x01,
    BF_FREQ_MON_HARD_EN_LSB = 0,

    /**
     * INTERRUPT_CNFG
     */
    BF_HZ_EN_SIZE = 1,
    BF_HZ_EN_MASK = 0x02,
    BF_HZ_EN_LSB = 1,

    /**
     * INTERRUPT_CNFG
     */
    BF_INT_POL_SIZE = 1,
    BF_INT_POL_MASK = 0x01,
    BF_INT_POL_LSB = 0,

    /**
     * INTERRUPTS1_STS
     * INTERRUPTS1_MASK_CNFG
     * INPUT_VALID1_STS
     * REMOTE_INPUT_VALID1_CNFG
     */
    BF_IN8_SIZE = 1,
    BF_IN8_MASK = 0x80,
    BF_IN8_LSB = 7,

    /**
     * INTERRUPTS1_STS
     * INTERRUPTS1_MASK_CNFG
     * INPUT_VALID1_STS
     * REMOTE_INPUT_VALID1_CNFG
     */
    BF_IN7_SIZE = 1,
    BF_IN7_MASK = 0x40,
    BF_IN7_LSB = 6,

    /**
     * INTERRUPTS1_STS
     * INTERRUPTS1_MASK_CNFG
     * INPUT_VALID1_STS
     * REMOTE_INPUT_VALID1_CNFG
     */
    BF_IN6_SIZE = 1,
    BF_IN6_MASK = 0x20,
    BF_IN6_LSB = 5,

    /**
     * INTERRUPTS1_STS
     * INTERRUPTS1_MASK_CNFG
     * INPUT_VALID1_STS
     * REMOTE_INPUT_VALID1_CNFG
     */
    BF_IN5_SIZE = 1,
    BF_IN5_MASK = 0x10,
    BF_IN5_LSB = 4,

    /**
     * INTERRUPTS1_STS
     * INTERRUPTS1_MASK_CNFG
     * INPUT_VALID1_STS
     * REMOTE_INPUT_VALID1_CNFG
     */
    BF_IN4_SIZE = 1,
    BF_IN4_MASK = 0x08,
    BF_IN4_LSB = 3,

    /**
     * INTERRUPTS1_STS
     * INTERRUPTS1_MASK_CNFG
     * INPUT_VALID1_STS
     * REMOTE_INPUT_VALID1_CNFG
     */
    BF_IN3_SIZE = 1,
    BF_IN3_MASK = 0x04,
    BF_IN3_LSB = 2,

    /**
     * INTERRUPTS1_STS
     * INTERRUPTS1_MASK_CNFG
     * INPUT_VALID1_STS
     * REMOTE_INPUT_VALID1_CNFG
     */
    BF_IN2_SIZE = 1,
    BF_IN2_MASK = 0x02,
    BF_IN2_LSB = 1,

    /**
     * INTERRUPTS1_STS
     * INTERRUPTS1_MASK_CNFG
     * INPUT_VALID1_STS
     * REMOTE_INPUT_VALID1_CNFG
     */
    BF_IN1_SIZE = 1,
    BF_IN1_MASK = 0x01,
    BF_IN1_LSB = 0,

    /**
     * INTERRUPTS2_STS
     * INTERRUPTS2_MASK_CNFG
     */
    BF_T0_OPERATING_MODE_STS_SIZE = 1,
    BF_T0_OPERATING_MODE_STS_MASK = 0x80,
    BF_T0_OPERATING_MODE_STS_LSB = 7,

    /**
     * INTERRUPTS2_STS
     * INTERRUPTS2_MASK_CNFG
     */
    BF_T0_MAIN_REF_FAIL_SIZE = 1,
    BF_T0_MAIN_REF_FAIL_MASK = 0x40,
    BF_T0_MAIN_REF_FAIL_LSB = 6,

    /**
     * INTERRUPTS2_STS
     * INTERRUPTS2_MASK_CNFG
     * INPUT_VALID2_STS
     * REMOTE_INPUT_VALID2_CNFG
     */
    BF_IN14_SIZE = 1,
    BF_IN14_MASK = 0x20,
    BF_IN14_LSB = 5,

    /**
     * INTERRUPTS2_STS
     * INTERRUPTS2_MASK_CNFG
     * INPUT_VALID2_STS
     * REMOTE_INPUT_VALID2_CNFG
     */
    BF_IN13_SIZE = 1,
    BF_IN13_MASK = 0x10,
    BF_IN13_LSB = 4,

    /**
     * INTERRUPTS2_STS
     * INTERRUPTS2_MASK_CNFG
     * INPUT_VALID2_STS
     * REMOTE_INPUT_VALID2_CNFG
     */
    BF_IN12_SIZE = 1,
    BF_IN12_MASK = 0x08,
    BF_IN12_LSB = 3,

    /**
     * INTERRUPTS2_STS
     * INTERRUPTS2_MASK_CNFG
     * INPUT_VALID2_STS
     * REMOTE_INPUT_VALID2_CNFG
     */
    BF_IN11_SIZE = 1,
    BF_IN11_MASK = 0x04,
    BF_IN11_LSB = 2,

    /**
     * INTERRUPTS2_STS
     * INTERRUPTS2_MASK_CNFG
     * INPUT_VALID2_STS
     * REMOTE_INPUT_VALID2_CNFG
     */
    BF_IN10_SIZE = 1,
    BF_IN10_MASK = 0x02,
    BF_IN10_LSB = 1,

    /**
     * INTERRUPTS2_STS
     * INTERRUPTS2_MASK_CNFG
     * INPUT_VALID2_STS
     * REMOTE_INPUT_VALID2_CNFG
     */
    BF_IN9_SIZE = 1,
    BF_IN9_MASK = 0x01,
    BF_IN9_LSB = 0,

    /**
     * INTERRUPTS3_STS
     * INTERRUPTS3_MASK_CNFG
     */
    BF_EX_SYNC_ALARM_SIZE = 1,
    BF_EX_SYNC_ALARM_MASK = 0x80,
    BF_EX_SYNC_ALARM_LSB = 7,

    /**
     * INTERRUPTS3_STS
     * INTERRUPTS3_MASK_CNFG
     */
    BF_T4_STS_SIZE = 1,
    BF_T4_STS_MASK = 0x40,
    BF_T4_STS_LSB = 6,

    /**
     * INTERRUPTS3_STS
     * INTERRUPTS3_MASK_CNFG
     */
    BF_INPUT_TO_T4_SIZE = 1,
    BF_INPUT_TO_T4_MASK = 0x10,
    BF_INPUT_TO_T4_LSB = 4,

    /**
     * INTERRUPTS3_STS
     * INTERRUPTS3_MASK_CNFG
     */
    BF_AMI2_VIOL_SIZE = 1,
    BF_AMI2_VIOL_MASK = 0x08,
    BF_AMI2_VIOL_LSB = 3,

    /**
     * INTERRUPTS3_STS
     * INTERRUPTS3_MASK_CNFG
     */
    BF_AMI2_LOS_SIZE = 1,
    BF_AMI2_LOS_MASK = 0x04,
    BF_AMI2_LOS_LSB = 2,

    /**
     * INTERRUPTS3_STS
     * INTERRUPTS3_MASK_CNFG
     */
    BF_AMI1_VIOL_SIZE = 1,
    BF_AMI1_VIOL_MASK = 0x02,
    BF_AMI1_VIOL_LSB = 1,

    /**
     * INTERRUPTS3_STS
     * INTERRUPTS3_MASK_CNFG
     */
    BF_AMI1_LOS_SIZE = 1,
    BF_AMI1_LOS_MASK = 0x01,
    BF_AMI1_LOS_LSB = 0,

    /**
     * MS_SL_CTRL_CNFG
     */
    BF_PPS_FAST_FREQ_LOCK_EN_SIZE = 1,
    BF_PPS_FAST_FREQ_LOCK_EN_MASK = 0x10,
    BF_PPS_FAST_FREQ_LOCK_EN_LSB = 4,

    /**
     * MS_SL_CTRL_CNFG
     */
    BF_PPS_FAST_PH_LOCK_EN_SIZE = 1,
    BF_PPS_FAST_PH_LOCK_EN_MASK = 0x08,
    BF_PPS_FAST_PH_LOCK_EN_LSB = 3,

    /**
     * MS_SL_CTRL_CNFG
     */
    BF_MS_SL_CTRL_SIZE = 1,
    BF_MS_SL_CTRL_MASK = 0x01,
    BF_MS_SL_CTRL_LSB = 0,

    /**
     * IN1_CNFG
     * IN2_CNFG
     */
    BF_400HZ_SEL_SIZE = 1,
    BF_400HZ_SEL_MASK = 0x40,
    BF_400HZ_SEL_LSB = 6,

    /**
     * IN1_CNFG
     * IN2_CNFG
     * IN3_CNFG
     * IN4_CNFG
     * IN5_CNFG
     * IN6_CNFG
     * IN7_CNFG
     * IN8_CNFG
     * IN9_CNFG
     * IN10_CNFG
     * IN11_CNFG
     * IN12_CNFG
     * IN13_CNFG
     * IN14_CNFG
     */
    BF_BUCKET_SEL_SIZE = 2,
    BF_BUCKET_SEL_MASK = 0x30,
    BF_BUCKET_SEL_LSB = 4,

    /**
     * IN1_CNFG
     * IN2_CNFG
     * IN3_CNFG
     * IN4_CNFG
     * IN5_CNFG
     * IN6_CNFG
     * IN7_CNFG
     * IN8_CNFG
     * IN9_CNFG
     * IN10_CNFG
     * IN11_CNFG
     * IN12_CNFG
     * IN13_CNFG
     * IN14_CNFG
     */
    BF_IN_FREQ_SIZE = 4,
    BF_IN_FREQ_MASK = 0x0F,
    BF_IN_FREQ_LSB = 0,

    /**
     * IN3_CNFG
     * IN4_CNFG
     * IN5_CNFG
     * IN6_CNFG
     * IN7_CNFG
     * IN8_CNFG
     * IN9_CNFG
     * IN10_CNFG
     * IN11_CNFG
     * IN12_CNFG
     * IN13_CNFG
     * IN14_CNFG
     */
    BF_DIRECT_DIV_SIZE = 1,
    BF_DIRECT_DIV_MASK = 0x80,
    BF_DIRECT_DIV_LSB = 7,

    /**
     * IN3_CNFG
     * IN4_CNFG
     * IN5_CNFG
     * IN6_CNFG
     * IN7_CNFG
     * IN8_CNFG
     * IN9_CNFG
     * IN10_CNFG
     * IN11_CNFG
     * IN12_CNFG
     * IN13_CNFG
     * IN14_CNFG
     */
    BF_LOCK_8K_SIZE = 1,
    BF_LOCK_8K_MASK = 0x40,
    BF_LOCK_8K_LSB = 6,

    /**
     * IN5_IN6_HF_DIV_CNFG
     */
    BF_IN6_DIV_SIZE = 2,
    BF_IN6_DIV_MASK = 0xC0,
    BF_IN6_DIV_LSB = 6,

    /**
     * IN5_IN6_HF_DIV_CNFG
     */
    BF_IN5_DIV_SIZE = 2,
    BF_IN5_DIV_MASK = 0x03,
    BF_IN5_DIV_LSB = 0,

    /**
     * PRE_DIV_CH_CNFG
     */
    BF_PRE_DIV_CH_VALUE_SIZE = 4,
    BF_PRE_DIV_CH_VALUE_MASK = 0x0F,
    BF_PRE_DIV_CH_VALUE_LSB = 0,

    /**
     * PRE_DIVN_LOW_CNFG
     */
    BF_PRE_DIVN_VALUE_A_SIZE = 8,
    BF_PRE_DIVN_VALUE_A_MASK = 0xFF,
    BF_PRE_DIVN_VALUE_A_LSB = 0,

    /**
     * PRE_DIVN_HIGH_CNFG
     */
    BF_PRE_DIVN_VALUE_B_SIZE = 7,
    BF_PRE_DIVN_VALUE_B_MASK = 0x7F,
    BF_PRE_DIVN_VALUE_B_LSB = 0,

    /**
     * IN1_IN2_SEL_PRIORITY_CNFG
     */
    BF_IN2_SEL_PRIORITY_SIZE = 4,
    BF_IN2_SEL_PRIORITY_MASK = 0xF0,
    BF_IN2_SEL_PRIORITY_LSB = 4,

    /**
     * IN1_IN2_SEL_PRIORITY_CNFG
     */
    BF_IN1_SEL_PRIORITY_SIZE = 4,
    BF_IN1_SEL_PRIORITY_MASK = 0x0F,
    BF_IN1_SEL_PRIORITY_LSB = 0,

    /**
     * IN3_IN4_SEL_PRIORITY_CNFG
     */
    BF_IN4_SEL_PRIORITY_SIZE = 4,
    BF_IN4_SEL_PRIORITY_MASK = 0xF0,
    BF_IN4_SEL_PRIORITY_LSB = 4,

    /**
     * IN3_IN4_SEL_PRIORITY_CNFG
     */
    BF_IN3_SEL_PRIORITY_SIZE = 4,
    BF_IN3_SEL_PRIORITY_MASK = 0x0F,
    BF_IN3_SEL_PRIORITY_LSB = 0,

    /**
     * IN5_IN6_SEL_PRIORITY_CNFG
     */
    BF_IN6_SEL_PRIORITY_SIZE = 4,
    BF_IN6_SEL_PRIORITY_MASK = 0xF0,
    BF_IN6_SEL_PRIORITY_LSB = 4,

    /**
     * IN5_IN6_SEL_PRIORITY_CNFG
     */
    BF_IN5_SEL_PRIORITY_SIZE = 4,
    BF_IN5_SEL_PRIORITY_MASK = 0x0F,
    BF_IN5_SEL_PRIORITY_LSB = 0,

    /**
     * IN7_IN8_SEL_PRIORITY_CNFG
     */
    BF_IN8_SEL_PRIORITY_SIZE = 4,
    BF_IN8_SEL_PRIORITY_MASK = 0xF0,
    BF_IN8_SEL_PRIORITY_LSB = 4,

    /**
     * IN7_IN8_SEL_PRIORITY_CNFG
     */
    BF_IN7_SEL_PRIORITY_SIZE = 4,
    BF_IN7_SEL_PRIORITY_MASK = 0x0F,
    BF_IN7_SEL_PRIORITY_LSB = 0,

    /**
     * IN9_IN10_SEL_PRIORITY_CNFG
     */
    BF_IN10_SEL_PRIORITY_SIZE = 4,
    BF_IN10_SEL_PRIORITY_MASK = 0xF0,
    BF_IN10_SEL_PRIORITY_LSB = 4,

    /**
     * IN9_IN10_SEL_PRIORITY_CNFG
     */
    BF_IN9_SEL_PRIORITY_SIZE = 4,
    BF_IN9_SEL_PRIORITY_MASK = 0x0F,
    BF_IN9_SEL_PRIORITY_LSB = 0,

    /**
     * IN11_IN12_SEL_PRIORITY_CNFG
     */
    BF_IN12_SEL_PRIORITY_SIZE = 4,
    BF_IN12_SEL_PRIORITY_MASK = 0xF0,
    BF_IN12_SEL_PRIORITY_LSB = 4,

    /**
     * IN11_IN12_SEL_PRIORITY_CNFG
     */
    BF_IN11_SEL_PRIORITY_SIZE = 4,
    BF_IN11_SEL_PRIORITY_MASK = 0x0F,
    BF_IN11_SEL_PRIORITY_LSB = 0,

    /**
     * IN13_IN14_SEL_PRIORITY_CNFG
     */
    BF_IN14_SEL_PRIORITY_SIZE = 4,
    BF_IN14_SEL_PRIORITY_MASK = 0xF0,
    BF_IN14_SEL_PRIORITY_LSB = 4,

    /**
     * IN13_IN14_SEL_PRIORITY_CNFG
     */
    BF_IN13_SEL_PRIORITY_SIZE = 4,
    BF_IN13_SEL_PRIORITY_MASK = 0x0F,
    BF_IN13_SEL_PRIORITY_LSB = 0,

    /**
     * PAGE_POINTER_CNFG
     */
    BF_PAGE_POINTER_SIZE = 1,
    BF_PAGE_POINTER_MASK = 0x01,
    BF_PAGE_POINTER_LSB = 0,

    /**
     * FREQ_MON_FACTOR_CNFG
     */
    BF_FREQ_MON_FACTOR_SIZE = 4,
    BF_FREQ_MON_FACTOR_MASK = 0x0F,
    BF_FREQ_MON_FACTOR_LSB = 0,

    /**
     * HARD_FREQ_MON_THRESHOLD_CNFG
     */
    BF_HARD_FREQ_MON_THRESHOLD_A_SIZE = 4,
    BF_HARD_FREQ_MON_THRESHOLD_A_MASK = 0xF0,
    BF_HARD_FREQ_MON_THRESHOLD_A_LSB = 4,

    /**
     * HARD_FREQ_MON_THRESHOLD_CNFG
     */
    BF_HARD_FREQ_MON_THRESHOLD_B_SIZE = 4,
    BF_HARD_FREQ_MON_THRESHOLD_B_MASK = 0x0F,
    BF_HARD_FREQ_MON_THRESHOLD_B_LSB = 0,

    /**
     * SOFT_FREQ_MON_THRESHOLD_CNFG
     */
    BF_SOFT_FREQ_MON_THRESHOLD_A_SIZE = 4,
    BF_SOFT_FREQ_MON_THRESHOLD_A_MASK = 0xF0,
    BF_SOFT_FREQ_MON_THRESHOLD_A_LSB = 4,

    /**
     * SOFT_FREQ_MON_THRESHOLD_CNFG
     */
    BF_SOFT_FREQ_MON_THRESHOLD_B_SIZE = 4,
    BF_SOFT_FREQ_MON_THRESHOLD_B_MASK = 0x0F,
    BF_SOFT_FREQ_MON_THRESHOLD_B_LSB = 0,

    /**
     * UPPER_THRESHOLD_0_CNFG
     * UPPER_THRESHOLD_1_CNFG
     * UPPER_THRESHOLD_2_CNFG
     * UPPER_THRESHOLD_3_CNFG
     */
    BF_UPPER_THRESHOLD_DATA_SIZE = 8,
    BF_UPPER_THRESHOLD_DATA_MASK = 0xFF,
    BF_UPPER_THRESHOLD_DATA_LSB = 0,

    /**
     * LOWER_THRESHOLD_0_CNFG
     * LOWER_THRESHOLD_1_CNFG
     * LOWER_THRESHOLD_2_CNFG
     * LOWER_THRESHOLD_3_CNFG
     */
    BF_LOWER_THRESHOLD_DATA_SIZE = 8,
    BF_LOWER_THRESHOLD_DATA_MASK = 0xFF,
    BF_LOWER_THRESHOLD_DATA_LSB = 0,

    /**
     * BUCKET_SIZE_0_CNFG
     * BUCKET_SIZE_1_CNFG
     * BUCKET_SIZE_2_CNFG
     * BUCKET_SIZE_3_CNFG
     */
    BF_BUCKET_SIZE_DATA_SIZE = 8,
    BF_BUCKET_SIZE_DATA_MASK = 0xFF,
    BF_BUCKET_SIZE_DATA_LSB = 0,

    /**
     * DECAY_RATE_0_CNFG
     * DECAY_RATE_1_CNFG
     * DECAY_RATE_2_CNFG
     * DECAY_RATE_3_CNFG
     */
    BF_DECAY_RATE_DATA_SIZE = 2,
    BF_DECAY_RATE_DATA_MASK = 0x03,
    BF_DECAY_RATE_DATA_LSB = 0,

    /**
     * IN_FREQ_READ_CH_CNFG
     */
    BF_IN_FREQ_READ_CH_SIZE = 4,
    BF_IN_FREQ_READ_CH_MASK = 0x0F,
    BF_IN_FREQ_READ_CH_LSB = 0,

    /**
     * IN_FREQ_READ_STS
     */
    BF_IN_FREQ_VALUE_SIZE = 8,
    BF_IN_FREQ_VALUE_MASK = 0xFF,
    BF_IN_FREQ_VALUE_LSB = 0,

    /**
     * IN1_IN2_STS
     */
    BF_IN2_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN2_FREQ_SOFT_ALARM_MASK = 0x80,
    BF_IN2_FREQ_SOFT_ALARM_LSB = 7,

    /**
     * IN1_IN2_STS
     */
    BF_IN2_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN2_FREQ_HARD_ALARM_MASK = 0x40,
    BF_IN2_FREQ_HARD_ALARM_LSB = 6,

    /**
     * IN1_IN2_STS
     */
    BF_IN2_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN2_NO_ACTIVITY_ALARM_MASK = 0x20,
    BF_IN2_NO_ACTIVITY_ALARM_LSB = 5,

    /**
     * IN1_IN2_STS
     */
    BF_IN2_PH_LOCK_ALARM_SIZE = 1,
    BF_IN2_PH_LOCK_ALARM_MASK = 0x10,
    BF_IN2_PH_LOCK_ALARM_LSB = 4,

    /**
     * IN1_IN2_STS
     */
    BF_IN1_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN1_FREQ_SOFT_ALARM_MASK = 0x08,
    BF_IN1_FREQ_SOFT_ALARM_LSB = 3,

    /**
     * IN1_IN2_STS
     */
    BF_IN1_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN1_FREQ_HARD_ALARM_MASK = 0x04,
    BF_IN1_FREQ_HARD_ALARM_LSB = 2,

    /**
     * IN1_IN2_STS
     */
    BF_IN1_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN1_NO_ACTIVITY_ALARM_MASK = 0x02,
    BF_IN1_NO_ACTIVITY_ALARM_LSB = 1,

    /**
     * IN1_IN2_STS
     */
    BF_IN1_PH_LOCK_ALARM_SIZE = 1,
    BF_IN1_PH_LOCK_ALARM_MASK = 0x01,
    BF_IN1_PH_LOCK_ALARM_LSB = 0,

    /**
     * IN3_IN4_STS
     */
    BF_IN4_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN4_FREQ_SOFT_ALARM_MASK = 0x80,
    BF_IN4_FREQ_SOFT_ALARM_LSB = 7,

    /**
     * IN3_IN4_STS
     */
    BF_IN4_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN4_FREQ_HARD_ALARM_MASK = 0x40,
    BF_IN4_FREQ_HARD_ALARM_LSB = 6,

    /**
     * IN3_IN4_STS
     */
    BF_IN4_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN4_NO_ACTIVITY_ALARM_MASK = 0x20,
    BF_IN4_NO_ACTIVITY_ALARM_LSB = 5,

    /**
     * IN3_IN4_STS
     */
    BF_IN4_PH_LOCK_ALARM_SIZE = 1,
    BF_IN4_PH_LOCK_ALARM_MASK = 0x10,
    BF_IN4_PH_LOCK_ALARM_LSB = 4,

    /**
     * IN3_IN4_STS
     */
    BF_IN3_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN3_FREQ_SOFT_ALARM_MASK = 0x08,
    BF_IN3_FREQ_SOFT_ALARM_LSB = 3,

    /**
     * IN3_IN4_STS
     */
    BF_IN3_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN3_FREQ_HARD_ALARM_MASK = 0x04,
    BF_IN3_FREQ_HARD_ALARM_LSB = 2,

    /**
     * IN3_IN4_STS
     */
    BF_IN3_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN3_NO_ACTIVITY_ALARM_MASK = 0x02,
    BF_IN3_NO_ACTIVITY_ALARM_LSB = 1,

    /**
     * IN3_IN4_STS
     */
    BF_IN3_PH_LOCK_ALARM_SIZE = 1,
    BF_IN3_PH_LOCK_ALARM_MASK = 0x01,
    BF_IN3_PH_LOCK_ALARM_LSB = 0,

    /**
     * IN5_IN6_STS
     */
    BF_IN6_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN6_FREQ_SOFT_ALARM_MASK = 0x80,
    BF_IN6_FREQ_SOFT_ALARM_LSB = 7,

    /**
     * IN5_IN6_STS
     */
    BF_IN6_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN6_FREQ_HARD_ALARM_MASK = 0x40,
    BF_IN6_FREQ_HARD_ALARM_LSB = 6,

    /**
     * IN5_IN6_STS
     */
    BF_IN6_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN6_NO_ACTIVITY_ALARM_MASK = 0x20,
    BF_IN6_NO_ACTIVITY_ALARM_LSB = 5,

    /**
     * IN5_IN6_STS
     */
    BF_IN6_PH_LOCK_ALARM_SIZE = 1,
    BF_IN6_PH_LOCK_ALARM_MASK = 0x10,
    BF_IN6_PH_LOCK_ALARM_LSB = 4,

    /**
     * IN5_IN6_STS
     */
    BF_IN5_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN5_FREQ_SOFT_ALARM_MASK = 0x08,
    BF_IN5_FREQ_SOFT_ALARM_LSB = 3,

    /**
     * IN5_IN6_STS
     */
    BF_IN5_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN5_FREQ_HARD_ALARM_MASK = 0x04,
    BF_IN5_FREQ_HARD_ALARM_LSB = 2,

    /**
     * IN5_IN6_STS
     */
    BF_IN5_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN5_NO_ACTIVITY_ALARM_MASK = 0x02,
    BF_IN5_NO_ACTIVITY_ALARM_LSB = 1,

    /**
     * IN5_IN6_STS
     */
    BF_IN5_PH_LOCK_ALARM_SIZE = 1,
    BF_IN5_PH_LOCK_ALARM_MASK = 0x01,
    BF_IN5_PH_LOCK_ALARM_LSB = 0,

    /**
     * IN7_IN8_STS
     */
    BF_IN8_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN8_FREQ_SOFT_ALARM_MASK = 0x80,
    BF_IN8_FREQ_SOFT_ALARM_LSB = 7,

    /**
     * IN7_IN8_STS
     */
    BF_IN8_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN8_FREQ_HARD_ALARM_MASK = 0x40,
    BF_IN8_FREQ_HARD_ALARM_LSB = 6,

    /**
     * IN7_IN8_STS
     */
    BF_IN8_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN8_NO_ACTIVITY_ALARM_MASK = 0x20,
    BF_IN8_NO_ACTIVITY_ALARM_LSB = 5,

    /**
     * IN7_IN8_STS
     */
    BF_IN8_PH_LOCK_ALARM_SIZE = 1,
    BF_IN8_PH_LOCK_ALARM_MASK = 0x10,
    BF_IN8_PH_LOCK_ALARM_LSB = 4,

    /**
     * IN7_IN8_STS
     */
    BF_IN7_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN7_FREQ_SOFT_ALARM_MASK = 0x08,
    BF_IN7_FREQ_SOFT_ALARM_LSB = 3,

    /**
     * IN7_IN8_STS
     */
    BF_IN7_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN7_FREQ_HARD_ALARM_MASK = 0x04,
    BF_IN7_FREQ_HARD_ALARM_LSB = 2,

    /**
     * IN7_IN8_STS
     */
    BF_IN7_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN7_NO_ACTIVITY_ALARM_MASK = 0x02,
    BF_IN7_NO_ACTIVITY_ALARM_LSB = 1,

    /**
     * IN7_IN8_STS
     */
    BF_IN7_PH_LOCK_ALARM_SIZE = 1,
    BF_IN7_PH_LOCK_ALARM_MASK = 0x01,
    BF_IN7_PH_LOCK_ALARM_LSB = 0,

    /**
     * IN9_IN10_STS
     */
    BF_IN10_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN10_FREQ_SOFT_ALARM_MASK = 0x80,
    BF_IN10_FREQ_SOFT_ALARM_LSB = 7,

    /**
     * IN9_IN10_STS
     */
    BF_IN10_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN10_FREQ_HARD_ALARM_MASK = 0x40,
    BF_IN10_FREQ_HARD_ALARM_LSB = 6,

    /**
     * IN9_IN10_STS
     */
    BF_IN10_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN10_NO_ACTIVITY_ALARM_MASK = 0x20,
    BF_IN10_NO_ACTIVITY_ALARM_LSB = 5,

    /**
     * IN9_IN10_STS
     */
    BF_IN10_PH_LOCK_ALARM_SIZE = 1,
    BF_IN10_PH_LOCK_ALARM_MASK = 0x10,
    BF_IN10_PH_LOCK_ALARM_LSB = 4,

    /**
     * IN9_IN10_STS
     */
    BF_IN9_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN9_FREQ_SOFT_ALARM_MASK = 0x08,
    BF_IN9_FREQ_SOFT_ALARM_LSB = 3,

    /**
     * IN9_IN10_STS
     */
    BF_IN9_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN9_FREQ_HARD_ALARM_MASK = 0x04,
    BF_IN9_FREQ_HARD_ALARM_LSB = 2,

    /**
     * IN9_IN10_STS
     */
    BF_IN9_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN9_NO_ACTIVITY_ALARM_MASK = 0x02,
    BF_IN9_NO_ACTIVITY_ALARM_LSB = 1,

    /**
     * IN9_IN10_STS
     */
    BF_IN9_PH_LOCK_ALARM_SIZE = 1,
    BF_IN9_PH_LOCK_ALARM_MASK = 0x01,
    BF_IN9_PH_LOCK_ALARM_LSB = 0,

    /**
     * IN11_IN12_STS
     */
    BF_IN12_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN12_FREQ_SOFT_ALARM_MASK = 0x80,
    BF_IN12_FREQ_SOFT_ALARM_LSB = 7,

    /**
     * IN11_IN12_STS
     */
    BF_IN12_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN12_FREQ_HARD_ALARM_MASK = 0x40,
    BF_IN12_FREQ_HARD_ALARM_LSB = 6,

    /**
     * IN11_IN12_STS
     */
    BF_IN12_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN12_NO_ACTIVITY_ALARM_MASK = 0x20,
    BF_IN12_NO_ACTIVITY_ALARM_LSB = 5,

    /**
     * IN11_IN12_STS
     */
    BF_IN12_PH_LOCK_ALARM_SIZE = 1,
    BF_IN12_PH_LOCK_ALARM_MASK = 0x10,
    BF_IN12_PH_LOCK_ALARM_LSB = 4,

    /**
     * IN11_IN12_STS
     */
    BF_IN11_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN11_FREQ_SOFT_ALARM_MASK = 0x08,
    BF_IN11_FREQ_SOFT_ALARM_LSB = 3,

    /**
     * IN11_IN12_STS
     */
    BF_IN11_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN11_FREQ_HARD_ALARM_MASK = 0x04,
    BF_IN11_FREQ_HARD_ALARM_LSB = 2,

    /**
     * IN11_IN12_STS
     */
    BF_IN11_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN11_NO_ACTIVITY_ALARM_MASK = 0x02,
    BF_IN11_NO_ACTIVITY_ALARM_LSB = 1,

    /**
     * IN11_IN12_STS
     */
    BF_IN11_PH_LOCK_ALARM_SIZE = 1,
    BF_IN11_PH_LOCK_ALARM_MASK = 0x01,
    BF_IN11_PH_LOCK_ALARM_LSB = 0,

    /**
     * IN13_IN14_STS
     */
    BF_IN14_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN14_FREQ_SOFT_ALARM_MASK = 0x80,
    BF_IN14_FREQ_SOFT_ALARM_LSB = 7,

    /**
     * IN13_IN14_STS
     */
    BF_IN14_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN14_FREQ_HARD_ALARM_MASK = 0x40,
    BF_IN14_FREQ_HARD_ALARM_LSB = 6,

    /**
     * IN13_IN14_STS
     */
    BF_IN14_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN14_NO_ACTIVITY_ALARM_MASK = 0x20,
    BF_IN14_NO_ACTIVITY_ALARM_LSB = 5,

    /**
     * IN13_IN14_STS
     */
    BF_IN14_PH_LOCK_ALARM_SIZE = 1,
    BF_IN14_PH_LOCK_ALARM_MASK = 0x10,
    BF_IN14_PH_LOCK_ALARM_LSB = 4,

    /**
     * IN13_IN14_STS
     */
    BF_IN13_FREQ_SOFT_ALARM_SIZE = 1,
    BF_IN13_FREQ_SOFT_ALARM_MASK = 0x08,
    BF_IN13_FREQ_SOFT_ALARM_LSB = 3,

    /**
     * IN13_IN14_STS
     */
    BF_IN13_FREQ_HARD_ALARM_SIZE = 1,
    BF_IN13_FREQ_HARD_ALARM_MASK = 0x04,
    BF_IN13_FREQ_HARD_ALARM_LSB = 2,

    /**
     * IN13_IN14_STS
     */
    BF_IN13_NO_ACTIVITY_ALARM_SIZE = 1,
    BF_IN13_NO_ACTIVITY_ALARM_MASK = 0x02,
    BF_IN13_NO_ACTIVITY_ALARM_LSB = 1,

    /**
     * IN13_IN14_STS
     */
    BF_IN13_PH_LOCK_ALARM_SIZE = 1,
    BF_IN13_PH_LOCK_ALARM_MASK = 0x01,
    BF_IN13_PH_LOCK_ALARM_LSB = 0,

    /**
     * PRIORITY_TABLE1_STS
     */
    BF_HIGHEST_PRIORITY_VALIDATED_SIZE = 4,
    BF_HIGHEST_PRIORITY_VALIDATED_MASK = 0xF0,
    BF_HIGHEST_PRIORITY_VALIDATED_LSB = 4,

    /**
     * PRIORITY_TABLE1_STS
     */
    BF_CURRENTLY_SELECTED_INPUTS_SIZE = 4,
    BF_CURRENTLY_SELECTED_INPUTS_MASK = 0x0F,
    BF_CURRENTLY_SELECTED_INPUTS_LSB = 0,

    /**
     * PRIORITY_TABLE2_STS
     */
    BF_THIRD_HIGHEST_PRIORITY_VALIDATED_SIZE = 4,
    BF_THIRD_HIGHEST_PRIORITY_VALIDATED_MASK = 0xF0,
    BF_THIRD_HIGHEST_PRIORITY_VALIDATED_LSB = 4,

    /**
     * PRIORITY_TABLE2_STS
     */
    BF_SECOND_HIGHEST_PRIORITY_VALIDATED_SIZE = 4,
    BF_SECOND_HIGHEST_PRIORITY_VALIDATED_MASK = 0x0F,
    BF_SECOND_HIGHEST_PRIORITY_VALIDATED_LSB = 0,

    /**
     * T0_INPUT_SEL_CNFG
     */
    BF_T0_INPUT_SEL_SIZE = 4,
    BF_T0_INPUT_SEL_MASK = 0x0F,
    BF_T0_INPUT_SEL_LSB = 0,

    /**
     * T4_INPUT_SEL_CNFG
     */
    BF_T4_LOCK_T0_SIZE = 1,
    BF_T4_LOCK_T0_MASK = 0x40,
    BF_T4_LOCK_T0_LSB = 6,

    /**
     * T4_INPUT_SEL_CNFG
     */
    BF_T0_FOR_T4_SIZE = 1,
    BF_T0_FOR_T4_MASK = 0x20,
    BF_T0_FOR_T4_LSB = 5,

    /**
     * T4_INPUT_SEL_CNFG
     */
    BF_T4_TEST_T0_PH_SIZE = 1,
    BF_T4_TEST_T0_PH_MASK = 0x10,
    BF_T4_TEST_T0_PH_LSB = 4,

    /**
     * T4_INPUT_SEL_CNFG
     */
    BF_T4_INPUT_SEL_SIZE = 4,
    BF_T4_INPUT_SEL_MASK = 0x0F,
    BF_T4_INPUT_SEL_LSB = 0,

    /**
     * OPERATING_STS
     */
    BF_EX_SYNC_ALARM_MON_SIZE = 1,
    BF_EX_SYNC_ALARM_MON_MASK = 0x80,
    BF_EX_SYNC_ALARM_MON_LSB = 7,

    /**
     * OPERATING_STS
     */
    BF_T4_DPLL_LOCK_SIZE = 1,
    BF_T4_DPLL_LOCK_MASK = 0x40,
    BF_T4_DPLL_LOCK_LSB = 6,

    /**
     * OPERATING_STS
     */
    BF_T0_DPLL_SOFT_FREQ_ALARM_SIZE = 1,
    BF_T0_DPLL_SOFT_FREQ_ALARM_MASK = 0x20,
    BF_T0_DPLL_SOFT_FREQ_ALARM_LSB = 5,

    /**
     * OPERATING_STS
     */
    BF_T4_DPLL_SOFT_FREQ_ALARM_SIZE = 1,
    BF_T4_DPLL_SOFT_FREQ_ALARM_MASK = 0x10,
    BF_T4_DPLL_SOFT_FREQ_ALARM_LSB = 4,

    /**
     * OPERATING_STS
     */
    BF_T0_DPLL_LOCK_SIZE = 1,
    BF_T0_DPLL_LOCK_MASK = 0x08,
    BF_T0_DPLL_LOCK_LSB = 3,

    /**
     * OPERATING_STS
     */
    BF_T0_DPLL_OPERATING_MODE_SIZE = 3,
    BF_T0_DPLL_OPERATING_MODE_MASK = 0x07,
    BF_T0_DPLL_OPERATING_MODE_LSB = 0,

    /**
     * T0_OPERATION_MODE_CNFG
     */
    BF_T0_WDCO_SIZE = 1,
    BF_T0_WDCO_MASK = 0x08,
    BF_T0_WDCO_LSB = 3,

    /**
     * T0_OPERATION_MODE_CNFG
     */
    BF_T0_OPERATING_MODE_SIZE = 3,
    BF_T0_OPERATING_MODE_MASK = 0x07,
    BF_T0_OPERATING_MODE_LSB = 0,

    /**
     * T4_OPERATION_MODE_CNFG
     */
    BF_T4_WDCO_SIZE = 1,
    BF_T4_WDCO_MASK = 0x08,
    BF_T4_WDCO_LSB = 3,

    /**
     * T4_OPERATION_MODE_CNFG
     */
    BF_T4_OPERATING_MODE_SIZE = 3,
    BF_T4_OPERATING_MODE_MASK = 0x07,
    BF_T4_OPERATING_MODE_LSB = 0,

    /**
     * T0_DPLL_APLL_PATH_CNFG
     */
    BF_T0_APLL_PATH_SIZE = 4,
    BF_T0_APLL_PATH_MASK = 0xF0,
    BF_T0_APLL_PATH_LSB = 4,

    /**
     * T0_DPLL_APLL_PATH_CNFG
     */
    BF_T0_GSM_OBSAI_16E1_16T1_SEL_SIZE = 2,
    BF_T0_GSM_OBSAI_16E1_16T1_SEL_MASK = 0x0C,
    BF_T0_GSM_OBSAI_16E1_16T1_SEL_LSB = 2,

    /**
     * T0_DPLL_APLL_PATH_CNFG
     */
    BF_T0_12E1_GPS_E3_T3_SEL_SIZE = 2,
    BF_T0_12E1_GPS_E3_T3_SEL_MASK = 0x03,
    BF_T0_12E1_GPS_E3_T3_SEL_LSB = 0,

    /**
     * T0_DPLL_START_BW_DAMPING_CNFG
     */
    BF_T0_DPLL_START_DAMPING_SIZE = 3,
    BF_T0_DPLL_START_DAMPING_MASK = 0xE0,
    BF_T0_DPLL_START_DAMPING_LSB = 5,

    /**
     * T0_DPLL_START_BW_DAMPING_CNFG
     */
    BF_T0_DPLL_START_BW_SIZE = 5,
    BF_T0_DPLL_START_BW_MASK = 0x1F,
    BF_T0_DPLL_START_BW_LSB = 0,

    /**
     * T0_DPLL_ACQ_BW_DAMPING_CNFG
     */
    BF_T0_DPLL_ACQ_DAMPING_SIZE = 3,
    BF_T0_DPLL_ACQ_DAMPING_MASK = 0xE0,
    BF_T0_DPLL_ACQ_DAMPING_LSB = 5,

    /**
     * T0_DPLL_ACQ_BW_DAMPING_CNFG
     */
    BF_T0_DPLL_ACQ_BW_SIZE = 5,
    BF_T0_DPLL_ACQ_BW_MASK = 0x1F,
    BF_T0_DPLL_ACQ_BW_LSB = 0,

    /**
     * T0_DPLL_LOCKED_BW_DAMPING_CNFG
     */
    BF_T0_DPLL_LOCKED_DAMPING_SIZE = 3,
    BF_T0_DPLL_LOCKED_DAMPING_MASK = 0xE0,
    BF_T0_DPLL_LOCKED_DAMPING_LSB = 5,

    /**
     * T0_DPLL_LOCKED_BW_DAMPING_CNFG
     */
    BF_T0_DPLL_LOCKED_BW_SIZE = 5,
    BF_T0_DPLL_LOCKED_BW_MASK = 0x1F,
    BF_T0_DPLL_LOCKED_BW_LSB = 0,

    /**
     * T0_BW_OVERSHOOT_CNFG
     */
    BF_AUTO_BW_SEL_SIZE = 1,
    BF_AUTO_BW_SEL_MASK = 0x80,
    BF_AUTO_BW_SEL_LSB = 7,

    /**
     * T0_BW_OVERSHOOT_CNFG
     */
    BF_T0_LIMIT_SIZE = 1,
    BF_T0_LIMIT_MASK = 0x08,
    BF_T0_LIMIT_LSB = 3,

    /**
     * PHASE_LOSS_COARSE_LIMIT_CNFG
     */
    BF_COARSE_PH_LOS_LIMT_EN_SIZE = 1,
    BF_COARSE_PH_LOS_LIMT_EN_MASK = 0x80,
    BF_COARSE_PH_LOS_LIMT_EN_LSB = 7,

    /**
     * PHASE_LOSS_COARSE_LIMIT_CNFG
     */
    BF_WIDE_EN_SIZE = 1,
    BF_WIDE_EN_MASK = 0x40,
    BF_WIDE_EN_LSB = 6,

    /**
     * PHASE_LOSS_COARSE_LIMIT_CNFG
     */
    BF_MULTI_PH_APP_SIZE = 1,
    BF_MULTI_PH_APP_MASK = 0x20,
    BF_MULTI_PH_APP_LSB = 5,

    /**
     * PHASE_LOSS_COARSE_LIMIT_CNFG
     */
    BF_MULTI_PH_8K_4K_2K_EN_SIZE = 1,
    BF_MULTI_PH_8K_4K_2K_EN_MASK = 0x10,
    BF_MULTI_PH_8K_4K_2K_EN_LSB = 4,

    /**
     * PHASE_LOSS_COARSE_LIMIT_CNFG
     */
    BF_PH_LOS_COARSE_LIMT_SIZE = 4,
    BF_PH_LOS_COARSE_LIMT_MASK = 0x0F,
    BF_PH_LOS_COARSE_LIMT_LSB = 0,

    /**
     * PHASE_LOSS_FINE_LIMIT_CNFG
     */
    BF_FINE_PH_LOS_LIMT_EN_SIZE = 1,
    BF_FINE_PH_LOS_LIMT_EN_MASK = 0x80,
    BF_FINE_PH_LOS_LIMT_EN_LSB = 7,

    /**
     * PHASE_LOSS_FINE_LIMIT_CNFG
     */
    BF_FAST_LOS_SW_SIZE = 1,
    BF_FAST_LOS_SW_MASK = 0x40,
    BF_FAST_LOS_SW_LSB = 6,

    /**
     * PHASE_LOSS_FINE_LIMIT_CNFG
     */
    BF_PH_LOS_FINE_LIMT_SIZE = 3,
    BF_PH_LOS_FINE_LIMT_MASK = 0x07,
    BF_PH_LOS_FINE_LIMT_LSB = 0,

    /**
     * T0_HOLDOVER_MODE_CNFG
     */
    BF_MAN_HOLDOVER_SIZE = 1,
    BF_MAN_HOLDOVER_MASK = 0x80,
    BF_MAN_HOLDOVER_LSB = 7,

    /**
     * T0_HOLDOVER_MODE_CNFG
     */
    BF_AUTO_AVG_SIZE = 1,
    BF_AUTO_AVG_MASK = 0x40,
    BF_AUTO_AVG_LSB = 6,

    /**
     * T0_HOLDOVER_MODE_CNFG
     */
    BF_FAST_AVG_SIZE = 1,
    BF_FAST_AVG_MASK = 0x20,
    BF_FAST_AVG_LSB = 5,

    /**
     * T0_HOLDOVER_MODE_CNFG
     */
    BF_READ_AVG_SIZE = 1,
    BF_READ_AVG_MASK = 0x10,
    BF_READ_AVG_LSB = 4,

    /**
     * T0_HOLDOVER_MODE_CNFG
     */
    BF_TEMP_HOLDOVER_MODE_SIZE = 2,
    BF_TEMP_HOLDOVER_MODE_MASK = 0x0C,
    BF_TEMP_HOLDOVER_MODE_LSB = 2,

    /**
     * HOLDOVER_FREQ_LOW_CNFG
     */
    BF_HOLDOVER_FREQ_A_SIZE = 8,
    BF_HOLDOVER_FREQ_A_MASK = 0xFF,
    BF_HOLDOVER_FREQ_A_LSB = 0,

    /**
     * HOLDOVER_FREQ_MID_CNFG
     */
    BF_HOLDOVER_FREQ_B_SIZE = 8,
    BF_HOLDOVER_FREQ_B_MASK = 0xFF,
    BF_HOLDOVER_FREQ_B_LSB = 0,

    /**
     * HOLDOVER_FREQ_HIGH_CNFG
     */
    BF_HOLDOVER_FREQ_C_SIZE = 8,
    BF_HOLDOVER_FREQ_C_MASK = 0xFF,
    BF_HOLDOVER_FREQ_C_LSB = 0,

    /**
     * T4_DPLL_APLL_PATH_CNFG
     */
    BF_T4_APLL_PATH_SIZE = 4,
    BF_T4_APLL_PATH_MASK = 0xF0,
    BF_T4_APLL_PATH_LSB = 4,

    /**
     * T4_DPLL_APLL_PATH_CNFG
     */
    BF_T4_GSM_OBSAI_16E1_16T1_SEL_SIZE = 2,
    BF_T4_GSM_OBSAI_16E1_16T1_SEL_MASK = 0x0C,
    BF_T4_GSM_OBSAI_16E1_16T1_SEL_LSB = 2,

    /**
     * T4_DPLL_APLL_PATH_CNFG
     */
    BF_T4_12E1_GPS_E3_T3_SEL_SIZE = 2,
    BF_T4_12E1_GPS_E3_T3_SEL_MASK = 0x03,
    BF_T4_12E1_GPS_E3_T3_SEL_LSB = 0,

    /**
     * T4_DPLL_LOCKED_BW_DAMPING_CNFG
     */
    BF_T4_DPLL_LOCKED_DAMPING_SIZE = 3,
    BF_T4_DPLL_LOCKED_DAMPING_MASK = 0xE0,
    BF_T4_DPLL_LOCKED_DAMPING_LSB = 5,

    /**
     * T4_DPLL_LOCKED_BW_DAMPING_CNFG
     */
    BF_T4_DPLL_LOCKED_BW_SIZE = 2,
    BF_T4_DPLL_LOCKED_BW_MASK = 0x03,
    BF_T4_DPLL_LOCKED_BW_LSB = 0,

    /**
     * CURRENT_FREQ_LOW_STS
     */
    BF_CURRENT_DPLL_FREQ_A_SIZE = 8,
    BF_CURRENT_DPLL_FREQ_A_MASK = 0xFF,
    BF_CURRENT_DPLL_FREQ_A_LSB = 0,

    /**
     * CURRENT_FREQ_MID_STS
     */
    BF_CURRENT_DPLL_FREQ_B_SIZE = 8,
    BF_CURRENT_DPLL_FREQ_B_MASK = 0xFF,
    BF_CURRENT_DPLL_FREQ_B_LSB = 0,

    /**
     * CURRENT_FREQ_HIGH_STS
     */
    BF_CURRENT_DPLL_FREQ_C_SIZE = 8,
    BF_CURRENT_DPLL_FREQ_C_MASK = 0xFF,
    BF_CURRENT_DPLL_FREQ_C_LSB = 0,

    /**
     * DPLL_FREQ_SOFT_LIMIT_CNFG
     */
    BF_FREQ_LIMT_PH_LOS_SIZE = 1,
    BF_FREQ_LIMT_PH_LOS_MASK = 0x80,
    BF_FREQ_LIMT_PH_LOS_LSB = 7,

    /**
     * DPLL_FREQ_SOFT_LIMIT_CNFG
     */
    BF_DPLL_FREQ_SOFT_LIMT_SIZE = 7,
    BF_DPLL_FREQ_SOFT_LIMT_MASK = 0x7F,
    BF_DPLL_FREQ_SOFT_LIMT_LSB = 0,

    /**
     * DPLL_FREQ_HARD_LIMIT_LOW_CNFG
     */
    BF_DPLL_FREQ_HARD_LIMT_A_SIZE = 8,
    BF_DPLL_FREQ_HARD_LIMT_A_MASK = 0xFF,
    BF_DPLL_FREQ_HARD_LIMT_A_LSB = 0,

    /**
     * DPLL_FREQ_HARD_LIMIT_MID_CNFG
     */
    BF_DPLL_FREQ_HARD_LIMT_B_SIZE = 8,
    BF_DPLL_FREQ_HARD_LIMT_B_MASK = 0xFF,
    BF_DPLL_FREQ_HARD_LIMT_B_LSB = 0,

    /**
     * CURRENT_DPLL_PHASE_LOW_STS
     */
    BF_CURRENT_PH_DATA_A_SIZE = 8,
    BF_CURRENT_PH_DATA_A_MASK = 0xFF,
    BF_CURRENT_PH_DATA_A_LSB = 0,

    /**
     * CURRENT_DPLL_PHASE_HIGH_STS
     */
    BF_CURRENT_PH_DATA_B_SIZE = 8,
    BF_CURRENT_PH_DATA_B_MASK = 0xFF,
    BF_CURRENT_PH_DATA_B_LSB = 0,

    /**
     * OUT1_FREQ_CNFG
     */
    BF_OUT1_PATH_SEL_SIZE = 4,
    BF_OUT1_PATH_SEL_MASK = 0xF0,
    BF_OUT1_PATH_SEL_LSB = 4,

    /**
     * OUT1_FREQ_CNFG
     */
    BF_OUT1_DIVIDER_SIZE = 4,
    BF_OUT1_DIVIDER_MASK = 0x0F,
    BF_OUT1_DIVIDER_LSB = 0,

    /**
     * OUT2_FREQ_CNFG
     */
    BF_OUT2_PATH_SEL_SIZE = 4,
    BF_OUT2_PATH_SEL_MASK = 0xF0,
    BF_OUT2_PATH_SEL_LSB = 4,

    /**
     * OUT2_FREQ_CNFG
     */
    BF_OUT2_DIVIDER_SIZE = 4,
    BF_OUT2_DIVIDER_MASK = 0x0F,
    BF_OUT2_DIVIDER_LSB = 0,

    /**
     * OUT3_FREQ_CNFG
     */
    BF_OUT3_PATH_SEL_SIZE = 4,
    BF_OUT3_PATH_SEL_MASK = 0xF0,
    BF_OUT3_PATH_SEL_LSB = 4,

    /**
     * OUT3_FREQ_CNFG
     */
    BF_OUT3_DIVIDER_SIZE = 4,
    BF_OUT3_DIVIDER_MASK = 0x0F,
    BF_OUT3_DIVIDER_LSB = 0,

    /**
     * OUT4_FREQ_CNFG
     */
    BF_OUT4_PATH_SEL_SIZE = 4,
    BF_OUT4_PATH_SEL_MASK = 0xF0,
    BF_OUT4_PATH_SEL_LSB = 4,

    /**
     * OUT4_FREQ_CNFG
     */
    BF_OUT4_DIVIDER_SIZE = 4,
    BF_OUT4_DIVIDER_MASK = 0x0F,
    BF_OUT4_DIVIDER_LSB = 0,

    /**
     * OUT5_FREQ_CNFG
     */
    BF_OUT5_PATH_SEL_SIZE = 4,
    BF_OUT5_PATH_SEL_MASK = 0xF0,
    BF_OUT5_PATH_SEL_LSB = 4,

    /**
     * OUT5_FREQ_CNFG
     */
    BF_OUT5_DIVIDER_SIZE = 4,
    BF_OUT5_DIVIDER_MASK = 0x0F,
    BF_OUT5_DIVIDER_LSB = 0,

    /**
     * OUT6_FREQ_CNFG
     */
    BF_OUT6_PATH_SEL_SIZE = 4,
    BF_OUT6_PATH_SEL_MASK = 0xF0,
    BF_OUT6_PATH_SEL_LSB = 4,

    /**
     * OUT6_FREQ_CNFG
     */
    BF_OUT6_DIVIDER_SIZE = 4,
    BF_OUT6_DIVIDER_MASK = 0x0F,
    BF_OUT6_DIVIDER_LSB = 0,

    /**
     * OUT7_FREQ_CNFG
     */
    BF_OUT7_PATH_SEL_SIZE = 4,
    BF_OUT7_PATH_SEL_MASK = 0xF0,
    BF_OUT7_PATH_SEL_LSB = 4,

    /**
     * OUT7_FREQ_CNFG
     */
    BF_OUT7_DIVIDER_SIZE = 4,
    BF_OUT7_DIVIDER_MASK = 0x0F,
    BF_OUT7_DIVIDER_LSB = 0,

    /**
     * OUT8_FREQ_CNFG
     */
    BF_OUT8_PATH_SEL_SIZE = 1,
    BF_OUT8_PATH_SEL_MASK = 0x80,
    BF_OUT8_PATH_SEL_LSB = 7,

    /**
     * OUT8_FREQ_CNFG
     */
    BF_OUT8_EN_SIZE = 1,
    BF_OUT8_EN_MASK = 0x40,
    BF_OUT8_EN_LSB = 6,

    /**
     * OUT8_FREQ_CNFG
     * OUT9_FREQ_CNFG
     */
    BF_T4_INPUT_FAIL_SIZE = 1,
    BF_T4_INPUT_FAIL_MASK = 0x20,
    BF_T4_INPUT_FAIL_LSB = 5,

    /**
     * OUT8_FREQ_CNFG
     */
    BF_AMI_OUT_DUTY_SIZE = 1,
    BF_AMI_OUT_DUTY_MASK = 0x10,
    BF_AMI_OUT_DUTY_LSB = 4,

    /**
     * OUT8_FREQ_CNFG
     */
    BF_400HZ_OUT_SEL_SIZE = 1,
    BF_400HZ_OUT_SEL_MASK = 0x08,
    BF_400HZ_OUT_SEL_LSB = 3,

    /**
     * OUT8_FREQ_CNFG
     */
    BF_OUT9_INV_SIZE = 1,
    BF_OUT9_INV_MASK = 0x04,
    BF_OUT9_INV_LSB = 2,

    /**
     * OUT8_FREQ_CNFG
     */
    BF_OUT7_INV_SIZE = 1,
    BF_OUT7_INV_MASK = 0x02,
    BF_OUT7_INV_LSB = 1,

    /**
     * OUT8_FREQ_CNFG
     */
    BF_OUT6_INV_SIZE = 1,
    BF_OUT6_INV_MASK = 0x01,
    BF_OUT6_INV_LSB = 0,

    /**
     * OUT9_FREQ_CNFG
     */
    BF_OUT9_PATH_SEL_SIZE = 1,
    BF_OUT9_PATH_SEL_MASK = 0x80,
    BF_OUT9_PATH_SEL_LSB = 7,

    /**
     * OUT9_FREQ_CNFG
     */
    BF_OUT9_EN_SIZE = 1,
    BF_OUT9_EN_MASK = 0x40,
    BF_OUT9_EN_LSB = 6,

    /**
     * OUT9_FREQ_CNFG
     */
    BF_OUT5_INV_SIZE = 1,
    BF_OUT5_INV_MASK = 0x10,
    BF_OUT5_INV_LSB = 4,

    /**
     * OUT9_FREQ_CNFG
     */
    BF_OUT4_INV_SIZE = 1,
    BF_OUT4_INV_MASK = 0x08,
    BF_OUT4_INV_LSB = 3,

    /**
     * OUT9_FREQ_CNFG
     */
    BF_OUT3_INV_SIZE = 1,
    BF_OUT3_INV_MASK = 0x04,
    BF_OUT3_INV_LSB = 2,

    /**
     * OUT9_FREQ_CNFG
     */
    BF_OUT2_INV_SIZE = 1,
    BF_OUT2_INV_MASK = 0x02,
    BF_OUT2_INV_LSB = 1,

    /**
     * OUT9_FREQ_CNFG
     */
    BF_OUT1_INV_SIZE = 1,
    BF_OUT1_INV_MASK = 0x01,
    BF_OUT1_INV_LSB = 0,

    /**
     * FR_MFR_SYNC_CNFG
     */
    BF_IN_2K_4K_8K_INV_SIZE = 1,
    BF_IN_2K_4K_8K_INV_MASK = 0x80,
    BF_IN_2K_4K_8K_INV_LSB = 7,

    /**
     * FR_MFR_SYNC_CNFG
     */
    BF_8K_EN_SIZE = 1,
    BF_8K_EN_MASK = 0x40,
    BF_8K_EN_LSB = 6,

    /**
     * FR_MFR_SYNC_CNFG
     */
    BF_2K_EN_SIZE = 1,
    BF_2K_EN_MASK = 0x20,
    BF_2K_EN_LSB = 5,

    /**
     * FR_MFR_SYNC_CNFG
     */
    BF_2K_8K_PUL_POSITION_SIZE = 1,
    BF_2K_8K_PUL_POSITION_MASK = 0x10,
    BF_2K_8K_PUL_POSITION_LSB = 4,

    /**
     * FR_MFR_SYNC_CNFG
     */
    BF_8K_INV_SIZE = 1,
    BF_8K_INV_MASK = 0x08,
    BF_8K_INV_LSB = 3,

    /**
     * FR_MFR_SYNC_CNFG
     */
    BF_8K_PUL_SIZE = 1,
    BF_8K_PUL_MASK = 0x04,
    BF_8K_PUL_LSB = 2,

    /**
     * FR_MFR_SYNC_CNFG
     */
    BF_2K_INV_SIZE = 1,
    BF_2K_INV_MASK = 0x02,
    BF_2K_INV_LSB = 1,

    /**
     * FR_MFR_SYNC_CNFG
     */
    BF_2K_PUL_SIZE = 1,
    BF_2K_PUL_MASK = 0x01,
    BF_2K_PUL_LSB = 0,

    /**
     * PHASE_MON_CNFG
     */
    BF_IN_NOISE_WINDOW_SIZE = 1,
    BF_IN_NOISE_WINDOW_MASK = 0x80,
    BF_IN_NOISE_WINDOW_LSB = 7,

    /**
     * PHASE_OFFSET_LOW_CNFG
     */
    BF_PH_OFFSET_A_SIZE = 8,
    BF_PH_OFFSET_A_MASK = 0xFF,
    BF_PH_OFFSET_A_LSB = 0,

    /**
     * PHASE_OFFSET_HIGH_CNFG
     */
    BF_PH_OFFSET_EN_SIZE = 1,
    BF_PH_OFFSET_EN_MASK = 0x80,
    BF_PH_OFFSET_EN_LSB = 7,

    /**
     * PHASE_OFFSET_HIGH_CNFG
     */
    BF_PH_OFFSET_B_SIZE = 2,
    BF_PH_OFFSET_B_MASK = 0x03,
    BF_PH_OFFSET_B_LSB = 0,

    /**
     * SYNC_MONITOR_CNFG
     */
    BF_SYNC_BYPASS_SIZE = 1,
    BF_SYNC_BYPASS_MASK = 0x80,
    BF_SYNC_BYPASS_LSB = 7,

    /**
     * SYNC_MONITOR_CNFG
     */
    BF_SYNC_MON_LIMT_SIZE = 3,
    BF_SYNC_MON_LIMT_MASK = 0x70,
    BF_SYNC_MON_LIMT_LSB = 4,

    /**
     * SYNC_PHASE_CNFG
     */
    BF_SYNC_EDGE_SIZE = 1,
    BF_SYNC_EDGE_MASK = 0x40,
    BF_SYNC_EDGE_LSB = 6,

    /**
     * SYNC_PHASE_CNFG
     */
    BF_SYNC_PH2_SIZE = 2,
    BF_SYNC_PH2_MASK = 0x0C,
    BF_SYNC_PH2_LSB = 2,

    /**
     * SYNC_PHASE_CNFG
     */
    BF_SYNC_PH1_SIZE = 2,
    BF_SYNC_PH1_MASK = 0x03,
    BF_SYNC_PH1_LSB = 0,

    /**
     * PROTECTION_CNFG
     */
    BF_PROTECTION_DATA_SIZE = 8,
    BF_PROTECTION_DATA_MASK = 0xFF,
    BF_PROTECTION_DATA_LSB = 0,

    /**
     * MPU_SEL_CNFG
     */
    BF_MPU_SEL_CNFG_SIZE = 3,
    BF_MPU_SEL_CNFG_MASK = 0x07,
    BF_MPU_SEL_CNFG_LSB = 0,


/* Page 1 releated bitfields */
    /**
     * DFS_OFF_CNFG
     */
    BF_T0_DFS_OFF_3_SIZE = 1,
    BF_T0_DFS_OFF_3_MASK = 0x80,
    BF_T0_DFS_OFF_3_LSB = 7,

    /**
     * DFS_OFF_CNFG
     */
    BF_T0_DFS_OFF_2_SIZE = 1,
    BF_T0_DFS_OFF_2_MASK = 0x40,
    BF_T0_DFS_OFF_2_LSB = 6,

    /**
     * DFS_OFF_CNFG
     */
    BF_T0_DFS_OFF_1_SIZE = 1,
    BF_T0_DFS_OFF_1_MASK = 0x20,
    BF_T0_DFS_OFF_1_LSB = 5,

    /**
     * DFS_OFF_CNFG
     */
    BF_T0_DFS_OFF_0_SIZE = 1,
    BF_T0_DFS_OFF_0_MASK = 0x10,
    BF_T0_DFS_OFF_0_LSB = 4,

    /**
     * DFS_OFF_CNFG
     */
    BF_T4_DFS_OFF_3_SIZE = 1,
    BF_T4_DFS_OFF_3_MASK = 0x08,
    BF_T4_DFS_OFF_3_LSB = 3,

    /**
     * DFS_OFF_CNFG
     */
    BF_T4_DFS_OFF_2_SIZE = 1,
    BF_T4_DFS_OFF_2_MASK = 0x04,
    BF_T4_DFS_OFF_2_LSB = 2,

    /**
     * DFS_OFF_CNFG
     */
    BF_T4_DFS_OFF_1_SIZE = 1,
    BF_T4_DFS_OFF_1_MASK = 0x02,
    BF_T4_DFS_OFF_1_LSB = 1,

    /**
     * DFS_OFF_CNFG
     */
    BF_T4_DFS_OFF_0_SIZE = 1,
    BF_T4_DFS_OFF_0_MASK = 0x01,
    BF_T4_DFS_OFF_0_LSB = 0,

    /**
     * T0_PPS_CNFG
     * T4_PPS_CNFG
     */
    BF_PPS_PHASE_SIZE = 2,
    BF_PPS_PHASE_MASK = 0x30,
    BF_PPS_PHASE_LSB = 4,

    /**
     * T0_PPS_CNFG
     * T4_PPS_CNFG
     */
    BF_PPS_PULSE_SIZE = 4,
    BF_PPS_PULSE_MASK = 0x0F,
    BF_PPS_PULSE_LSB = 0,

    /**
     * PH_SLOPE_CNFG
     */
    BF_T0_PH_SLOPE_SIZE = 2,
    BF_T0_PH_SLOPE_MASK = 0x0C,
    BF_T0_PH_SLOPE_LSB = 2,

    /**
     * PH_SLOPE_CNFG
     */
    BF_T4_PH_SLOPE_SIZE = 2,
    BF_T4_PH_SLOPE_MASK = 0x03,
    BF_T4_PH_SLOPE_LSB = 0,

    /**
     * T0_ICP_CTRL_CNFG
     * T4_ICP_CTRL_CNFG
     */
    BF_ICP_CTRL_CODE_SIZE = 5,
    BF_ICP_CTRL_CODE_MASK = 0x1F,
    BF_ICP_CTRL_CODE_LSB = 0,

/*==================End of Bitfield Definitions=======================*/

    REGMAP_MAX
};


/** @}*/

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __PUMA_CMN_REGISTER_DEF_H__ */
