/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "osNumeric.h"

#include "Define.h"
#include "Hal.h"
#include "DpllCnfg.h"
#include "RegisterDef.h"
#include "Dpll.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/** @brief Tag used to indicating invalid register offset */
#define INVALID_REGISTER_OFFSET ((T_osUint8)(-1))

/**
 * @brief Structure Translate from Sonet/GigE bitfield to path configuration
 */
typedef struct S_idtSonetGigEthBitfield2PathConfig
{
    T_idtDpllInstance inputDpll_m; /**< input DPLL type */
    T_idtSonetGigEthOutput outputFreq_m; /**< output frequency */
} T_idtSonetGigEthBitfield2PathConfig;

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/** @brief Invalid output path configuration. */
const T_IDTPathConfiguration IDTDpll_pathConfigurationNULL_c = {
    Network_MAX, /**< networkType_m */
    DPLLOutputSelA_MAX, /**< T0SelA_m */
    DPLLOutputSelB_MAX, /**< T0SelB_m */
    DPLLOutputSelA_MAX, /**< T4SelA_m */
    DPLLOutputSelB_MAX, /**< T4SelB_m */
    { SonetGigEthOutput_MAX,  /**< sonetGigE_ma[0] */
      SonetGigEthOutput_MAX } /**< sonetGigE_ma[1] */
};

/**
 * @brief Structure used to determine which register to access for bandwidth limiting
 */
const T_osUint8 IDTDpll_bandwidthLimitRegisters_a[Dpll_MAX][dpllBandwidthLimit_MAX] =
{
    { /* T0 DPLL */
        REG_T0_DPLL_START_BW_DAMPING_CNFG, /**< Starting Stage */
        REG_T0_DPLL_ACQ_BW_DAMPING_CNFG, /**< Acquire Stage */
        REG_T0_DPLL_LOCKED_BW_DAMPING_CNFG /**< Locked Stage */
    },
    { /* T4 DPLL */
        INVALID_REGISTER_OFFSET, /**< Starting Stage */
        INVALID_REGISTER_OFFSET, /**< Acquire Stage */
        REG_T4_DPLL_LOCKED_BW_DAMPING_CNFG /**< Locked Stage */
    } };

/**
 * @brief Table to translate from Sonet/GigE path configuration to bitfield value
 */
const T_idtSonetGigEthSelector IDTDpll_sonetGigEthPathConfig2Bitfield[Dpll_MAX][SonetGigEthOutput_MAX] =
{
    { SonetGigEthSel_T0DpllSonet, /**< T0 input  SONET/SDH 622.08MHz output */
      SonetGigEthSel_T0Dpll10GbE, /**< T0 input Ethernet 625 MHz output */
      SonetGigEthSel_T0Dpll10GbE_6664 /**< T0 input Ethernet OTN 625(66/64) MHz output */
    },
    { SonetGigEthSel_T4DpllSonet, /**< T4 input  SONET/SDH 622.08MHz output */
      SonetGigEthSel_T4Dpll10GbE, /**< T4 input Ethernet 625 MHz output */
      SonetGigEthSel_T4Dpll10GbE_6664 /**< T4 input Ethernet OTN 625(66/64) MHz output */
    } };

/**
 * @brief Table to translate Sonet/GigE path bitfield value to path configuration
 */
static const T_idtSonetGigEthBitfield2PathConfig IDTDpll_sonetGigEthBitfield2PathConfig[SonetGigEthSel_MAX] =
{
    { Dpll_T0,  SonetGigEthOutput_Sonet }, /**< SonetGigESel_T0Dpll62208 */
    { Dpll_MAX, SonetGigEthOutput_MAX }, /**< Invalid value (1) */
    { Dpll_MAX, SonetGigEthOutput_MAX }, /**< Invalid value (2) */
    { Dpll_MAX, SonetGigEthOutput_MAX }, /**< Invalid value (3) */
    { Dpll_T4,  SonetGigEthOutput_Sonet }, /**< SonetGigESel_T4Dpll62208 */
    { Dpll_MAX, SonetGigEthOutput_MAX }, /**< Invalid value (5) */
    { Dpll_MAX, SonetGigEthOutput_MAX }, /**< Invalid value (6) */
    { Dpll_MAX, SonetGigEthOutput_MAX }, /**< Invalid value (7) */
    { Dpll_T0,  SonetGigEthOutput_10GbE }, /**< SonetGigESel_T0Dpll62500 */
    { Dpll_T0,  SonetGigEthOutput_10GbE_6664 }, /**< SonetGigESel_T0Dpll62500_6466 */
    { Dpll_MAX, SonetGigEthOutput_MAX }, /**< Invalid value (10) */
    { Dpll_MAX, SonetGigEthOutput_MAX }, /**< Invalid value (11) */
    { Dpll_T4,  SonetGigEthOutput_10GbE }, /**< SonetGigESel_T4Dpll62500 */
    { Dpll_T4,  SonetGigEthOutput_10GbE_6664 } /**< SonetGigESel_T4Dpll62500_6466 */
};

/**
 * @brief Table to translate DPLL path selector A to bitfield value 
 */
static const T_osUint8 IDTDpll_dpllSelAPathConfig2Bitfield[Dpll_MAX][DPLLOutputSelA_MAX] =
{
    { /* T0 */
        T0DpllSelA_16E1, /**< DPLLOuptutSelA_16E1 */
        T0DpllSelA_16T1, /**< DPLLOutputSelA_16T1 */
        T0DpllSelA_GSM, /**< DPLLOutputSelA_GSM */
        T0DpllSelA_OBSAI, /**< DPLLOutputSelA_OBSAI */
        T0DpllSelA_MAX  /**< DPLLOutputSelA_GPS */
    },
    { /* T4 */
        T4DpllSelA_16E1, /**< DPLLOuptutSelA_16E1 */
        T4DpllSelA_16T1, /**< DPLLOutputSelA_16T1 */
        T4DpllSelA_GSM, /**< DPLLOutputSelA_GSM */
        T4DpllSelA_MAX, /**< DPLLOutputSelA_OBSAI */
        T4DpllSelA_GPS  /**< DPLLOutputSelA_GPS */
    } };

/**
 * @brief Table to translate DPLL path selector A to bitfield value 
 */
static const T_osUint8 IDTDpll_dpllSelBPathConfig2Bitfield[Dpll_MAX][DPLLOutputSelB_MAX] =
{
    { /* T0 */
        T0DpllSelB_12E1, /**< DPLLOuptutSelB_12E1 */
        T0DpllSelB_GPS, /**< DPLLOutputSelB_GPS */
        T0DpllSelB_E3, /**< DPLLOutputSelB_E3 */
        T0DpllSelB_T3,  /**< DPLLOutputSelB_T3 */
        T0DpllSelB_MAX /**< DPLLOutputSelB_24T1 */
    },
    { /* T4 */
        T4DpllSelB_12E1, /**< DPLLOuptutSelB_12E1 */
        T4DpllSelB_MAX, /**< DPLLOutputSelB_GPS */
        T4DpllSelB_E3, /**< DPLLOutputSelB_E3 */
        T4DpllSelB_T3,  /**< DPLLOutputSelB_T3 */
        T4DpllSelB_24T1 /**< DPLLOutputSelB_24T1 */
    } };

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

/**
 * @brief Function to determine type of DPLL give a DPLL instance.
 *
 * This functions is useful in determining supported features of each DPLL
 *
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return Dpll_T0- T0 DPLL, Dpll_T4- T4 DPLL, Dpll_MAX- DPLL instance not supported
 */
T_idtDpllType IDTDpll_GetTypeOfDpll(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nDpll<DPLL_INSTANCE_MAX);

    return (hdlr->deviceConfig_m->pllExt2IntXlate_ma[nDpll]);
}

/**
 * @brief Set the DPLL as automatic reference clock selection
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param mode 0-In manual mode, 1-In auto mode
 */
void IDTDpll_SetAutoSelInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                             T_osUint8 mode)
{
    T_idtDpllType dpllType;

    OS_LOGGER(5, ("%s dev/%s nDpll/%d\n", __FUNCTION__, hdlr->name_m, nDpll));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERT(mode<2);

    if (dpllType == Dpll_T0)
    {
        IDTHAL_WriteBitfield(hdlr, REG_T0_INPUT_SEL_CNFG, BF_T0_INPUT_SEL_MASK,
                             BF_T0_INPUT_SEL_LSB, mode?0:1);
    }
    else
    {
        IDTHAL_WriteBitfield(hdlr, REG_T4_INPUT_SEL_CNFG, BF_T4_INPUT_SEL_MASK,
                             BF_T4_INPUT_SEL_LSB, mode?0:1);
    }
}

/**
 * @brief Get automatic input selection status
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return 0-In manual mode, 1-In auto mode
 */
T_osUint8 IDTDpll_GetAutoSelInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    if (dpllType == Dpll_T0)
    {
        return IDTHAL_ReadBitfield(hdlr, REG_T0_INPUT_SEL_CNFG, BF_T0_INPUT_SEL_MASK,
                                   BF_T0_INPUT_SEL_LSB) == 0 ? 1 : 0;
    }
    else
    {
        return IDTHAL_ReadBitfield(hdlr, REG_T4_INPUT_SEL_CNFG, BF_T4_INPUT_SEL_MASK,
                                   BF_T4_INPUT_SEL_LSB) == 0 ? 1 : 0;
    }
}

/**
 * @brief Set the DPLL to force to lock to a specified reference clock
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nInputNo The number of a specified input clock
 */
void IDTDpll_SetForceSelInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                              T_osUint8 nInputNo)
{
    T_idtDpllType dpllType;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nInputNo/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nInputNo));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    if (dpllType == Dpll_T0)
    {
        IDTHAL_WriteBitfield(hdlr, REG_T0_INPUT_SEL_CNFG, BF_T0_INPUT_SEL_MASK,
                             BF_T0_INPUT_SEL_LSB, nInputNo);
    }
    else
    {
        IDTHAL_WriteBitfield(hdlr, REG_T4_INPUT_SEL_CNFG, BF_T4_INPUT_SEL_MASK,
                             BF_T4_INPUT_SEL_LSB, nInputNo);
    }
}

/**
 * @brief Set the DPLL to force to lock to a specified reference clock
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return Port number of selected input clock
 */
T_osUint8 IDTDpll_GetForceSelInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    if (dpllType == Dpll_T0)
    {
        return IDTHAL_ReadBitfield(hdlr, REG_T0_INPUT_SEL_CNFG, BF_T0_INPUT_SEL_MASK,
                                   BF_T0_INPUT_SEL_LSB);
    }
    else
    {
        return IDTHAL_ReadBitfield(hdlr, REG_T4_INPUT_SEL_CNFG, BF_T4_INPUT_SEL_MASK,
                                   BF_T4_INPUT_SEL_LSB);
    }
}

/**
 * @brief   Get input number of a valid clock with the highest priority
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return  nInput The number of input clock with highest priority
 */
T_osUint8 IDTDpll_Get1stPriorityInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    return IDTHAL_ReadBitfield(hdlr, REG_PRIORITY_TABLE1_STS,
                               BF_HIGHEST_PRIORITY_VALIDATED_MASK,
                               BF_HIGHEST_PRIORITY_VALIDATED_LSB);
}

/**
 * @brief   Get input number of a valid clock with the second highest priority
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return  nInput The number of input clock with second highest priority
 */
T_osUint8 IDTDpll_Get2ndPriorityInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    return IDTHAL_ReadBitfield(hdlr, REG_PRIORITY_TABLE2_STS,
                               BF_SECOND_HIGHEST_PRIORITY_VALIDATED_MASK,
                               BF_SECOND_HIGHEST_PRIORITY_VALIDATED_LSB);
}

/**
 * @brief   Get input number of a valid clock with the third highest priority
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return  nInput The number of input clock with third highest priority
 */
T_osUint8 IDTDpll_Get3rdPriorityInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    return IDTHAL_ReadBitfield(hdlr, REG_PRIORITY_TABLE2_STS,
                               BF_THIRD_HIGHEST_PRIORITY_VALIDATED_MASK,
                               BF_THIRD_HIGHEST_PRIORITY_VALIDATED_LSB);
}

/**
 * @brief   Get the current selected input clock
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return  nInput the number of selected input clock
 */
T_osUint8 IDTDpll_GetCurrentSelectInput(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    return IDTHAL_ReadBitfield(hdlr, REG_PRIORITY_TABLE1_STS,
                               BF_CURRENTLY_SELECTED_INPUTS_MASK, BF_CURRENTLY_SELECTED_INPUTS_LSB);
}

/**
 * @brief   Set DPLL working at the specified operating mode
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param	nMode <ul><li>Automatic_Mode = Automatic mode</li>
 *		       <li>Force_FreeRun_Mode = Force to Free-Run mode</li>
 *		       <li>Force_Holdover_Mode = Force to Holdover mode</li>
 *		       <li>Force_Locked_Mode = Force to Locked mode</li>
 *		       <li>Force_Pre_Locked2_Mode = Force to Pre-Locked2 mode</li>
 *		       <li>Force_Pre_Locked_Mode = Force to Pre-Lock mode</li>
 *		       <li>Force_Lost_Phase_Mode = Force to Lost-Phase mode</li>
 *		       <li>Dco_Mode = DCO mode</li></ul>
 */
void IDTDpll_SetDpllOperMod(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                            T_idtDpllOperMode nMode)
{
    T_idtDpllType dpllType;
    T_osUint8 bfValue = nMode;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nMode/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nMode));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Check if we support DCO mode for this DPLL */
    if (nMode == Dco_Mode) {
        OS_ASSERTS(hdlr->deviceConfig_m->dcoModeSupport_ma[nDpll],
                    ("DPLL %d does not supprt DCO mode\n",nDpll+1));        
    }

    if (nMode == Dco_Mode)
        bfValue = Force_Holdover_Mode;

    if (dpllType == Dpll_T0)
    {
        IDTHAL_WriteBitfield(hdlr, REG_T0_OPERATION_MODE_CNFG,
                             BF_T0_OPERATING_MODE_MASK, BF_T0_OPERATING_MODE_LSB, bfValue);
    }
    else
    {
        IDTHAL_WriteBitfield(hdlr, REG_T4_OPERATION_MODE_CNFG,
                             BF_T4_OPERATING_MODE_MASK, BF_T4_OPERATING_MODE_LSB, bfValue);
    }

    if (nMode == Dco_Mode)
        bfValue = 1;
    else
    {
        /* In all other modes, disable DCO mode */
        bfValue = 0;
    }

    if (dpllType == Dpll_T0)
    {
        IDTHAL_WriteBitfield(hdlr, REG_T0_OPERATION_MODE_CNFG, BF_T0_WDCO_MASK,
                             BF_T0_WDCO_LSB, bfValue);
    }
    else
    {
        IDTHAL_WriteBitfield(hdlr, REG_T4_OPERATION_MODE_CNFG, BF_T4_WDCO_MASK,
                             BF_T4_WDCO_LSB, bfValue);
    }
}

/**
 * @brief   Set DPLL working at the specified operating mode
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return   nMode <ul><li>Automatic_Mode = Automatic mode</li>
 *             <li>Force_FreeRun_Mode = Force to Free-Run mode</li>
 *             <li>Force_Holdover_Mode = Force to Holdover mode</li>
 *             <li>Force_Locked_Mode = Force to Locked mode</li>
 *             <li>Force_Pre_Locked2_Mode = Force to Pre-Locked2 mode</li>
 *             <li>Force_Pre_Locked_Mode = Force to Pre-Lock mode</li>
 *             <li>Force_Lost_Phase_Mode = Force to Lost-Phase mode</li>
 *             <li>Dco_Mode = DCO mode</li></ul>
 */
T_idtDpllOperMode IDTDpll_GetDpllOperMod(T_idtDrvHdlr hdlr,
                                         T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint8 bfValue;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    if (dpllType == Dpll_T0)
    {
        bfValue =
            IDTHAL_ReadBitfield(hdlr, REG_T0_OPERATION_MODE_CNFG, BF_T0_WDCO_MASK,
                                BF_T0_WDCO_LSB);
    }
    else
    {
        bfValue =
            IDTHAL_ReadBitfield(hdlr, REG_T4_OPERATION_MODE_CNFG, BF_T4_WDCO_MASK,
                                BF_T4_WDCO_LSB);
    }

    if (bfValue == 1)
        return Dco_Mode;

    if (dpllType == Dpll_T0)
    {
        return (T_idtDpllOperMode) IDTHAL_ReadBitfield(hdlr, REG_T0_OPERATION_MODE_CNFG,
                                                       BF_T0_OPERATING_MODE_MASK, BF_T0_OPERATING_MODE_LSB);
    }
    else
    {
        return (T_idtDpllOperMode) IDTHAL_ReadBitfield(hdlr, REG_T4_OPERATION_MODE_CNFG,
                                                       BF_T4_OPERATING_MODE_MASK, BF_T4_OPERATING_MODE_LSB);
    }
}

/**
 * @brief   Inquire if the DPLL is in lock state
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return	nState <ul><li>0= DPLL is in unlocked state</li>
 *			<li>1= DPLL is in locked state</li></ul>
 */
T_osUint8 IDTDpll_IsDpllLocked(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    if (dpllType == Dpll_T0)
    {
        return (IDTHAL_ReadBitfield(hdlr, REG_OPERATING_STS, BF_T0_DPLL_LOCK_MASK,
                                    BF_T0_DPLL_LOCK_LSB));
    }
    else
    {
        return (IDTHAL_ReadBitfield(hdlr, REG_OPERATING_STS, BF_T4_DPLL_LOCK_MASK,
                                    BF_T4_DPLL_LOCK_LSB));
    }
}

/**
 * @brief Set the bandwidth and damping factor used for bandwidth limiting
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param stage Bandwidth stage
 * @param   nBandWidth the bandwidth of DPLL
 *                     <ul><li>dpllBandwidth_0p5mHz - 0.5 mHz</li>
 *                            <li>dpllBandwidth_1mHz -  1  mHz</li>
 *                            <li>dpllBandwidth_2mHz -  2  mHz</li>
 *                            <li>dpllBandwidth_4mHz -  4  mHz</li>
 *                            <li>dpllBandwidth_8mHz -  8  mHz</li>
 *                            <li>dpllBandwidth_15mHz - 15  mHz</li>
 *                            <li>dpllBandwidth_30mHz - 30  mHz</li>
 *                            <li>dpllBandwidth_60mHz - 60  mHz</li>
 *                            <li>dpllBandwidth_0p1Hz - 0.1 Hz</li>
 *                            <li>dpllBandwidth_0p3Hz - 0.3 Hz</li>
 *                            <li>dpllBandwidth_0p6Hz - 0.6 Hz</li>
 *                            <li>dpllBandwidth_1p2Hz - 1.2 Hz</li>
 *                            <li>dpllBandwidth_2p5Hz - 2.5 Hz</li>
 *                            <li>dpllBandwidth_4Hz -  4  Hz</li>
 *                            <li>dpllBandwidth_8Hz -  8  Hz</li>
 *                            <li>dpllBandwidth_18Hz - 18  Hz</li>
 *                            <li>dpllBandwidth_35Hz - 35  Hz</li>
 *                            <li>dpllBandwidth_70Hz - 70  Hz</li>
 *                            <li>dpllBandwidth_560Hz - 560 Hz</li></ul>
 * @param   nDamping the damping factor of DPLL
 *                     <ul><li>DampingFactor_1p2 -  1.2</li>
 *                            <li>DampingFactor_2p5 - 2.5</li>
 *                            <li>DampingFactor_5 - 5</li>
 *                            <li>DampingFactor_10 - 10</li>
 *                            <li>DampingFactor_20 - 20</li></ul>
 *                        
 */
void IDTDpll_SetBandWidthDamping(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                 T_idtBandwidthLimitStage stage, T_idtDpllBandwidth nBandWidth,
                                 T_idtDampingFactor nDamping)
{
    T_idtDpllType dpllType;
    T_osUint8 regOffset;
    T_osUint8 regVal;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d stage/%d nBandWidth/%d nDamping/%d\n", __FUNCTION__, hdlr->name_m, nDpll, stage, nBandWidth, nDamping));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nDamping<DampingFactor_MAX);
    OS_ASSERT(stage<dpllBandwidthLimit_MAX);
    OS_ASSERT(nBandWidth<dpllBandwidth_MAX);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    regOffset = IDTDpll_bandwidthLimitRegisters_a[dpllType][stage];
    OS_ASSERTS(regOffset!=INVALID_REGISTER_OFFSET,
                ("Bandwidth limit stage (%d) not supported my DPLL/%d\n", 
                 (int)stage, nDpll));

    /* T0 but is it running on T4 mode ? */
    if ((dpllType == Dpll_T0) && 
        (hdlr->deviceConfig_m->t0PllMode_m == Dpll_T4)) {
        OS_ASSERTS( (stage == dpllBandwidthLimitLocked),
                     ("DPLL %d does not support START/ACQUIRE stage"
                      " bandwith damping\n",nDpll+1) );
        OS_ASSERT( (nBandWidth >= dpllBandwidth_8Hz) &&
                    (nBandWidth <= dpllBandwidth_560Hz));
    }

    regVal = IDTHal_Read(hdlr, regOffset);

    /* Some devices have a T4 instead of a T0 */
    if (dpllType == Dpll_T0)
    {
        IDTHAL_ModifyBitfield(regVal, BF_T0_DPLL_LOCKED_BW_MASK,
                              BF_T0_DPLL_LOCKED_BW_LSB, (T_osUint8) nBandWidth);
    }
    else
    {
        /* T4 has a smaller bandwidth */
        OS_ASSERT(nBandWidth>dpllBandwidth_8Hz);

        /* Normalize bandwidth to 0..3 */
        nBandWidth -= dpllBandwidth_18Hz;

        IDTHAL_ModifyBitfield(regVal, BF_T4_DPLL_LOCKED_BW_MASK,
                              BF_T4_DPLL_LOCKED_BW_LSB, (T_osUint8) nBandWidth);
    }

    IDTHAL_ModifyBitfield(regVal, BF_T0_DPLL_LOCKED_DAMPING_MASK,
                          BF_T0_DPLL_LOCKED_DAMPING_LSB, (T_osUint8) nDamping);

    IDTHal_Write(hdlr, regOffset, regVal);
}

/**
 * @brief Get configured bandwidth and damping factor for bandwidth limiting
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param stage Bandwidth stage
 * @param nBandWidth_p the bandwidth of DPLL
 *                        <ul><li>dpllBandwidth_0p5mHz - 0.5 mHz</li>
 *                            <li>dpllBandwidth_1mHz -  1  mHz</li>
 *                            <li>dpllBandwidth_2mHz -  2  mHz</li>
 *                            <li>dpllBandwidth_4mHz -  4  mHz</li>
 *                            <li>dpllBandwidth_8mHz -  8  mHz</li>
 *                            <li>dpllBandwidth_15mHz - 15  mHz</li>
 *                            <li>dpllBandwidth_30mHz - 30  mHz</li>
 *                            <li>dpllBandwidth_60mHz - 60  mHz</li>
 *                            <li>dpllBandwidth_0p1Hz - 0.1 Hz</li>
 *                            <li>dpllBandwidth_0p3Hz - 0.3 Hz</li>
 *                            <li>dpllBandwidth_0p6Hz - 0.6 Hz</li>
 *                            <li>dpllBandwidth_1p2Hz - 1.2 Hz</li>
 *                            <li>dpllBandwidth_2p5Hz - 2.5 Hz</li>
 *                            <li>dpllBandwidth_4Hz -  4  Hz</li>
 *                            <li>dpllBandwidth_8Hz -  8  Hz</li>
 *                            <li>dpllBandwidth_18Hz - 18  Hz</li>
 *                            <li>dpllBandwidth_35Hz - 35  Hz</li>
 *                            <li>dpllBandwidth_70Hz - 70  Hz</li>
 *                            <li>dpllBandwidth_560Hz - 560 Hz</li></ul>
 * @param nDamping_p the damping factor of DPLL
 *                          <ul><li>DampingFactor_1p2 -  1.2</li>
 *                            <li>DampingFactor_2p5 - 2.5</li>
 *                            <li>DampingFactor_5 - 5</li>
 *                            <li>DampingFactor_10 - 10</li>
 *                            <li>DampingFactor_20 - 20</li></ul>
 */
void IDTDpll_GetBandWidthDamping(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                 T_idtBandwidthLimitStage stage, T_idtDpllBandwidth *nBandWidth_p,
                                 T_idtDampingFactor *nDamping_p)
{
    T_idtDpllType dpllType;
    T_osUint8 regOffset;
    T_osUint8 regVal;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT_PTR_NULL(nBandWidth_p);
    OS_ASSERT_PTR_NULL(nDamping_p);
    OS_ASSERT(stage<dpllBandwidthLimit_MAX);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    regOffset = IDTDpll_bandwidthLimitRegisters_a[dpllType][stage];
    OS_ASSERTS(regOffset!=INVALID_REGISTER_OFFSET,
                ("Bandwidth limit stage (%d) not supported my DPLL\n", stage));

    /* T0 but is it running on T4 mode ? */
    if ((dpllType == Dpll_T0) && 
        (hdlr->deviceConfig_m->t0PllMode_m == Dpll_T4))
        OS_ASSERTS( (stage == dpllBandwidthLimitLocked),
                     ("DPLL %d does not support START/ACQUIRE stage"
                      " bandwith damping\n",nDpll+1) );

    regVal = IDTHal_Read(hdlr, regOffset);

    /* Some devices have a T4 instead of a T0 */
    if (dpllType == Dpll_T0)
    {
        *nBandWidth_p = IDTHAL_GetBitfield(regVal, BF_T0_DPLL_LOCKED_BW_MASK,
                                           BF_T0_DPLL_LOCKED_BW_LSB);
    }
    else
    {
        /* T4 has a smaller bandwidth */
        *nBandWidth_p = IDTHAL_GetBitfield(regVal, BF_T4_DPLL_LOCKED_BW_MASK,
                                           BF_T4_DPLL_LOCKED_BW_LSB);
        /* Normalize bandwidth to 0..3 */
        *nBandWidth_p = (T_idtDpllBandwidth)(*nBandWidth_p+dpllBandwidth_18Hz);

    }

    *nDamping_p = IDTHAL_GetBitfield(regVal, BF_T0_DPLL_LOCKED_DAMPING_MASK,
                                     BF_T0_DPLL_LOCKED_DAMPING_LSB);
}

/**
 * @brief Set device to select a suitable bandwidth and damping factor
 * automatically.
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param	nEnable <ul><li> 0= Only locked bandwidth and damping factor selected</li>
 *			 <li>1= enable automatic selection</li></ul>
 */
void IDTDpll_SetAutoSelBandWidth(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                 T_osUint8 nEnable)
{
    T_idtDpllType dpllType;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nEnable/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nEnable));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    if (dpllType == Dpll_T0)
    {
        IDTHAL_WriteBitfield(hdlr, REG_T0_BW_OVERSHOOT_CNFG,
                             BF_AUTO_BW_SEL_MASK, BF_AUTO_BW_SEL_LSB, nEnable);
    }
    else
    {
        OS_ASSERTS(0, ("DPLL %d does not support this feature.\n",nDpll));
        return;
    }
}

/**
 * @brief Get auto bandwidth/damping factor status
 * automatically.
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return 0-manual bandwidth/damping factor mode, 1-automatic bandwidth/damping factor mode
 */
T_osUint8 IDTDpll_GetAutoSelBandWidth(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    if (dpllType == Dpll_T0)
    {
        return IDTHAL_ReadBitfield(hdlr, REG_T0_BW_OVERSHOOT_CNFG,
                                   BF_AUTO_BW_SEL_MASK, BF_AUTO_BW_SEL_LSB);
    }
    else
    {
        OS_ASSERTS(0, ("DPLL %d does not support this feature.\n",nDpll));
        return 0;
    }
}

/**
 *
 * @brief Set DPLL integral path value frozen
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param	nEnable <ul><li>0= Disable the integral path frozen</li>
 *			 <li>1= Enable the integral path frozen</li></ul>
 */
void IDTDpll_SetDpllFrozen(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                           T_osUint8 nEnable)
{
    T_idtDpllType dpllType;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nEnable/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nEnable));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    if (dpllType == Dpll_T0)
    {
        IDTHAL_WriteBitfield(hdlr, REG_T0_BW_OVERSHOOT_CNFG, BF_T0_LIMIT_MASK,
                             BF_T0_LIMIT_LSB, nEnable);
    }
    else
    {
        OS_ASSERTS(0, ("DPLL %d does not support this feature.\n",nDpll));
        return;
    }
}

/**
 *
 * @brief Get DPLL integral path value frozen
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return <ul><li>0= Disable the integral path frozen</li>
 *           <li>1= Enable the integral path frozen</li></ul>
 */
T_osUint8 IDTDpll_GetDpllFrozen(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    if (dpllType == Dpll_T0)
    {
        return IDTHAL_ReadBitfield(hdlr, REG_T0_BW_OVERSHOOT_CNFG, BF_T0_LIMIT_MASK,
                                   BF_T0_LIMIT_LSB);
    }
    else
    {
        OS_ASSERTS(0, ("DPLL %d does not support this feature.\n",nDpll));
        return 0;
    }
}

/**
 * @brief   Set the coarse phase lose detector enable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param	nEnable <ul><li>0= Disable the coarse phase detector</li>
 *			 <li>1= Enable the coarse phase detector</li></ul>
 * @param	nWide	 <ul><li>0= Wide range disable</li>
 *			 <li>1= Wide range enable</li></ul>
 * @param	nMultiPhase <ul><li>0= Limit to +/- 1UI</li>
 *			     <li>1= Limit to PH_LOS_COARSE_LIMT register</li></ul>
 * @param	nMultiPh8kEn <ul><li>0= Limit to +/- 1UI when reference clock 8/4/2KHz clock</li>
 *			      <li>1= Limit to PH_LOS_COARSE_LIMT register</li></ul>
 *
 */
void IDTDpll_SetPhaseCoarseDetector(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                    T_osUint8 nEnable, T_osUint8 nWide, T_osUint8 nMultiPhase,
                                    T_osUint8 nMultiPh8kEn)
{
    T_idtDpllType dpllType;
    T_osUint8 nIntInd;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nEnable/%d nWide/%d "
                "nMultiPhase/%d nMultiPh8kEn/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nEnable, nWide, nMultiPhase, nMultiPh8kEn));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nEnable<2);
    OS_ASSERT(nWide<2);
    OS_ASSERT(nMultiPhase<2);
    OS_ASSERT(nMultiPh8kEn<2);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    /* Provision */
    nIntInd = IDTHal_Read(hdlr, REG_PHASE_LOSS_COARSE_LIMIT_CNFG);
    {
        /* Enable/Disable coarse phase detection */
        IDTHAL_ModifyBitfield(nIntInd, 
                              BF_COARSE_PH_LOS_LIMT_EN_MASK,
                              BF_COARSE_PH_LOS_LIMT_EN_LSB, 
                              nEnable);

        /* Wide range enable/disable */
        IDTHAL_ModifyBitfield(nIntInd, 
                              BF_WIDE_EN_MASK, 
                              BF_WIDE_EN_LSB, 
                              nWide);

        IDTHAL_ModifyBitfield(nIntInd, 
                              BF_MULTI_PH_APP_MASK,
                              BF_MULTI_PH_APP_LSB, 
                              nMultiPhase);

        IDTHAL_ModifyBitfield(nIntInd, 
                              BF_MULTI_PH_8K_4K_2K_EN_MASK,
                              BF_MULTI_PH_8K_4K_2K_EN_LSB, 
                              nMultiPh8kEn);
    }
    IDTHal_Write(hdlr, REG_PHASE_LOSS_COARSE_LIMIT_CNFG, nIntInd);
}

/**
 * @brief   Set the coarse phase lose detector enable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param   nEnable_p <ul><li>0= Disable the coarse phase detector</li>
 *           <li>1= Enable the coarse phase detector</li></ul>
 * @param   nWide_p <ul><li>0= Wide range disable</li>
 *               <li>1= Wide range enable</li></ul>
 * @param   nMultiPhase_p <ul><li>0= Limit to +/- 1UI</li>
 *               <li>1= Limit to PH_LOS_COARSE_LIMT register</li></ul>
 * @param   nMultiPh8kEn_p <ul><li>0= Limit to +/- 1UI when reference clock 8/4/2KHz clock</li>
 *               <li>1= Limit to PH_LOS_COARSE_LIMT register</li></ul>
 *
 */
void IDTDpll_GetPhaseCoarseDetector(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                    T_osUint8 *nEnable_p, T_osUint8 *nWide_p, T_osUint8 *nMultiPhase_p,
                                    T_osUint8 *nMultiPh8kEn_p)
{
    T_idtDpllType dpllType;
    T_osUint8 nIntInd;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT_PTR_NULL(nEnable_p);
    OS_ASSERT_PTR_NULL(nWide_p);
    OS_ASSERT_PTR_NULL(nMultiPhase_p);
    OS_ASSERT_PTR_NULL(nMultiPh8kEn_p);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    /* Provision */
    nIntInd = IDTHal_Read(hdlr, REG_PHASE_LOSS_COARSE_LIMIT_CNFG);
    {
        /* Enable/Disable coarse phase detection */
        *nEnable_p = IDTHAL_GetBitfield(nIntInd,
                                        BF_COARSE_PH_LOS_LIMT_EN_MASK,
                                        BF_COARSE_PH_LOS_LIMT_EN_LSB);

        /* Wide range enable/disable */
        *nWide_p = IDTHAL_GetBitfield(nIntInd, 
                                      BF_WIDE_EN_MASK, 
                                      BF_WIDE_EN_LSB);

        *nMultiPhase_p = IDTHAL_GetBitfield(nIntInd, 
                                            BF_MULTI_PH_APP_MASK,
                                            BF_MULTI_PH_APP_LSB);

        *nMultiPh8kEn_p =
            IDTHAL_GetBitfield(nIntInd, 
                               BF_MULTI_PH_8K_4K_2K_EN_MASK,
                               BF_MULTI_PH_8K_4K_2K_EN_LSB);
    }
}

/**
 * @brief   Set the limitation of the coarse phase lose detector
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nLimit Limitation of coarse phase loss detector. <ul>
 *                    <li>coarsePhaseLimit_1 - +/- 1 UI</li>
 *                    <li>coarsePhaseLimit_3 - +/- 3 UI</li>
 *                    <li>coarsePhaseLimit_7 - +/- 7 UI</li>
 *                    <li>coarsePhaseLimit_15 - +/- 15 UI</li>
 *                    <li>coarsePhaseLimit_31 - +/- 31 UI</li>
 *                    <li>coarsePhaseLimit_63 - +/- 63 UI</li>
 *                    <li>coarsePhaseLimit_127 - +/- 127 UI</li>
 *                    <li>coarsePhaseLimit_255 - +/- 255 UI</li>
 *                    <li>coarsePhaseLimit_511 - +/- 511 UI</li>
 *                    <li>coarsePhaseLimit_1023 - +/- 1023 UI T0 only</li></ul>
 */
void IDTDpll_SetPhaseCoarseLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                 T_idtCoarsePhaseLimit nLimit)
{
    T_idtDpllType dpllType;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nLimit/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nLimit));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    IDTHAL_WriteBitfield(hdlr, REG_PHASE_LOSS_COARSE_LIMIT_CNFG,
                         BF_PH_LOS_COARSE_LIMT_MASK, BF_PH_LOS_COARSE_LIMT_LSB, nLimit);
}

/**
 * @brief Get the limitation of the coarse phase lose detector
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return Limitation of coarse phase loss detector. <ul>
 *                    <li>coarsePhaseLimit_1 - +/- 1 UI</li>
 *                    <li>coarsePhaseLimit_3 - +/- 3 UI</li>
 *                    <li>coarsePhaseLimit_7 - +/- 7 UI</li>
 *                    <li>coarsePhaseLimit_15 - +/- 15 UI</li>
 *                    <li>coarsePhaseLimit_31 - +/- 31 UI</li>
 *                    <li>coarsePhaseLimit_63 - +/- 63 UI</li>
 *                    <li>coarsePhaseLimit_127 - +/- 127 UI</li>
 *                    <li>coarsePhaseLimit_255 - +/- 255 UI</li>
 *                    <li>coarsePhaseLimit_511 - +/- 511 UI</li>
 *                    <li>coarsePhaseLimit_1023 - +/- 1023 UI T0 only</li></ul>
 */
T_idtCoarsePhaseLimit IDTDpll_GetPhaseCoarseLimit(T_idtDrvHdlr hdlr,
                                                  T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    return (T_idtCoarsePhaseLimit) IDTHAL_ReadBitfield(hdlr, REG_PHASE_LOSS_COARSE_LIMIT_CNFG,
                                                       BF_PH_LOS_COARSE_LIMT_MASK, BF_PH_LOS_COARSE_LIMT_LSB);
}

/**
 * @brief Set the fine phase lose detector enable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param	nEnable <ul><li>0= Disable the fine phase detector</li>
 *			 <li>1= Enable the fine phase detector</li></ul>
 */
void IDTDpll_SetPhaseFineDetectorEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                        T_osUint8 nEnable)
{
    T_idtDpllType dpllType;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nEnable/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nEnable));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    IDTHAL_WriteBitfield(hdlr, REG_PHASE_LOSS_FINE_LIMIT_CNFG,
                         BF_FINE_PH_LOS_LIMT_EN_MASK, BF_FINE_PH_LOS_LIMT_EN_LSB, nEnable);
}

/**
 * @brief Get the fine phase lose detector enable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return <ul><li>0= Disable the fine phase detector</li>
 *           <li>1= Enable the fine phase detector</li></ul>
 */
T_osUint8 IDTDpll_GetPhaseFineDetectorEnable(T_idtDrvHdlr hdlr,
                                              T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    return IDTHAL_ReadBitfield(hdlr, REG_PHASE_LOSS_FINE_LIMIT_CNFG,
                               BF_FINE_PH_LOS_LIMT_EN_MASK, BF_FINE_PH_LOS_LIMT_EN_LSB);
}

/**
 * @brief   Set the limitation of the fine phase lose detector
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param	nLimit Limitation of fine phase loss detector
 *     <ul><li>finePhaseLimit_45_90degree = +/- 45-90 degrees</li>
 *      <li>finePhaseLimit_90_180degree = +/- 90-180 degrees</li>
 *      <li>finePhaseLimit_180_360degree = +/- 180-360 degrees</li>
 *      <li>finePhaseLimit_20_25nanosec = +/- 20-25 nanoseconds</li>
 *      <li>finePhaseLimit_60_65nanosec = +/- 60-65 nanoseconds</li>
 *      <li>finePhaseLimit_120_125nanosec = +/- 120-125 nanoseconds</li>
 *      <li>finePhaseLimit_950_955nanosec = +/- 950-955 nanoseconds</li></ul>
 */
void IDTDpll_SetPhaseFineLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                               T_idtFinePhaseLimit nLimit)
{
    T_idtDpllType dpllType;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nLimit/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nLimit));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT((nLimit<finePhaseLimit_MAX)&&(nLimit!=0));
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    IDTHAL_WriteBitfield(hdlr, REG_PHASE_LOSS_FINE_LIMIT_CNFG,
                         BF_PH_LOS_FINE_LIMT_MASK, BF_PH_LOS_FINE_LIMT_LSB, nLimit);
}

/**
 * @brief   Set the limitation of the fine phase lose detector
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return Limitation of fine phase loss detector
 *     <ul><li>finePhaseLimit_45_90degree = +/- 45-90 degrees</li>
 *      <li>finePhaseLimit_90_180degree = +/- 90-180 degrees</li>
 *      <li>finePhaseLimit_180_360degree = +/- 180-360 degrees</li>
 *      <li>finePhaseLimit_20_25nanosec = +/- 20-25 nanoseconds</li>
 *      <li>finePhaseLimit_60_65nanosec = +/- 60-65 nanoseconds</li>
 *      <li>finePhaseLimit_120_125nanosec = +/- 120-125 nanoseconds</li>
 *      <li>finePhaseLimit_950_955nanosec = +/- 950-955 nanoseconds</li></ul>
 */
T_idtFinePhaseLimit IDTDpll_GetPhaseFineLimit(T_idtDrvHdlr hdlr,
                                              T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);

    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    return (T_idtFinePhaseLimit) IDTHAL_ReadBitfield(hdlr, REG_PHASE_LOSS_FINE_LIMIT_CNFG,
                                                     BF_PH_LOS_FINE_LIMT_MASK, BF_PH_LOS_FINE_LIMT_LSB);
}

/**
 * @brief Set reference clock switch if a fast loss occurs
 * @param hdlr Device driver handler
 * @param nEnable <ul><li>0= Disable switch, DPLL enters temporary holdover state</li>
 *                    <li>1= Enable switch, DPLL enters Phase-loss state</li></ul>
 */
void IDTDpll_SetFastLossSwitch(T_idtDrvHdlr hdlr, T_osUint8 nEnable)
{
    OS_LOGGER(5,
               ("%s dev/%s nEnable/%d\n", __FUNCTION__, hdlr->name_m, nEnable));
    OS_ASSERT_PTR_NULL(hdlr);

    IDTHAL_WriteBitfield(hdlr, REG_PHASE_LOSS_FINE_LIMIT_CNFG,
                         BF_FAST_LOS_SW_MASK, BF_FAST_LOS_SW_LSB, nEnable);
}

/**
 * @brief Get reference clock switch if a fast loss occurs
 * @param hdlr Device driver handler
 * @return <ul><li>0= Disable switch, DPLL enters temporary holdover state</li>
 *             <li>1= Enable switch, DPLL enters Phase-loss state</li></ul>
 */
T_osUint8 IDTDpll_GetFastLossSwitch(T_idtDrvHdlr hdlr)
{
    OS_ASSERT_PTR_NULL(hdlr);

    return IDTHAL_ReadBitfield(hdlr, REG_PHASE_LOSS_FINE_LIMIT_CNFG,
                               BF_FAST_LOS_SW_MASK, BF_FAST_LOS_SW_LSB);
}

/**
 * @brief Set calculation mode of holdover frequency and read back mode
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nMode <ul><li>Holdover_Instantaneous = automatic instantaneous mode</li>
 *			        <li>Holdover_Slow_Average = automatic slow average</li>
 *			        <li>Holdover_Fast_Average = Automatic fast average</li>
 *		            <li>Holdover_Manual = Manual</li></ul>
 * @param nReadMode <ul><li>0= Don't read value from device</li>
 *                      <li>1= Read value calculated by device back to the
 *                          holdover frequency register(0x5D,0x5E and 0x5F)</li></ul>
 */
void IDTDpll_SetHoldoverMode(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                             T_idtDpllHoldover nMode, T_osUint8 nReadMode)
{
    T_idtDpllType dpllType;
    T_osUint8 nIntInd;

    OS_LOGGER(5,
               ("%s dev/%s nMode/%d nReadMode/%d\n", __FUNCTION__, hdlr->name_m, nMode, nReadMode));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERT(nMode<Holdover_MAX);
    OS_ASSERTS(dpllType==Dpll_T0,
                ("DPLL %d does not support this feature.\n",nDpll));
    OS_ASSERT(nReadMode<2);

    nIntInd = IDTHal_Read(hdlr, REG_T0_HOLDOVER_MODE_CNFG);

    IDTHAL_ModifyBitfield(nIntInd, BF_READ_AVG_MASK, BF_READ_AVG_LSB,
                          nReadMode);

    /* Mode = { HOLDOVER, AUTO AVERAGE, FAST AVERAGE } */
    IDTHAL_ModifyBitfield(nIntInd,
                          (BF_FAST_AVG_MASK | BF_AUTO_AVG_MASK | BF_MAN_HOLDOVER_MASK),
                          BF_FAST_AVG_LSB, nMode);

    IDTHal_Write(hdlr, REG_T0_HOLDOVER_MODE_CNFG, nIntInd);
}

/**
 * @brief Get calculation mode of holdover frequency and read back mode
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nMode_p <ul><li>Holdover_Instantaneous = automatic instantaneous mode</li>
 *                  <li>Holdover_Slow_Average = automatic slow average</li>
 *                  <li>Holdover_Fast_Average = Automatic fast average</li>
 *                  <li>Holdover_Manual = Manual</li></ul>
 * @param nReadMode_p <ul><li>0= Don't read value from device</li>
 *                        <li>1= Read value calculated by device back to the
 *                            holdover frequency register(0x5D,0x5E and 0x5F)</li></ul>
 */
void IDTDpll_GetHoldoverMode(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                             T_idtDpllHoldover *nMode_p, T_osUint8 *nReadMode_p)
{
    T_idtDpllType dpllType;
    T_osUint8 nIntInd;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERT_PTR_NULL(nMode_p);
    OS_ASSERT_PTR_NULL(nReadMode_p);
    OS_ASSERTS(dpllType==Dpll_T0,
                ("DPLL %d does not support this feature.\n",nDpll));

    nIntInd = IDTHal_Read(hdlr, REG_T0_HOLDOVER_MODE_CNFG);

    *nReadMode_p =
        IDTHAL_GetBitfield(nIntInd, BF_READ_AVG_MASK, BF_READ_AVG_LSB);

    /* Mode = { HOLDOVER, AUTO AVERAGE, FAST AVERAGE } */
    *nMode_p = (T_idtDpllHoldover) IDTHAL_GetBitfield(nIntInd,
                                                      (BF_FAST_AVG_MASK | BF_AUTO_AVG_MASK | BF_MAN_HOLDOVER_MASK),
                                                      BF_FAST_AVG_LSB);
}

/**
 * @brief   Set calculation mode of temporary holdover frequency
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param	nMode 	<ul><li>Temp_Same_As_Full_Holdover = Same as the DPLL holdover mode</li>
 * 			<li>Temp_Holdover_Instantaneous = Automatic instantaneous mode</li>
 * 			<li>Temp_Holdover_Fast_Average = Automatic fast average</li>
 * 			<li>Temp_Holdover_Slow_Average = Automatic slow average</li></ul>
 */
void IDTDpll_SetTempHoldoverMode(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                 T_idtTemp_holdover nMode)
{
    T_idtDpllType dpllType;

    OS_LOGGER(5, ("%s dev/%s nMode/%d\n", __FUNCTION__, hdlr->name_m, nMode));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERT(nMode<Temp_Holdover_MAX);
    OS_ASSERTS(dpllType==Dpll_T0,
                ("DPLL %d does not support this feature.\n",nDpll));

    IDTHAL_WriteBitfield(hdlr, REG_T0_HOLDOVER_MODE_CNFG,
                         BF_TEMP_HOLDOVER_MODE_MASK, BF_TEMP_HOLDOVER_MODE_LSB, nMode);
}

/**
 * @brief   Set calculation mode of temporary holdover frequency
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return <ul><li>Temp_Same_As_Full_Holdover = Same as the DPLL holdover mode</li>
 *             <li>Temp_Holdover_Instantaneous = Automatic instantaneous mode</li>
 *             <li>Temp_Holdover_Fast_Average = Automatic fast average</li>
 *             <li>Temp_Holdover_Slow_Average = Automatic slow average</li></ul>
 */
T_idtTemp_holdover IDTDpll_GetTempHoldoverMode(T_idtDrvHdlr hdlr,
                                               T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERTS(dpllType==Dpll_T0,
                ("DPLL %d does not support this feature.\n",nDpll));

    return (T_idtTemp_holdover) IDTHAL_ReadBitfield(hdlr, REG_T0_HOLDOVER_MODE_CNFG,
                                                    BF_TEMP_HOLDOVER_MODE_MASK, BF_TEMP_HOLDOVER_MODE_LSB);
}

/**
 * @brief Get  holdover frequency offset (in parts per trillion). In DCO
 * mode, this function returns current DCO offset (in parts per trillion).
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return Parts per trillion
 */
long IDTDpll_GetHoldoverFreq(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint32 nOffset;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    /* Combine 3 registers into value */
    nOffset = (T_osUint32) IDTHal_Read(hdlr, REG_HOLDOVER_FREQ_LOW_CNFG);
    nOffset |= (T_osUint32) (IDTHal_Read(hdlr, REG_HOLDOVER_FREQ_MID_CNFG)
                             << BF_HOLDOVER_FREQ_A_SIZE);
    nOffset |= (T_osUint32) (IDTHal_Read(hdlr, REG_HOLDOVER_FREQ_HIGH_CNFG)
                             << (BF_HOLDOVER_FREQ_A_SIZE + BF_HOLDOVER_FREQ_B_SIZE));

    /* Convert from 24 bit 2's compliment to standard long */
    if (nOffset & 0x00800000)
    {
        /* Negative value (sign extend) */
        return ((long)(nOffset | ~(0x7FFFFF)) * DCO_PPT2DCOUNITS);
    }
    else
    {
        /* Positive value - no changes required */
        return (nOffset * DCO_PPT2DCOUNITS);
    }
}

/**
 * @brief Set DCO frequency to device. Note this function should only
 * be called when DCO mode is enabled.
 *
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param pptOffset Parts per trillion offset.
 */
void IDTDpll_SetHoldoverFreq(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, long pptOffset)
{
    T_idtDpllType dpllType;
    T_osUint32 registerOffsetVal;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d pptOffset/%ld\n", __FUNCTION__, hdlr->name_m, nDpll, pptOffset));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Check if we support DCO mode for this DPLL - we can't write to
       this register if DCO mode is not supported. */
    OS_ASSERTS(hdlr->deviceConfig_m->dcoModeSupport_ma[nDpll],
                ("DPLL %d does not supprt DCO mode\n",nDpll+1));        

    /* Translate parts per trillion to register value
     *
     * register value is in 24 bit 2's compliment */
    if (pptOffset < 0)
    {
        /* Negative values */
        registerOffsetVal = (T_osUint32) (-pptOffset / DCO_PPT2DCOUNITS);
        registerOffsetVal = (~(registerOffsetVal) + 1) & 0x00FFFFFF;
    }
    else
    {
        /* Positive values */
        registerOffsetVal = (T_osUint32) (pptOffset / DCO_PPT2DCOUNITS);
    }

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    /* Split up offset value into 3 parts for 3 registers */
    IDTHal_Write(hdlr, REG_HOLDOVER_FREQ_LOW_CNFG,
                 (registerOffsetVal & BF_HOLDOVER_FREQ_A_MASK));
    IDTHal_Write(hdlr, REG_HOLDOVER_FREQ_MID_CNFG,
                 ((registerOffsetVal >> BF_HOLDOVER_FREQ_A_SIZE)
                  & BF_HOLDOVER_FREQ_B_MASK));
    IDTHal_Write(hdlr, REG_HOLDOVER_FREQ_HIGH_CNFG,
                 ((registerOffsetVal
                   >> (BF_HOLDOVER_FREQ_B_SIZE + BF_HOLDOVER_FREQ_A_SIZE))
                  & BF_HOLDOVER_FREQ_C_MASK));
}

/**
 * @brief Get current frequency offset of DPLL in parts per trillion (PPT)
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return	Frequency offset in parts per trillion
 */
long IDTDpll_GetCurrentDpllFreqOffset(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint32 nOffset;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    /* Combine 3 registers into value */
    nOffset = (T_osUint32) IDTHal_Read(hdlr, REG_CURRENT_FREQ_LOW_STS);
    nOffset |= (T_osUint32) (IDTHal_Read(hdlr, REG_CURRENT_FREQ_MID_STS)
                             << BF_CURRENT_DPLL_FREQ_A_SIZE);
    nOffset |= (T_osUint32) (IDTHal_Read(hdlr, REG_CURRENT_FREQ_HIGH_STS)
                             << (BF_CURRENT_DPLL_FREQ_A_SIZE + BF_CURRENT_DPLL_FREQ_B_SIZE));

    /* Convert from 24 bit 2's compliment to standard long */
    if (nOffset & 0x00800000)
    {
        /* Negative value (sign extend) */
        return ((long)(nOffset | ~(0x7FFFFF)) * DCO_PPT2DCOUNITS);
    }
    else
    {
        /* Positive value - no changes required */
        return (nOffset * DCO_PPT2DCOUNITS);
    }
}

/**
 * @brief   Set the limitation of Soft alarm of DPLL
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nLimit A 2' complement signed integer represents limitation of soft alarm.
 * 			     It is a multiple, the real limitation is:  nLimit * 0.724
 */
void IDTDpll_SetDpllSoftAlarmLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                   T_osUint8 nLimit)
{
    T_idtDpllType dpllType;

    OS_LOGGER(5,
               ("%s dev/%s nLimit/%d\n", __FUNCTION__, hdlr->name_m, nLimit));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERTS(dpllType==Dpll_T0,
                ("DPLL %d does not support this feature.\n",nDpll));
    OS_ASSERT(nLimit<(1<<BF_DPLL_FREQ_SOFT_LIMT_SIZE));

    IDTHAL_WriteBitfield(hdlr, REG_DPLL_FREQ_SOFT_LIMIT_CNFG,
                         BF_DPLL_FREQ_SOFT_LIMT_MASK, BF_DPLL_FREQ_SOFT_LIMT_LSB, nLimit);
}

/**
 * @brief Get the limitation of Soft alarm of DPLL
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return a 2' complement signed integer represents limitation of soft alarm.
 *          It is a multiple, the real limitation is:  nLimit * 0.724
 */
T_osUint8 IDTDpll_GetDpllSoftAlarmLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERTS(dpllType==Dpll_T0,
                ("DPLL %d does not support this feature.\n",nDpll));

    return IDTHAL_ReadBitfield(hdlr, REG_DPLL_FREQ_SOFT_LIMIT_CNFG,
                               BF_DPLL_FREQ_SOFT_LIMT_MASK, BF_DPLL_FREQ_SOFT_LIMT_LSB);
}

/**
 * @brief   Set the limitation of Hard alarm of DPLL
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nLimit A 2' complement signed integer represents limitation of hard alarm.
 *			     It is a multiple, the real limitation is:  nLimit * 0.0014
 */
void IDTDpll_SetDpllHardAlarmLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                   T_osUint16 nLimit)
{
    T_idtDpllType dpllType;

    OS_LOGGER(5,
               ("%s dev/%s nLimit/%d\n", __FUNCTION__, hdlr->name_m, nLimit));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERTS(dpllType==Dpll_T0,
                ("DPLL %d does not support this feature.\n",nDpll));
    OS_ASSERT(nLimit<(1<<(BF_DPLL_FREQ_HARD_LIMT_A_SIZE +
                           BF_DPLL_FREQ_HARD_LIMT_B_SIZE)));

    IDTHal_Write(hdlr, REG_DPLL_FREQ_HARD_LIMIT_LOW_CNFG,
                 (nLimit & BF_DPLL_FREQ_HARD_LIMT_A_MASK));
    IDTHal_Write(hdlr, REG_DPLL_FREQ_HARD_LIMIT_MID_CNFG,
                 ((nLimit >> BF_DPLL_FREQ_HARD_LIMT_A_SIZE)
                  & BF_DPLL_FREQ_HARD_LIMT_B_MASK));
}

/**
 * @brief Get the limitation of Hard alarm of DPLL
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return a 2' complement signed integer represents limitation of hard alarm.
 *          It is a multiple, the real limitation is:  nLimit * 0.0014
 */
T_osUint16 IDTDpll_GetDpllHardAlarmLimit(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERTS(dpllType==Dpll_T0,
                ("DPLL %d does not support this feature.\n",nDpll));

    return (T_osUint16) (IDTHal_Read(hdlr, REG_DPLL_FREQ_HARD_LIMIT_LOW_CNFG)
                        | (IDTHal_Read(hdlr, REG_DPLL_FREQ_HARD_LIMIT_MID_CNFG) << 
                           BF_DPLL_FREQ_HARD_LIMT_A_SIZE));
}

/**
 * @brief Set DPLL in unlocked state when a Hard alarm raised
 * @param hdlr Device driver handler
 * @param nEnable <ul>
 *               <li>0= Ignore Hard alarm, DPLL continues operation</li>
 *               <li>1= DPLL is unlock if Hard alarm raised</li></ul>
 */
void IDTDpll_SetDpllHardAlarmEnable(T_idtDrvHdlr hdlr, T_osUint8 nEnable)
{
    OS_LOGGER(5,
               ("%s dev/%s nEnable/%d\n", __FUNCTION__, hdlr->name_m, nEnable));
    OS_ASSERT_PTR_NULL(hdlr);

    IDTHAL_WriteBitfield(hdlr, REG_DPLL_FREQ_SOFT_LIMIT_CNFG,
                         BF_FREQ_LIMT_PH_LOS_MASK, BF_FREQ_LIMT_PH_LOS_LSB, nEnable);
}

/**
 * @brief Get DPLL in unlocked state when a Hard alarm raised
 * @param hdlr Device driver handler
 * @return <ul><li>0= Ignore Hard alarm, DPLL continues operation</li>
 *                     <li>1= DPLL is unlock if Hard alarm raised</li></ul>
 */
T_osUint8 IDTDpll_GetDpllHardAlarmEnable(T_idtDrvHdlr hdlr)
{
    OS_ASSERT_PTR_NULL(hdlr);

    return IDTHAL_ReadBitfield(hdlr, REG_DPLL_FREQ_SOFT_LIMIT_CNFG,
                               BF_FREQ_LIMT_PH_LOS_MASK, BF_FREQ_LIMT_PH_LOS_LSB);
}

/**
 * @brief   Get the current phase error of DPLL
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return	nError: a 2' complement signed integer represents current phase error.
 * 			It is a multiple, the real phase error is:  nError * 0.0014
 */
T_osUint16 IDTDpll_GetCurrentPhaseError(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint16 nError;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Select Dpll */
    IDTHAL_WriteBitfield(hdlr, REG_T4_T0_SEL_CNFG, BF_T4_T0_SEL_MASK,
                         BF_T4_T0_SEL_LSB, dpllType);

    nError = (T_osUint16) (IDTHal_Read(hdlr, REG_CURRENT_DPLL_PHASE_LOW_STS));
    nError |= (T_osUint16) (IDTHal_Read(hdlr, REG_CURRENT_DPLL_PHASE_HIGH_STS)
                           << BF_CURRENT_PH_DATA_A_SIZE);
    return nError;
}

/**
 * @brief Set Sonet/GigE path configuration.
 * @param hdlr Device driver handler
 * @param instance Sonet/GigE path instance 0..1
 * @param inputDpll Dpll instance
 * @param outputFreq Output frequency
 */
void IDTDpll_SetSonetGigEthOutputPath(T_idtDrvHdlr hdlr, T_osUint8 instance,
                               T_idtDpllInstance inputDpll, T_idtSonetGigEthOutput outputFreq)
{
    T_idtDpllType dpllType;
    T_idtSonetGigEthSelector bfVal;

    OS_LOGGER(5,
               ("%s dev/%s instance/%d inputDpll/%d outputFreq/%d\n", __FUNCTION__, hdlr->name_m, instance, inputDpll, outputFreq));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERTS(instance<hdlr->deviceConfig_m->numberOfSonetGigEthPath_m,
                ("Sonet GigEth path instance %d not supported (%d)\n",
                 instance, hdlr->deviceConfig_m->numberOfSonetGigEthPath_m));
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, inputDpll, dpllType);
    OS_ASSERT(outputFreq<SonetGigEthOutput_MAX);

    /* Convert { dpllType, outputFreq } to bitfield value */
    bfVal = IDTDpll_sonetGigEthPathConfig2Bitfield[dpllType][outputFreq];

    /* Really, Sonet/GigE pathes are not tied to a particular DPLL */
    if (instance == 0)
    {
        if ((SonetGigEthSel_T0Dpll10GbE_6664 == bfVal)||
            (SonetGigEthSel_T4Dpll10GbE_6664 == bfVal))
        {
            IDTHal_Write(hdlr, 0x2D, 0x01);
            IDTHal_Write(hdlr, 0x3C, 0x13);
            IDTHal_Write(hdlr, 0x2D, 0x00);
            IDTHAL_WriteBitfield(hdlr, REG_T0_DPLL_APLL_PATH_CNFG,
                             BF_T0_APLL_PATH_MASK, BF_T0_APLL_PATH_LSB,
                             SonetGigEthSel_T0Dpll10GbE_6664 == bfVal ?
                             SonetGigEthSel_T0DpllSonet : SonetGigEthSel_T4DpllSonet);
        }
        else
        {
            IDTHal_Write(hdlr, 0x2D, 0x01);
            IDTHal_Write(hdlr, 0x3C, 0x03);
            IDTHal_Write(hdlr, 0x2D, 0x00);
        }

        IDTHAL_WriteBitfield(hdlr, REG_T0_DPLL_APLL_PATH_CNFG,
                             BF_T0_APLL_PATH_MASK, BF_T0_APLL_PATH_LSB, (T_osUint8)bfVal);
    }
    else
    {
        if ((SonetGigEthSel_T0Dpll10GbE_6664 == bfVal)||
            (SonetGigEthSel_T4Dpll10GbE_6664 == bfVal))
        {
            IDTHal_Write(hdlr, 0x2D, 0x01);
            IDTHal_Write(hdlr, 0x46, 0x13);
            IDTHal_Write(hdlr, 0x2D, 0x00);
            IDTHAL_WriteBitfield(hdlr, REG_T4_DPLL_APLL_PATH_CNFG,
                             BF_T4_APLL_PATH_MASK, BF_T4_APLL_PATH_LSB,
                             SonetGigEthSel_T0Dpll10GbE_6664 == bfVal ?
                             SonetGigEthSel_T0DpllSonet : SonetGigEthSel_T4DpllSonet);
        }
        else
        {
            IDTHal_Write(hdlr, 0x2D, 0x01);
            IDTHal_Write(hdlr, 0x46, 0x03);
            IDTHal_Write(hdlr, 0x2D, 0x00);
        }

        IDTHAL_WriteBitfield(hdlr, REG_T4_DPLL_APLL_PATH_CNFG,
                             BF_T4_APLL_PATH_MASK, BF_T4_APLL_PATH_LSB, (T_osUint8)bfVal);
    }
}

/**
 * @brief Retrieve Sonet/GigE path configuration.
 * @param hdlr Device driver handler
 * @param instance Sonet/GigE path instance 0..1
 * @param inputDpll_p Provisioned DPLL instance of Sonet/GigE path
 * @param outputFreq_p Provisioned output frequency of Sonet/GigE path
 */
void IDTDpll_GetSonetGigEthOutputPath(T_idtDrvHdlr hdlr, T_osUint8 instance,
                               T_idtDpllInstance *inputDpll_p, T_idtSonetGigEthOutput *outputFreq_p)
{
    T_idtSonetGigEthSelector bfVal;
    const T_idtSonetGigEthBitfield2PathConfig *ptr;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(instance<hdlr->deviceConfig_m->numberOfSonetGigEthPath_m);
    OS_ASSERT_PTR_NULL(inputDpll_p);
    OS_ASSERT_PTR_NULL(outputFreq_p);

    /* Really, Sonet/GigE pathes are not tied to a particular DPLL */
    if (instance == 0)
        bfVal =
            (T_idtSonetGigEthSelector) IDTHAL_ReadBitfield(hdlr, REG_T0_DPLL_APLL_PATH_CNFG,
                                                    BF_T0_APLL_PATH_MASK, BF_T0_APLL_PATH_LSB);
    else
        bfVal =
            (T_idtSonetGigEthSelector) IDTHAL_ReadBitfield(hdlr, REG_T4_DPLL_APLL_PATH_CNFG,
                                                    BF_T4_APLL_PATH_MASK, BF_T4_APLL_PATH_LSB);

    /* Convert bitfield value to path config */
    ptr = &IDTDpll_sonetGigEthBitfield2PathConfig[bfVal];
    OS_ASSERT(ptr->inputDpll_m != DPLL_INSTANCE_MAX);

    *outputFreq_p = ptr->outputFreq_m;

    /* Translate DPLL type to instance */
    *inputDpll_p = hdlr->deviceConfig_m->pllInt2ExtXlate_ma[ptr->inputDpll_m];
}

/**
 * @brief Set DPLL output path selector A.
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param path DPLL selector path
 */
void IDTDpll_SetDpllOutputPathSelectorA(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                        T_idtDpllOutputSelectorA path)
{
    T_idtDpllType dpllType;
    T_osUint8 bfVal;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d path/%d\n", __FUNCTION__, hdlr->name_m, nDpll, path));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERT(path<DPLLOutputSelA_MAX);

    /* Translate from DPLL selector A output path to bitfield value */
    bfVal = IDTDpll_dpllSelAPathConfig2Bitfield[dpllType][path];

    if (dpllType == Dpll_T0)
    {
        OS_ASSERTS(bfVal != T0DpllSelA_MAX,
                    ("Path %d not supported by DPLL %d\n",
                     path, nDpll));
        IDTHAL_WriteBitfield(hdlr, REG_T0_DPLL_APLL_PATH_CNFG,
                             BF_T0_GSM_OBSAI_16E1_16T1_SEL_MASK,
                             BF_T0_GSM_OBSAI_16E1_16T1_SEL_LSB, bfVal);
    }
    else
    {
        OS_ASSERTS(bfVal != T4DpllSelA_MAX,
                    ("Path %d not supported by DPLL %d\n",
                     path, nDpll));
        IDTHAL_WriteBitfield(hdlr, REG_T4_DPLL_APLL_PATH_CNFG,
                             BF_T4_GSM_OBSAI_16E1_16T1_SEL_MASK,
                             BF_T4_GSM_OBSAI_16E1_16T1_SEL_LSB, bfVal);
    }
}

/**
 * @brief Get DPLL output path selector A.
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return DPLL selector path
 */
T_idtDpllOutputSelectorA IDTDpll_GetDpllOutputPathSelectorA(T_idtDrvHdlr hdlr,
                                                            T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint8 bfVal;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    if (dpllType == Dpll_T0)
    {
        bfVal = IDTHAL_ReadBitfield(hdlr, REG_T0_DPLL_APLL_PATH_CNFG,
                                    BF_T0_GSM_OBSAI_16E1_16T1_SEL_MASK,
                                    BF_T0_GSM_OBSAI_16E1_16T1_SEL_LSB);
        if (bfVal == 3)
            return DPLLOutputSelA_OBSAI;
        else
            return (T_idtDpllOutputSelectorA) bfVal;
    }
    else
    {
        bfVal = IDTHAL_ReadBitfield(hdlr, REG_T4_DPLL_APLL_PATH_CNFG,
                                    BF_T4_GSM_OBSAI_16E1_16T1_SEL_MASK,
                                    BF_T4_GSM_OBSAI_16E1_16T1_SEL_LSB);
        if (bfVal == 3)
            return DPLLOutputSelA_GPS;
        else
            return (T_idtDpllOutputSelectorA) bfVal;
    }
}

/**
 * @brief Set DPLL output path selector B.
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param path DPLL selector path
 */
void IDTDpll_SetDpllOutputPathSelectorB(T_idtDrvHdlr             hdlr, 
                                        T_idtDpllInstance        nDpll,
                                        T_idtDpllOutputSelectorB path)
{
    T_idtDpllType dpllType;
    T_osUint8 bfVal;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d path/%d\n", __FUNCTION__, hdlr->name_m, nDpll, path));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERT(path<DPLLOutputSelB_MAX);

    /* Translate from DPLL selector A output path to bitfield value */
    bfVal = IDTDpll_dpllSelBPathConfig2Bitfield[dpllType][path];

    if (dpllType == Dpll_T0)
    {
        OS_ASSERTS(bfVal != T0DpllSelB_MAX,
                    ("Path %d not supported by DPLL %d\n",
                     path, nDpll));

        IDTHAL_WriteBitfield(hdlr, 
                             REG_T0_DPLL_APLL_PATH_CNFG,
                             BF_T0_12E1_GPS_E3_T3_SEL_MASK, 
                             BF_T0_12E1_GPS_E3_T3_SEL_LSB,
                             bfVal);
    }
    else
    {
        OS_ASSERTS(bfVal != T4DpllSelB_MAX,
                    ("Path %d not supported by DPLL %d\n",
                     path, nDpll));
        IDTHAL_WriteBitfield(hdlr, 
                             REG_T4_DPLL_APLL_PATH_CNFG,
                             BF_T4_12E1_GPS_E3_T3_SEL_MASK, 
                             BF_T4_12E1_GPS_E3_T3_SEL_LSB,
                             bfVal);
    }
}

/**
 * @brief Get DPLL output path selector B.
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return DPLL selector path
 */
T_idtDpllOutputSelectorB IDTDpll_GetDpllOutputPathSelectorB(T_idtDrvHdlr hdlr,
                                                            T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint8 bfVal;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    if (dpllType == Dpll_T0)
    {
        bfVal = IDTHAL_ReadBitfield(hdlr,  
                                    REG_T0_DPLL_APLL_PATH_CNFG,
                                    BF_T0_12E1_GPS_E3_T3_SEL_MASK, 
                                    BF_T0_12E1_GPS_E3_T3_SEL_LSB);
        if (bfVal == 1)
            return DPLLOutputSelB_GPS;
        else
            return (T_idtDpllOutputSelectorB) bfVal;
    }
    else
    {
        bfVal = IDTHAL_ReadBitfield(hdlr, REG_T4_DPLL_APLL_PATH_CNFG,
                                    BF_T4_12E1_GPS_E3_T3_SEL_MASK, 
                                    BF_T4_12E1_GPS_E3_T3_SEL_LSB);
        if (bfVal == 1)
            return DPLLOutputSelB_24T1;
        else
            return (T_idtDpllOutputSelectorB) bfVal;
    }
}

/**
 * @brief   Set ETH Enable/Disable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nDisable <ul><li>0= Enable</li>
 *                       <li>1= Disable</li></ul>
 */
void IDTDpll_SetDpllEthPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                  T_osUint8 nDisable)
{
    T_idtDpllType dpllType;
    T_osUint8 bfMask;
    T_osUint8 bfLsb;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nDisable/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nDisable));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {
        if (dpllType == Dpll_T0)
        {
            bfMask = BF_T0_DFS_OFF_3_MASK;
            bfLsb = BF_T0_DFS_OFF_3_LSB;
        }
        else
        {
            bfMask = BF_T4_DFS_OFF_3_MASK;
            bfLsb = BF_T4_DFS_OFF_3_LSB;
        }
        IDTHAL_WriteBitfield(hdlr, REG_DFS_OFF_CNFG, bfMask, bfLsb, nDisable);
    }
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);
}

/**
 * @brief Get ETH Enable/Disable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return <ul><li>0= Enable</li>
 *             <li>1= Disable</li></ul>
 */
T_osUint8 IDTDpll_GetDpllEthPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint8 bfMask;
    T_osUint8 bfLsb;
    T_osUint8 retVal;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Change to page 1 */
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {
        if (dpllType == Dpll_T0)
        {
            bfMask = BF_T0_DFS_OFF_3_MASK;
            bfLsb = BF_T0_DFS_OFF_3_LSB;
        }
        else
        {
            bfMask = BF_T4_DFS_OFF_3_MASK;
            bfLsb = BF_T4_DFS_OFF_3_LSB;
        }
        retVal = IDTHAL_ReadBitfield(hdlr, REG_DFS_OFF_CNFG, bfMask, bfLsb);
    }
    /* Change back to page 0 */
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);

    return retVal;
}

/**
 * @brief Set DPLL output path selector B Enable/Disable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nDisable <ul><li>0= Enable</li>
 *                     <li>1= Disable</li></ul>
 */
void IDTDpll_SetDpllSelBPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                   T_osUint8 nDisable)
{
    T_idtDpllType dpllType;
    T_osUint8 bfMask;
    T_osUint8 bfLsb;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nDisable/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nDisable));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {
        if (dpllType == Dpll_T0)
        {
            bfMask = BF_T0_DFS_OFF_2_MASK;
            bfLsb = BF_T0_DFS_OFF_2_LSB;
        }
        else
        {
            bfMask = BF_T4_DFS_OFF_2_MASK;
            bfLsb = BF_T4_DFS_OFF_2_LSB;
        }
        IDTHAL_WriteBitfield(hdlr, REG_DFS_OFF_CNFG, bfMask, bfLsb, nDisable);
    }
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);
}

/**
 * @brief Get DPLL output path selector B Enable/Disable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return <ul><li>0= Enable</li>
 *             <li>1= Disable</li></ul>
 */
T_osUint8 IDTDpll_GetDpllSelBPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint8 bfMask;
    T_osUint8 bfLsb;
    T_osUint8 retVal;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {
        if (dpllType == Dpll_T0)
        {
            bfMask = BF_T0_DFS_OFF_2_MASK;
            bfLsb = BF_T0_DFS_OFF_2_LSB;
        }
        else
        {
            bfMask = BF_T4_DFS_OFF_2_MASK;
            bfLsb = BF_T4_DFS_OFF_2_LSB;
        }
        retVal = IDTHAL_ReadBitfield(hdlr, REG_DFS_OFF_CNFG, bfMask, bfLsb);
    }
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);

    return retVal;
}

/**
 * @brief   Set 16E1/16T1 Enable/Disable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nDisable <ul><li>0= Enable</li>
 *                       <li>1= Disable</li></ul>
 */
void IDTDpll_SetDpll16E116T1Enable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                   T_osUint8 nDisable)
{
    T_idtDpllType dpllType;
    T_osUint8 bfMask;
    T_osUint8 bfLsb;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nDisable/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nDisable));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {
        if (dpllType == Dpll_T0)
        {
            bfMask = BF_T0_DFS_OFF_1_MASK;
            bfLsb = BF_T0_DFS_OFF_1_LSB;
        }
        else
        {
            bfMask = BF_T4_DFS_OFF_1_MASK;
            bfLsb = BF_T4_DFS_OFF_1_LSB;
        }
        IDTHAL_WriteBitfield(hdlr, REG_DFS_OFF_CNFG, bfMask, bfLsb, nDisable);
    }
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);
}

/**
 * @brief Get 16E1/16T1 Enable/Disable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return <ul><li>0= Enable</li>
 *             <li>1= Disable</li></ul>
 */
T_osUint8 IDTDpll_GetDpll16E116T1Enable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint8 bfMask;
    T_osUint8 bfLsb;
    T_osUint8 retVal;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {
        if (dpllType == Dpll_T0)
        {
            bfMask = BF_T0_DFS_OFF_1_MASK;
            bfLsb = BF_T0_DFS_OFF_1_LSB;
        }
        else
        {
            bfMask = BF_T4_DFS_OFF_1_MASK;
            bfLsb = BF_T4_DFS_OFF_1_LSB;
        }
        retVal = IDTHAL_ReadBitfield(hdlr, REG_DFS_OFF_CNFG, bfMask, bfLsb);
    }
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);

    return retVal;
}

/**
 * @brief Set DPLL output path selector A Enable/Disable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nDisable <ul><li>0= Enable</li>
 *                       <li>1= Disable</li></ul>
 */
void IDTDpll_SetDpllSelAPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                                   T_osUint8 nDisable)
{
    T_idtDpllType dpllType;
    T_osUint8 bfMask;
    T_osUint8 bfLsb;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nDisable/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nDisable));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {
        if (dpllType == Dpll_T0)
        {
            bfMask = BF_T0_DFS_OFF_0_MASK;
            bfLsb = BF_T0_DFS_OFF_0_LSB;
        }
        else
        {
            bfMask = BF_T4_DFS_OFF_0_MASK;
            bfLsb = BF_T4_DFS_OFF_0_LSB;
        }
        IDTHAL_WriteBitfield(hdlr, REG_DFS_OFF_CNFG, bfMask, bfLsb, nDisable);
    }
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);
}

/**
 * @brief Set DPLL output path selector A Enable/Disable
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return <ul><li>0= Enable</li>
 *             <li>1= Disable</li></ul>
 */
T_osUint8 IDTDpll_GetDpllSelAPathEnable(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint8 bfMask;
    T_osUint8 bfLsb;
    T_osUint8 retVal;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {
        if (dpllType == Dpll_T0)
        {
            bfMask = BF_T0_DFS_OFF_0_MASK;
            bfLsb = BF_T0_DFS_OFF_0_LSB;
        }
        else
        {
            bfMask = BF_T4_DFS_OFF_0_MASK;
            bfLsb = BF_T4_DFS_OFF_0_LSB;
        }
        retVal = IDTHAL_ReadBitfield(hdlr, REG_DFS_OFF_CNFG, bfMask, bfLsb);
    }
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);

    return retVal;
}


/**
 * @brief Function to set the frequency monitoring factor. Hard and soft alarm
 * thresholds use this factor as their base unit.
 *
 * @param hdlr Device driver handler
 * @param freqMonFactor Frequency monitoring factor value.
 *                      <ul><li>FreqMonFactor_0p0032 - 0.0032 ppm</li>
 *                          <li>FreqMonFactor_0p0064 - 0.0064 ppm</li>
 *                          <li>FreqMonFactor_0p0127 - 0.0127 ppm</li>
 *                          <li>FreqMonFactor_0p0257 - 0.0257 ppm</li>
 *                          <li>FreqMonFactor_0p0514 - 0.0514 ppm</li>
 *                          <li>FreqMonFactor_0p1030 - 0.103  ppm</li>
 *                          <li>FreqMonFactor_0p2060 - 0.206  ppm</li>
 *                          <li>FreqMonFactor_0p4120 - 0.412  ppm</li>
 *                          <li>FreqMonFactor_0p8230 - 0.823  ppm</li>
 *                          <li>FreqMonFactor_1p6460 - 1.646  ppm</li>
 *                          <li>FreqMonFactor_3p2920 - 3.29   ppm</li>
 *                          <li>FreqMonFactor_3p8100 - 3.81   ppm</li>
 *                          <li>FreqMonFactor_4p6000 - 4.6    ppm</li></ul>
 */
void IDTDpll_SetFrequencyMonitoringFactor(T_idtDrvHdlr hdlr,
                                          T_idtFreqMonFactor freqMonFactor)
{
    OS_LOGGER(5,
               ("%s dev/%s freqMonFactor/%d\n", __FUNCTION__, hdlr->name_m, freqMonFactor));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(freqMonFactor<FreqMonFactor_MAX);

    IDTHAL_WriteBitfield(hdlr, REG_FREQ_MON_FACTOR_CNFG,
                         BF_FREQ_MON_FACTOR_MASK, BF_FREQ_MON_FACTOR_LSB, freqMonFactor);
}

/**
 * @brief Function to get the frequency monitoring factor. Hard and soft alarm
 * thresholds use this factor as their base unit.
 *
 * @param hdlr Device driver handler
 * @return Frequency monitoring factor value.
 *                      <ul><li>FreqMonFactor_0p0032 - 0.0032 ppm</li>
 *                          <li>FreqMonFactor_0p0064 - 0.0064 ppm</li>
 *                          <li>FreqMonFactor_0p0127 - 0.0127 ppm</li>
 *                          <li>FreqMonFactor_0p0257 - 0.0257 ppm</li>
 *                          <li>FreqMonFactor_0p0514 - 0.0514 ppm</li>
 *                          <li>FreqMonFactor_0p1030 - 0.103  ppm</li>
 *                          <li>FreqMonFactor_0p2060 - 0.206  ppm</li>
 *                          <li>FreqMonFactor_0p4120 - 0.412  ppm</li>
 *                          <li>FreqMonFactor_0p8230 - 0.823  ppm</li>
 *                          <li>FreqMonFactor_1p6460 - 1.646  ppm</li>
 *                          <li>FreqMonFactor_3p2920 - 3.29   ppm</li>
 *                          <li>FreqMonFactor_3p8100 - 3.81   ppm</li>
 *                          <li>FreqMonFactor_4p6000 - 4.6    ppm</li></ul>
 */
T_idtFreqMonFactor IDTDpll_GetFrequencyMonitoringFactor(T_idtDrvHdlr hdlr)
{
    OS_ASSERT_PTR_NULL(hdlr);

    return (T_idtFreqMonFactor) IDTHAL_ReadBitfield(hdlr, REG_FREQ_MON_FACTOR_CNFG,
                                                    BF_FREQ_MON_FACTOR_MASK, BF_FREQ_MON_FACTOR_LSB);
}

/**
 * @brief Set the frequency hard alarm accepting thresholds of
 * reference clock monitoring.
 *
 * @param hdlr Device driver handler
 * @param accepting Accepting threshold (ppm) is a multiple of frequency
 * monitoring factor value.
 *            Accepting Threshold = (AcceptingValue + 1)*(FreqMonFactor)
 * @param rejecting Rejecting threshold (ppm) is a multiple of frequency
 * monitoring factor value.
 *            Rejecting Threshold = (RejectingValue + 1)*(FreqMonFactor)
 */
void IDTDpll_SetHardAlarmThreshold(T_idtDrvHdlr hdlr, T_osUint8 accepting,
                                   T_osUint8 rejecting)
{
    OS_LOGGER(5,
               ("%s dev/%s accepting/%d rejecting/%d\n", __FUNCTION__, hdlr->name_m, accepting, rejecting));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(accepting<(1<<BF_HARD_FREQ_MON_THRESHOLD_A_SIZE));
    OS_ASSERT(rejecting<(1<<BF_HARD_FREQ_MON_THRESHOLD_B_SIZE));

    IDTHAL_WriteBitfield(hdlr, REG_HARD_FREQ_MON_THRESHOLD_CNFG,
                         BF_HARD_FREQ_MON_THRESHOLD_A_MASK, BF_HARD_FREQ_MON_THRESHOLD_A_LSB,
                         accepting);
    IDTHAL_WriteBitfield(hdlr, REG_HARD_FREQ_MON_THRESHOLD_CNFG,
                         BF_HARD_FREQ_MON_THRESHOLD_B_MASK, BF_HARD_FREQ_MON_THRESHOLD_B_LSB,
                         rejecting);
}

/**
 * @brief Get the frequency hard alarm accepting thresholds of
 * reference clock monitoring.
 *
 * @param hdlr Device driver handler
 * @param accepting_p Accepting threshold (ppm) is a multiple of frequency
 * monitoring factor value.
 *            Accepting Threshold = (AcceptingValue + 1)*(FreqMonFactor)
 * @param rejecting_p Rejecting threshold (ppm) is a multiple of frequency
 * monitoring factor value.
 *            Rejecting Threshold = (RejectingValue + 1)*(FreqMonFactor)
 */
void IDTDpll_GetHardAlarmThreshold(T_idtDrvHdlr hdlr, T_osUint8 *accepting_p,
                                   T_osUint8 *rejecting_p)
{
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT_PTR_NULL(accepting_p);
    OS_ASSERT_PTR_NULL(rejecting_p);

    *accepting_p = IDTHAL_ReadBitfield(hdlr, REG_HARD_FREQ_MON_THRESHOLD_CNFG,
                                       BF_HARD_FREQ_MON_THRESHOLD_A_MASK, BF_HARD_FREQ_MON_THRESHOLD_A_LSB);
    *rejecting_p = IDTHAL_ReadBitfield(hdlr, REG_HARD_FREQ_MON_THRESHOLD_CNFG,
                                       BF_HARD_FREQ_MON_THRESHOLD_B_MASK, BF_HARD_FREQ_MON_THRESHOLD_B_LSB);
}

/**
 * @brief Set the frequency soft alarm accepting thresholds of
 * reference clock monitoring.
 *
 * @param hdlr Device driver handler
 * @param accepting Accepting threshold (ppm) is a multiple of frequency
 * monitoring factor value.
 *            Accepting Threshold = (AcceptingValue + 1)*(FreqMonFactor)
 * @param rejecting Rejecting threshold (ppm) is a multiple of frequency
 * monitoring factor value.
 *            Rejecting Threshold = (RejectingValue + 1)*(FreqMonFactor)
 */
void IDTDpll_SetSoftAlarmThreshold(T_idtDrvHdlr hdlr, T_osUint8 accepting,
                                   T_osUint8 rejecting)
{
    OS_LOGGER(5,
               ("%s dev/%s accepting/%d rejecting/%d\n", __FUNCTION__, hdlr->name_m, accepting, rejecting));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(accepting<(1<<BF_SOFT_FREQ_MON_THRESHOLD_A_SIZE));
    OS_ASSERT(rejecting<(1<<BF_SOFT_FREQ_MON_THRESHOLD_B_SIZE));

    IDTHAL_WriteBitfield(hdlr, REG_SOFT_FREQ_MON_THRESHOLD_CNFG,
                         BF_SOFT_FREQ_MON_THRESHOLD_A_MASK, BF_SOFT_FREQ_MON_THRESHOLD_A_LSB,
                         accepting);
    IDTHAL_WriteBitfield(hdlr, REG_SOFT_FREQ_MON_THRESHOLD_CNFG,
                         BF_SOFT_FREQ_MON_THRESHOLD_B_MASK, BF_SOFT_FREQ_MON_THRESHOLD_B_LSB,
                         rejecting);
}

/**
 * @brief Get the frequency soft alarm accepting thresholds of
 * reference clock monitoring.
 *
 * @param hdlr Device driver handler
 * @param accepting_p Accepting threshold (ppm) is a multiple of frequency
 * monitoring factor value.
 *            Accepting Threshold = (AcceptingValue + 1)*(FreqMonFactor)
 * @param rejecting_p Rejecting threshold (ppm) is a multiple of frequency
 * monitoring factor value.
 *            Rejecting Threshold = (RejectingValue + 1)*(FreqMonFactor)
 */
void IDTDpll_GetSoftAlarmThreshold(T_idtDrvHdlr hdlr, T_osUint8 *accepting_p,
                                   T_osUint8 *rejecting_p)
{
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT_PTR_NULL(accepting_p);
    OS_ASSERT_PTR_NULL(rejecting_p);

    *accepting_p = IDTHAL_ReadBitfield(hdlr, REG_SOFT_FREQ_MON_THRESHOLD_CNFG,
                                       BF_SOFT_FREQ_MON_THRESHOLD_A_MASK, BF_SOFT_FREQ_MON_THRESHOLD_A_LSB);
    *rejecting_p = IDTHAL_ReadBitfield(hdlr, REG_SOFT_FREQ_MON_THRESHOLD_CNFG,
                                       BF_SOFT_FREQ_MON_THRESHOLD_B_MASK, BF_SOFT_FREQ_MON_THRESHOLD_B_LSB);
}

/**
 * @brief Set charge pump control
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nIcp Charge pump current selection ( 0 - 0x1F );
 *                          10 = 40uA;
 */
void IDTDpll_SetIcpCtrl(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll, T_osUint8 nIcp)
{
    T_idtDpllType dpllType;
    T_osUint8 nPage;
    T_osUint8 regAddr;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nIcp/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nIcp));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERT(nIcp<(1<<BF_ICP_CTRL_CODE_SIZE));

    nPage = IDTHal_Read(hdlr, REG_PAGE_POINTER_CNFG);
    if ((nPage & 0x01) == 0)
        IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {
		T_osUint8 regVal;
        if (dpllType == Dpll_T0)
        {
            regAddr = REG_T0_ICP_CTRL_CNFG;
        }
        else
        {
            regAddr = REG_T4_ICP_CTRL_CNFG;
        }

        /* Need to write twice for ICP writes */
		regVal = IDTHal_Read(hdlr,regAddr);
		IDTHAL_ModifyBitfield(regVal,BF_ICP_CTRL_CODE_MASK,BF_ICP_CTRL_CODE_LSB,nIcp);
		IDTHal_Write(hdlr,regAddr,regVal);
		IDTHal_Write(hdlr,regAddr,regVal);
    }
    if ((nPage & 0x01) == 0)
        IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);
}

/**
 * @brief Get charge pump control
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return Charge pump current selection ( 0 - 0x1F );
 *                          10 = 40uA;
 */
T_osUint8 IDTDpll_GetIcpCtrl(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint8 nPage;
    T_osUint8 regAddr;
    T_osUint8 regVal;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Change page if required */
    nPage = IDTHal_Read(hdlr, REG_PAGE_POINTER_CNFG);
    if ((nPage & 0x01) == 0)
        IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);

    if (dpllType == Dpll_T0)
    {
        regAddr = REG_T0_ICP_CTRL_CNFG;
    }
    else
    {
        regAddr = REG_T4_ICP_CTRL_CNFG;
    }
    regVal = IDTHAL_ReadBitfield(hdlr, regAddr, BF_ICP_CTRL_CODE_MASK,
                                 BF_ICP_CTRL_CODE_LSB);
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);

    return regVal;
}

/**
 * @brief Set DPLL phase slope for DPLLs
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nPhaseSlope <ul><li>phaseSlope_gr1244st3 = GR-1244 ST3 (61us/s)</li>
 *                        <li>phaseSlope_gr1244st3e = GR-1244 ST2,3E,ST3 (855ns/s)</li>
 *                        <li>phaseSlope_gr813 = GR-813,G.8262 (7.5us/s)</li>
 *                        <li>phaseSlope_noLimit = No limitation</li></ul>
 */
void IDTDpll_SetPhaseSlope(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll,
                           T_idtPhaseSlope nPhaseSlope)
{
    T_idtDpllType dpllType;
    T_osUint8 nPage;
    T_osUint8 bfMask;
    T_osUint8 bfLsb;

    OS_LOGGER(5,
               ("%s dev/%s nDpll/%d nPhaseSlope/%d\n", __FUNCTION__, hdlr->name_m, nDpll, nPhaseSlope));
    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    OS_ASSERTS( (hdlr->deviceConfig_m->phaseSlopeLimiting_ma[nDpll]),
                 ("DPLL %d does not support phase slope limiting\n",nDpll+1) );

    nPage = IDTHal_Read(hdlr, REG_PAGE_POINTER_CNFG);
    if ((nPage & 0x01) == 0)
        IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {
        if (dpllType == Dpll_T0)
        {
            bfMask = BF_T0_PH_SLOPE_MASK;
            bfLsb = BF_T0_PH_SLOPE_LSB;
        }
        else
        {
            bfMask = BF_T4_PH_SLOPE_MASK;
            bfLsb = BF_T4_PH_SLOPE_LSB;
        }
        IDTHAL_WriteBitfield(hdlr, REG_PH_SLOPE_CNFG, bfMask, bfLsb,
                             nPhaseSlope);
    }
    if ((nPage & 0x01) == 0)
        IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);
}

/**
 * @brief Get DPLL phase slope for DPLLs
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @return Phase slope <ul><li>phaseSlope_gr1244st3 = GR-1244 ST3 (61us/s)</li>
 *                        <li>phaseSlope_gr1244st3e = GR-1244 ST2,3E,ST3 (855ns/s)</li>
 *                        <li>phaseSlope_gr813 = GR-813,G.8262 (7.5us/s)</li>
 *                        <li>phaseSlope_noLimit = No limitation</li></ul>
 */
T_idtPhaseSlope IDTDpll_GetPhaseSlope(T_idtDrvHdlr hdlr, T_idtDpllInstance nDpll)
{
    T_idtDpllType dpllType;
    T_osUint8 nPage;
    T_osUint8 bfMask;
    T_osUint8 bfLsb;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    OS_ASSERTS( (hdlr->deviceConfig_m->phaseSlopeLimiting_ma[nDpll]),
                 ("DPLL %d does not support phase slope limiting\n",nDpll+1) );

    nPage = IDTHal_Read(hdlr, REG_PAGE_POINTER_CNFG);
    if ((nPage & 0x01) == 0)
        IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);

    if (dpllType == Dpll_T0)
    {
        bfMask = BF_T0_PH_SLOPE_MASK;
        bfLsb = BF_T0_PH_SLOPE_LSB;
    }
    else
    {
        bfMask = BF_T4_PH_SLOPE_MASK;
        bfLsb = BF_T4_PH_SLOPE_LSB;
    }
    return (T_idtPhaseSlope) IDTHAL_ReadBitfield(hdlr, REG_PH_SLOPE_CNFG, bfMask, bfLsb);
}

/**
 * @brief Enable/Disable 1 PPS fast lock
 * @param hdlr Device driver handler
 * @param nEnableFreq Enable (1) / Disable (0) frequency locking
 * @param nEnablePhase Enable (1) / Disable (0) phase locking
 */
void IDTDpll_SetPpsFastLock(T_idtDrvHdlr hdlr, T_osUint8 nEnableFreq,
                            T_osUint8 nEnablePhase)
{
    T_osUint8 regVal;

    OS_LOGGER(5,
               ("%s dev/%s nEnableFreq/%d nEnablePhase/%d\n", __FUNCTION__, hdlr->name_m, nEnableFreq, nEnablePhase));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nEnablePhase<(1<<BF_PPS_FAST_PH_LOCK_EN_SIZE));
    OS_ASSERT(nEnableFreq<(1<<BF_PPS_FAST_FREQ_LOCK_EN_SIZE));

    regVal = IDTHal_Read(hdlr, REG_MS_SL_CTRL_CNFG);

    IDTHAL_ModifyBitfield(regVal, BF_PPS_FAST_FREQ_LOCK_EN_MASK,
                          BF_PPS_FAST_FREQ_LOCK_EN_LSB, nEnableFreq);

    IDTHAL_ModifyBitfield(regVal, BF_PPS_FAST_PH_LOCK_EN_MASK,
                          BF_PPS_FAST_PH_LOCK_EN_LSB, nEnablePhase);

    IDTHal_Write(hdlr, REG_MS_SL_CTRL_CNFG, regVal);
}

/**
 * @brief Get enable/disable 1 PPS fast lock status
 * @param hdlr Device driver handler
 * @param nEnableFreq_p Enable (1) / Disable (0) frequency locking
 * @param nEnablePhase_p Enable (1) / Disable (0) phase locking
 */
void IDTDpll_GetPpsFastLock(T_idtDrvHdlr hdlr, T_osUint8 *nEnableFreq_p,
                            T_osUint8 *nEnablePhase_p)
{
    T_osUint8 regVal;

    OS_ASSERT_PTR_NULL(hdlr);

    regVal = IDTHal_Read(hdlr, REG_MS_SL_CTRL_CNFG);

    *nEnableFreq_p = IDTHAL_GetBitfield(regVal, 
                                        BF_PPS_FAST_FREQ_LOCK_EN_MASK,
                                        BF_PPS_FAST_FREQ_LOCK_EN_LSB);

    *nEnablePhase_p = IDTHAL_GetBitfield(regVal, 
                                         BF_PPS_FAST_PH_LOCK_EN_MASK,
                                         BF_PPS_FAST_PH_LOCK_EN_LSB);

}

/**
 * @brief Set the configuration when phase transient occurs
 * @param hdlr Device driver handler
 * @param   nEnable  <ul><li>0= Disable the phase transient monitor</li>
 *           <li>1= Enable the phase transient monitor</li></ul>
 */
void IDTDpll_SetPhaseTransient(T_idtDrvHdlr hdlr, T_osUint8 nEnable)
{

    OS_LOGGER(5,
               ("%s dev/%s nEnable/%d\n", __FUNCTION__, hdlr->name_m, nEnable));
    OS_ASSERT_PTR_NULL(hdlr);

    IDTHAL_WriteBitfield(hdlr, REG_PHASE_MON_CNFG, BF_IN_NOISE_WINDOW_MASK,
                         BF_IN_NOISE_WINDOW_LSB, nEnable);
}

/**
 * @brief Get the configuration when phase transient occurs
 * @param hdlr Device driver handler
 * @return <ul><li>0= Disable the phase transient monitor</li>
 *             <li>1= Enable the phase transient monitor</li></ul>
 */
T_osUint8 IDTDpll_GetPhaseTransient(T_idtDrvHdlr hdlr)
{
    OS_ASSERT_PTR_NULL(hdlr);

    return IDTHAL_ReadBitfield(hdlr, REG_PHASE_MON_CNFG, BF_IN_NOISE_WINDOW_MASK,
                               BF_IN_NOISE_WINDOW_LSB);

}


/**
 * @brief   Set the configuration of hitless switching feature
 * @param hdlr Device driver handler
 * @param   nEnable  <ul><li>0= Disable the hitless switching feature;</li>
 *           <li>1= Enable the hitless switching feature;</li></ul>
 * @param   nFrozen  <ul><li>0= hitless switching feature is not frozen;</li>
 *           <li>1= HS feature is frozen, ignore the further hitless switching events</li></ul>
 */
void IDTDpll_SetHitlessSwitchingFeature(T_idtDrvHdlr hdlr, T_osUint8 nEnable, T_osUint8 nFrozen)
{
    T_osUint8 nIntInd;

    OS_LOGGER(5,
               ("%s dev/%s nEnable/%d nFrozen/%d\n", __FUNCTION__, hdlr->name_m, nEnable, nFrozen));
    OS_ASSERT_PTR_NULL(hdlr);

    nIntInd = IDTHal_Read(hdlr, REG_MON_SW_HS_CNFG);

    IDTHAL_ModifyBitfield(nIntInd, BF_HS_EN_MASK, BF_HS_EN_LSB, nEnable);

    IDTHAL_ModifyBitfield(nIntInd, BF_HS_FREZ_MASK, BF_HS_FREZ_LSB, nFrozen);

    IDTHal_Write(hdlr, REG_MON_SW_HS_CNFG, nIntInd);
}

/**
 * @brief Get the configuration of hitless switching feature
 * @param hdlr Device driver handler
 * @param   nEnable_p  <ul><li>0= Disable the hitless switching feature;</li>
 *           <li>1= Enable the hitless switching feature;</li></ul>
 * @param   nFrozen_p  <ul><li>0= hitless switching feature is not frozen;</li>
 *           <li>1= HS feature is frozen, ignore the further hitless switching events</li></ul>
 */
void IDTDpll_GetHitlessSwitchingFeature(T_idtDrvHdlr hdlr, T_osUint8 *nEnable_p,
                                        T_osUint8 *nFrozen_p)
{
    T_osUint8 nIntInd;

    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT_PTR_NULL(nEnable_p);
    OS_ASSERT_PTR_NULL(nFrozen_p);

    nIntInd = IDTHal_Read(hdlr, REG_MON_SW_HS_CNFG);

    *nEnable_p = IDTHAL_GetBitfield(nIntInd, BF_HS_EN_MASK, BF_HS_EN_LSB);
    *nFrozen_p = IDTHAL_GetBitfield(nIntInd, BF_HS_FREZ_MASK, BF_HS_FREZ_LSB);
}

/**
 * @brief   Set the phase offset value
 * @param hdlr Device driver handler
 * @param   nOffset Phase offset= nOffset * 0x61 ns;
 */
void IDTDpll_SetPhaseOffset(T_idtDrvHdlr hdlr, T_osUint16 nOffset)
{

    OS_LOGGER(5,
               ("%s dev/%s nOffset/%d\n", __FUNCTION__, hdlr->name_m, nOffset));
    OS_ASSERT_PTR_NULL(hdlr);

    IDTHal_Write(hdlr, REG_PHASE_OFFSET_LOW_CNFG,
                 (T_osUint8) (nOffset & BF_PH_OFFSET_A_MASK));
    IDTHAL_WriteBitfield(hdlr, REG_PHASE_OFFSET_HIGH_CNFG, BF_PH_OFFSET_B_MASK,
                         BF_PH_OFFSET_B_LSB, (nOffset >> BF_PH_OFFSET_A_SIZE));
}

/**
 * @brief Get the phase offset value
 * @param hdlr Device driver handler
 * @return Phase offset= nOffset * 0x61 ns;
 */
T_osUint16 IDTDpll_GetPhaseOffset(T_idtDrvHdlr hdlr)
{
    T_osUint16 retVal;

    OS_ASSERT_PTR_NULL(hdlr);

    retVal = IDTHal_Read(hdlr, REG_PHASE_OFFSET_LOW_CNFG);
    retVal |= IDTHAL_ReadBitfield(hdlr, REG_PHASE_OFFSET_HIGH_CNFG, 
                                  BF_PH_OFFSET_B_MASK,
                                  BF_PH_OFFSET_B_LSB) << BF_PH_OFFSET_A_SIZE;
    return retVal;
}


/**
 * @brief Set PPS phase and pulse for DPLL
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nPhase  Phase position \see T_idtDpllPpsPhasePostion
 * @param nPulse  Pulse width \see T_idtDpllPpsPulseWidth
 */
void IDTDpll_SetPpsPhase(T_idtDrvHdlr             hdlr, 
                         T_idtDpllInstance        nDpll, 
                         T_idtDpllPpsPhasePostion nPhase, 
                         T_idtDpllPpsPulseWidth   nPulse)
{
    T_osUint8 nPage;
    T_osUint8 nIntInd;
    T_idtDpllType dpllType;

    OS_LOGGER(5,
               ("%s dev/%s nPhase/%d nPulse/%d\n", __FUNCTION__, hdlr->name_m, nPhase, nPulse));
    OS_ASSERT_PTR_NULL(hdlr);
    OS_ASSERT(nPhase<PpsPhasePostion_MAX);
    OS_ASSERT(nPulse<PpsPulseWidth_MAX);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);

    /* Register is located on page 1, need to change page to
       access register */
    nPage = IDTHal_Read(hdlr, REG_PAGE_POINTER_CNFG);
    if ((nPage & 0x01) == 0)
        IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {

        if (dpllType ==  Dpll_T0)
        {
            nIntInd = IDTHal_Read(hdlr, REG_T0_PPS_CNFG);

            IDTHAL_ModifyBitfield(nIntInd, BF_PPS_PHASE_MASK, BF_PPS_PHASE_LSB, nPhase);

            IDTHAL_ModifyBitfield(nIntInd, BF_PPS_PULSE_MASK, BF_PPS_PULSE_LSB, nPulse);

            IDTHal_Write(hdlr, REG_T0_PPS_CNFG, nIntInd);
        }
        else
        {
            /* T4 DPLL */
            nIntInd = IDTHal_Read(hdlr, REG_T4_PPS_CNFG);

            IDTHAL_ModifyBitfield(nIntInd, BF_PPS_PHASE_MASK, BF_PPS_PHASE_LSB, nPhase);

            IDTHAL_ModifyBitfield(nIntInd, BF_PPS_PULSE_MASK, BF_PPS_PULSE_LSB, nPulse);

            IDTHal_Write(hdlr, REG_T4_PPS_CNFG, nIntInd);
        }
    }
    if ((nPage & 0x01) == 0)
        IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);
}

/**
 * @brief Get PPS phase and pulse for DPLL
 * @param hdlr Device driver handler
 * @param nDpll DPLL instance
 * @param nPhase_p  Phase position \see T_idtDpllPpsPhasePostion
 * @param nPulse_p  Pulse width \see T_idtDpllPpsPulseWidth
 */
void IDTDpll_GetPpsPhase(T_idtDrvHdlr hdlr,
                         T_idtDpllInstance nDpll,
                         T_idtDpllPpsPhasePostion *nPhase_p, 
                         T_idtDpllPpsPulseWidth   *nPulse_p)
{
    T_osUint8 nPage;
    T_osUint8 nIntInd;
    T_idtDpllType dpllType;

    OS_ASSERT_PTR_NULL(hdlr);
    IDT_TRANSLATE_EXT_TO_INT_DPLL(hdlr, nDpll, dpllType);
    OS_ASSERT_PTR_NULL(nPhase_p);
    OS_ASSERT_PTR_NULL(nPulse_p);

    /* Register is located on page 1, need to change page to
       access register */
    nPage = IDTHal_Read(hdlr, REG_PAGE_POINTER_CNFG);
    if ((nPage & 0x01) == 0)
        IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x01);
    {

        if (dpllType ==  Dpll_T0)
        {
            nIntInd = IDTHal_Read(hdlr, REG_T0_PPS_CNFG);
        }
        else
        {
            /* T4 DPLL */
            nIntInd = IDTHal_Read(hdlr, REG_T4_PPS_CNFG);
        }
        *nPhase_p = IDTHAL_GetBitfield(nIntInd, BF_PPS_PHASE_MASK, BF_PPS_PHASE_LSB);

        *nPulse_p = IDTHAL_GetBitfield(nIntInd, BF_PPS_PULSE_MASK, BF_PPS_PULSE_LSB);
    }
    if ((nPage & 0x01) == 0)
        IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0x00);
}
