/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <check.h>

#include "osNumeric.h"
#include "Define.h"
#include "DpllCnfg.h"

#include "check_main.h"
/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

START_TEST (test_SetAutoSelInput)
{
    IDTDpll_SetAutoSelInput(hdlr, _i, 0);
    fail_unless(IDTDpll_GetAutoSelInput(hdlr, _i) == 0,
                "IDTDpll_SetAutoSelInput 0 not working");
    IDTDpll_SetAutoSelInput(hdlr, _i, 1);
    fail_unless(IDTDpll_GetAutoSelInput(hdlr, _i) == 1,
                "IDTDpll_SetAutoSelInput 1 not working");
}
END_TEST

START_TEST (test_SetForceSelInput)
{
    T_osUint8 nInputNo;

    for (nInputNo=1; nInputNo<IDT_DRV_MAX_INPUT+1; nInputNo++)
    { 
        T_osUint8 rdInputNo;

        IDTDpll_SetForceSelInput(hdlr, _i, nInputNo);
        rdInputNo = IDTDpll_GetForceSelInput(hdlr, _i);
        fail_unless(nInputNo == rdInputNo,
                    "IDTDpll_SetForceSelInput "
                    "nInputNo/%d:%d not working",
                    nInputNo, rdInputNo);
    }
}
END_TEST

START_TEST (test_GetInputPriority)
{
    T_osUint8 temp;

    /* We just want to check to see that it's not going to trap */
    temp = IDTDpll_Get1stPriorityInput(hdlr, _i);
    temp = IDTDpll_Get2ndPriorityInput(hdlr, _i);
    temp = IDTDpll_Get3rdPriorityInput(hdlr, _i);

    fail_unless((temp<=IDT_DRV_MAX_INPUT),
                "Priority out of range %d",
                temp);
}
END_TEST

START_TEST (test_GetCurrentSelectInput)
{
    T_osUint8 temp;

    /* We just want to check to see that it's not going to trap */
    temp = IDTDpll_GetCurrentSelectInput(hdlr, _i);
    fail_unless((temp<=IDT_DRV_MAX_INPUT),
                "Selected input out of range %d",
                temp);
}
END_TEST

START_TEST (test_DpllOperMode)
{
    T_idtDpllOperMode nMode;

    for (nMode=Automatic_Mode;
         nMode<Dco_Mode;
         nMode=(T_idtDpllOperMode)nMode+1)
    {
        T_idtDpllOperMode rdMode;

        IDTDpll_SetDpllOperMod(hdlr, _i, nMode);
        rdMode =  IDTDpll_GetDpllOperMod(hdlr, _i);
        fail_unless(nMode == rdMode,
                    "IDTDpll_SetDpllOperMode nDpll/%d "
                    "nMode/%d:%d not working",
                    _i, nMode, rdMode);
    }
}
END_TEST

START_TEST (test_IsDpllLocked)
{
    fail_unless(IDTDpll_IsDpllLocked(hdlr, _i) <= 1,
                "IsDpllLocked not working");
}
END_TEST

START_TEST (test_BandWidthDamping)
{
    T_idtBandwidthLimitStage stage;
    T_idtDpllBandwidth nBandWidth;
    T_idtDampingFactor nDamping;

    for (stage=dpllBandwidthLimitStart;
         stage<dpllBandwidthLimit_MAX;
         stage=(T_idtBandwidthLimitStage)stage+1)
    {

        /* Skip for T4 typed DPLL */
        if (((_i == DPLL_INSTANCE_2) && 
            (stage != dpllBandwidthLimitLocked)) ||
            ((_i == DPLL_INSTANCE_1) &&
             (hdlr->deviceConfig_m->t0PllMode_m == Dpll_T4)))
            continue;

        for (nBandWidth=dpllBandwidth_0p5mHz;
             nBandWidth<dpllBandwidth_MAX; 
             nBandWidth=(T_idtDampingFactor)nBandWidth+1)
        {

            /* Skip for T4 typed DPLL */
            if (((_i == DPLL_INSTANCE_2) && 
                 (nBandWidth < dpllBandwidth_18Hz)) ||
                ((_i == DPLL_INSTANCE_1) &&
                 (hdlr->deviceConfig_m->t0PllMode_m == Dpll_T4)))
                continue;

            for (nDamping=DampingFactor_1p2;
                 nDamping<DampingFactor_MAX;
                 nDamping=(T_idtDampingFactor)nDamping+1)
            {
                T_idtDpllBandwidth rdBandWidth;
                T_idtDampingFactor rdDamping;

                IDTDpll_SetBandWidthDamping(hdlr, 
                                            _i, 
                                            stage, 
                                            nBandWidth, 
                                            nDamping);
                IDTDpll_GetBandWidthDamping(hdlr, 
                                            _i, 
                                            stage, 
                                            &rdBandWidth, 
                                            &rdDamping);
                fail_unless(((nBandWidth==rdBandWidth)&&
                             (nDamping==rdDamping)),
                            "SetBandWidthDamping "
                            "nDpll/%d stage/%d "
                            "nBandWidth/%d:%d "
                            "nDamping/%d:%d not working",
                            _i, stage, 
                            nBandWidth, rdBandWidth, 
                            nDamping, rdDamping);
            }
        }
    }
}
END_TEST

START_TEST (test_BandWidthDampingAssert)
{
    /* Should Assert */
    IDTDpll_SetBandWidthDamping(hdlr, 
                                DPLL_INSTANCE_2, 
                                (T_idtBandwidthLimitStage)_i, 
                                dpllBandwidth_0p5mHz, 
                                DampingFactor_1p2);
}
END_TEST

START_TEST (test_AutoSelBandWidth)
{
    IDTDpll_SetAutoSelBandWidth(hdlr, _i, 0);
    fail_unless(IDTDpll_GetAutoSelBandWidth(hdlr, _i) == 0,
                "SetAutoSelBandWidth 0 not working");

    IDTDpll_SetAutoSelBandWidth(hdlr, _i, 1);
    fail_unless(IDTDpll_GetAutoSelBandWidth(hdlr, _i) == 1,
                "SetAutoSelBandWidth 1 not working");
}
END_TEST

START_TEST (test_DpllFrozen)
{
    IDTDpll_SetDpllFrozen(hdlr, _i, 0);
    fail_unless(IDTDpll_GetDpllFrozen(hdlr, _i) == 0,
                "SetDpllFrozen 0 not working");

    IDTDpll_SetDpllFrozen(hdlr, _i, 1);
    fail_unless(IDTDpll_GetDpllFrozen(hdlr, _i) == 1,
                "SetDpllFrozen 1 not working");
}
END_TEST

START_TEST (test_PhaseCoarseDetector)
{
    T_osUint8 i;

    T_osUint8 nEnable;
    T_osUint8 nWide;
    T_osUint8 nMultiPhase;
    T_osUint8 nMultiPh8kEn;

    for (i=0; i<(1<<4); i++)
    {
        IDTDpll_SetPhaseCoarseDetector(hdlr, _i,
                                       (i&1)?1:0, /* Enable */
                                       (i&2)?1:0, /*nWide */
                                       (i&4)?1:0, /* nMultiPhase */
                                       (i&8)?1:0); /* nMultiPh8kEn */
        IDTDpll_GetPhaseCoarseDetector(hdlr, _i,
                                       &nEnable, 
                                       &nWide, 
                                       &nMultiPhase,
                                       &nMultiPh8kEn);

        fail_unless(((nEnable == ((i&1)?1:0)) &&
                     (nWide == ((i&2)?1:0)) &&
                     (nMultiPhase == ((i&4)?1:0)) &&
                     (nMultiPh8kEn == ((i&8)?1:0))),
                    "IDTDpll_SetPhaseCoarseDetector "
                    "nEnable/%d:%d nWide/%d:%d "
                    "nMultiPhase/%d:%d nMultiPh8kEn/%d:%d "
                    "not working",
                    nEnable,(i&1)?1:0,
                    nWide,(i&2)?1:0,
                    nMultiPhase,(i&4)?1:0,
                    nMultiPh8kEn,(i&8)?1:0);
    }
}
END_TEST

START_TEST (test_PhaseCoarseLimit)
{
    T_idtCoarsePhaseLimit nLimit;

    for (nLimit=coarsePhaseLimit_1;
         nLimit<coarsePhaseLimit_MAX;
         nLimit=(T_idtCoarsePhaseLimit)nLimit+1)
    {
        T_idtCoarsePhaseLimit rdLimit;

        IDTDpll_SetPhaseCoarseLimit(hdlr, _i, nLimit);
        rdLimit = IDTDpll_GetPhaseCoarseLimit(hdlr, _i);
        fail_unless(nLimit == rdLimit,
                    "SetPhaseCoarseLimit nDpll/%d "
                    "nLimit/%d:%d not working",
                    _i, nLimit, rdLimit);
    }
}
END_TEST

START_TEST (test_PhaseFineDetectorEnable)
{
    IDTDpll_SetPhaseFineDetectorEnable(hdlr, _i, 0);
    fail_unless(IDTDpll_GetPhaseFineDetectorEnable(hdlr, _i) == 0,
                "SetPhaseFineDetectorEnable 0 not working");

    IDTDpll_SetPhaseFineDetectorEnable(hdlr, _i, 1);
    fail_unless(IDTDpll_GetPhaseFineDetectorEnable(hdlr, _i) == 1,
                "SetPhaseFineDetectorEnable 1 not working");
}
END_TEST

START_TEST (test_PhaseFineLimit)
{
    T_idtFinePhaseLimit nLimit;

    for (nLimit=finePhaseLimit_45_90degree;
         nLimit<finePhaseLimit_MAX;
         nLimit=(T_idtFinePhaseLimit)nLimit+1)
    {
        T_idtFinePhaseLimit rdLimit;

        IDTDpll_SetPhaseFineLimit(hdlr, _i, nLimit);
        rdLimit = IDTDpll_GetPhaseFineLimit(hdlr, _i);

        fail_unless(nLimit == rdLimit,
                    "SetPhaseFineLimit nDpll/%d "
                    "nLimit/%d:%d not working",
                    _i, nLimit, rdLimit);
    }
}
END_TEST

START_TEST (test_FastLossSwitch)
{
    IDTDpll_SetFastLossSwitch(hdlr, 0);
    fail_unless(IDTDpll_GetFastLossSwitch(hdlr) == 0,
                "SetFastLossSwitch 0 not working");

    IDTDpll_SetFastLossSwitch(hdlr, 1);
    fail_unless(IDTDpll_GetFastLossSwitch(hdlr) == 1,
                "SetFastLossSwitch 1 not working");
}
END_TEST

START_TEST (test_HoldoverMode)
{
    T_idtDpllHoldover nMode;
    T_osUint8 nReadMode;

    for (nMode=Holdover_Instantaneous;
         nMode<Holdover_MAX;
         nMode=(T_idtDpllHoldover)nMode+1)
        for (nReadMode=0; nReadMode<2; nReadMode++)
        {
            T_idtDpllHoldover rdMode;
            T_osUint8 rdReadMode;

            IDTDpll_SetHoldoverMode(hdlr, _i,  nMode,  nReadMode);
            IDTDpll_GetHoldoverMode(hdlr, _i,  &rdMode,  &rdReadMode);
            fail_unless(((rdMode == nMode) && 
                         (rdReadMode == nReadMode)),
                        "SetHoldoverMode nMode/%d:%d "
                        "nReadMode/%d:%d not working",
                        nMode, rdMode, nReadMode, rdReadMode);
                        
        }
}
END_TEST

START_TEST (test_TempHoldoverMode)
{
    T_idtTemp_holdover nMode;

    for (nMode=Temp_Same_As_Full_Holdover;
         nMode<Temp_Holdover_MAX;
         nMode=(T_idtTemp_holdover)nMode+1)
    {
        T_idtTemp_holdover rdMode;

        IDTDpll_SetTempHoldoverMode(hdlr, _i, nMode);
        rdMode =  IDTDpll_GetTempHoldoverMode(hdlr, _i);
        fail_unless(nMode == rdMode,
                    "SetTempHoldoverMode nMode/%d:%d "
                    "not working",
                    nMode, rdMode);
    }
}
END_TEST

START_TEST (test_GetHoldoverFreq)
{
    /* Check the we don't assert */
    IDTDpll_GetHoldoverFreq(hdlr, _i);
}
END_TEST

START_TEST (test_HoldoverFreq)
{
    IDTDpll_GetHoldoverFreq(hdlr, _i);
}
END_TEST

START_TEST (test_GetCurrentDpllFreqOffset)
{
    /* Check that we don't assert */
    IDTDpll_GetCurrentDpllFreqOffset(hdlr, _i); 
}
END_TEST

START_TEST (test_DpllSoftAlarmLimit)
{
    T_osUint8 nLimit;

    for (nLimit=120; nLimit<128; nLimit++)
    {
        T_osUint8 rdLimit;

        IDTDpll_SetDpllSoftAlarmLimit(hdlr, _i,  nLimit);
        rdLimit = IDTDpll_GetDpllSoftAlarmLimit(hdlr, _i);
        fail_unless(nLimit==rdLimit,
                    "SetDpllSoftAlarmLimit  nLimit/%d:%d not working",
                    nLimit,rdLimit);
    }
}
END_TEST

START_TEST (test_DpllHardAlarmLimit)
{
    T_osUint16 nLimit;

    for (nLimit=250; nLimit<270; nLimit++)
    {
        T_osUint16 rdLimit;

        IDTDpll_SetDpllHardAlarmLimit(hdlr, _i,  nLimit);
        rdLimit = IDTDpll_GetDpllHardAlarmLimit(hdlr, _i);
        fail_unless(nLimit==rdLimit,
                    "SetDpllHardAlarmLimit nLimit/%d:%d not working",
                    nLimit,rdLimit);
    }
}
END_TEST

START_TEST (test_DpllHardAlarmEnable)
{
    IDTDpll_SetDpllHardAlarmEnable(hdlr, 0);
    fail_unless(IDTDpll_GetDpllHardAlarmEnable(hdlr) == 0,
                "SetDpllHardAlarmEnable 0 not working");

    IDTDpll_SetDpllHardAlarmEnable(hdlr, 1);
    fail_unless(IDTDpll_GetDpllHardAlarmEnable(hdlr) == 1,
                "SetDpllHardAlarmEnable 1 not working");
}
END_TEST

START_TEST (test_GetCurrentPhaseError)
{
    /* Just check that we don't assert */
    IDTDpll_GetCurrentPhaseError(hdlr, _i);
}
END_TEST

START_TEST (test_SonetGigEthOutputPath)
{
        T_idtSonetGigEthOutput outputFreq;
        T_osUint8 nSonetGigEthPathN;
        T_osUint8 numOfonetGigEth;

        numOfonetGigEth = hdlr->deviceConfig_m->numberOfSonetGigEthPath_m;
        for (nSonetGigEthPathN=0; nSonetGigEthPathN<numOfonetGigEth; nSonetGigEthPathN++)
                for (outputFreq=SonetGigEthOutput_Sonet;
                     outputFreq<SonetGigEthOutput_MAX;
                     outputFreq=(T_idtSonetGigEthOutput)outputFreq+1)
        {
            T_idtDpllInstance rdInputDpll;
                T_idtSonetGigEthOutput rdOutputFreq;

                IDTDpll_SetSonetGigEthOutputPath(hdlr, nSonetGigEthPathN, _i, outputFreq);
                IDTDpll_GetSonetGigEthOutputPath(hdlr, nSonetGigEthPathN, &rdInputDpll, &rdOutputFreq);
            fail_unless(((outputFreq == rdOutputFreq) &&
                         (_i == rdInputDpll)),
                            " SetSonetGigEthOutputPath instance/%d "
                        "inputDpll/%d outputFreq/%d "
                        "not working",
                        nSonetGigEthPathN, _i, rdInputDpll, 
                        outputFreq, rdOutputFreq);
        }
}
END_TEST

START_TEST (test_DpllOutputPathSelectorA)
{
    T_idtDpllOutputSelectorA path;

    for (path=DPLLOutputSelA_16E1;
         path<DPLLOutputSelA_MAX;
         path=(T_idtDpllOutputSelectorA)path+1)
    {
        T_idtDpllOutputSelectorA rdPath;

        if (((_i==DPLL_INSTANCE_1) && (path == DPLLOutputSelA_GPS)) ||
            ((_i==DPLL_INSTANCE_2) && (path == DPLLOutputSelA_OBSAI)))
            continue;

        IDTDpll_SetDpllOutputPathSelectorA(hdlr, _i, path);
        rdPath = IDTDpll_GetDpllOutputPathSelectorA(hdlr, _i);
        fail_unless(path == rdPath,
                    "SetDpllOutputPathSelectorA nDpll/%d "
                    "path/%d:%d not working",
                    _i, path, rdPath);
    }
}
END_TEST

START_TEST (test_DpllOutputPathSelectorB)
{
    T_idtDpllOutputSelectorB path;

    for (path=DPLLOutputSelB_12E1;
         path<DPLLOutputSelB_MAX;
         path=(T_idtDpllOutputSelectorB)path+1)
    {
        T_idtDpllOutputSelectorB rdPath;

        if (((_i==DPLL_INSTANCE_1) && (path == DPLLOutputSelB_24T1)) ||
            ((_i==DPLL_INSTANCE_2) && (path == DPLLOutputSelB_GPS)))
            continue;

        IDTDpll_SetDpllOutputPathSelectorB(hdlr, _i,  path);
        rdPath = IDTDpll_GetDpllOutputPathSelectorB(hdlr, _i);
        fail_unless(path == rdPath,
                    "SetDpllOutputPathSelectorB nDpll/%d "
                    "path/%d:%d not working",
                    _i, path, rdPath);
    }
}
END_TEST

START_TEST (test_DpllEthPathEnable)
{
    IDTDpll_SetDpllEthPathEnable(hdlr, _i, 0);
    fail_unless(IDTDpll_GetDpllEthPathEnable(hdlr, _i) == 0,
                "SetDpllEthPathEnable 0 not working");

    IDTDpll_SetDpllEthPathEnable(hdlr, _i, 1);
    fail_unless(IDTDpll_GetDpllEthPathEnable(hdlr, _i) == 1,
                "SetDpllEthPathEnable 1 not working");

}
END_TEST

START_TEST (test_DpllSelBPathEnable)
{
    IDTDpll_SetDpllSelBPathEnable(hdlr, _i, 0);
    fail_unless(IDTDpll_GetDpllSelBPathEnable(hdlr, _i) == 0,
                "SetDpllSelBPathEnable 0 not working");

    IDTDpll_SetDpllSelBPathEnable(hdlr, _i, 1);
    fail_unless(IDTDpll_GetDpllSelBPathEnable(hdlr, _i) == 1,
                "SetDpllSelBPathEnable 1 not working");
}
END_TEST

START_TEST (test_Dpll16E116T1Enable)
{
    IDTDpll_SetDpll16E116T1Enable(hdlr, _i, 0);
    fail_unless(IDTDpll_GetDpll16E116T1Enable(hdlr, _i) == 0,
                "SetDpll16E116T1Enable 0 not working");

    IDTDpll_SetDpll16E116T1Enable(hdlr, _i, 1);
    fail_unless(IDTDpll_GetDpll16E116T1Enable(hdlr, _i) == 1,
                "SetDpll16E116T1Enable 1 not working");
}
END_TEST

START_TEST (test_DpllSelAPathEnable)
{
    IDTDpll_SetDpllSelAPathEnable(hdlr, _i, 0);
    fail_unless(IDTDpll_GetDpllSelAPathEnable(hdlr, _i) == 0,
                "SetDpllSelAPathEnable 0 not working");

    IDTDpll_SetDpllSelAPathEnable(hdlr, _i, 1);
    fail_unless(IDTDpll_GetDpllSelAPathEnable(hdlr, _i) == 1,
                "SetDpllSelAPathEnable 1 not working");
}
END_TEST

START_TEST (test_FrequencyMonitoringFactor)
{
    T_idtFreqMonFactor freqMonFactor;

    for (freqMonFactor=FreqMonFactor_0p0032;
         freqMonFactor<FreqMonFactor_MAX;
         freqMonFactor=(T_idtFreqMonFactor)freqMonFactor+1)
    {
        T_idtFreqMonFactor rdFreqMonFactor;

        IDTDpll_SetFrequencyMonitoringFactor(hdlr, freqMonFactor);
        rdFreqMonFactor = IDTDpll_GetFrequencyMonitoringFactor(hdlr);
        fail_unless(freqMonFactor == rdFreqMonFactor,
                    "SetFrequencyMonitoringFactor "
                    "freqMonFactor/%d:%d not working",
                    freqMonFactor, rdFreqMonFactor);
    }
}
END_TEST

START_TEST (test_HardAlarmThreshold)
{
    T_osUint8 accepting;
    T_osUint8 rejecting;

    for (accepting=8; accepting<16; accepting++)
        for (rejecting=0; rejecting<8; rejecting++)
        {
            T_osUint8 rdAccepting;
            T_osUint8 rdRejecting;

            IDTDpll_SetHardAlarmThreshold(hdlr, accepting, rejecting);
            IDTDpll_GetHardAlarmThreshold(hdlr, &rdAccepting, &rdRejecting);
            fail_unless(((accepting == rdAccepting) &&
                         (rejecting == rdRejecting)),
                        "SetHardAlarmThreshold accepting/%d:%d "
                        "rejecting/%d:%d not working",
                        accepting, rdAccepting, 
                        rejecting, rdRejecting);
        }
}
END_TEST

START_TEST (test_SoftAlarmThreshold)
{
    T_osUint8 accepting;
    T_osUint8 rejecting;

    for (accepting=0; accepting<8; accepting++)
        for (rejecting=8; rejecting<16; rejecting++)
        {
            T_osUint8 rdAccepting;
            T_osUint8 rdRejecting;

            IDTDpll_SetSoftAlarmThreshold (hdlr, accepting, rejecting);
            IDTDpll_GetSoftAlarmThreshold(hdlr, &rdAccepting, &rdRejecting);
            fail_unless(((accepting == rdAccepting) &&
                         (rejecting == rdRejecting)),
                        "SetSoftAlarmThreshold accepting/%d:%d "
                        "rejecting/%d:%d not working",
                        accepting, rdAccepting, 
                        rejecting, rdRejecting);
        }
}
END_TEST

START_TEST (test_IcpCtrl)
{
    T_osUint8 nIcp;

    for (nIcp=0; nIcp<16; nIcp++)
    {
        T_osUint8 rdIcp;

        IDTDpll_SetIcpCtrl(hdlr, _i, nIcp);
        rdIcp = IDTDpll_GetIcpCtrl(hdlr, _i);
        fail_unless(nIcp == rdIcp,
                    "SetIcpCtrl nDpll/%d nIcp/%d:%d not working",
                    _i, nIcp, rdIcp);
    }
}
END_TEST

START_TEST (test_PhaseSlope)
{
    T_idtPhaseSlope nPhaseSlope;

    for (nPhaseSlope=phaseSlope_gr1244st3;
         nPhaseSlope<phaseSlope_MAX;
         nPhaseSlope=(T_idtPhaseSlope)nPhaseSlope+1)
    {
        T_idtPhaseSlope rdPhaseSlope;

        IDTDpll_SetPhaseSlope(hdlr, _i,  nPhaseSlope);
        rdPhaseSlope = IDTDpll_GetPhaseSlope(hdlr, _i);
        fail_unless(nPhaseSlope == rdPhaseSlope,
                    "SetPhaseSlope nDpll/%d nPhaseSlope/%d "
                    "not working",
                    _i, nPhaseSlope, rdPhaseSlope);
    }
}
END_TEST

START_TEST (test_PpsFastLock)
{
    T_osUint8 nEnablePhase;

    for (nEnablePhase=0; nEnablePhase<2; nEnablePhase++)
    {
        T_osUint8 rdEnableFreq;
        T_osUint8 rdEnablePhase;

        IDTDpll_SetPpsFastLock(hdlr,0, nEnablePhase);
        IDTDpll_GetPpsFastLock(hdlr, &rdEnableFreq, &rdEnablePhase);
        fail_unless(((rdEnableFreq == 0) &&
                     (nEnablePhase == rdEnablePhase)),
                    "SetPpsFastLock nEnableFreq/1:%d "
                    "nEnablePhase/%d:%d not working",
                    rdEnableFreq, nEnablePhase, rdEnablePhase);
        IDTDpll_SetPpsFastLock(hdlr, 1, nEnablePhase);
        IDTDpll_GetPpsFastLock(hdlr, &rdEnableFreq, &rdEnablePhase);
        fail_unless(((rdEnableFreq == 1) &&
                     (nEnablePhase == rdEnablePhase)),
                    "SetPpsFastLock nEnableFreq/1:%d "
                    "nEnablePhase/%d:%d not working",
                    rdEnableFreq, nEnablePhase, rdEnablePhase);
    }
}
END_TEST

START_TEST (test_HitlessSwitchingFeature)
{
    T_osUint8 rdEnable;
    T_osUint8 rdFrozen;

    IDTDpll_SetHitlessSwitchingFeature(hdlr, 0, 0);
    IDTDpll_GetHitlessSwitchingFeature(hdlr, &rdEnable, &rdFrozen);
    fail_unless(((rdEnable == 0) &&
                 (rdFrozen == 0)),
                "SetHitlessSwitchingFeature nEnable/0:%d "
                "nFrozen/0:%d not working",
                rdEnable, rdFrozen);
    IDTDpll_SetHitlessSwitchingFeature(hdlr, 0, 1);
    IDTDpll_GetHitlessSwitchingFeature(hdlr, &rdEnable, &rdFrozen);
    fail_unless(((rdEnable == 0) &&
                 (rdFrozen == 1)),
                "SetHitlessSwitchingFeature nEnable/0:%d "
                "nFrozen/1:%d not working",
                rdEnable, rdFrozen);
    IDTDpll_SetHitlessSwitchingFeature(hdlr, 1, 0);
    IDTDpll_GetHitlessSwitchingFeature(hdlr, &rdEnable, &rdFrozen);
    fail_unless(((rdEnable == 1) &&
                 (rdFrozen == 0)),
                "SetHitlessSwitchingFeature nEnable/1:%d "
                "nFrozen/0:%d not working",
                rdEnable, rdFrozen);
    IDTDpll_SetHitlessSwitchingFeature(hdlr, 1, 1);
    IDTDpll_GetHitlessSwitchingFeature(hdlr, &rdEnable, &rdFrozen);
    fail_unless(((rdEnable == 1) &&
                 (rdFrozen == 1)),
                "SetHitlessSwitchingFeature nEnable/1:%d "
                "nFrozen/1:%d not working",
                rdEnable, rdFrozen);
}
END_TEST

START_TEST (test_PhaseTransient)
{
    IDTDpll_SetPhaseTransient(hdlr, 0);
    fail_unless(IDTDpll_GetPhaseTransient(hdlr) == 0,
                "SetPhaseTransient 0 not working");

    IDTDpll_SetPhaseTransient(hdlr, 1);
    fail_unless(IDTDpll_GetPhaseTransient(hdlr) == 1,
                "SetPhaseTransient 1 not working");
}
END_TEST

START_TEST (test_PhaseOffset)
{
    T_osUint16 nOffset;

    for (nOffset=250; nOffset<300; nOffset++)
    {
        T_osUint16 rdOffset;

        IDTDpll_SetPhaseOffset(hdlr, nOffset);
        rdOffset = IDTDpll_GetPhaseOffset(hdlr);
        fail_unless(nOffset == rdOffset,
                    "SetPhaseOffset nOffset/%d:%d not working",
                    _i, nOffset, rdOffset);
    }
}
END_TEST

START_TEST (test_PpsPhase)
{
    T_idtDpllPpsPhasePostion nPhase; 
    T_idtDpllPpsPulseWidth   nPulse;
    T_idtDpllPpsPhasePostion rdPhase;
    T_idtDpllPpsPulseWidth   rdPulse;

    for (nPhase=PpsPhasePostion_0degree;
         nPhase<PpsPhasePostion_MAX;
         nPhase = (T_idtDpllPpsPhasePostion)(nPhase+1))
        for (nPulse=PpsPulseWidth_0pt5sec;
             nPulse<PpsPulseWidth_MAX;
             nPulse = (T_idtDpllPpsPulseWidth)(nPulse+1))
        {
            IDTDpll_SetPpsPhase(hdlr, 
                                _i, 
                                nPhase, 
                                nPulse); 
            IDTDpll_GetPpsPhase(hdlr, 
                                _i, 
                                &rdPhase, 
                                &rdPulse); 
            fail_unless(((nPulse == rdPulse) && (nPhase == rdPhase)),
                        "IDTDpll_SetPpsPhase nPhase/%d:%d "
                        "nPulse/%d:%d not working",
                        nPhase,rdPhase, nPulse,rdPulse);
        }
}
END_TEST

Suite *DpllCnfg_suite (void)
{
    T_osUint8 i;
    Suite *s = suite_create ("DpllCnfg");

    /* General test case */
    TCase *tc_dpllCnfg = tcase_create ("DpllCnfg");

    tcase_add_checked_fixture (tc_dpllCnfg,
                               initalizeDriver, 
                               deinitilizeDriver);

    for (i=DPLL_INSTANCE_1; i<DPLL_INSTANCE_MAX; i++)
    {
        T_osUint8 dpllType;
        T_idtDpllType t0PllMode;

        dpllType = hdlr->deviceConfig_m->pllExt2IntXlate_ma[i];
        t0PllMode = hdlr->deviceConfig_m->t0PllMode_m;

        if (dpllType != Dpll_MAX)
        {
            tcase_add_loop_test (tc_dpllCnfg, test_SetAutoSelInput, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_SetForceSelInput, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_DpllOperMode, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_IsDpllLocked, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_BandWidthDamping, 
                                 i, i+1);
            
            if (dpllType == Dpll_T0)
            {
                tcase_add_loop_test (tc_dpllCnfg, test_AutoSelBandWidth, 
                                     i, i+1);
                tcase_add_loop_test (tc_dpllCnfg, test_DpllFrozen,
                                     i, i+1);
            }
            tcase_add_loop_test (tc_dpllCnfg, test_PhaseCoarseDetector, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_PhaseCoarseLimit, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_PhaseFineDetectorEnable, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_PhaseFineLimit, 
                                 i, i+1);
            if (dpllType == Dpll_T0)
            {
                tcase_add_loop_test (tc_dpllCnfg, test_HoldoverMode,
                                     i, i+1);
                tcase_add_loop_test (tc_dpllCnfg, test_TempHoldoverMode, 
                                     i, i+1);
            }
            tcase_add_loop_test (tc_dpllCnfg, test_GetHoldoverFreq, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_HoldoverFreq, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_GetCurrentDpllFreqOffset, 
                                 i, i+1);
            if (dpllType == Dpll_T0)
            {
                tcase_add_loop_test (tc_dpllCnfg, test_DpllSoftAlarmLimit, 
                                     i, i+1);
                tcase_add_loop_test (tc_dpllCnfg, test_DpllHardAlarmLimit, 
                                     i, i+1);
            }
            tcase_add_loop_test (tc_dpllCnfg, test_GetCurrentPhaseError, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_SonetGigEthOutputPath, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_DpllOutputPathSelectorA, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_DpllOutputPathSelectorB, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_DpllEthPathEnable, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_DpllSelBPathEnable, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_DpllEthPathEnable, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_DpllSelBPathEnable, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_Dpll16E116T1Enable, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_DpllSelAPathEnable, 
                                 i, i+1);
            tcase_add_loop_test (tc_dpllCnfg, test_IcpCtrl, 
                                 i, i+1);
            if ((dpllType == Dpll_T0) &&
                (hdlr->deviceConfig_m->phaseSlopeLimiting_ma[i]))
            {
                if (Dpll_T0 == t0PllMode)
                {
                    tcase_add_loop_test (tc_dpllCnfg, test_PhaseSlope, 
                                         i, i+1);
                }
                else
                {
                    /* T4 Dpll Mode does not support this feature,
                                         we should get an assert */
                    tcase_add_loop_test_raise_signal(tc_dpllCnfg, 
                                                     test_PhaseSlope, 
                                                     6, i, i+1);
                }
            }
            tcase_add_loop_test (tc_dpllCnfg, test_PpsPhase ,
                                 i, i+1);
        }
        else 
        {
            /* Unsupported DPLL - it should assert */
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_SetAutoSelInput, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_SetForceSelInput, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_DpllOperMode, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_IsDpllLocked, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_BandWidthDamping, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_AutoSelBandWidth, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_DpllFrozen,
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_PhaseCoarseDetector, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_PhaseCoarseLimit, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_PhaseFineDetectorEnable, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_PhaseFineLimit, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_HoldoverMode,
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_TempHoldoverMode, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_GetHoldoverFreq, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_HoldoverFreq, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_GetCurrentDpllFreqOffset, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_DpllSoftAlarmLimit, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_DpllHardAlarmLimit, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_GetCurrentPhaseError, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_SonetGigEthOutputPath, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_DpllOutputPathSelectorA, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_DpllOutputPathSelectorB, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_DpllEthPathEnable, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_DpllSelBPathEnable, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_DpllEthPathEnable, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_DpllSelBPathEnable, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_Dpll16E116T1Enable, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_DpllSelAPathEnable, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_IcpCtrl, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_PhaseSlope, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                              test_PpsPhase , 
                                              6, i, i+1);
        }
    }
    tcase_add_loop_test_raise_signal (tc_dpllCnfg, 
                                      test_BandWidthDampingAssert,
                                      6, /* Abort signal */
                                      dpllBandwidthLimitStart,
                                      dpllBandwidthLimitAcquire+1);
    tcase_add_test (tc_dpllCnfg, test_GetInputPriority);
    tcase_add_test (tc_dpllCnfg, test_GetCurrentSelectInput);
    tcase_add_test (tc_dpllCnfg, test_FastLossSwitch);
    tcase_add_test (tc_dpllCnfg, test_DpllHardAlarmEnable);
    tcase_add_test (tc_dpllCnfg, test_FrequencyMonitoringFactor);
    tcase_add_test (tc_dpllCnfg, test_HardAlarmThreshold);
    tcase_add_test (tc_dpllCnfg, test_SoftAlarmThreshold);
    tcase_add_test (tc_dpllCnfg, test_PpsFastLock);
    tcase_add_test (tc_dpllCnfg, test_HitlessSwitchingFeature);
    tcase_add_test (tc_dpllCnfg, test_PhaseTransient);
    tcase_add_test (tc_dpllCnfg, test_PhaseOffset);
 
    suite_add_tcase (s, tc_dpllCnfg);

    return s;
}


