/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <check.h>

#include "osNumeric.h"
#include "Define.h"
#include "General.h"

#include "check_main.h"
/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

START_TEST (test_DeviceID)
{
        fail_unless(IDTGeneral_GetDeviceID(hdlr) == 0x3391,
                    "GetDeviceID not working");
}
END_TEST
START_TEST (test_NetworkType)
{
        IDTGeneral_SetNetworkType(hdlr, Network_SDH);
        fail_unless(IDTGeneral_GetNetworkType(hdlr) == Network_SDH,
                    "Set/Get for SDH not working");
        IDTGeneral_SetNetworkType(hdlr, Network_SONET);
        fail_unless(IDTGeneral_GetNetworkType(hdlr) == Network_SONET,
                    "Set/Get for SONET not working");
}
END_TEST

START_TEST (test_RevertiveMode)
{
        IDTGeneral_SetRevertiveMode(hdlr, 1);
        fail_unless(IDTGeneral_GetRevertiveMode(hdlr) == 1,
                    "SetRevertiveMode 1 not working");
        IDTGeneral_SetRevertiveMode(hdlr, 0);
        fail_unless(IDTGeneral_GetRevertiveMode(hdlr) == 0,
                    "SetRevertiveMode 0 not working");
}
END_TEST

START_TEST (test_TimeoutValue)
{
        phaseTimeoutMultiplier_t nMultiFactor;
        T_osUint8                nTimeout;

        for (nMultiFactor=phaseTimeoutMultiplier_2; 
             nMultiFactor<phaseTimeoutMultiplier_MAX;
             nMultiFactor = (phaseTimeoutMultiplier_t)(nMultiFactor+1))
                for (nTimeout=0; 
                     nTimeout<(1<<BF_TIMEOUT_VALUE_SIZE);
                     nTimeout++)

                {
                        phaseTimeoutMultiplier_t rdMultiFactor;
                        T_osUint8                rdTimeout;

                        IDTGeneral_SetTimeoutValue(hdlr, nMultiFactor, nTimeout);
                        IDTGeneral_GetTimeoutValue(hdlr, &rdMultiFactor, &rdTimeout);
                        fail_unless(((nMultiFactor == rdMultiFactor)&&
                                     (nTimeout == rdTimeout)),
                                    "SetTimeoutValue fail nMultiFactor/%d "
                                    "nTimeout/%d",
                                    nMultiFactor, nTimeout);
                }
}
END_TEST

START_TEST (test_OscilatorActiveEdge)
{
        IDTGeneral_SetOscActiveEdge(hdlr, 1);
        fail_unless(IDTGeneral_GetOscActiveEdge(hdlr) == 1,
                    "SetOscActiveEdge 1 not working");
        IDTGeneral_SetOscActiveEdge(hdlr, 0);
        fail_unless(IDTGeneral_GetOscActiveEdge(hdlr) == 0,
                    "SetOscActiveEdge 0 not working");
}
END_TEST

START_TEST (test_UltraFastSwitch)
{
        IDTGeneral_SetUltraFastSwitch(hdlr, 1);
        fail_unless(IDTGeneral_GetUltraFastSwitch(hdlr) == 1,
                    "SetUltraFastSwitch 1 not working");
        IDTGeneral_SetUltraFastSwitch(hdlr, 0);
        fail_unless(IDTGeneral_GetUltraFastSwitch(hdlr) == 0,
                    "SetUltraFastSwitch 0 not working");
}
END_TEST

START_TEST (test_OscilatorFrequencyOffset)
{
#if 0
        long i;
        long freqOffsetInPpt = 0;
        long readBackFreqOffset;

        for (i=0; i<(1<<23); i++)
        {
/* Because of translation between ppt and register val,
 *
 *   REG VAL = PPT * (10/84)
 *           = 
 *   PPT     = REG VAL * (84/10)
 */


                freqOffsetInPpt = i*9; /* Technially 8.4 */
                SetOscFreqOffset(hdlr, freqOffsetInPpt);

                readBackFreqOffset = GetOscFreqOffset(hdlr);


        }
        fail_unless(money_amount (m) == 5,
                    "Amount was %d, instead of 5", money_amount (m));
        fail_if (strcmp (money_currency (m), "USD") != 0,
                 "Currency not set correctly on creation");
#endif
}
END_TEST

START_TEST (test_FlagOnTDO)
{
        IDTGeneral_SetFlagOnTDO(hdlr, 1);
        fail_unless(IDTGeneral_GetFlagOnTDO(hdlr) == 1,
                    "SetFlagOnTDO 1 not working");
        IDTGeneral_SetFlagOnTDO(hdlr, 0);
        fail_unless(IDTGeneral_GetFlagOnTDO(hdlr) == 0,
                    "SetFlagOnTDO 0 not working");
}
END_TEST

START_TEST (test_HardAlarm)
{
        IDTGeneral_SetHardAlarm(hdlr, 1);
        fail_unless(IDTGeneral_GetHardAlarm(hdlr) == 1,
                    "SetHardAlarm 1 not working");
        IDTGeneral_SetHardAlarm(hdlr, 0);
        fail_unless(IDTGeneral_GetHardAlarm(hdlr) == 0,
                    "SetHardAlarm 0 not working");
}
END_TEST

START_TEST (test_RegisterProtection)
{
        IDTGeneral_SetProtectMode(hdlr);
        IDTGeneral_SetSingleUnprotectMode(hdlr);
        IDTGeneral_SetFullyUnprotectMode(hdlr);
}
END_TEST

Suite *General_suite (void)
{
        Suite *s = suite_create ("General");

        /* General test case */
        TCase *tc_general = tcase_create ("General");

        tcase_add_checked_fixture (tc_general,
                                   initalizeDriver, 
                                   deinitilizeDriver);
        tcase_add_test (tc_general, test_DeviceID);
        tcase_add_test (tc_general, test_NetworkType);
        tcase_add_test (tc_general, test_RevertiveMode);
        tcase_add_test (tc_general, test_TimeoutValue);
        tcase_add_test (tc_general, test_OscilatorActiveEdge);
        tcase_add_test (tc_general, test_UltraFastSwitch);
        tcase_add_test (tc_general, test_OscilatorFrequencyOffset);
        tcase_add_test (tc_general, test_HardAlarm);
        tcase_add_test (tc_general, test_FlagOnTDO);
        tcase_add_test (tc_general, test_RegisterProtection);
        suite_add_tcase (s, tc_general);

        return s;
}


