/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <check.h>

#include "Define.h"
#include "osNumeric.h"
#include "Input.h"

#include "check_main.h"
/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

START_TEST (test_InputPriority)
{
    T_osUint8 nPriority;

    for (nPriority=0; nPriority<16; nPriority++)
    {
        T_osUint8 rdPriority;

        IDTInput_SetPriority(hdlr, _i, DPLL_INSTANCE_1, nPriority);
        rdPriority = IDTInput_GetPriority(hdlr, _i, DPLL_INSTANCE_1);

        fail_unless(nPriority == rdPriority,
                    "SetPriority nInputNo/%d "
                    "nDpll/%d nPriority/%d:%d not working",
                    _i, DPLL_INSTANCE_1, nPriority, rdPriority);
    }
}
END_TEST

START_TEST (test_InputFrequency)
{
    T_idtInputFrequencyBitfieldValue nFrequency;

    for (nFrequency=Freq_8K;
         nFrequency<Freq_MAX;
         nFrequency=(T_idtInputFrequencyBitfieldValue)nFrequency+1)
    {
        T_idtInputFrequencyBitfieldValue rdFrequency;
        T_osUint8 internalPort;

        /* Skip AMI port if frequency is not 8kHz */
        IDT_TRANSLATE_EXT_TO_INT_INPORT(hdlr,_i,internalPort);
        if ((nFrequency != Freq_8K) && (internalPort<3))
            continue;

        IDTInput_SetInputFrequency(hdlr, _i, nFrequency);
        rdFrequency =  IDTInput_GetInputFrequency(hdlr, _i);
        fail_unless(nFrequency == rdFrequency,
                    "SetInputFrequency nInputNo/%d "
                    "nFrequency/%d:%d not working",
                    _i, nFrequency, rdFrequency);
    }
}
END_TEST

START_TEST (test_ActivityMonitor)
{
    T_osUint8 nBucket;

    for (nBucket=0; nBucket<4; nBucket++)
    {
        T_osUint8 rdBucket;

        IDTInput_SetActivityMonitor(hdlr, _i, nBucket);
        rdBucket = IDTInput_GetActivityMonitor(hdlr, _i);
        fail_unless(nBucket == rdBucket,
                    "SetActivityMonitor nInputNo/%d"
                    " nBucket/%d:%d not working",
                    _i, nBucket, rdBucket);
    }
}
END_TEST

void foo(T_idtDrvHdlr hdlr);
void foo(T_idtDrvHdlr hdlr)
{
    int nUpper;
    int nLower;
    int nSize;
    int nDecay;
    int bucket = 0;

    /* Not really a system test but it checks the interface */
    for (nUpper=250; nUpper<255; nUpper++)
    {
        for (nLower=250; nLower<255; nLower++)
        {
            for (nSize=250; nSize<255; nSize++)
            {
                for (nDecay=250; nDecay<256; nDecay++)
                {
                    T_osUint8 rdUpper;
                    T_osUint8 rdLower;
                    T_osUint8 rdSize;
                    T_osUint8 rdDecay;

                    IDTInput_SetBucketConfig(hdlr, bucket, 
                                             nUpper, 
                                             nLower, 
                                             nSize, 
                                             nDecay);
                    IDTInput_GetBucketConfig(hdlr, bucket,
                                             &rdUpper, 
                                             &rdLower, 
                                             &rdSize, 
                                             &rdDecay);
#if 0
                    fail_unless(((nUpper == rdUpper) &&
                                 (nLower == rdLower) &&
                                 (nSize == rdSize) &&
                                 (nDecay == rdDecay)),
                                "SetBucketConfig "
                                "nBucket/%d nUpper/%d:%d"
                                " nLower/%d:%d "
                                "nSize/%d:%d "
                                "nDecay/%d:%d "
                                "not working",
                                bucket, 
                                nUpper, rdUpper, 
                                nLower, rdLower, 
                                nSize, rdSize, 
                                nDecay, rdDecay);
#endif
                }
            }
        }
    }
}

START_TEST (test_BucketConfig)
{
    foo(hdlr);
}
END_TEST

START_TEST (test_PreDivider)
{
    T_idtInputDividerMux nDivider;

    if (_i<3) return;

    for (nDivider=Divider_Bypass; 
         nDivider<Divider_MAX;
         nDivider=(T_idtInputDividerMux)nDivider+1)
    {
        T_idtInputDividerMux rdDivider;

        IDTInput_SetPreDivider(hdlr, _i, nDivider);
        rdDivider =  IDTInput_GetPreDivider(hdlr, _i);

        fail_unless(nDivider == rdDivider,
                    "SetPreDivider nInputNo/%d nDivider/%d:%d "
                    "not working",
                    _i, nDivider, rdDivider);
    }
}
END_TEST

START_TEST (test_DivisionFactor)
{
    T_osUint16 nFactor;

    if (_i<3) return;

    /* Reduce tests set for speed */
    for (nFactor=250; nFactor<750; nFactor++)
    {
        T_osUint16 rdFactor;

        IDTInput_SetDivisionFactor(hdlr, _i, nFactor);
        rdFactor = IDTInput_GetDivisionFactor(hdlr, _i);

        fail_unless(nFactor == rdFactor,
                    "SetDivisionFactor nInputNo/%d "
                    "nFactor/%d:%d not working",
                    _i, nFactor, rdFactor);
    }
}
END_TEST

START_TEST (test_InputHFdivider)
{
    T_idtHighFreqencyDivisor nUsage;

    for (nUsage=HF_Bypass; 
         nUsage<HF_MAX;
         nUsage=(T_idtHighFreqencyDivisor)nUsage+1)
    {
        T_idtHighFreqencyDivisor rdUsage;

        IDTInput_SetInputHFdivider(hdlr, _i, nUsage);
        rdUsage = IDTInput_GetInputHFdivider(hdlr, _i);
        fail_unless(nUsage == rdUsage,
                    "SetInputHFdivider "
                    "nInputNo/%d nUsage/%d not working",
                    _i, nUsage);
    }
}
END_TEST

START_TEST (test_InputStatus)
{
    IDTInput_GetInputFreqOffset(hdlr, _i);
    IDTInput_GetInputClockStatus(hdlr, _i);
    IDTInput_GetInputClockValidation(hdlr, _i);
}
END_TEST

START_TEST (test_AllInputs)
{
    IDTInput_EnableAllInputs(hdlr);
    IDTInput_DisableAllInputs(hdlr);
}
END_TEST

START_TEST (test_InputEnable)
{
    IDTInput_SetInputEnable(hdlr, _i, 1);
    fail_unless(IDTInput_GetInputEnable(hdlr, _i) == 1,
                "SetInputEnable 1 not working");
    IDTInput_SetInputEnable(hdlr, _i, 0);
    fail_unless(IDTInput_GetInputEnable(hdlr, _i) == 0,
                "SetInputEnable 0 not working");
}
END_TEST

START_TEST (test_SyncFrequency)
{
    T_idtInputSyncFreq i;

    for (i=0; i<SYNC_MAX; i=(T_idtInputSyncFreq)(i+1))
    {
        IDTInput_SetSyncFrequency(hdlr,i);
        fail_unless(i == IDTInput_GetSyncFrequency(hdlr));
    }
}
END_TEST

START_TEST (test_SyncSampling)
{
    T_externalSyncSampling sampling;

    for (sampling=externalSyncSampling_onTarget;
         sampling<externalSyncSampling_MAX;
         sampling=((T_externalSyncSampling)sampling+1)) {
        IDTInput_SetSyncSampling(hdlr, _i, sampling);
        fail_unless(sampling == IDTInput_GetSyncSampling(hdlr, _i));
    }
}
END_TEST

START_TEST (test_SetSyncEdge)
{
    T_externalSyncEdge edge;

    for (edge=externalSyncEdge_FallingEdge;
         edge<externalSyncEdge_MAX;
         edge = ((T_externalSyncEdge)edge+1)) {
        IDTInput_SetSyncEdge(hdlr, edge);
        fail_unless(edge == IDTInput_GetSyncEdge(hdlr));
    }
}
END_TEST

START_TEST (test_SyncAlarmRange)
{
    T_externalSyncAlarmRange range;

    for (range=externalSyncAlarmRange_1;
         range<externalSyncAlarmRange_MAX;
         range = ((T_externalSyncAlarmRange)range+1)) {
        IDTInput_SetSyncAlarmRange(hdlr, range);
        fail_unless(range == IDTInput_GetSyncAlarmRange(hdlr));
    }
}
END_TEST

START_TEST (test_SetSyncBypass)
{
    T_externalSyncBypass bypass;

    for (bypass=externalSyncBypass_ExSync1;
         bypass<externalSyncBypass_MAX;
         bypass = ((T_externalSyncBypass)bypass+1)) {
        IDTInput_SetSyncBypass(hdlr, bypass);
        fail_unless(bypass == IDTInput_GetSyncBypass(hdlr));
    }
}
END_TEST

START_TEST (test_SetSyncEnable)
{
    T_externalSyncEnable enable;

    for (enable=externalSyncEnable_Disable;
         enable<externalSyncEnable_MAX;
         enable = ((T_externalSyncEnable)enable+1)) {
        IDTInput_SetSyncEnable(hdlr, enable);
        fail_unless(enable == IDTInput_GetSyncEnable(hdlr));
    }
}
END_TEST

START_TEST (test_AmiInputFrequency)
{
    int i;
    T_idtAmiInputFrequencyBitFieldValue amiSignal;

    for (i=0; i<AmiFreq_MAX; i++)
    {
        IDTInput_SetAmiInputFrequency(
            hdlr, _i, 
            (T_idtAmiInputFrequencyBitFieldValue)i);
        amiSignal = IDTInput_GetAmiInputFrequency(hdlr,
                                                _i);
        fail_unless(amiSignal == 
                    (T_idtAmiInputFrequencyBitFieldValue)i);
    }
}
END_TEST


Suite *Input_suite (void)
{
    T_osUint8 i;
    Suite *s = suite_create ("Input");

    /* General test case */
    TCase *tc_input = tcase_create ("Input");

    tcase_add_checked_fixture (tc_input,
                               initalizeDriver, 
                               deinitilizeDriver);

    tcase_add_test (tc_input, test_BucketConfig);
    tcase_add_test (tc_input, test_AllInputs);
    tcase_add_test (tc_input, test_SyncFrequency);
    tcase_add_loop_test(tc_input, test_SyncSampling,
                        1,2);
    tcase_add_test(tc_input, test_SetSyncEdge);
    tcase_add_test(tc_input, test_SyncAlarmRange);
    tcase_add_test(tc_input, test_SetSyncBypass);
    tcase_add_test(tc_input, test_SetSyncEnable);

    for (i=1; i<IDT_DRV_MAX_INPUT; i++)
    {
        T_osUint8 internalPort;

        internalPort = hdlr->deviceConfig_m->inputExt2IntXlate_ma[i];
        if (internalPort != 0)
        {
            tcase_add_loop_test (tc_input, test_InputPriority, i,i+1);
            tcase_add_loop_test (tc_input, test_InputFrequency, i,i+1);
            tcase_add_loop_test (tc_input, test_ActivityMonitor, i,i+1);
            tcase_add_loop_test (tc_input, test_PreDivider, i,i+1);
            tcase_add_loop_test (tc_input, test_DivisionFactor, i,i+1);
            tcase_add_loop_test (tc_input, test_InputStatus, i,i+1);
            if ((internalPort == 5) || (internalPort == 6))
                tcase_add_loop_test (tc_input, 
                                     test_InputHFdivider, 
                                     i,i+1);
            tcase_add_loop_test (tc_input, test_InputEnable, i,i+1);
            if ((internalPort == 1) || (internalPort == 2))
                tcase_add_loop_test (tc_input, 
                                     test_AmiInputFrequency, 
                                     i,i+1);
        }
        else 
        {
            /* Not supported, so it should assert */
            tcase_add_loop_test_raise_signal (tc_input, 
                                              test_InputPriority, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_input, 
                                              test_InputFrequency, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_input, 
                                              test_ActivityMonitor, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_input, 
                                              test_PreDivider, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_input, 
                                              test_DivisionFactor, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_input, 
                                              test_InputStatus, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_input, 
                                              test_InputHFdivider, 
                                              6, i, i+1);
            tcase_add_loop_test_raise_signal (tc_input, 
                                              test_InputEnable, 
                                              6, i, i+1);
        }
    }

    suite_add_tcase (s, tc_input);

    return s;
}


