/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <check.h>

#include "osNumeric.h"
#include "Define.h"
#include "Output.h"
#include "Dpll.h"
#include "Apll.h"

#include "check_main.h"
/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/
/* Ethernet 25MHz */
static T_idtApllConfigPerOutput apll1ConfigPerOutput =
{
    APLL_CONFIG0,
    APLL_CLK_SEL_INTERNAL,
    2430,
    3125,
    4,
    0,
    APLL_XTAL_1_APLL1,
    APLL_OUT_SIG_LVPECL,
    APLL_OUTPUT_ENABLE
};

static T_idtApllConfigPerOutput apll2ConfigPerOutput =
{
    APLL_CONFIG0,
    APLL_CLK_SEL_INTERNAL,
    2430,
    3125,
    4,
    0,
    APLL_XTAL_4_APLL2,
    APLL_OUT_SIG_LVPECL,
    APLL_OUTPUT_ENABLE
};

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/

static T_osUint8  setupOutputMux(T_idtDrvHdlr hdlr,
                                 T_idtDpllInstance nDpll,
                                 T_outputPathType  nPath)
{
    switch (nPath)
    {

    case OutputPath_SonetGigEth:
    case OutputPath_7776kHz:
    case OutputPath_Eth:
    case OutputPath_16E1_16T1:
    {
        /* No mux to configure */
        return 1;
        break;
    }

    case OutputPath_12E1_E3_T3:
    {
        IDTDpll_SetDpllOutputPathSelectorB(
            hdlr,
            nDpll, 
            DPLLOutputSelB_12E1);
        break;
    }

    case OutputPath_GPS:
    {
        if (nDpll == DPLL_INSTANCE_1)
        {
            IDTDpll_SetDpllOutputPathSelectorB(
                hdlr,
                nDpll, 
                DPLLOutputSelB_GPS);
        }
        else if (nDpll == DPLL_INSTANCE_2)
        {
            IDTDpll_SetDpllOutputPathSelectorA(
                hdlr,
                nDpll, 
                DPLLOutputSelA_GPS);
        }
        break;
    }

    case OutputPath_GSM_16E1_16T1:
    {
        IDTDpll_SetDpllOutputPathSelectorA(
            hdlr, 
            nDpll, 
            DPLLOutputSelA_GSM);
        break;
    }

    case OutputPath_OBSAI:
    {
        if (nDpll == DPLL_INSTANCE_1)
        {
            IDTDpll_SetDpllOutputPathSelectorA(
                hdlr, 
                nDpll, 
                DPLLOutputSelA_OBSAI);
        }
        else if (nDpll == DPLL_INSTANCE_2)
        {
            /* DPLL 2 does not support OBSAI */
            return 0;
        }
        break;
    }
    case OutputPath_24T1:
    {
        if (nDpll == DPLL_INSTANCE_1)
        {
            /* DPLL 1 does not support 24T1 */
            return 0;
        }
        else if (nDpll == DPLL_INSTANCE_2)
        {
            IDTDpll_SetDpllOutputPathSelectorB(
                hdlr, 
                nDpll, 
                DPLLOutputSelB_24T1);
        }
        break;
    }
    default:
        OS_ERROR(("Unknown path %d\n",nPath));
        return 0;
        break;
    }

    return 1;
}

START_TEST (test_OutputConfig)
{
    T_idtDpllInstance nDpll;
    T_outputPathType  nPath;
    T_osUint8        nFactor;
    T_idtApllConfigPerOutput tmpApllConfigPerOutput;
    T_idtOutputApllConfig outputApll;
    IDT_GET_OUTPUT_APLL_CONFIG(hdlr, _i, outputApll);

    for (nDpll=DPLL_INSTANCE_1; 
         nDpll<DPLL_INSTANCE_MAX;
         nDpll=(T_idtDpllInstance)nDpll+1)
        for(nPath=OutputPath_SonetGigEth;
            nPath<OutputPath_MAX;
            nPath=(T_outputPathType)nPath+1)
            for(nFactor=1; 
                nFactor<(1<<BF_OUT1_DIVIDER_SIZE);
                nFactor++)
            {
                T_idtDpllInstance rdDpll;
                T_outputPathType  rdPath;
                T_osUint8         rdFactor;
                T_osUint8         isValidPathDpllCombo = 0;
                T_osBool          dev317port3 = E_osFalse;

                memset(&tmpApllConfigPerOutput, 
                       0,
                       sizeof(tmpApllConfigPerOutput));

               /* If we don't support this DPLL (skip). Don't worry
                * we'll check for this in the DPLL unit tests */
                if (IDTDpll_GetTypeOfDpll(hdlr,nDpll) == Dpll_MAX)
                    continue;

                /* Ouputs 8/9 don't have
                 * provisionable output
                 * dividers */
                if ((_i == 8) || (_i == 9))
                {
                    if (nFactor != 1)
                        continue;
                    if ((_i == 8) && (nPath != OutputPath_7776kHz))
                        continue;
                    if ((_i == 9) && (nPath != OutputPath_16E1_16T1))
                        continue;
                }

                /* Need to setup output mux first or get may fail
                 *
                 * Get routine relies on output muxes to determine
                 * dpll and path */
                isValidPathDpllCombo = setupOutputMux(hdlr,nDpll,nPath);
                if (!isValidPathDpllCombo)
                    continue;

                if ((IDT_8V89317 == hdlr->deviceType_m) && (3 == _i))
                {
                    dev317port3 = E_osTrue;
                }

                if (APLL_INST_1 == outputApll.apllInst)
                {
                    IDTOutput_SetOutputConfig(hdlr, 
                                              _i, 
                                              nDpll, 
                                              nPath, 
                                              nFactor,
                                              &apll1ConfigPerOutput);
                }
                else
                {
                     T_idtApllDividerValue configured;

                     if (dev317port3)
                     {
                        configured = apll2ConfigPerOutput.outputDiv;
                        apll2ConfigPerOutput.outputDiv = 2;
                     }

                     IDTOutput_SetOutputConfig(hdlr, 
                                              _i, 
                                              nDpll, 
                                              nPath, 
                                              nFactor,
                                              &apll2ConfigPerOutput);

                     if (dev317port3)
                        apll2ConfigPerOutput.outputDiv = configured;
                }

                IDTOutput_GetOutputConfig(hdlr,  
                                          _i,
                                          &rdDpll, 
                                          &rdPath, 
                                          &rdFactor,
                                          &tmpApllConfigPerOutput);
                
                if (APLL_INST_MAX == outputApll.apllInst)
                {
                    /* Don't check APLL config for output that does not go through APLL */
                    fail_unless(((nDpll == rdDpll)&&
                                (nPath == rdPath)&&
                                (nFactor == rdFactor)),
                            "SetOutputConfig "
                            "not working "
                            "nOutN/%d nDpll/%d:%d "
                            "nPath/%d:%d nFactor/%d:%d",
                            _i, 
                            nDpll, rdDpll,
                            nPath, rdPath,
                            nFactor, rdFactor);
                }
                else if (APLL_INST_1 == outputApll.apllInst)
                {   
                    fail_unless(((nDpll == rdDpll)&&
                                (nPath == rdPath)&&
                                (nFactor == rdFactor)&&
                                !memcmp(&apll1ConfigPerOutput,
                                        &tmpApllConfigPerOutput,
                                        sizeof(T_idtApllConfigPerOutput))),
                            "SetOutputConfig "
                            "not working "
                            "nOutN/%d nDpll/%d:%d "
                            "nPath/%d:%d nFactor/%d:%d",
                            _i, 
                            nDpll, rdDpll,
                            nPath, rdPath,
                            nFactor, rdFactor);
                }
                else
                {
                    T_idtApllDividerValue configured;

                    if (dev317port3)
                    {
                       configured = apll2ConfigPerOutput.outputDiv;
                       apll2ConfigPerOutput.outputDiv = 2;
                    }

                    fail_unless(((nDpll == rdDpll)&&
                                (nPath == rdPath)&&
                                (nFactor == rdFactor)&&
                                !memcmp(&apll2ConfigPerOutput,
                                        &tmpApllConfigPerOutput,
                                        sizeof(T_idtApllConfigPerOutput))),
                            "SetOutputConfig "
                            "not working "
                            "nOutN/%d nDpll/%d:%d "
                            "nPath/%d:%d nFactor/%d:%d",
                            _i, 
                            nDpll, rdDpll,
                            nPath, rdPath,
                            nFactor, rdFactor);

                    if (dev317port3)
                        apll2ConfigPerOutput.outputDiv = configured;
                }
            }
}
END_TEST

START_TEST (test_OutputInvert)
{
    IDTOutput_SetOutputInvert(hdlr, _i, 0);
    fail_unless(IDTOutput_GetOutputInvert(hdlr, _i) == 0,
                "SetOutputInvert nOutN/%d 0 not working",_i);
    IDTOutput_SetOutputInvert(hdlr, _i, 1);
    fail_unless(IDTOutput_GetOutputInvert(hdlr, _i) == 1,
                "SetOutputInvert nOutN/%d 1 not working",_i);
}
END_TEST

START_TEST (test_OutputEnable)
{
    IDTOutput_SetOutputEnable(hdlr, _i, 0);
    fail_unless(IDTOutput_GetOutputEnable(hdlr, _i) == 0,
                "SetOutputEnable nOutN/%d 0 not working",_i);
    IDTOutput_SetOutputEnable(hdlr, _i, 1);
    fail_unless(IDTOutput_GetOutputEnable(hdlr, _i) == 1,
                "SetOutputEnable nOutN/%d 1 not working",_i);
}
END_TEST

START_TEST (test_OutputInterfaceDifferential)
{
    T_outputInterface interface;

    for (interface=OUTPUTIF_LVDS; 
         interface<=OUTPUTIF_PECL; 
         interface = (T_outputInterface)interface+1)
    {
        IDTOutput_SetOuputpuInterface(hdlr, _i, interface);
        fail_unless(IDTOutput_GetOuputpuInterface(hdlr, _i) == interface,
                    "SetOuputpuInterface nOutN/%d "
                    "interface/%d not working", _i, interface);
    }
}
END_TEST

START_TEST (test_OutputInterfaceCmos)
{
    /* Test for static interface types */
    IDTOutput_SetOuputpuInterface(hdlr, _i, OUTPUTIF_CMOS);
    fail_unless(IDTOutput_GetOuputpuInterface(hdlr, _i) == OUTPUTIF_CMOS,
                "GetOuputpuInterface nOutN/%d "
                "interface/%d not working", _i, OUTPUTIF_CMOS);
}
END_TEST

START_TEST (test_OutputInterfaceAmi)
{
    /* Test for static interface types */
    IDTOutput_SetOuputpuInterface(hdlr, _i, OUTPUTIF_AMI);
    fail_unless(IDTOutput_GetOuputpuInterface(hdlr, _i) == OUTPUTIF_AMI,
                "GetOuputpuInterface nOutN/%d "
                "interface/%d not working", _i, OUTPUTIF_AMI);
}
END_TEST

START_TEST (test_FrameSync)
{
    int mask;
    T_osUint8 enable;
    T_osUint8 invert;
    T_osUint8 pulse;

    for (mask=0; mask<8; mask++)
    {
        IDTOutput_SetFrameSync(hdlr, frameSync_FRSYNC,
                               (mask&0x1)?0:1, /* is1Pps */
                               (mask&0x2)?0:1, /* invert */
                               (mask&0x4)?0:1); /* pulse */
        IDTOutput_GetFrameSync(hdlr,
                               frameSync_FRSYNC,
                               &enable, &invert, &pulse);
        fail_unless(((enable==(T_osUint8)((mask&0x1)?0:1)) &&
                     (invert==(T_osUint8)((mask&0x2)?0:1)) &&
                     (pulse==(T_osUint8)((mask&0x4)?0:1))),
                    "IDTOutput_SetFrameSync not working, enable/%d:%d"
                    " invert/%d:%d pulse/%d:%d\n",
                    enable,((mask&0x1)?0:1),
                    invert,((mask&0x2)?0:1),
                    pulse,((mask&0x4)?0:1));

        IDTOutput_SetFrameSync(hdlr, frameSync_MFRSYNC,
                               (mask&0x1)?0:1, /* is1Pps */
                               (mask&0x2)?0:1, /* invert */
                               (mask&0x4)?0:1); /* pulse */
        IDTOutput_GetFrameSync(hdlr,
                               frameSync_MFRSYNC,
                               &enable, &invert, &pulse);
        fail_unless(((enable==(T_osUint8)((mask&0x1)?0:1)) &&
                     (invert==(T_osUint8)((mask&0x2)?0:1)) &&
                     (pulse==(T_osUint8)((mask&0x4)?0:1))),
                    "IDTOutput_SetFrameSync not working, enable/%d:%d"
                    " invert/%d:%d pulse/%d:%d\n",
                    enable,((mask&0x1)?0:1),
                    invert,((mask&0x2)?0:1),
                    pulse,((mask&0x4)?0:1));
    }
}
END_TEST

START_TEST (test_FrameSyncPulseEdge)
{
    IDTOutput_SetFrameSyncPulseEdge(hdlr, 1);
    fail_unless(IDTOutput_GetFrameSyncPulseEdge(hdlr) == 1,
                "IDTOutput_SetFrameSyncPulseEdge not working 1");
    IDTOutput_SetFrameSyncPulseEdge(hdlr, 0);
    fail_unless(IDTOutput_GetFrameSyncPulseEdge(hdlr) == 0,
                "IDTOutput_SetFrameSyncPulseEdge not working 0");
}
END_TEST

Suite *Output_suite (void)
{
    T_osUint8 i;
    Suite *s = suite_create ("Output");

    /* Output test case */
    TCase *tc_output = tcase_create ("Output");

    tcase_add_checked_fixture (tc_output,
                               initalizeDriver, 
                               deinitilizeDriver);

    for (i=1; i<=IDT_DRV_MAX_OUTPUT; i++)
    {
        T_osUint8 internalPort;

        internalPort = hdlr->deviceConfig_m->outputExt2IntXlate_ma[i];
        if (internalPort != 0)
        {
            tcase_add_loop_test (tc_output, 
                                 test_OutputConfig,
                                 i, i+1);
            if ((internalPort != 6) && (internalPort != 7) &&
                (internalPort != 10) && (internalPort != 11))
                tcase_add_loop_test (tc_output, 
                                     test_OutputInvert,
                                     i, i+1);
            if ((internalPort == 8) || (internalPort == 9))
                tcase_add_loop_test (tc_output, 
                                     test_OutputEnable,
                                     i, i+1);
            if ((internalPort == 6) || (internalPort == 7))
            {
                tcase_add_loop_test (tc_output, 
                                     test_OutputInterfaceDifferential,
                                     i, i+1);
            }
            else if (internalPort == 8)
            {
                tcase_add_loop_test (tc_output, 
                                     test_OutputInterfaceAmi,
                                     i, i+1);
            }
            else
            {
                tcase_add_loop_test (tc_output, 
                                     test_OutputInterfaceCmos,
                                     i, i+1);
            }
        }
        else
        {
            /* Not supported, so it should assert */

            /* Ports 8/9 are always available */
            if ((internalPort != 8) && (internalPort != 9))
                tcase_add_loop_test_raise_signal (tc_output, 
                                                  test_OutputConfig,
                                                  6, i, i+1);

            if ((internalPort == 6) || (internalPort == 7) ||
                (internalPort == 10) || (internalPort == 11))
                tcase_add_loop_test_raise_signal (tc_output, 
                                                  test_OutputInvert,
                                                  6, i, i+1);
            if ((internalPort != 8) && (internalPort != 9))
                tcase_add_loop_test_raise_signal (tc_output, 
                                                  test_OutputEnable,
                                                  6, i, i+1);
            if ((internalPort == 6) || (internalPort == 7))
            {
                tcase_add_loop_test_raise_signal(
                    tc_output, 
                    test_OutputInterfaceAmi,
                    6, i, i+1);
                tcase_add_loop_test_raise_signal(
                    tc_output, 
                    test_OutputInterfaceCmos,
                    6, i, i+1);
            }
            else if (internalPort == 8)
            {
                tcase_add_loop_test_raise_signal(
                    tc_output, 
                    test_OutputInterfaceCmos,
                    6, i, i+1);
                tcase_add_loop_test_raise_signal(
                    tc_output, 
                    test_OutputInterfaceDifferential,
                    6, i, i+1);
            }
            else
            {
                tcase_add_loop_test_raise_signal(
                    tc_output, 
                    test_OutputInterfaceAmi,
                    6, i, i+1);
                tcase_add_loop_test_raise_signal(
                    tc_output, 
                    test_OutputInterfaceDifferential,
                    6, i, i+1);
            }
        }
    }

    tcase_add_test (tc_output, test_FrameSync);
    tcase_add_test (tc_output, test_FrameSyncPulseEdge);

    suite_add_tcase (s, tc_output);

    return s;
}


