#include <stdlib.h>

#include "check.h"

#include "ReadWrite_sim.h"
#include "idt82V3911.h"

#include "check_main.h"

#define I2C_ADDR_FAKE   (0x50)

static T_idtApllXtal vcxoInfo[] = {
    { APLL_XTAL_1_APLL1, APLL_XTAL_FREQ_25000KHz },
    { APLL_XTAL_4_APLL2, APLL_XTAL_FREQ_25000KHz }
};

static const T_idtDrvAccess accessMethods =  {
    Init_Sim,
    DeInit_Sim,
    RDByte_Sim,
    WRByte_Sim,
    NULL  /* Simulated */
};

T_idtDrvHdlr hdlr;

void initalizeDriver (void)
{
        OS_SETLOGLVL(0);
        hdlr = IDTGeneral_InitDriver("check3911", 
                                     accessMethods, 
                                     IDT_82V3911, 
                                     vcxoInfo, 
                                     2, 
                                     I2C_ADDR_FAKE, 
                                     1);
        Init_Sim(NULL);
        IDTGeneral_InitDevice(hdlr);
}

void deinitilizeDriver(void)
{
    IDTGeneral_DeInitDriver(hdlr);
    DeInit_Sim(NULL);
}

Suite *TopLevel_suite (void)
{
    Suite *s = suite_create ("TopLevel");
    return s;
}

int main (int argc, char *argv[])
{
  int      number_failed;
  SRunner *sr = srunner_create (TopLevel_suite());

  /* Specify output log file */
  srunner_set_log (sr, "test.log");

/*  srunner_set_fork_status(sr, CK_NOFORK);*/

  /* Temporarily setup device driver handle so test setup will
   * work */
  hdlr = IDTGeneral_InitDriver("check3911", 
                               accessMethods, 
                               IDT_82V3911, 
                               vcxoInfo, 
                               2, 
                               I2C_ADDR_FAKE,
                               1);
  {
      srunner_add_suite (sr, General_suite());
      srunner_add_suite (sr, Input_suite());
      srunner_add_suite (sr, DpllCnfg_suite());
      srunner_add_suite (sr, Output_suite());
      srunner_add_suite (sr, InputFreq_suite());
  }
  IDTGeneral_DeInitDriver(hdlr);

  srunner_run_all (sr, CK_NORMAL);

  number_failed = srunner_ntests_failed (sr);

  srunner_free (sr);
  return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
