/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "RegisterDef.h"
#include "ApllRegDef.h"
#include "annotate.h"

#define FGETS_BUF_SIZE  255

extern const RegisterName_t Page0RegisterNameArray[];
extern const int Page0RegisterNameArraySize;
extern const RegisterName_t Page1RegisterNameArray[];
extern const int Page1RegisterNameArraySize;
extern const RegisterName_t ApllRegisterNameArray[];
extern const int ApllRegisterNameArraySize;

static FILE *infile = NULL;
static FILE *outfile = NULL;
static int rgfFileVersion;

static void checkRgfFileVersion(void)
{
    char buf[FGETS_BUF_SIZE];

    if (!infile)
        return;

    while (fgets(buf, FGETS_BUF_SIZE, infile)) {
        if (strstr(buf, "[Page0_Parameters]")) {
            rgfFileVersion = 2;
            goto RET;
        }

        if (strstr(buf, "[Parameters]")) {
            rgfFileVersion = 1;
            goto RET;
        }
    }

RET:
    rewind(infile);
    return;
}

static int processRegValStr(char* buf, char* regLocation, const RegisterName_t *registerNameArray, int arraySize)
{
    char     addrString[4];
    char     valString[3];
    long int addr;
    long int val;
    int i, j;
    int found = 0;

    if (!buf || !regLocation || !registerNameArray || !outfile)
        return found;

    /* Extract register address */
    strncpy(addrString,regLocation+3,3);
    addrString[3] = '\0';
    if (addrString[0] == '0')
        addr = strtol(addrString,NULL,16);
    else {
        addr = 0x0FF&(strtol(addrString,NULL,16));
        addr &= 0x7F;
    }

    /* Extract register value */
    strncpy(valString,regLocation+11,2);
    valString[2] = '\0';
    val = strtol(valString,NULL,16);

    /* Look for register name given address */
    for (i=0; i<arraySize; i++) {

        const RegisterName_t *regPtr;

        regPtr = &registerNameArray[i];
        /* Matching address ? */
        if (regPtr->addrOff_m == addr) {

            /* Strip newline/carriage return */
            if (buf[strlen(buf)-1] == '\n')
                buf[strlen(buf)-1] = '\0';

            /* Append register name */
            fprintf(outfile,"%s # %s\n",
                    buf, regPtr->name_m);
            found = 1;

            /* Enhanced annotation... bitfields! 
             *
             *  For every register, we display what each bitfield is
             *  set to. */
            for (j=0; j<regPtr->numberOfBf_m; j++) {

                const BitfieldName_t *bfPtr;

                bfPtr = &regPtr->bfArray[j];
                fprintf(outfile,"             # %s",
                        bfPtr->name_m);
                if (bfPtr->size_m > 1)
                    fprintf(outfile,"[%d:%d]",
                            (bfPtr->size_m+bfPtr->lsb_m-1),
                            bfPtr->lsb_m);
                else
                    fprintf(outfile,"[%d]",
                            bfPtr->lsb_m);
                fprintf(outfile," = 0x%02X\n",
                        (((unsigned char)val)&bfPtr->mask_m)>>bfPtr->lsb_m);

            }
            break;
        }
    }

    return found;
}

static void processRgfVersion1(void)
{
    char buf[FGETS_BUF_SIZE];
    char *tempPtr = NULL;
    int found;

    if (!infile || !outfile)
        return;

    do {
            if (!fgets(buf,FGETS_BUF_SIZE,infile)) 
                continue;

            /* Search for "Reg.* = " string 
             *
             *   Register lines are formatted:
             *      Reg[ADDR] = [VALUE]
             */
            if ((tempPtr = strstr(buf,"Reg"))) {
                if (strstr(tempPtr+3," = ")) {
                    if (buf[3] == '0') {
                        found = processRegValStr(buf,
                                                 tempPtr,
                                                 Page0RegisterNameArray,
                                                 Page0RegisterNameArraySize);
                    }
                    else if (buf[3] == '1') {
                        found = processRegValStr(buf,
                                                 tempPtr,
                                                 ApllRegisterNameArray,
                                                 ApllRegisterNameArraySize);
                    }
                    else
                        found = 0;

                    if (found)
                        continue;

                    /* Not found... fall through and just print out line */
                }
            }

            /* Skip over non-register lines */
            fputs(buf,outfile);
    }while (!feof(infile));
}

static void processRgfVersion2(void)
{
    char buf[FGETS_BUF_SIZE];
    char *tempPtr = NULL;
    int found, page = 0;

    const RegisterName_t *regNameArray = NULL;
    int arraySize;

    if (!infile || !outfile)
        return;

    do {
            if (!fgets(buf,FGETS_BUF_SIZE,infile))
                continue;

            if (strstr(buf, "[Page0_Parameters]") ||
                strstr(buf, "[Page0_T0_Parameters]") ||
                strstr(buf, "[Page0_T4_Parameters]")) {
                    page = 0;
            }

            if (strstr(buf, "[Page1_Parameters]")) {
                page = 1;
            }

            /* Search for "Reg.* = " string 
             *
             *   Register lines are formatted:
             *      Reg[ADDR] = [VALUE]
             */
            if ((tempPtr = strstr(buf,"Reg"))) {
                if (strstr(tempPtr+3," = ")) {
                    if (page == 0) {
                        if (buf[3] == '0') {
                            /* Page 0 register */
                            regNameArray = Page0RegisterNameArray;
                            arraySize = Page0RegisterNameArraySize;
                        } else if (buf[3] == '1') {
                            /* APLL register */
                            regNameArray = ApllRegisterNameArray;
                            arraySize = ApllRegisterNameArraySize;
                        } else {
                            regNameArray = NULL;
                            arraySize = 0;
                        }
                    } else if (page == 1) {
                        /* Page 1 register */
                        regNameArray = Page1RegisterNameArray;
                        arraySize = Page1RegisterNameArraySize;
                    } else {
                        regNameArray = NULL;
                        arraySize = 0;
                    }
                
                    found = processRegValStr(buf,
                                             tempPtr,
                                             regNameArray,
                                             arraySize);

                    if (found)
                        continue;

                    /* Not found... fall through and just print out line */
                }
            }

            /* Skip over non-register lines */
            fputs(buf,outfile);
    }while (!feof(infile));
}

static void processRgfVersionUnknown(void)
{
    processRgfVersion1();
}

int main(int argc, char *argv[])
{
    if (argc != 2) {
        printf("%s usage:\n"
                "   %s [RGF FILE] <[OUTFILE]>\n",
                argv[0],argv[0]);
        return 1;
    }

    infile = fopen(argv[1],"r+t");
    if (argc>2)
        outfile = fopen(argv[2],"w+t");
    else
        outfile = fopen("temp.txt","w+t");

    checkRgfFileVersion();

    if (rgfFileVersion == 1)
        processRgfVersion1();
    else if (rgfFileVersion == 2)
        processRgfVersion2();
    else
        processRgfVersionUnknown();

    fclose(outfile);
    fclose(infile); 

    return 0;
}
