/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <stdlib.h>
#include <string.h>

#include "osAdapt.h"
#include "Define.h"
#include "Hal.h"
#include "load.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/
#define APLL_REG_MAP_SIZE   (17)

/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
static void processApllRegistersSection(T_idtReadFromFile inputFunc,
                                        T_idtDrvHdlr      hdlr)
{
    char tempString[200];
    T_osUint8 writeCache[256];
    T_osUint8 writeUsed[256];
    int i = 0;

    OS_ZERO(writeCache,sizeof(T_osUint8)*256);
    OS_ZERO(writeUsed,sizeof(T_osUint8)*256);

    /* Cache writes */
    while (inputFunc(tempString,200) && i < APLL_REG_MAP_SIZE*2)
    {
        char *regAddrString;
        int regAddr;
        char *regValString;
        int regVal;

        if (!strstr(tempString, "Reg"))
            break;
           
        /* Process line */
        regAddrString = strtok(tempString," ");
        if (!regAddrString) continue;
        /* AND with '0x0FF' to strip off leading '1' to get real address */
        regAddr = 0x0FF&((int)strtol(regAddrString+3,NULL,16));
        
        if ((regAddr<0) || (regAddr>255)) continue;

        /* Skip '=' */
        strtok(NULL," ");

        regValString = strtok(NULL," ");
        if (!regValString) continue;
        regVal = (int)strtol(regValString,NULL,16);

        /* Save to cache - mark also that we're using this address */
        writeUsed[regAddr] = 1;
        writeCache[regAddr] = regVal;
        i++;
    }

    /* Write cache to device */
    for (i=0; i<256; i++)
    {
        if (writeUsed[i])
        {
            /* Write to device */
            IDTHal_Write_Ext(hdlr, i, writeCache[i]);
        }
    }
}

static void processRegistersSection(T_idtReadFromFile inputFunc,
                                    T_idtDrvHdlr      hdlr)
{
    char tempString[200];
    T_osUint8 writeCache[256];
    T_osUint8 writeUsed[256];
    int i;

    OS_ZERO(writeCache,sizeof(T_osUint8)*256);
    OS_ZERO(writeUsed,sizeof(T_osUint8)*256);

    /* Cache writes */
    while (inputFunc(tempString,200))
    {
        char *regAddrString;
        int regAddr;
        char *regValString;
        int regVal;

        if (!strstr(tempString, "Reg"))
            break;
           
        /* Process line */
        regAddrString = strtok(tempString," ");
        if (!regAddrString) continue;
        regAddr = (int)strtol(regAddrString+3,NULL,16);
        if ((regAddr<0) || (regAddr>255)) continue;

        /* Skip '=' */
        strtok(NULL," ");

        regValString = strtok(NULL," ");
        if (!regValString) continue;
        regVal = (int)strtol(regValString,NULL,16);

        /* Save to cache - mark also that we're using this address */
        writeUsed[regAddr] = 1;
        writeCache[regAddr] = regVal;
    }

    /* Write cache to device */
    for (i=0; i<256; i++)
    {
        if (writeUsed[i])
        {
            /* Write to device */
            IDTHal_Write(hdlr, i, writeCache[i]);
        }
    }
}

static void processPreDividerSection(T_idtReadFromFile inputFunc,
                                     T_idtDrvHdlr      hdlr)
{
    char tempString[200];
    T_osUint16 valArray[IDT_DRV_MAX_INPUT+1];
    int i;

    /* Initialize array */
    for (i=0; i<(IDT_DRV_MAX_INPUT+1); i++)
        valArray[i] = 0;

    while (inputFunc(tempString,200))
    {
        char *inputString;
        int input;
        char *regAddrString;
        char *regValString;
        int regVal;

        if (!strstr(tempString, "IN"))
            return;
           
        /* Process line */
        inputString = strtok(tempString," ");
        if (!inputString) continue;

        /* Convert to integer */
        input = (int)strtol(inputString+2,&regAddrString,16);
        if ((input<0) || (input>IDT_DRV_MAX_INPUT))
            continue;

        /* Skip '=' */
        strtok(NULL," ");

        regValString = strtok(NULL," ");
        if (!regValString) continue;
        regVal = (int)strtol(regValString,NULL,16);

        /* Write to device */
        if (regAddrString[0] == 'L')
        {
		   valArray[input] = regVal;
        }
        if (regAddrString[0] == 'H')
        {
		   valArray[input] |= (regVal<<BF_PRE_DIVN_VALUE_A_SIZE);
        }
    }

    for (i=3; i<=IDT_DRV_MAX_INPUT; i++)
    {
        IDTHal_Write(hdlr, REG_PRE_DIV_CH_CNFG,i);
        IDTHal_Write(hdlr, REG_PRE_DIVN_LOW_CNFG, 
                     valArray[i] & BF_PRE_DIVN_VALUE_A_MASK);
        IDTHal_Write(hdlr, REG_PRE_DIVN_HIGH_CNFG, 
                     ((valArray[i] >> BF_PRE_DIVN_VALUE_A_SIZE) &
                      BF_PRE_DIVN_VALUE_B_MASK));
    }

}

void loadVersion1(T_idtReadFromFile inputFunc,
                  T_idtDrvHdlr      hdlr)
{
    char tempString[200];

    while (inputFunc(tempString,200))
        if (strstr(tempString, "[Info]"))
            break;
    while (inputFunc(tempString,200))
        if (strstr(tempString, "Total = "))
            break;
    while (inputFunc(tempString,200))
        if (strstr(tempString, "Date = "))
            break;
    while (inputFunc(tempString,200))
        if (strstr(tempString, "Title = "))
            break;

    while (inputFunc(tempString,200))
        if (strstr(tempString, "[Parameters]"))
            break;
    processRegistersSection(inputFunc,hdlr);
}

void loadVersion2(T_idtReadFromFile inputFunc,
                  T_idtDrvHdlr      hdlr)
{
    char tempString[200];

    while (inputFunc(tempString,200))
        if (strstr(tempString, "[Info]"))
            break;
    while (inputFunc(tempString,200))
        if (strstr(tempString, "Total = "))
            break;
    while (inputFunc(tempString,200))
        if (strstr(tempString, "Date = "))
            break;
    while (inputFunc(tempString,200))
        if (strstr(tempString, "Title = "))
            break;

    /* Select page 0 */
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0);
    while (inputFunc(tempString,200))
        if (strstr(tempString, "[Page0_Parameters]"))
            break;
    processApllRegistersSection(inputFunc,hdlr);
    processRegistersSection(inputFunc,hdlr);

    /* Input pre-dividers */
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0);
    IDTHal_Write(hdlr, REG_T4_T0_SEL_CNFG,0x00);
    processPreDividerSection(inputFunc,hdlr);

    /* Select T0 DPLL */
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0);
    IDTHal_Write(hdlr, REG_T4_T0_SEL_CNFG,0x00);
    processRegistersSection(inputFunc,hdlr);

    /* Select T4 DPLL */
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0);
    IDTHal_Write(hdlr, REG_T4_T0_SEL_CNFG,0x10);
    processRegistersSection(inputFunc,hdlr);

    /* Select page 0 */
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 1);
    processRegistersSection(inputFunc,hdlr);

    /* IMPORTANT - restore back page/Dpll selection registers */
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0);
    IDTHal_Write(hdlr, REG_T4_T0_SEL_CNFG,0x00);
}
