/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <stdlib.h>
#include <stdarg.h>

#include "osAdapt.h"
#include "Define.h"
#include "ApllRegDef.h"
#include "Hal.h"
#include "Dpll.h"
#include "save.h"

/*---------------------------------------------------------------------
 * T Y P E D E F S,    E N U M S   &    D E F I N E S
 *---------------------------------------------------------------------*/
typedef struct S_idtRegExclude {
    const T_osUint8 *regExcludeTable_m;
    const T_osUint8 *regExcludeTablePage1_m;
} T_idtRegExclude;

#define LAST_EXCLUSION 0xFF
/*---------------------------------------------------------------------
 * V A R I A B L E S    &    C O N S T A N T S
 *---------------------------------------------------------------------*/
static const char *idtSaveRestore_deviceType2StringTable[IDT_DEVICE_MAX] = {
    "82V3910", /**< 82V3910 Device */
    "82V3911", /**< 82V3911 Device */
    "82V33930", /**< 82V33930 Device */
    "8V89316", /**< 8V89316 Device */
    "8V89317"  /**< 8V89317 Device */
};

static const T_idtRegExclude idtSaveRestore_DeviceExcludeRegisters[IDT_DEVICE_MAX] = {
    { NULL, 
      NULL }, /**< 82V3910 Register exclusion table */
    { NULL,
      NULL }, /**< 82V3911 Register exclusion table */
    { NULL,
      NULL }, /**< 82V33930 Register exclusion table */
    { NULL,
      NULL }, /**< 8V89316 Register exclusion table */
    { NULL,
      NULL }, /**< 8V89317 Register exclusion table */
};

static const T_osUint8 idtSaveRestore_ValidRegisters[] = {
    REG_ID_LOW,
    REG_ID_HIGH,
    REG_MPU_PIN_STS,
    REG_NOMINAL_FREQ_LOW_CNFG,
    REG_NOMINAL_FREQ_MID_CNFG,
    REG_NOMINAL_FREQ_HIGH_CNFG,
    REG_T4_T0_SEL_CNFG,
    REG_PHASE_ALARM_TIME_OUT_CNFG,
    REG_INPUT_MODE_CNFG,
    REG_DIFFERENTIAL_IN_OUT_CNFG,
    REG_MON_SW_HS_CNFG,
    REG_INTERRUPT_CNFG,
    REG_INTERRUPTS1_STS,
    REG_INTERRUPTS2_STS,
    REG_INTERRUPTS3_STS,
    REG_INTERRUPTS1_MASK_CNFG,
    REG_INTERRUPTS2_MASK_CNFG,
    REG_INTERRUPTS3_MASK_CNFG,
    REG_MS_SL_CTRL_CNFG,
    REG_IN1_CNFG,
    REG_IN2_CNFG,
    REG_IN3_CNFG,
    REG_IN4_CNFG,
    REG_IN5_IN6_HF_DIV_CNFG,
    REG_IN5_CNFG,
    REG_IN6_CNFG,
    REG_IN7_CNFG,
    REG_IN8_CNFG,
    REG_IN9_CNFG,
    REG_IN10_CNFG,
    REG_IN11_CNFG,
    REG_IN12_CNFG,
    REG_IN13_CNFG,
    REG_IN14_CNFG,
    REG_PRE_DIV_CH_CNFG,
    REG_PRE_DIVN_LOW_CNFG,
    REG_PRE_DIVN_HIGH_CNFG,
    REG_IN1_IN2_SEL_PRIORITY_CNFG,
    REG_IN3_IN4_SEL_PRIORITY_CNFG,
    REG_IN5_IN6_SEL_PRIORITY_CNFG,
    REG_IN7_IN8_SEL_PRIORITY_CNFG,
    REG_IN9_IN10_SEL_PRIORITY_CNFG,
    REG_IN11_IN12_SEL_PRIORITY_CNFG,
    REG_IN13_IN14_SEL_PRIORITY_CNFG,
    REG_PAGE_POINTER_CNFG,
    REG_FREQ_MON_FACTOR_CNFG,
    REG_HARD_FREQ_MON_THRESHOLD_CNFG,
    REG_SOFT_FREQ_MON_THRESHOLD_CNFG,
    REG_UPPER_THRESHOLD_0_CNFG,
    REG_LOWER_THRESHOLD_0_CNFG,
    REG_BUCKET_SIZE_0_CNFG,
    REG_DECAY_RATE_0_CNFG,
    REG_UPPER_THRESHOLD_1_CNFG,
    REG_LOWER_THRESHOLD_1_CNFG,
    REG_BUCKET_SIZE_1_CNFG,
    REG_DECAY_RATE_1_CNFG,
    REG_UPPER_THRESHOLD_2_CNFG,
    REG_LOWER_THRESHOLD_2_CNFG,
    REG_BUCKET_SIZE_2_CNFG,
    REG_DECAY_RATE_2_CNFG,
    REG_UPPER_THRESHOLD_3_CNFG,
    REG_LOWER_THRESHOLD_3_CNFG,
    REG_BUCKET_SIZE_3_CNFG,
    REG_DECAY_RATE_3_CNFG,
    REG_IN_FREQ_READ_CH_CNFG,
    REG_IN_FREQ_READ_STS,
    REG_IN1_IN2_STS,
    REG_IN3_IN4_STS,
    REG_IN5_IN6_STS,
    REG_IN7_IN8_STS,
    REG_IN9_IN10_STS,
    REG_IN11_IN12_STS,
    REG_IN13_IN14_STS,
    REG_INPUT_VALID1_STS,
    REG_INPUT_VALID2_STS,
    REG_REMOTE_INPUT_VALID1_CNFG,
    REG_REMOTE_INPUT_VALID2_CNFG,
    REG_PRIORITY_TABLE1_STS,
    REG_PRIORITY_TABLE2_STS,
    REG_T0_INPUT_SEL_CNFG,
    REG_T4_INPUT_SEL_CNFG,
    REG_OPERATING_STS,
    REG_T0_OPERATION_MODE_CNFG,
    REG_T4_OPERATION_MODE_CNFG,
    REG_T0_DPLL_APLL_PATH_CNFG,
    REG_T0_DPLL_START_BW_DAMPING_CNFG,
    REG_T0_DPLL_ACQ_BW_DAMPING_CNFG,
    REG_T0_DPLL_LOCKED_BW_DAMPING_CNFG,
    REG_T0_BW_OVERSHOOT_CNFG,
    REG_PHASE_LOSS_COARSE_LIMIT_CNFG,
    REG_PHASE_LOSS_FINE_LIMIT_CNFG,
    REG_T0_HOLDOVER_MODE_CNFG,
    REG_HOLDOVER_FREQ_LOW_CNFG,
    REG_HOLDOVER_FREQ_MID_CNFG,
    REG_HOLDOVER_FREQ_HIGH_CNFG,
    REG_T4_DPLL_APLL_PATH_CNFG,
    REG_T4_DPLL_LOCKED_BW_DAMPING_CNFG,
    REG_CURRENT_FREQ_LOW_STS,
    REG_CURRENT_FREQ_MID_STS,
    REG_CURRENT_FREQ_HIGH_STS,
    REG_DPLL_FREQ_SOFT_LIMIT_CNFG,
    REG_DPLL_FREQ_HARD_LIMIT_LOW_CNFG,
    REG_DPLL_FREQ_HARD_LIMIT_MID_CNFG,
    REG_CURRENT_DPLL_PHASE_LOW_STS,
    REG_CURRENT_DPLL_PHASE_HIGH_STS,
    REG_OUT1_FREQ_CNFG,
    REG_OUT2_FREQ_CNFG,
    REG_OUT3_FREQ_CNFG,
    REG_OUT4_FREQ_CNFG,
    REG_OUT5_FREQ_CNFG,
    REG_OUT6_FREQ_CNFG,
    REG_OUT7_FREQ_CNFG,
    REG_OUT8_FREQ_CNFG,
    REG_OUT9_FREQ_CNFG,
    REG_FR_MFR_SYNC_CNFG,
    REG_PHASE_MON_CNFG,
    REG_PHASE_OFFSET_LOW_CNFG,
    REG_PHASE_OFFSET_HIGH_CNFG,
    REG_SYNC_MONITOR_CNFG,
    REG_SYNC_PHASE_CNFG,
    REG_PROTECTION_CNFG,
    REG_MPU_SEL_CNFG
};

static const int idtSaveRestore_ValidRegisterTableSize =
    (sizeof(idtSaveRestore_ValidRegisters)/sizeof(T_osUint8));

static const T_osUint8 idtSaveRestore_T4Registers[] = {
    REG_IN1_IN2_SEL_PRIORITY_CNFG,
    REG_IN3_IN4_SEL_PRIORITY_CNFG,
    REG_IN5_IN6_SEL_PRIORITY_CNFG,
    REG_IN7_IN8_SEL_PRIORITY_CNFG,
    REG_IN9_IN10_SEL_PRIORITY_CNFG,
    REG_IN11_IN12_SEL_PRIORITY_CNFG,
    REG_IN13_IN14_SEL_PRIORITY_CNFG,
    REG_PRIORITY_TABLE1_STS,
    REG_PRIORITY_TABLE2_STS,
    REG_PHASE_LOSS_COARSE_LIMIT_CNFG,
    REG_PHASE_LOSS_FINE_LIMIT_CNFG,
    REG_HOLDOVER_FREQ_LOW_CNFG,
    REG_HOLDOVER_FREQ_MID_CNFG,
    REG_HOLDOVER_FREQ_HIGH_CNFG
};

static const T_osUint8 idtSaveRestore_page1Registers[] = {
    REG_DFS_OFF_CNFG,
    REG_T0_PPS_CNFG,
    REG_PH_SLOPE_CNFG,
    REG_T0_ICP_CTRL_CNFG,
    REG_T4_ICP_CTRL_CNFG,
    REG_T4_PPS_CNFG
};

static const T_osUint8 idtSaveRestore_apllRegisters[] = {
    REG_CONTROL,
    REG_CLK_SEL,
    REG_PDSEL0_LSB,
    REG_PDSEL0_MSB,
    REG_PDSEL1_LSB,
    REG_PDSEL1_MSB,
    REG_M0_LSB,
    REG_M0_MSB,
    REG_M1_LSB,
    REG_M1_MSB,
    REG_ODBSELA0,
    REG_ODBSELA1,
    REG_ODBSELB0,
    REG_ODBSELB1,
    REG_VCXO_SEL,
    REG_CONFIG_QA,
    REG_CONFIG_QB
};

static const int idtSaveRestore_apllRegisterTableSize =
    (sizeof(idtSaveRestore_apllRegisters)/sizeof(T_osUint8));

/*---------------------------------------------------------------------
 * F U N C T I O N    I M P L E M E N T A T I O N
 *---------------------------------------------------------------------*/
static int idtSaveRestore_SkipRegister(T_osUint8    regAddr,
                                       int          isPage1,
                                       T_idtDrvHdlr hdlr)
{
    const T_idtRegExclude *exclusionTable;
    const T_osUint8 *excludedRegTable;
    int i;

    exclusionTable = &idtSaveRestore_DeviceExcludeRegisters[
        hdlr->deviceType_m];
    if (!isPage1)
        excludedRegTable = exclusionTable->regExcludeTable_m;
    else
        excludedRegTable = exclusionTable->regExcludeTablePage1_m;

    if (!excludedRegTable)
        return 0;

    /* Cycle through excluded register list. If we match then we
     * should ignore this register. */
    for (i=0; excludedRegTable[i]!=LAST_EXCLUSION; i++)
    {
        if (regAddr == excludedRegTable[i])
            return 1;
    }

    /* Register is ok it save */
    return 0;
}

static void idtSaveRestore_SaveHeaderVersion1(T_idtOutputToFile outputFunc,
                                              T_idtDrvHdlr      hdlr)
{
    outputFunc("[Info]\n");
    outputFunc("Total = %d\n",
               idtSaveRestore_ValidRegisterTableSize + 2*idtSaveRestore_apllRegisterTableSize);
    outputFunc("Date = %s\n", OS_CTIME());
    outputFunc("Title = --- IDT %s Registers ---\n",
               idtSaveRestore_deviceType2StringTable[hdlr->deviceType_m]);
}

static void idtSaveRestore_SaveHeaderVersion2(T_idtOutputToFile outputFunc,
                                              T_idtDrvHdlr      hdlr)
{
    outputFunc("[Info]\n");
    outputFunc("Total = %d\n",
               idtSaveRestore_ValidRegisterTableSize + 2*idtSaveRestore_apllRegisterTableSize);
    outputFunc("Date = %s\n", OS_CTIME());
    outputFunc("Title = --- IDT %s Registers ---\n",
               idtSaveRestore_deviceType2StringTable[hdlr->deviceType_m]);
}

static void idtSaveRestore_SavePage0T0(T_idtOutputToFile outputFunc,
                                       T_idtDrvHdlr      hdlr,
                                       int               isVersion1)
{
    int i; 
    T_osUint8 *saveCache = NULL;

    saveCache = (T_osUint8*)OS_MALLOC(idtSaveRestore_ValidRegisterTableSize*
                        sizeof(T_osUint8));
    if (!saveCache)
    {
        OS_ERROR(("Uanble to allocate cache size %d\n",
                  idtSaveRestore_ValidRegisterTableSize*
                  sizeof(T_osUint8)));
        return;
    }

    /* Select page 0 */
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0);

    /* Select T0 DPLL */
    IDTHal_Write(hdlr, REG_T4_T0_SEL_CNFG,0x00);

    /* Read register values in to cache */
    for (i=0; i<idtSaveRestore_ValidRegisterTableSize; i++)
    {
        saveCache[i] = IDTHal_Read(hdlr, 
                                   idtSaveRestore_ValidRegisters[i]);
    }

    /* Output cache to file.
     *
     * Wondering why we would do this? Well it's because we output
     * the registers in reverse order then we read them. Don't ask me
     * why, it's the way the GUI does it and we don't want to mess
     * with that... 
     */
    for (i=idtSaveRestore_ValidRegisterTableSize; i>0; i--)
    {
        /* Skip excluded registers */
        if (idtSaveRestore_SkipRegister(idtSaveRestore_ValidRegisters[i-1],
                                        0, /* Page 0 */
                                        hdlr)) continue;
        
        outputFunc("Reg%03X = 0x%02X\n", 
                   idtSaveRestore_ValidRegisters[i-1],saveCache[i-1]);
    }

    /* Free allocated cache */
    OS_FREE(saveCache);
}

static void idtSaveRestore_SavePreDividers(T_idtOutputToFile outputFunc,
                                           T_idtDrvHdlr      hdlr,
                                           int               isVersion1)
{
    int i;
    T_osUint8 regVal;

    outputFunc("[Page0_PreDivider_Parameters]\n");
    for (i=1; i<=IDT_DRV_MAX_INPUT; i++)
    {
        int internalPort;

        internalPort = hdlr->deviceConfig_m->inputExt2IntXlate_ma[i];
        if (internalPort == 0) continue; /* Skip unsupported ports */
        if (internalPort < 3) continue; /* Skip AMI ports */

        /* Select input port */
        IDTHal_Write(hdlr, REG_PRE_DIV_CH_CNFG,internalPort);

        regVal = IDTHal_Read(hdlr, REG_PRE_DIVN_LOW_CNFG);
        outputFunc("IN%dL = 0x%02X\n",
                   i,regVal);
        regVal = IDTHal_Read(hdlr, REG_PRE_DIVN_HIGH_CNFG);
        outputFunc("IN%dH = 0x%02X\n",
                   i,regVal);
    }
}

static void idtSaveRestore_SaveT0(T_idtOutputToFile outputFunc,
                                  T_idtDrvHdlr      hdlr,
                                  int               isVersion1)
{
    int i; 
    T_idtDpllType dpllType;

    /* If DPLL #2 not supported then there is no need to do this
       section since it's done because we need to differentiate
       between DPLLs */
    dpllType = ((T_idtDpllType)
                hdlr->deviceConfig_m->pllExt2IntXlate_ma[DPLL_INSTANCE_2]);
    if (dpllType == Dpll_MAX) return;

    /* Select T0 DPLL */
    IDTHal_Write(hdlr, REG_T4_T0_SEL_CNFG,0x00);

    outputFunc("[Page0_T0_Parameters]\n");
    for (i=0; i<(sizeof(idtSaveRestore_T4Registers)/sizeof(T_osUint8)); i++)
    {
        T_osUint8 regVal = IDTHal_Read(hdlr, 
                                        idtSaveRestore_T4Registers[i]);
        /* Skip excluded registers */
        if (idtSaveRestore_SkipRegister(idtSaveRestore_T4Registers[i],
                                        0, /* Page 0 */
                                        hdlr)) continue;

        outputFunc("Reg%03X = 0x%02X\n", 
                   idtSaveRestore_T4Registers[i], regVal);
    }
}

static void idtSaveRestore_SaveT4(T_idtOutputToFile outputFunc,
                                  T_idtDrvHdlr      hdlr,
                                  int               isVersion1)
{
    int i; 
    T_idtDpllType dpllType;

    /* If DPLL #2 not supported then there is no need to dos this
       section since it's done because we need to differentiate
       between DPLLs */
    dpllType = ((T_idtDpllType)
                hdlr->deviceConfig_m->pllExt2IntXlate_ma[DPLL_INSTANCE_2]);
    if (dpllType == Dpll_MAX) return;

    /* Select T4 DPLL */
    IDTHal_Write(hdlr, REG_T4_T0_SEL_CNFG,0x10);

    outputFunc("[Page0_T4_Parameters]\n");
    for (i=0; i<(sizeof(idtSaveRestore_T4Registers)/sizeof(T_osUint8)); i++)
    {
        T_osUint8 regVal = IDTHal_Read(hdlr, 
                                        idtSaveRestore_T4Registers[i]);
        /* Skip excluded registers */
        if (idtSaveRestore_SkipRegister(idtSaveRestore_T4Registers[i],
                                        0, /* Page 0 */
                                        hdlr)) continue;

        outputFunc("Reg%03X = 0x%02X\n", 
                   idtSaveRestore_T4Registers[i], regVal);
    }
}


static void idtSaveRestore_SavePage1(T_idtOutputToFile outputFunc,
                                     T_idtDrvHdlr      hdlr,
                                     int               isVersion1)
{
    int i; 

    /* Select page 0 */
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 1);

    outputFunc("[Page1_Parameters]\n");
    for (i=0; i<(sizeof(idtSaveRestore_page1Registers)/sizeof(T_osUint8)); i++)
    {
        T_osUint8 regVal = IDTHal_Read(hdlr, 
                                        idtSaveRestore_page1Registers[i]);

        /* Skip excluded registers */
        if (idtSaveRestore_SkipRegister(idtSaveRestore_page1Registers[i],
                                        1, /* Page 1 */
                                        hdlr)) continue;

        outputFunc("Reg%03X = 0x%02X\n", 
                   idtSaveRestore_page1Registers[i], regVal);
    }
}

static void idtSaveRestore_SaveApll(T_idtOutputToFile outputFunc,
                                    T_idtDrvHdlr      hdlr,
                                    int               isVersion1)
{
    int i;
    T_osUint8 *saveCache = NULL;

    saveCache = (T_osUint8*)OS_MALLOC(idtSaveRestore_apllRegisterTableSize*
                                      sizeof(T_osUint8));

    if (!saveCache) 
    {
        OS_ERROR(("Uanble to allocate cache size %d\n",
                  idtSaveRestore_apllRegisterTableSize*
                  sizeof(T_osUint8)));
        return;
    }

    /* Reads APLL2 registers into cache */
    for (i=0; i<idtSaveRestore_apllRegisterTableSize; i++)
    {
        saveCache[i] = IDTHal_Read_Ext(hdlr,
                                       0x80|idtSaveRestore_apllRegisters[i]);
    }

    /* Output APLL2 cache into file */
    for (i=idtSaveRestore_apllRegisterTableSize; i>0; i--)
    {
        outputFunc("Reg%03X = 0x%02X\n", 
                   0x180|idtSaveRestore_apllRegisters[i-1],saveCache[i-1]);
    }

    /* Reads APLL1 registers into cache */
    for (i=0; i<idtSaveRestore_apllRegisterTableSize; i++)
    {
        saveCache[i] = IDTHal_Read_Ext(hdlr,
                                       idtSaveRestore_apllRegisters[i]);
    }

    /* Output APLL1 cache into file */
    for (i=idtSaveRestore_apllRegisterTableSize; i>0; i--)
    {
        outputFunc("Reg%03X = 0x%02X\n", 
                   0x100|idtSaveRestore_apllRegisters[i-1],saveCache[i-1]);
    }
}

void saveVersion1(T_idtOutputToFile outputFunc,
                  T_idtDrvHdlr      hdlr)
{
    idtSaveRestore_SaveHeaderVersion1(outputFunc, hdlr);
    outputFunc("[Parameters]\n");
    idtSaveRestore_SaveApll(outputFunc, hdlr, 1);
    idtSaveRestore_SavePage0T0(outputFunc, hdlr, 1);
}

void saveVersion2(T_idtOutputToFile outputFunc,
                  T_idtDrvHdlr      hdlr)
{
    idtSaveRestore_SaveHeaderVersion2(outputFunc, hdlr);
    outputFunc("[Page0_Parameters]\n");
    idtSaveRestore_SaveApll(outputFunc, hdlr, 0);
    idtSaveRestore_SavePage0T0(outputFunc, hdlr, 0);
    idtSaveRestore_SavePreDividers(outputFunc, hdlr, 0);
    idtSaveRestore_SaveT0(outputFunc, hdlr, 0);
    idtSaveRestore_SaveT4(outputFunc, hdlr, 0);
    idtSaveRestore_SavePage1(outputFunc, hdlr, 0);

    /* IMPORTANT - restore back page/Dpll selection registers */
    IDTHal_Write(hdlr, REG_PAGE_POINTER_CNFG, 0);
    IDTHal_Write(hdlr, REG_T4_T0_SEL_CNFG,0x00);
}
